Package: sebkc
Type: Package
Title: Surface Energy Balance and Crop Coefficient Evapotranspiration
        Estimation
Version: 1.0-6
Date: 2026-07-15
Authors@R: person(given = "George", family = "Owusu",
    email = "owusugeorge@ug.edu.gh", role = c("aut", "cre"))
Description: Computes and integrates surface energy balance components of
    evapotranspiration (ET), sensible heat (H), soil heat flux (G) and net
    radiation (Rn) into the Food and Agriculture Organization (FAO)
    Irrigation and Drainage Paper 56 (FAO-56) water balance model. The
    package can perform single crop coefficient (Kc), dual Kc and the
    integration of thermal-based evaporative fractions in a water balance
    model. The surface energy balance models include Two-Source Surface
    Energy Balance (TSEB) models, namely the Priestley-Taylor TSEB
    (TSEB-PT), the Penman-Monteith TSEB (TSEB-PM) and TSEB-Parallel, as
    well as One-Source Surface Energy Balance (OSEB) models, namely the
    Surface Energy Balance Algorithm for Land (SEBAL), Mapping
    Evapotranspiration at high Resolution with Internalized Calibration
    (METRIC), Surface Energy Balance Index (SEBI), Simplified Surface
    Energy Balance (SSEB) and Surface Energy Balance System (SEBS). Methods
    are described in Allen et al. (2007)
    <doi:10.1061/(ASCE)0733-9437(2007)133:4(380)> and Bastiaanssen et al.
    (1998) <doi:10.1016/S0022-1694(98)00253-4>.
Depends: R (>= 3.5.0), raster (>= 2.5), sp (>= 1.2)
Imports: gstat, stats, graphics, grDevices, utils
License: GPL-2
Encoding: UTF-8
URL: https://github.com/gowusu/sebkc
BugReports: https://github.com/gowusu/sebkc/issues
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-15 11:34:25 UTC; PC
Author: George Owusu [aut, cre]
Maintainer: George Owusu <owusugeorge@ug.edu.gh>
Repository: CRAN
Date/Publication: 2026-07-23 12:40:08 UTC
Built: R 4.6.0; ; 2026-07-23 14:38:50 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(ETo,default)
S3method(ETohr,default)
S3method(coldTs,default)
S3method(coldTs2,default)
S3method(coldhot,default)
S3method(hotTs,default)
S3method(hotTs2,default)
S3method(kc,default)
S3method(landsat578,default)
S3method(plot,kc)
S3method(sebal,default)
S3method(sebi,default)
S3method(sebkcstack,default)
S3method(sebs,default)
S3method(sseb,default)
S3method(ssebi,default)
S3method(tseb,default)
export(ETo)
export(ETohr)
export(biomass)
export(cal.kc)
export(coldTs)
export(coldTs2)
export(coldhot)
export(cropsebkc)
export(dl)
export(fit.kc)
export(fitting)
export(hotTs)
export(hotTs2)
export(invdist)
export(kc)
export(landsat578)
export(lonlatreproject)
export(mapmatrix)
export(rastercon)
export(sebal)
export(sebi)
export(sebkc.tryCatch)
export(sebkcstack)
export(sebs)
export(sseb)
export(ssebi)
export(tseb)
export(wdi)
export(weather)
export(writesebkc)
export(xyextract)
import(raster)
import(sp)
importFrom(grDevices,dev.new)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(gstat,gstat)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,pf)
importFrom(utils,data)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.csv)
importFrom(utils,write.table)

# Surface Energy Balance and Crop Coefficient Estimation with R

This R package computes and integrates surface energy balance into FAO56 crop water balance model

Specifically, the package can perform the following functions:

+   Single crop cofficient modelling
+   Dual crop cofficient modelling 
+   the integration of thermal-based Evaporative Fractions in a water balance model
+   The computation of Two Source Surface Energy Balance (TSEB) model such as such as TSEB-PT, 
    TSEB-PM and TSEB-Parallel 
+   The computation of One Source Surface Energy Balance (OSEB) models such as SEBAL, METRIC, SEBI, SSEB and SEBS 

## Installation
You can install the latest development version from github with,
 <pre><code>
 
  if (!require(devtools)) {
  install.packages("devtools")
  }

if (packageVersion("devtools") < 1.6) {
  install.packages("devtools")
  }
  if(!require(sp)){
  install.packages("sp")
}
if(!require(rgdal)){
  install.packages("rgdal")
}
if(!require(raster)){
  install.packages("raster")
}
if(!require(gstat)){
  install.packages("gstat")
}
if(!require(httr)){
  install.packages("httr")
  }
if(!require(sebkc)){
devtools::install_github("gowusu/sebkc")
  }
</code></pre>


**Author**: George Owusu <owusugeorge@ug.edu.gh> This is part of the author's PhD program.

#' @title Radiometric correction and processing of Landsat Images
#'
#' @description \code{landsat578} performs radiometric collection of 
#' landsat 5,7,and 8 images. It returns albedo, NDVI, 
#' fractional vegetation cover, surface temperature
#' SAVI,radiance and reflectance of the image
#' @param data An original folder containing Landsat data or
#' Raster \code{\link[raster]{brick}}  or raster 
#' \code{\link[raster]{stack}}. Arrange the bands in ascending order:
#'  c(band1,band2,band3,band4,band5,band6,band7) for Landsat 5 and 7.
#'  And a similar  order for Landsat 8.  On folder see \code{\link{sebkcstack}}
#'  See also details.
#' @param sensor Numeric. Type of landsat satellite; it takes 5,7, or 8
#' @param gain Band-specific sensor gain. It takes either "high" or "low"
#' @inheritParams sebal
#' @inheritParams sebkcstack
#' @param weights albedo bands weights for each band.
#' list as in order c(band1,band2,band3,band4,band5,band6,band7) 
#' for landsat 5 and 7.
#'  The default is "auto" where 
#' sebal weights are used.
#' @param grescale list as in order weights. Band-specific sensor values. 
#' The default for landsat 5 and 7 is from Chander et.al (2009).
#' For Landsat 8 use ML Radiance scale in the metadata;
#' i.e. Band specific multiplicative rescaling factor from the metadata 
#' (MTL file) (RADIANCE_MULT_BAND_x, where x is the band number).
#' @param brescale list. Band-specific sensor values.
#' The default for landsat 5 and 7 is from Chander et.al (2009).
#' for Landsat 8 use AL Radiance scale in the metadata;
#' i.e. Band specific multiplicative rescaling factor from the metadata 
#' (MTL file) (RADIANCE_MULT_BAND_x, where x is the band number). 
#' @param ESUNx list. The mean solar exo-atmospheric irradiance for each band.
#' It is used for reflectance correction. The default is taken from SEBAL manual
#' @param K1 Temperature constants for each Landsat image. 
#' Default is taken from landsat handbook
#' @param K2 Temperature constants for each Landsat image. 
#' Default is taken from landsat handbook
#' @param Rp path radiance in the 10.4-12.5 um band.
#' MODTRAN runs can accurately estimate this. 
#' The default is taken from Allen et al(2007) 
#' to be 0.91,
#' @param Rsky narrow band downward thermal radiation. 
#' MODTRAN runs can accurately estimate this.
#' The default is taken from Allen et al(2007) to be 1.32 
#' @param tNB narrow band transmissivity of air (10.4-12.5) um range.
#' MODTRAN runs can accurately estimate this.
#' The default is taken from Allen et al(2007) to be 0.866 
#' @param date DATE_ACQUIRED of the image, see metadata
#' @param Mp Reflectance scale for Landsat 8. Band specific multiplicative 
#' rescaling factor from the metadata (MTL file) (REFLECTANCE_MULT_BAND_x, 
#' where x is the band number).
#' @param Ap Reflectance scale for Landsat 8. Band specific additive rescaling
#' factor from the metadata (MTL file) (REFLECTANCE_ADD_BAND_x,
#' where x is the band number)
#' @author George Owusu
#' @details If you set the data to landsat folder that contains 
#' landsat band and meta data \code{\link{sebkcstack}} is used to
#' compute the parameters. The user must specify welev and may
#' use default parameters. 
#'  The data must always be a raster brick 
#' or stack. To use the default values please order bind the bands
#'  c(band1,band2,band3,band4,band5,band6,band7). 
#'  The defaults values are for landsat 5 and 7. 
#' For landsat 8, use the meta data. 
#' @return 
#'  \describe{
#' \item{albedo:}{ albedo values}
#' \item{Ts:}{ Surface Temperature}
#' \item{NDVI:}{ NDVI values}
#' \item{SAVI:}{ SAVI values}
#' \item{fc:}{ Fractional vegetation cover}
#' \item{radiance:}{ radiance values}
#' \item{reflectance:}{ TOA reflectance}
#' }
#' @references 
#'  \itemize{
#' \item Chander, G., Markham, B. L., & Helder, D. L. 2009. 
#' Summary of current radiometric calibration coefficients for 
#' Landsat MSS, TM, ETM+, and EO-1 ALI sensors. 
#' Remote Sensing of Environment, 113(5): 893-903.
#' \item Waters, R. 2002. SEBAL: Surface Energy Balance Algorithms for Land. 
#' Idaho Implementation, Advanced Training and Users Manual. Kimberly, Idaho: 
#' University of Idaho.
#' \item Survey, U. S. G. 2015. Landsat 8 (L8) data users handbook: 97p.
#'  from https://landsat.usgs.gov/documents/Landsat8DataUsersHandbook.pdf
#' }
#' @examples
#' \dontrun{
#' #Automatic detection of parameters
#' folder=system.file("extdata","stack",package="sebkc")
#' modauto=landsat578(data=folder, welev=362)
#' 
#' #Manual estimation with input parameters
#' landsat7_6feb2004=brick(system.file("extdata","landsat7_6feb2004.grd",
#' package="sebkc"))
#' mod=landsat578(data=landsat7_6feb2004,welev=317.1,sensor=7,gain="high",
#' sunelev=50.71154048,weights="auto",grescale="auto",brescale="auto",
#' ESUN="auto",K1=666.09,K2=1282.71,date="2002-2-6")
#' #acces albedo value
#' albedo=mod$albedo
#' }
#' @export
#' @rdname landsat578

landsat578<-function(data,welev,sensor=NULL,gain="low",
sunelev=NULL,date=NULL,weights="auto",grescale="auto",brescale="auto",
ESUNx="auto",Mp=NULL,Ap=NULL,K1=NULL,K2=NULL,Rp=0.91,Rsky=1.32,tNB=0.886,
DOY=NULL,clip=NULL,folder=NULL) UseMethod ("landsat578")
#' @export
#' @rdname landsat578
landsat578.default<-function(data=NULL,welev,sensor=NULL,gain="low",
                             sunelev=NULL,date=NULL,
weights="auto",grescale="auto",brescale="auto",ESUNx="auto",Mp=NULL,
Ap=NULL,K1=NULL,K2=NULL,Rp=0.91,Rsky=1.32,tNB=0.886,DOY=NULL,clip=NULL,folder=NULL){
  stack=NULL
  
  if(is.null(data)&&!is.null(folder)){
   data=folder 
  }
  if(is.null(data)&&is.null(folder)){
  return(message("Please provide data or folder path"))
      }
  folder=NULL

  if(inherits(data, "character")||inherits(data, "sebkcstack")){
    folder=data
    if(inherits(data, "sebkcstack")){
      file.info2=TRUE
    }else{
      file.info2=file.info(folder)[["isdir"]]
    }
    if(file.info2==TRUE&&!is.na(file.info2)){
      if(inherits(data, "sebkcstack")){
        stack=data
      }else{
        stack=sebkcstack(data,clip=clip) 
      }
      data=stack$data
      sensor=stack$sensor
      satellite=sensor
      sunelev=stack$sunelev
      date=stack$date
      #gain="high"
      grescale=stack$grescale
      brescale=stack$brescale
      Ap=stack$Ap
      Ap[is.na(Ap)]=0
      Mp=stack$Mp
      Mp[is.na(Mp)]=0
      if(sensor==8){
      K1=stack$K1
      K2=stack$K2
      K11=stack$K11
      K21=stack$K21
      }
        }
  }
  
satellite=sensor
thisdate=date
if(!is.numeric(DOY)){
DOY=strptime(thisdate,"%Y-%m-%d")$yday+1
if(is.na(DOY)){
  DOY=strptime(thisdate,"%Y-%m-%d")$yday+1
}
}



dr=1+0.033*cos(DOY*((2*pi)/365))
cos_teta=cos(((90-sunelev)/360)*2*pi)
tsw=0.75+2*1e-05*welev
weights5=c(0.293,0.274,0.233,0.157,0.033,0,0.011)
weights7=c(0.293,0.274,0.231,0.156,0.034,0,0.012)
weights8=c(0.293,0.274,0.231,0.156,0.034,0.012,0)
#landsat 8 weights from Silva et al (2016): 
#DOI: http://dx.doi.org/10.1590/1807-1929/agriambi.v20n1p3-8
weights8=c(0.300,0.277,0.233,0.143,0.035,0.012,0)


lowGrescale7=c(1.180709,1.209843,0.942520,0.969291,
0.191220,0.067087,0.066496)
lowBrescale7=c(-7.38,-7.61,-5.94,-6.07,-1.19,-0.07,-0.42)
ESUNx7=c(1997,1812,1533,1039,230.8,1,84.9)

highGrescale7=c(0.77874,0.798819,0.621654,0.639764,
0.12622,0.037205,0.043898)
highBrescale7=c(-6.98,-7.2,-5.62,-5.74,-1.13,3.16,-0.39)

##############################
Grescale5after=c(0.765827,1.448189,1.043976,0.876024,0.120354,
0.055376,0.065551)
Brescale5after=c(-2.29,-4.29,-2.21,-2.39,-0.49,1.18,-0.22)
ESUNx5=c(1983,1796,1536,1031,220,1,83.44)

Grescale5before=c(0.671339,1.322205,1.043976,0.876024,0.120354,
0.055376,0.065551)
Brescale5before=c(-2.19,-4.16,-2.21,-2.39,-0.49,1.18,-0.22)

if(satellite==7||satellite=="ETM+"||satellite=="7"){
ESUN=ESUNx7
weight=weights7
if(is.null(K1)){
K1=666.09
}
if(is.null(K2)){
  K2=1282.71
}
if(gain=="high"){
Grescale=highGrescale7
Brescale=highBrescale7
}
if(gain=="low"){
Grescale=lowGrescale7
Brescale=lowBrescale7
}
}
#thisdate=paste(year,month,day,sep="-")
beforedate = as.POSIXct("1991-12-31",format='%Y-%m-%d')
afterdate = as.POSIXct(thisdate,format='%Y-%m-%d')
timediff=as.numeric(difftime(afterdate,beforedate,units="days"))
if(satellite==5||satellite=="MSS"||satellite=="5"){
ESUN=ESUNx5
weight=weights5
if(is.null(K1)){
  K1=607.76
}
if(is.null(K2)){
  K2=1260.56
}

if(timediff>0){
Grescale=Grescale5after
Brescale=Brescale5after
}
if(timediff<0){
Grescale=Grescale5before
Brescale=Brescale5before
}
}
#Radiance
if(is.numeric(grescale)){
Grescale=grescale

}

if(is.numeric(brescale)){
Brescale=brescale

}
message("Computing Radiance: this may take several minutes........")

radiance=(Grescale*data)+Brescale

#reflection
if(ESUNx!="auto"){
ESUN=ESUNx
}

message("Computing Reflectance")

if(satellite==8||satellite=="8"){
weight=weights8
Mp[7]=0
Ap[7]=0
reflection=((Mp*data)+Ap)/sin(sunelev*(pi/180))
#plot(data)
#print(Ap)

}else{
  reflection=(pi*radiance)/(ESUN*cos_teta*dr)
}
#albedo weights
if(!is.numeric(weights)){
if(weights!="auto"){
weight=weights
}
}
weights=reflection*weight

message("Computing Albedo")
#albedo toa
albedo_toa=sum(weights)

#albedo
albedo=(albedo_toa-0.03)/(tsw^2)

message("Computing NDVI")

NDVI=(reflection[[4]]-reflection[[3]])/(reflection[[4]]+reflection[[3]])
fc=((NDVI-cellStats(NDVI,min))/(cellStats(NDVI,max)-cellStats(NDVI,min)))^2
NDVI=rastercon(NDVI< (-1),-1,rastercon(NDVI>1,1,NDVI))
message("Computing SAVI")
SAVI=(1.5*(reflection[[4]]-reflection[[3]]))/
(0.5+(reflection[[4]]+reflection[[3]]))
message("Computing LAI")
LAI=(5.434*SAVI)+1.5416
message("Computing emissivities")
eNB=rastercon( NDVI<0 & albedo<0.47,0.99,rastercon(LAI>=3,0.98,0.97+
                                                     (LAI*0.0033)))
#Rc=radiance[[6]]

Ts=NULL
#temperature in Kelvin
if(satellite==8)
{
radiance6=radiance[[7]]  
}else{
radiance6=radiance[[6]]  
}
if(!is.null(K1)&&!is.null(K2)){
  message("Computing corrected thermal radiance")
Rc=((radiance6-Rp)/tNB)-(1-eNB)*Rsky
message("Computing Surface temperature")
Ts=K2/(log(((eNB*K1)/Rc)+1))
}

factor<-list(albedo=albedo,Ts=Ts,NDVI=NDVI,SAVI=SAVI,fc=fc,radiance=radiance,
             reflectance=reflection,DOY=DOY,sunelev=sunelev,welev=welev,
             date=date,LAI=LAI,eNB=eNB,time=stack$time,date=stack$date,folder=folder )


factor$call<-match.call()

class(factor)<-"landsat578"
factor
}

#' Hourly FAO-56 and ASCE-EWRI Reference Evapotranspiration
#' 
#' The function calculates hourly or less than hour FAO-56 (grass) and 
#' ASCE-EWRI Reference (alfalfa) Evapotranspiration (ETo ETr). 
#' The "surface" parameter determines #' the type of ET. 
#' The function can also be used to scale hourly or less than hour actual evapotranspiration.
#' The daily ET is implemented in \code{\link{ETo}}.
#' @param Tmean Numeric. Hourly Mean Air Temperature in degree Celsius
#' @param RH Hourly Mean Relative Humidity in percent
#' @param Lz longitude of the centre of the local time zone [degrees west of Greenwich].
#' MUST BE POSITIVE
#' Lz = 0 degrees for Greenwich.
#' @param Lm longitude of the measurement site [degrees west of Greenwich]. MUST BE POSITIVE
#' @param t1 The length of the calculation period in hour; 1 for hour, 0.5 for 30 minutes
#' 0.25 for 15 minutes. 
#' @param time The midpoint of the time of measurement[hour]; for example
#'  time is 12.5 for a period between 12:00 and 13:00
#' @param ETins model. It takes ET model from \code{\link{sebal}} or \code{\link{tseb}} and
#' retrieve instantaneous ET estimates. This helps to estimate ET24, T24, E24, EF, Kc, EFs, EFc
#' @param Rn net radiation [MJ/m2/hr]
#' @param Rs solar radiation [MJ/m2/hr]
#' @param map spatial raster data for inverse distance interpolation
# @param scaling numeric. The type of scaling models to upscale instantanous ET to 
# daily ET. it takes 1 is Evaporative Fraction (Bastiaanssen et al 1998); 
# 2 for Sine Function  (Jackson et al 1983); 3 for Solar Radiation (French et al 2005),
# 4 for time scaling of alfalfa (Allen et al 2007); 5 for time scaling of grass ()
#' @inheritParams sebal
#' @inheritParams ETo 
#' @inheritParams tseb
#' @inheritParams weather
#' @seealso \code{\link{ETo}}
#' @details 
#' \code{\link{sebal}}, \code{\link{sebi}},\code{\link{sebs}},\code{\link{sseb}},
#' \code{\link{tseb}} 
#' @author George Owusu
#' @param period character. Time of day for the calculation: "daytime" or "nighttime". The default is "daytime".
#' @return 
#' \describe{
#' \item{ETo:}{hour reference ETo [mm/hr]}
#' \item{ETa:}{hour actual ET. It is not NULL if EF has a numeric value}
#' \item{ETc:}{hour crop ET. It is not NULL if Kc has a numeric value}
#' \item{Rn:}{ Net  Radiation  [W/m2] }
#' \item{Rnwm_2:}{ Net Radiation [MJ/m2/hr] }
#' \item{y:}{ psychrometric constant [kPa/degree Celsius]}
#' \item{slope:}{ slope of saturation vapour pressure 
#' curve [kPa/degree Celsius]}
#' \item{vpd:}{ Vapour pressure deficit[kPa]}
#' }
#' @examples
#' \donttest{
#' #FAO56 Example 19 14:00-15:00 h using Cd = 0.24 instead 0.34 in FAO56
#' ET1hr=ETohr(Tmean=38,RH=52,DOY=274,Lz=15,t1=1,time=14.5,Lm=16.25,latitude=16.22,uz=3.3,Rs=2.450)
#' ET1hr$ETo
#' }
#' @references
#' \itemize{ 
#' \item ALLEN, R. G., PEREIRA, L. S., RAES, D., & SMITH, M. 1998. 
#' Crop Evapotranspiration (guidelines for computing crop water requirements) 
#' FAO Irrigation and Drainage Paper No. 56: FAO.
#' 
#' \item Jackson, R. D., Hatfield, J. L., Reginato, R. J., Idso, S. B., & Pinter Jr, P. J. (1983). 
#' Estimation of daily evapotranspiration from one time-of-day measurements. 
#' Agricultural Water Management, 7(1-3), 351-362. doi: http://dx.doi.org/10.1016/0378-3774(83)90095-1
#' 
#' \item French, A. N., Fitzgerald, G., Hunsaker, D., Barnes, E., Clarke, T., Lesch, S., . . .
#'  Pinter, P. (2005). 
#' Estimating spatially distributed cotton water use from thermal infrared aerial imagery. 
#' Paper presented at the World Water Congress 2005: 
#' Impacts of Global Climate Change - Proceedings of the 2005 World Water and 
#' Environmental Resources Congress, Reston, Va.
#' 
#' \item Colaizzi, P. D., Kustas, W. P., Anderson, M. C., Agam, N., Tolk, J. A., 
#' Evett, S. R., . . . O'Shaughnessy, S. A. (2012). Two-source energy balance model
#'  estimates of evapotranspiration using component and composite surface temperatures. 
#'  Advances in Water Resources, 50, 134-151. 
#'  doi: http://dx.doi.org/10.1016/j.advwatres.2012.06.004
#'  }
#' @export
#' @rdname ETohr 
ETohr=function(Tmean,RH,DOY,Lz,t1,time,Lm,latitude,longitude=NULL,n=NULL,uz=2,Rs=NULL,
Rn=NULL,G=NULL,altitude=NULL,z=NULL, Krs=0.16,Tmax=NULL,Tmin=NULL,albedo=0.23,as=0.25,bs=0.5,
ETins=NULL,ETr24=NULL,wmo=NULL, airport=NULL,map=NULL,surface="grass",period="daytime")UseMethod ("ETohr")
#' @importFrom gstat gstat 
#' @import sp 
#' @export
#' @rdname ETohr 
ETohr.default=function(Tmean,RH,DOY,Lz,t1=1,time,Lm,latitude,longitude=NULL,n=NULL,uz=2,Rs=NULL,
  Rn=NULL,G=NULL,altitude=NULL,z=NULL, Krs=0.16,Tmax=NULL,Tmin=NULL,albedo=0.23,as=0.25,bs=0.5,
  ETins=NULL,ETr24=NULL,wmo=NULL, airport=NULL,map=NULL,surface="grass",period="daytime"){
  filedata=NULL
  timestep="hour"
  message("Computing ETr")
    if(!is.null(wmo)||!is.null(airport)){
   Tmean=sebkc.tryCatch(Tmean)$value
   
   if(inherits(Tmean, "simpleError")){
    Tmean=NULL 
   }
   if(is.null(Tmean$time)){
    time=time 
   }else{
     time=NULL
   }
   Tmean=sebkc.tryCatch(weather(data=Tmean,airport = airport,wmo=wmo, date=DOY,time=time))$value
   if(inherits(Tmean, "simpleError")){
    return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo",Tmean))) 
   }
   Tmean= Tmean$WMO$hour
   
     
  }
  
 
  
if(is.character(Tmean)||!is.null(names(Tmean))){
  
  if(!is.null(names(Tmean))){
    if(!is.null(Tmean$WMO$hour)){
      Tmean=Tmean$WMO$hour
    }
    filedata=Tmean 
  }else{
    filedata= read.csv(Tmean)
  }
   #print(names(filedata))
  if(!is.null(filedata$Tmean)){
    Tmean=filedata$Tmean
  }
   if(!is.null(filedata$RH)){
     RH=filedata$RH
   }
   if(!is.null(filedata$DOY)){
     DOY=filedata$DOY
   }
   
   if(!is.null(filedata$Lz)){
     Lz=filedata$Lz
   }
  if(!is.null(filedata$t1)){
    t1=filedata$t1
  } 
  if(!is.null(filedata$time)){
    time=filedata$time
  }  
   if(!is.null(filedata$uz)){
     uz=filedata$uz
   } 
   if(!is.null(filedata$latitude)){
     latitude=filedata$latitude
   } 
   if(!is.null(filedata$Lm)){
     Lm=filedata$Lm
   } 
   if(!is.null(filedata$z)){
     z=filedata$z
   } 
   if(!is.null(filedata$altitude)){
     altitude=filedata$altitude
   } 
   if(!is.null(filedata$Rs)){
     Rs=filedata$Rs
   }
   
   if(!is.null(filedata$Rn)){
     Rn=filedata$Rn
   }
   
   if(!is.null(filedata$n)){
     n=filedata$n
   }
   if(!is.null(filedata$Krs)){
     Krs=filedata$Krs
   }
  if(!is.null(filedata$Tmax)){
    Tmax=filedata$Tmax
  }
  if(!is.null(filedata$Tmin)){
    Tmin=filedata$Tmin
  }
   if(!is.null(filedata$albedo)){
     albedo=filedata$albedo
   }
   if(!is.null(filedata$as)){
     as=filedata$as
   }
   if(!is.null(filedata$bs)){
     bs=filedata$bs
   }
   if(!is.null(filedata$EF)){
     EF=filedata$EF
   }
   if(!is.null(filedata$Kc)){
     Kc=filedata$Kc
   }
  
  if(!is.null(filedata$timestep)){
    timestep=filedata$timestep
  }
  
  if(!is.null(filedata$surface)){
    surface=filedata$surface
  }
  
  if(!is.null(filedata$period)){
    period=filedata$period
  } 
  
  }  

  
#print(cbind(Tamax,Tamin))
#check hr or time
Cn=37
Cd=0.34
if(surface=="grass"){
  if(timestep=="hourly"||timestep=="hours"||timestep=="Hourly"||timestep=="hour"){
    Cn=37
    if(period=="nighttime"||period=="night"){
      Cd=0.96
    }else{
      Cd=0.24
    }
  }
}

if(surface=="ETr"||surface=="alfalfa"||surface=="Alfalfa"){
  if(timestep=="hourly"||timestep=="hours"||timestep=="Hourly"||timestep=="hour"){
    Cn=66
    
    if(period=="nighttime"||period=="night"){
      Cd=1.7
    }else{
      Cd=0.25
    }
  }else{
    Cn=1600
    Cd=0.38
  }
  
 
  
}

DOY2=DOY
if(!is.numeric(DOY)){
DOY=strptime(DOY,"%d-%m-%Y")$yday+1
if(is.na(DOY[1])){
  DOY=strptime(DOY2,"%Y-%m-%d")$yday+1
}
if(is.na(DOY[1])){
  DOY=strptime(DOY2,"%m/%d/%Y")$yday+1
}
if(is.na(DOY[1])){
  DOY=strptime(DOY2,"%m-%d-%Y")$yday+1
}
}

#print(DOY)
if(z!=2&&!is.null(z)){
u2=uz*(4.87/log((67.8*z)-5.42))
}else{
u2=uz
}
if(is.null(altitude)){
P=101.3
}else{
P=101.3*((293-(0.0065*altitude))/293)^5.26
}


slope=(4098*(0.6108*exp((17.27*Tmean)/(Tmean+237.3))))/(Tmean+237.3)^2
cp=1.013*10^-3
ratio=0.622
vaporization=2.45
y=(cp*P)/(ratio*vaporization)
es=0.6108*exp((17.27*Tmean)/(Tmean+237.3))
ea=es*(RH/100)
vpd=es-ea

#radiation
if(is.null(Rn)||is.null(Rs)){
lat=(pi/180)*latitude
Gsc=0.0820
dr=1+0.033*cos(((2*pi)/365)*DOY)
declination=0.409*sin((((2*pi*DOY)/(365)))-1.39)
ws=acos(-tan(lat)*tan(declination))
b=(2*pi*(DOY-81))/364
Sc=0.1645*sin(2*b)-0.1255*cos(b)-0.025*sin(b)
#w=(pi/12)*((time+(0.06667*(Lz-Lm)+Sc))-12)
w=(pi/12)*((time+0.06667*(Lz-Lm)+Sc)-12)
ws1=w-((pi*t1)/24)
ws2=w+((pi*t1)/24)
Ra=((12*60)/pi)*Gsc*dr*((ws2-ws1)*sin(lat)*sin(declination)+cos(lat)*cos(declination)*(sin(ws2)-sin(ws1)))
N=(24/pi)*ws
if(is.null(Rs)){
if(is.null(n)){
if(is.null(Tmax)||is.null(Tmin)||is.null(Krs)){
 message("daily Tmax, Tmin and Krs needed, please") 
}  
  Rs=Krs*sqrt(Tmax-Tmin)*Ra  
}else{  
  
  nN=n/N
#Rs=(as+(bs*nN))*Ra
#print(Rs)
#rc2=0.4560+(0.3566*nN)+(0.1874*((nN)^2))
rc=0.3139+(0.2480*nN)+(0.1276*((nN)^2))
#rc=0.1715+(0.8419*nN)+(-0.3206*((nN)^2))

if(n==0){
 rc=0.1811 
}
Rs=rc*Ra
}
#print(Rs)
#print(rc2*Ra)

}
#Rso=(0.75+(2*10^-5)*altitude)*Ra
Rso=(as+bs)*Ra
RsRso=Rs/Rso
Rns=(1-albedo)*Rs
stefan=2.043*10^-10
Rnl=(stefan*((Tmean+273.16)^4))*(0.34-(0.14*sqrt(ea)))*((1.35*RsRso)-0.35)
if(is.null(Rn)){
  Rn=Rns-Rnl
}
}
if(is.null(G)){
G=0.1*Rn
if(period=="night"||period=="nighttime"){
G=0.5*Rn  
}
}

wind=(1+(Cd*u2))
sloping=slope/(slope+(y*wind))
y2=y/(slope+y*(wind))
ES=0.408*(Rn-G)*sloping
yvpd=Cn/(Tmean+273.16)*u2*(vpd)*y2
ETo=ES+yvpd
ETc=NULL
#print(list(ES=ES,y2=y2,slope=slope,yvpd=yvpd,sloping=sloping,Cd=Cd,Cn=Cn))

ETa=NULL
E24=T24=EFc=EFs=EF=ET24=NULL
if(!is.null(ETins)){
  ETins.class=tolower(class(ETins))
  vaporisation=2430000
  if(ETins.class=="sebal"||ETins.class=="tseb"
     ||ETins.class=="metric"){
    if(!is.null(ETins$vaporisation)){
      #print(ETins)
      vaporisation=ETins$vaporisation
    } 
    
    ETint=(t1*3600*ETins$LE)/vaporisation
    
    EF=ETint/ETo

    if(ETins.class=="tseb"){
      Tint=(t1*3600*ETins$LEc)/vaporisation
      Sint=(t1*3600*ETins$LEs)/vaporisation
      #print(Tint)
      EFc=Tint/ETo
      EFs=Sint/ETo
      if(!is.null(ETr24)){
        E24=EFs*ETr24 
        T24=EFc*ETr24
      }
      }
  
  }else{
   EF=ETins/ETo 
  }
  if(!is.null(ETr24)){
    ET24=EF*ETr24 
  }
}
#print(paste("Cn",Cn,"Cd",Cd,"ETo",ETo))

if(!is.null(longitude)||!is.null(map)){
  if(length(longitude)>1){
    map=invdist(longitude,latitude,Rn,ext=map)
    Rn <- map$var
    map=invdist(longitude,latitude,ETo,ext=map)
    ETo <- map$var
  }
}
if(!is.null(filedata)){
  filedata$ETo=ETo  
  filedata$Rn=Rn  
  filedata$slope=slope  
  filedata$vpd=vpd 
  filedata$RH=RH  
  filedata$u2=u2
  filedata$Tmean=Tmean
  
}
factor<-list(ETo=ETo,Rn=11.6*Rn,Rs=Rs,G=G,Rn_mj_m2_hr=Rn,y=y,slope=slope,vpd=vpd,ETa=ET24,E24=E24,T24=T24,EFc=EFc,EFs=EFs,
             EF=EF,Kc=EF,xy=data,ETc=ETc,u2=u2,DOY=DOY2,
             latitude=latitude,Lm=Lm,data=filedata)
factor$call<-match.call()
class(factor)<-"ETohr"
factor
}

#file="C:/Users/GeoKings/Documents/George Owusu/UG PhD/literature/sebal/NASAweather1983-20052.csv"
#data=read.csv("C:/Users/GeoKings/Documents/George Owusu/UG PhD/literature/sebal/NASAweather1983-20052.csv")

#mod=ETo(file)
#mod=ETo(data)

#' 24 Hour FAO 56 and ASCE-EWRI Reference Evapotranspiration ETo
#'
#' @param Tmax Numeric. Maximum air Temperature in degree Celsius
#' @param Tmin Numeric. Minimum air Temperature in degree Celsius
#' @param latitude geographical coordinates in decimal degrees. It should be negative
#' for southern hemisphere
#' @param longitude The longitude of the measurement site 
#' i.e. geographical coordinates in decimal degrees for the weather station.
#'  It should be negative for West and positive for East.
#' @param Rs shortwave incoming radiation [MJ/m2/day]
#' @param z The height above surface where wind speed was measured, in metres. 
#' @param uz The wind speed in m/s
#' @param altitude The elevation of the weather station, in metres
#' @param RHmax Maximum Relative Humidity in percent
#' @param RHmin Minimum Relative Humidity in percent
#' @param n Sunshine hours 
#' @param Krs Relationship between the fraction of extraterrestrial radiation that reaches the 
#' earth's surface, Rs/Ra, and the air temperature difference Tmax - Tmin for
#' interior (Krs = 0.16) and coastal (Krs = 0.19) regions
#' @param albedo albedo values. For grass it is 0.23[dimensionless] 
#' @param as Regression constant, intercept, expressing the fraction of extra-terrestrial 
#' radiation reaching the earth on overcast days (n = 0); 
#' either a calibrated value or as = 0.25 can be used.
#' @param bs regression constant, slope, expressing 
#' the fraction of extraterrestrial radiation
#' reaching the earth on overcast days (n = 0); either a calibrated value or bs = 0.5 can be used.
#' @param Rn Net daily radiation in MJ/m2/day. If it is NULL, 
#' it will be computed
#' @param scale character The type of scale models to upscale instantaneous ET to 
#' daily ET. it takes "Evaporative Fraction" (Bastiaanssen et al 1998); 
#' "Sine Function"  (Jackson et al 1983); "Solar Radiation" (French et al 2005). 
#' Remember the blank spaces between the parameters. 
#' You can also respectively use "evaporative" or "solar" or "sine".
# ,4 for time scale of alfalfa (Allen et al 2007); 5 for time scale of grass ()
#' @param surface character. It is either "grass" or "alfalfa"
#' @param Kc Crop coefficient
#' @param G Daily Soil heat flux in W/m2 
#' @param EF Spatial. ET fraction [-] from \link{sebal} or \link{tseb} \link{sebs}.
#'  It is advisable to link it to any of the two functions. 
#' @inheritParams sebal
#' @inheritParams weather
#' @inheritParams ETohr
#' @seealso 
#' \code{\link{ETohr}}
#' @author George Owusu
#' @return 
#' \describe{
#' \item{ETo:}{ 24 hour reference ETo [mm/day]}
#' \item{ETa:}{ 24 hour actual ET. It is not NULL if EF has a numeric value}
#' \item{ETc:}{ 24 hour crop ET. It is not NULL if Kc has a numeric value}
#' \item{Rn:}{ Net daily Radiation  [W/m2] }
#' \item{Rn_mj_m2_day:}{ Net daily Radiation [MJ/m2/day]}
#' \item{y:}{ psychrometric constant [kPa/degree Celsius]}
#' \item{slope:}{ slope of saturation vapour pressure 
#' curve [kPa/degree Celsius]}
#' \item{vpd:}{ Vapour pressure deficit[kPa]}
#' }
#' @examples
#' \donttest{
#' #with minimal data
#' PET=ETo(Tmax=31,Tmin=28,latitude=5.6,DOY="6-2-2001")
#' PET$ETo
#' #with enough data as in FAO56 EXAMPLE 18 on 6 July in Uccle (Brussels, Belgium) 
#' #located at 50 degrees 48 minutes N and at 100 m above sea level
#' PET24=ETo(Tmax=21.5,Tmin=12.3,z=10,uz=2.78,altitude=100,
#' RHmax=84,RHmin=63,n=9.25,DOY=187,latitude=50.8)
#' PET24$ETo
#' }
#' @references ALLEN, R. G., PEREIRA, L. S., RAES, D., & SMITH, M. 1996. 
#' Crop Evapotranspiration (guidelines for computing crop water requirements) 
#' FAO Irrigation and Drainage Paper No. 56: FAO.
#' @export
#' @rdname ETo 
ETo=function(Tmax,Tmin,DOY,latitude,longitude=NULL,z=NULL,uz=2,
  altitude=NULL,RHmax=NULL,RHmin=NULL,n=NULL,Krs=0.16,albedo=0.23,
      as=0.25,bs=0.5,Rs=NULL,Rn=NULL,G=0,EF=NULL,wmo=NULL, airport=NULL,scale="sine function",Kc=NULL,
  surface="grass") UseMethod ("ETo")
#' @importFrom gstat gstat 
#' @import sp 
#' @export
#' @rdname ETo 
ETo.default=function(Tmax,Tmin,DOY,latitude,longitude=NULL,z=NULL,uz=2,altitude=NULL,
RHmax=NULL,RHmin=NULL,n=NULL,Krs=0.16,albedo=0.23,
as=0.25,bs=0.5,Rs=NULL,Rn=NULL,G=0,EF=NULL,wmo=NULL, airport=NULL,scale="evaporative fraction",Kc=NULL,
surface="grass"){
  message("Computing ETr24 or ETo24")
  thislat=sebkc.tryCatch(latitude)$value
  if(inherits(thislat, "simpleError")){
    return(message("latitude is needed"))
    
  }
  filedata=NULL
  timestep="24-h"
  period="daytime"
  
  if(!is.null(wmo)||!is.null(airport)){
    Tmax=sebkc.tryCatch(Tmax)$value
    
    if(inherits(Tmax, "simpleError")){
      Tmax=NULL 
    }
   #print(Tmax)
    Tmax=sebkc.tryCatch(weather(data=Tmax,airport = airport,wmo=wmo, date=DOY))$value
    if(inherits(Tmax, "simpleError")){
      return (message(Tmax)) 
    }
    Tmax= Tmax$WMO$day
    
  }
  #altitude=welev
if(is.character(Tmax)||!is.null(names(Tmax))){
  
  if(!is.null(names(Tmax))){
    if(!is.null(Tmax$WMO$hour)){
      Tmax=Tmax$WMO$day
    }
    filedata=Tmax 
  }else{
    filedata= read.csv(Tmax)
  }
   #print(names(filedata))
  Tmax=filedata$Tmax
   if(!is.null(filedata$Tmin)){
     Tmin=filedata$Tmin
   }
  
   
   if(!is.null(filedata$DOY)){
     DOY=filedata$DOY
   }
   
   
   if(!is.null(filedata$uz)){
     uz=filedata$uz
   } 
   if(!is.null(filedata$latitude)){
     latitude=filedata$latitude
   } 
   if(!is.null(filedata$longitude)){
     longitude=filedata$longitude
   } 
   if(!is.null(filedata$z)){
     z=filedata$z
   } 
   if(!is.null(filedata$altitude)){
     altitude=filedata$altitude
   } 
   if(!is.null(filedata$RHmin)){
     RHmin=filedata$RHmin
   }
   if(!is.null(filedata$RHmax)){
     RHmax=filedata$RHmax
   }
   if(!is.null(filedata$Rn)){
     Rn=filedata$Rn
   }
   
   if(!is.null(filedata$n)){
     n=filedata$n
   }
   if(!is.null(filedata$Krs)){
     Krs=filedata$Krs
   }
   if(!is.null(filedata$albedo)){
     albedo=filedata$albedo
   }
   if(!is.null(filedata$as)){
     as=filedata$as
   }
   if(!is.null(filedata$bs)){
     bs=filedata$bs
   }
   if(!is.null(filedata$EF)){
     EF=filedata$EF
   }
   if(!is.null(filedata$Kc)){
     Kc=filedata$Kc
   }
  
  if(!is.null(filedata$timestep)){
    timestep=filedata$timestep
  }
  
  if(!is.null(filedata$surface)){
    surface=filedata$surface
  }
  
  if(!is.null(filedata$period)){
    period=filedata$period
  } 
  
  }  

Tamax=Tmax
Tamin=Tmin
#print(cbind(Tamax,Tamin))
#check day or time
Cn=900
Cd=0.34
if(surface=="grass"){
  if(timestep=="hourly"||timestep=="hours"||timestep=="Hourly"||timestep=="hour"){
    Cn=37
    if(period=="nighttime"||period=="night"){
      Cd=0.96
    }else{
      Cd=0.24
    }
  }
}

if(surface=="ETr"||surface=="alfalfa"||surface=="Alfalfa"){
  if(timestep=="hourly"||timestep=="hours"||timestep=="Hourly"||timestep=="hour"){
    Cn=66
    
    if(period=="nighttime"||period=="night"){
      Cd=1.7
    }else{
      Cd=0.25
    }
  }else{
    Cn=1600
    Cd=0.38
  }
  
 
  
}

DOY2=DOY
if(!is.numeric(DOY)){
DOY=strptime(DOY,"%d-%m-%Y")$yday+1
if(is.na(DOY[1])){
  DOY=strptime(DOY2,"%Y-%m-%d")$yday+1
}

if(is.na(DOY[1])){
  DOY=strptime(DOY2,"%m/%d/%Y")$yday+1
}
if(is.na(DOY[1])){
  DOY=strptime(DOY2,"%m-%d-%Y")$yday+1
}
}

#print(DOY)
if(z!=2&&!is.null(z)){
u2=uz*(4.87/log((67.8*z)-5.42))
}else{
u2=uz
}
if(is.null(altitude)){
P=101.3
}else{
P=101.3*((293-(0.0065*altitude))/293)^5.26
}

Tmean=(Tamax+Tamin)/2

slope=(4098*(0.6108*exp((17.27*Tmean)/(Tmean+237.3))))/(Tmean+237.3)^2
cp=1.013*10^-3
ratio=0.622
vaporization=2.45
y=(cp*P)/(ratio*vaporization)
etmax=0.6108*exp((17.27*Tamax)/(Tamax+237.3))
etmin=0.6108*exp((17.27*Tamin)/(Tamin+237.3))
es=(etmax+etmin)/2
if(is.null(RHmax)||is.null(RHmin)){
ea=etmin
}else{
ea=((etmin*(RHmax/100))+(etmax*(RHmin/100)))/2
}


vpd=es-ea

#radiation
if(is.null(Rn)){
lat=(pi/180)*latitude
Gsc=0.0820
dr=1+0.033*cos(((2*pi)/365)*DOY)
declination=0.409*sin((((2*pi*DOY)/(365)))-1.39)
ws=acos(-tan(lat)*tan(declination))
Ra=((24*60)/pi)*Gsc*dr*((ws*sin(lat)*sin(declination))+(cos(lat)*cos(declination)*sin(ws)))
N=(24/pi)*ws
nN=n/N
if(is.null(Rs)){
if(is.null(n)){
  if(is.null(Tmax)||is.null(Tmin)||is.null(Krs)){
    message("daily Tmax, Tmin and Krs needed, please") 
  } 
Rs=Krs*sqrt(Tamax-Tamin)*Ra
}else{
Rs=(as+(bs*nN))*Ra
}
}
if(as==0.25&&bs==0.5&&!is.null(altitude)){
  Rso=(0.75+(2*10^-5)*altitude)*Ra  
}else{
  Rso=(as+bs)*Ra
}
#
RsRso=Rs/Rso
Rns=(1-albedo)*Rs
tmaxk=Tamax+273.16
tmink=Tamin+273.16
stefan=4.903*10^-9
Rnl=(stefan*(((tmaxk^4)+(tmink^4))/2))*(0.34-(0.14*sqrt(ea)))*((1.35*RsRso)-0.35)
Rn=Rns-Rnl
}
wind=(1+(Cd*u2))
sloping=slope/(slope+(y*wind))
y2=y/(slope+y*(wind))
ES=0.408*(Rn-G)*sloping
yvpd=Cn/(Tmean+273.16)*u2*(vpd)*y2
ETo=ES+yvpd
ETc=NULL
if(!is.null(Kc)){
 ETc=ETo*Kc 
}
ETa=NULL
if(!is.null(EF)){
  model=NULL
  EF.class=tolower(class(EF))
  if(!is.numeric(scale)){
   scale= tolower(scale)
   if(scale=="evaporative"||scale=="evaporative fraction"||scale=="fraction"){
  scale=1
   }
   if(scale=="sine"||scale=="sine function"||scale=="function"){
     scale=2
   }
   if(scale=="solar"||scale=="radiation"||scale=="solar radiation"){
     scale=3
   }
     }
  if(EF.class=="sebal"||EF.class=="tseb"
     ||EF.class=="metric"){
    model=EF
    if(!is.null(EF$EF)){
      EF=EF$EF
    }else{
      EF=1.1*(EF$LE/(EF$Rn-EF$G))
      } 
    
  }
  
  if(!is.null(longitude)||!is.null(EF)){
    if(length(longitude)>1){
    map=invdist(longitude,latitude,Rn,ext=EF)
    Rn <- map$var
    map=invdist(longitude,latitude,ETo,ext=EF)
    ETo <- map$var
    }
  }
  #Evaporative Fraction
if(scale==1){
  ETa=(EF*Rn)/2.45
}
  #Sine Function
  if(!is.null(model)){
    #print(model$LE)
  if(scale==2){
    ETa=((model$LE/model$Rs)*Rs)/2.45
  }
  #Solar Radiation
  if(scale==3){
    ETa=(EF*Rs*(model$Rn/model$Rs))/2.45
      }
  }
}
#print(paste("Cn",Cn,"Cd",Cd,"ETo",ETo))
if(is.null(RHmin)){
  RHmin=(etmin/etmax)*100
}

if(!is.null(filedata)){
  filedata$ETo=ETo  
  filedata$Rn=11.6*Rn  
  filedata$slope=slope  
  filedata$vpd=vpd 
  filedata$RHmin=RHmin  
  filedata$u2=u2
  
}
factor<-list(ETo=ETo,Rn=11.6*Rn,Rs=Rs,Rn_mj_m2_day=Rn,y=y,slope=slope,vpd=vpd,ETa=ETa,
             EF=EF,xy=data,ETc=ETc,u2=u2,RHmin=RHmin,DOY=DOY2,
             latitude=latitude,longitude=longitude,data=filedata,N=N,nN=nN,Ra=Ra,Rns=Rns,Rnl=Rnl)
factor$call<-match.call()
class(factor)<-"ETo"
factor
}

#file="C:/Users/GeoKings/Documents/George Owusu/UG PhD/literature/sebal/NASAweather1983-20052.csv"
#data=read.csv("C:/Users/GeoKings/Documents/George Owusu/UG PhD/literature/sebal/NASAweather1983-20052.csv")

#mod=ETo(file)
#mod=ETo(data)

#' @title Retrieval of Weather Data  from WMO and NASA Surface meteorology and Solar Energy
#' 
#' @description The function retrieves weather data from the World Meteorological Organization
#'  and NASA SSE. The retrieval is based on airports  IATA or ICAO codes or WMO weather station 
#'  codes. The actual WMO data is from Weather Underground . Internet connection  is therefore 
#'  needed before data can be accessed
#'
#' @param data dataframe that can contain all or part of the input data 
#' @param wmo numeric. World Meteorological Organization weather station code.
#' You can use the  Worldwide Station List at 
#' \url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
#' \url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]
#' @param airport numeric. IATA or ICAO code. They are alphabetically  listed at 
#' \url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]
#' @param date date in the form of "YYYY-mm-dd" for example "2016-01-01"
#' @param time time in the form of "H:M" or decimal hours only. For example "14:21" or 14.35
#' @param NASA.SSE list of  NASA SSE data dates in the form of "YYYY-mm-dd".
#' For example NASA.SSE=list(from=""2001-01-01"", to="2001-04-01"). Note that SSE date
#' is from 1983 to 2005: 
#' \url{https://power.larc.nasa.gov/}
#' [Accessed on 2016-5-6] 
#' if the  parameter is not provided, date parameter will be used 
#' provided longitude and latitude are provided.
#' @param folder The path of the folder where the data can be written. 
#' @inheritParams ETo
#' @author George Owusu
#'
#' @return WMO and NASE.SSE. The WMO contains hour and day data. The NASE.SSE contains 
#' meta and data. See example below.
#' @references 
#' \itemize{
#' \item Worldwide Station List.\url{https://www.wetterzentrale.de/} 
#' [Accessed on 2016-5-6] 
#' \item List of airports by IATA code. 
#' \url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code} [Accessed on 2016-5-6] 
#' \item Weather Underground \url{https://www.wunderground.com/} [Accessed on 2016-5-6]
#' }
#' @export
#'
#' @examples
#' \dontrun{
#' #all hours data at Lincoln Airport, Nabraska
#' lincoln=weather(airport = "KLNK", date="2011-08-04")
#' lincoln$WMO$hour
#' lincoln$WMO$day
#' #current hour data at London
#' london.now=weather(airport = "LON", date=format(Sys.time(), "%Y-%m-%d"),
#' time=format(Sys.time(), "%H:%M"))
#' london.now$WMO$hour
#' #NASA SSE data for Kumasi, Ghana
#' kumasi=weather(longitude = -1.5,latitude = 6.7,NASA.SSE = list(from="2001-04-01",
#' to="2002-04-01"))
#' 
#' #Hour, daily and SSE data at JFK 
#' JFK=weather(airport = "JFK", date="2001-02-02",time="17:20", latitude=40.6413, 
#' longitude=73.7781)
#' JFK$WMO$hour
#' JFK$WMO$day
#' JFK$NASA.SEE$data
#' JFK$NASA.SEE$meta
#' 
#' #' #Hour, daily and SSE slice data at Kotoka International Airport, ACCRA 
#' accra=weather(airport = "ACC", date="2001-02-02",time="17:20", latitude=40.6413, longitude=73.7781, 
#' NASA.SSE=list(from="2001-01-01",to="2004-01-01"))
#' accra$WMO$hour
#' accra$WMO$hour
#' accra$NASA.SEE$data
#' accra$NASA.SEE$meta
#' 
#' #Using wmo code
#' kumasi.now=weather(wmo="65442", date=format(Sys.time(), "%Y-%m-%d"),time=format(Sys.time(), 
#' "%H:%M"))
#' kumasi.now$WMO$hour
#' 
#' #retrieve daily data for a specific period i.e 1 month
#' lincoln=weather(airport = "KLNK", date=c("2011-08-04","2011-09-04"))
#' #kumasi and accra data for a time slice. May not work for different 
#' #geographical regions; write the results to a file
#' accrakumasi=weather(airport ="ACC", date=c("2011-08-04","2011-10-04"),
#' folder="C:/Users/george/Documents/")
#' }

#now data
#kumasi.now=weather(wmo="65442", date=format(Sys.time(), "%Y-%m-%d"),time=format(Sys.time(), "%H:%M"))
#kumasi.now$WMO$hour
weather=function (data=NULL,wmo=NULL, airport=NULL, date="YYYY-m-d",time=NULL,
  latitude=NULL,longitude=NULL,NASA.SSE=list(from="YYYY-m-d",to="YYYY-m-d"),folder=NULL){
  invisible(NULL)
  if(!is.null(data)){
    if(!is.null(data$date)){
      date=unique(data$date)
    }
    if(!is.null(data$time)){
      time=unique(data$time)
    }
    
    if(!is.null(data$wmo)){
      wmo=unique(data$wmo)
    }
    if(!is.null(data$airpot)){
      airpot=unique(data$airpot)
    }
    if(!is.null(data$latitude)){
      latitude=unique(data$latitude)
    }
    if(!is.null(data$longitude)){
      longitude=unique(data$longitude)
    }
    if(!is.null(data$NASA.SSE)){
      NASA.SSE=unique(data$NASA.SSE)
    }
  }
  
 
  if(length(date)>1){
    date=c(date[1],date[length(date)]) 
  }
 
  if(length(wmo)>1||length(airport)>1){
    
    if(length(wmo)>1){
      ID=wmo
    }else{
      ID=airport
    }
    
   
    
   
     # if(!is.null(date$from)||!is.null(date$to)){
      #  date=seq(as.Date(date$from), as.Date(date$to), "days")
     # }else{
      #  
      #}
      
 hour=NULL
 day=NULL
    for (i in 1:length(ID)){
      if(length(wmo)==length(ID)){
        thiswmo=wmo[[i]]
        thisairport=NULL
      }else{
        thisairport=airport[[i]]
        thiswmo=NULL
      }
      if(length(time)>1){
        thistime=time[[i]]
        
      }else{
        thistime=time
      }
      
      if(length(latitude)>1){
        thislatitude=latitude[[i]]
        
      }else{
        thislatitude=latitude
      }
      if(length(longitude)>1){
        thislongitude=longitude[[i]]
        
      }else{
        thislongitude=longitude
      }
     
      model=weather(data=NULL,wmo=thiswmo, airport=thisairport, date=date,time=thistime,
         latitude=NULL,longitude=NULL,NASA.SSE=list(from="YYYY-m-d",to="YYYY-m-d"))
      #if(length(day)==length(model$WMO$day)){
      day=sebkc.tryCatch(rbind(day,model$WMO$day))$value
      #}
      hour=sebkc.tryCatch(rbind(hour,model$WMO$hour))$value
      
    }
 if(!is.null(folder)){
   write.csv(hour,paste(folder,"/",date,"/WMO.hour.csv",sep=""))
   write.csv(day,paste(folder,"/",date,"/WMO.day.csv",sep=""))
    }
 list(WMO=list(hour=hour,day=day))
 
    
  }else{
  NASA=NULL  
  meta=NULL
  
 if(!is.null(time)){
   if(is.character(time)&&grep(":",time)){
   time=strsplit(time,":")
   time=as.numeric(time[[1]][1])+(as.numeric(time[[1]][2])/60)
   }
   time=as.numeric(time)
 }
  
  date=as.character(date)
  date1=date
  if(date=="YYYY-mm-dd"){
    date=readline(prompt = "Enter Date in format YYYY-m-d:")
    #return(print("Date in format YYYY-mm-dd is needed"))
  }
  
  date=gsub("-","/",date)
#WMO data
  hour.data=NULL
  day.data=NULL
  station=airport
  if(!is.null(wmo)||!is.null(airport)){
  if(!is.null(airport)){
  if(length(date)<=1){
    
  url.hr=paste("https://www.wunderground.com/history/airport/",airport,"/", date,"/DailyHistory.html?format=1", sep="")
   url.day=paste("https://www.wunderground.com/history/airport/",airport,"/", date,"/CustomHistory.html?format=1", sep="")
  }else{
    date1a=date[1]
    date1b=date[2]
    date1=date1a
    enddate=strsplit(date1b,"/")[[1]]
    url.hr=paste("https://www.wunderground.com/history/airport/",airport,"/", date1a,"/DailyHistory.html?format=1", sep="")
    url.hr=paste("https://www.wunderground.com/history/airport/",airport,"/", date1a,"/DailyHistory.html?dayend=",enddate[3],"&monthend=",enddate[2],"&yearend=",enddate[1],"&format=1", sep="")
    url.day=paste("https://www.wunderground.com/history/airport/",airport,"/", date1a,"/CustomHistory.html?dayend=",enddate[3],"&monthend=",enddate[2],"&yearend=",enddate[1],"&format=1", sep="")
  }
  }
  if(!is.null(wmo)){
    station=wmo
    #65442 kumasi
    if(length(date)<=1){
      
      url.hr=paste("https://www.wunderground.com/history/airport/",wmo,"/", date,"/DailyHistory.html?format=1", sep="")
      url.day=paste("https://www.wunderground.com/history/airport/",wmo,"/", date,"/CustomHistory.html?format=1", sep="")
    }else{
      date1a=date[1]
      date1b=date[2]
      date1=date1a
      enddate=strsplit(date1b,"/")[[1]]
      url.hr=paste("https://www.wunderground.com/history/airport/",wmo,"/", date1a,"/DailyHistory.html?format=1", sep="")
      url.hr=paste("https://www.wunderground.com/history/airport/",wmo,"/", date1a,"/DailyHistory.html?dayend=",enddate[3],"&monthend=",enddate[2],"&yearend=",enddate[1],"&format=1", sep="")
      url.day=paste("https://www.wunderground.com/history/airport/",wmo,"/", date1a,"/CustomHistory.html?dayend=",enddate[3],"&monthend=",enddate[2],"&yearend=",enddate[1],"&format=1", sep="")
    }
   # url.hr=paste("https://www.wunderground.com/history/wmo/",wmo,"/", date,"/DailyHistory.html?format=1", sep="")
   # url.day=paste("https://www.wunderground.com/history/wmo/",wmo,"/", date,"/CustomHistory.html?format=1", sep="")
    }
  
   hour.data=sebkc.tryCatch(read.csv(url.hr,stringsAsFactors=FALSE))$value
   #print(hour.data)
   if(inherits(hour.data, "simpleError")){
     message(paste("WMO Hourly data not available for ",wmo,airport, "on", date ))
   }
   day.data=sebkc.tryCatch(read.csv(url.day,stringsAsFactors=FALSE))$value
   if(inherits(day.data, "simpleError")){
     message(paste("WMO Daily data not available for ",wmo,airport, "on", date ))
   }
   
     
     if(!inherits(day.data, "simpleError")){
       thisnames=names (day.data)
       #imperial units
       if(length(grep("TemperatureF",thisnames)>0)){
         day.data$Tmax=sebkc.tryCatch((as.numeric(day.data$Max.TemperatureF)-32)/1.8)$value
         day.data$Tmin=sebkc.tryCatch((as.numeric(day.data$Min.TemperatureF)-32)/1.8)$value
                  day.data$uz=as.numeric(day.data$Mean.Wind.SpeedMPH)*0.44704 
         day.data$P=as.numeric(as.character(day.data$PrecipitationIn)) *25.4
        # DOY=strptime(day.data[[1]],"%m/%d/%Y")
         day.data$DOY=strftime(strptime(day.data[[1]],"%m/%d/%Y"),"%Y-%m-%d")
       }else{
         #metric units
         day.data$Tmax=sebkc.tryCatch((as.numeric(day.data$Max.TemperatureC)))$value
         day.data$Tmin=sebkc.tryCatch((as.numeric(day.data$Min.TemperatureC)))$value
         #print("yes")
         
         #print(names(day.data))
         day.data$uz=as.numeric(day.data$Mean.Wind.SpeedKm.h)*0.27778 
         day.data$P=(day.data$Precipitationmm)
         day.data$P=as.numeric(as.character(day.data$P))
         day.data$DOY=as.character(day.data[[1]])
       }
       
       day.data$P[is.na(day.data$P)]=0.001
       day.data$RHmin=day.data$Min.Humidity 
       day.data$RHmax=day.data$Max.Humidity
       #day.data$DOY=as.character(as.Date(day.data[[1]],"%Y-%m-%d"))
       day.data$station=station
       day.data$longitude=longitude
       day.data$latitude=latitude
     }
   if(length(date)>1){
     day.data$station=station
     hour.data$station=station
     if(!is.null(folder)){
       write.csv(hour.data,paste(folder,"/",station,".WMO.hour.csv",sep=""))
       write.csv(day.data,paste(folder,"/",station,".WMO.day.csv",sep=""))
     }
     return(list(WMO=list(hour=hour.data,day=day.data),NASA.SEE=list(data=NASA,meta=meta)))
   }
   if(!is.null(time)){
     time=round(time)
     hour.data$time=round(
       as.numeric(format(strptime(hour.data[[1]], "%I:%M %p"), "%H")) +         # hours component
         (as.numeric(format(strptime(hour.data[[1]], "%I:%M %p"), "%M")) / 60),2  # rounded minutes component
     )

     #hour.data=hour.data[hour.data$time==time,]
     if(!inherits(hour.data, "simpleError")){
     hour.data$time2=abs(hour.data$time-time)
     thisort=sort(hour.data$time2,decreasing = FALSE)[1:2]
     hour.data2=hour.data[hour.data$time2==thisort[1]|hour.data$time2==thisort[2],]
     hour.data3= hour.data2[1,]
     for (i in 1:length(hour.data2)){
     mod=sebkc.tryCatch(lm(hour.data2[[i]]~hour.data2$time))$value
     if(inherits(mod, "simpleError")){
    output=paste(hour.data2[[i]][1],hour.data2[[i]][2],sep="-")  
     }else{
     slope=coef(mod)[[2]]
     intercept=coef(mod)[[1]]
     output=(slope*time)+intercept 
     }
     hour.data3[i]=output
     #hour.data3=rbind(hour.data3,output)
     #print(output)
     }
     hour.data=hour.data3
     hour.data$time2=NULL
     thisnames=names (hour.data)
     #imperial units
     if(length(grep("TemperatureF",thisnames)>0)){
     hour.data$Tmean=(as.numeric(hour.data$TemperatureF)-32)/1.8
     hour.data$uz=as.numeric(hour.data$Wind.SpeedMPH)*0.44704 
     hour.data$P=as.numeric(hour.data$PrecipitationIn)*25.4 
     }else{
       #print(hour.data)
       hour.data$Tmean=(as.numeric(hour.data$TemperatureC))
       if(length(grep("-Calm",hour.data$Wind.SpeedKm.h))>0){
         wind=as.numeric(strsplit(hour.data$Wind.SpeedKm.h,"-")[[1]][1])
       }else{
        wind= hour.data$Wind.SpeedKm.h
       }
       hour.data$uz=as.numeric(wind)*0.27778 
       hour.data$P=as.numeric(hour.data$Precipitationmm) 
     }
     
     hour.data$RH=hour.data$Humidity 
     hour.data$DOY=date1 
     hour.data$station=station
     hour.data$longitude=longitude
     hour.data$latitude=latitude
   }
   
   
   }
  
   #rint(day.data)
   if(!inherits(day.data, "simpleError")){
     thisnames=names (day.data)
     #imperial units
     if(length(grep("TemperatureF",thisnames)>0)){
   day.data$Tmax=sebkc.tryCatch((as.numeric(day.data$Max.TemperatureF)-32)/1.8)$value
   day.data$Tmin=sebkc.tryCatch((as.numeric(day.data$Min.TemperatureF)-32)/1.8)$value
   if(nrow(hour.data)>0){
     #hour.data$Tmin=NA
     hour.data$Tmin=sebkc.tryCatch(day.data$Tmin)$value
     hour.data$Tmax=sebkc.tryCatch(day.data$Tmax)$value
   }
   
   day.data$uz=as.numeric(day.data$Mean.Wind.SpeedMPH)*0.44704 
   day.data$P=(as.numeric(as.character(day.data$PrecipitationIn) ))*25.4
     }else{
       #metric units
       day.data$Tmax=sebkc.tryCatch((as.numeric(day.data$Max.TemperatureC)))$value
       day.data$Tmin=sebkc.tryCatch((as.numeric(day.data$Min.TemperatureC)))$value
       #print("yes")
       if(nrow(hour.data)>0){
         #hour.data$Tmin=NA
         hour.data$Tmin=sebkc.tryCatch(day.data$Tmin)$value
         hour.data$Tmax=sebkc.tryCatch(day.data$Tmax)$value
       }
       #print(names(day.data))
       day.data$uz=as.numeric(as.character(day.data$Mean.Wind.SpeedKm.h))*0.27778 
       day.data$P=(day.data$Precipitationmm)
       day.data$P=as.numeric(as.character(day.data$P))
     
     }
     #day.data$P=as.numeric(as.character(day.data$P))
     day.data$P[is.na(day.data$P)]=0.001
     day.data$RHmin=day.data$Min.Humidity 
     day.data$RHmax=day.data$Max.Humidity
     day.data$DOY=date1
     day.data$station=station
     day.data$longitude=longitude
     day.data$latitude=latitude
   }
   if(is.null(station)){
     station=NA
   }
  }
   #day
NASA.this=NULL
   #NASA SSE
   if(!is.null(latitude)||!is.null(longitude)||NASA.SSE$from!="YYYY-m-d"){
     NASA.this=TRUE
     if(NASA.SSE$from=="YYYY-m-d"){
       start.date=date
     }else{
       start.date= NASA.SSE$from
     }
     if(NASA.SSE$to=="YYYY-m-d"){
       end.date=date
     }else{
       end.date= NASA.SSE$to
     }
     start.date=gsub("/","-",start.date)
     end.date=gsub("/","-",end.date)
     start.date=strsplit(start.date,"-")[[1]]
     end.date=strsplit(end.date,"-")[[1]]
     start.date=as.numeric(start.date)
     end.date=as.numeric(end.date)
     
    url <- paste("https://eosweb.larc.nasa.gov/cgi-bin/sse/daily.cgi?email=skip%40larc.nasa.gov&step=1&lat=",
                 latitude,"&lon=",longitude,"&sitelev=&ms=",start.date[2],"&ds=",start.date[3],"&ys=",start.date[1],"&me=",end.date[2],"&de=",end.date[3],"&ye=",end.date[1],"&p=swv_dwn&p=avg_kt&p=clr_sky&p=clr_dif&p=clr_dnr&p=clr_kt&p=lwv_dwn&p=toa_dwn&p=PS&p=TSKIN&p=T10M&p=T10MN&p=T10MX&p=Q10M&p=RH10M&p=DFP10M&submit=Submit&plot=swv_dwn",sep="")
   html <- sebkc.tryCatch(paste(readLines(url), collapse="\n"))$value
   
   sensor=strsplit(html, "<a href=")
   link=paste("https://eosweb.larc.nasa.gov",strsplit(sensor[[1]][9],"\"")[[1]][2],sep="")
   meta=sebkc.tryCatch(read.delim(link,sep=" ",stringsAsFactors = FALSE,header=FALSE))$value[1:12,]
   #print(meta[1:12,])
   elevation=sebkc.tryCatch(as.numeric(read.delim(link,sep=" ",stringsAsFactors = FALSE,header=FALSE)[4,][10][[1]]))$value
   NASA=sebkc.tryCatch(read.table(file=link,skip=23,header=TRUE))$value 
   if(inherits(NASA, "simpleError")){
     message(paste("NASA SSE data not Available in ",start.date[1]))
   }
   NASA$altitude=elevation
   NASA$Tmax=NASA$T10MX
   NASA$Tmin=NASA$T10MN
   NASA$DOY=paste(NASA$YEAR,NASA$MO,NASA$DY,sep="-")
   NASA$Rs=(NASA$swv_dwn*41.666666666666664)*0.0864
   }
   
   if(!is.null(folder)){
     write.csv(hour.data,paste(folder,"/",station,".WMO.hour.csv",sep=""))
     write.csv(day.data,paste(folder,"/",station,".WMO.day.csv",sep=""))
     if(!is.null(NASA.this)){
       write.csv(NASA,paste(folder,"NASA.data.csv"))
       write.csv(day.data,paste(folder,"NASA.meta.csv"))
       write.csv(NASA,paste(folder,"/",station,".NASA.data.csv",sep=""))
       write.csv(meta,paste(folder,"/",station,".NASA.meta.csv",sep=""))
     }
     
   }
   list(WMO=list(hour=hour.data,day=day.data),NASA.SEE=list(data=NASA,meta=meta))
  }
}
#all hours 
#lincoln=weather(airport = "KLNK", date="2011-08-04")
#lincoln=weather(airport = "KLNK", date=format(Sys.time(), "%Y-%m-%d"),time=format(Sys.time(), "%H:%M"))
#london=weather(airport = "LON", date=format(Sys.time(), "%Y-%m-%d"),time=format(Sys.time(), "%H:%M"))

#lincoln$WMO$hour
#NASA SSE data
#kumasi=weather(wmo="65442", date="2016-05-05",longitude = -1.5,latitude = 6.7,NASA.SSE = list(from="2001-04-01",to="2002-04-01"))

#now data
#kumasi.now=weather(wmo="65442", date=format(Sys.time(), "%Y-%m-%d"),time=format(Sys.time(), "%H:%M"))
#kumasi.now$WMO$hour

#generic function
#default function
#' @title  R computation of Surface Energy Balance Algorithm for Land 
#' and METRIC
#' 
#' @description This R function computes Bastiaanssen (1998) surface energy 
#' balance components of evapotranspiration (ET), Sensible heat(H), 
#' soil heat flux(G) and Net Radiation(Rn).
#'
#' @author George Owusu
#' @inheritParams ETohr
#' @inheritParams ETo
#' @inheritParams sebkcstack
#' @inheritParams weather
#' 
#' @param albedo A RasterLayer data that has albedo values. 
#' You can also provide file path or location on your computer
#' @param Ts A RasterLayer data that indicates radiometric 
#' surface temperature values from remote sensing image, 
#' preferably in Kelvin (K). You can also point to raster file on your 
#' computer.
#' @param NDVI A RasterLayer data that indicates NDVI 
#' (Normalized Difference Vegetation Index) values. You can also point to a 
#' file on your computer.
#' @param SAVI A RasterLayer data that indicates 
#' SAVI(Soil-adjusted Vegetation Index) values. 
#'  You can also point to file on your computer.
#' @param t1 numeric. The length of the calculation period in hour; 1 for hour, 0.5 for 30 minutes 0.25 for 15 minutes
#' @param xycold numeric or "auto". A list of x and y coordinates of a 
#' cold pixel in the form of c(x,y). If it set to "auto", \code{\link{coldTs}} 
#' will be used to compute it.
#' @param xyhot numeric or "auto". A list of x and y coordinates of a 
#' hot pixel in the form c(x,y). If it is set to "auto", \code{\link{hotTs}} 
#' will be used to compute it.
#' @param DOY Numeric or Date [YYYY-mm-dd]. Day of the Year. 
#' If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY
#' @param sunelev Angle of Sun elevation in degrees, 
#' as found in the meta data of the satellite image
#' @param welev Weather station elevation in meters
#' @param zx The height above the weather station where the 
#' wind speed is measured [m]
#' @param u The satellites  overpass time wind speed at the weather station [m/s]
#' @param zomw the roughness length for the weather station surface [m]
#' @param zom the momentum roughness length for each pixel [m]. 
#' You can also point to a raster file on your computer
#' @param LAI Leaf Area Index, dimensionless
#' @param DEM A digital elevation model[m]
#' @param lapse A local lapse rate that is applied to correct DEM [K/m]. 
#' Default value is 0.0065
#' @param Rn24 A 24 hour net radiation [W/m2]. It is needed to estimate daily ET. 
#' It can be computed with \code{\link{ETo}} with even a minimal data of 
#' Tmax and Tmin. 
#'  You can also point to raster file on your computer
#' @param ETr an alfalfa or grass reference ET at the time of satellites  overpass. 
#' It only needed if you set model to "metric" [mm/hr]
#' @param ETr24 daily grass or alfalfa reference Evapotranspiration (mm/day)
#' @param model character. The type of model. It takes either "SEBAL" or "METRIC"
#' @param iter.max maximum iterations of sensible heat calculation.
#' @param folder An original  directory of the images that contains all 
#' landsat 5,7 or 8 bands and metadata.
#' At the moment only Landsat folder is supported 
#' @return 
#' \describe{
#' \item{LE:}{  latent heat flux [W/m2]}
#' \item{Rn:}{ Net Radiation [W/m2]}
#' \item{H: }{ Sensible Heat Flux [W/m2]}
#' \item{G:}{ Soil heat flux [W/m2]}
#' \item{ETins:}{ instantaneous ET if model = "METRIC" [mm/hr]}
#' \item{ET24:}{ 24 hour Evapotranspiration only if Rn24 is not NULL [mm/day]}
#' \item{ETrF:}{ METRIC ET fraction [dimensionless]}
#' \item{EF:}{  ET fraction [dimensionless]}
#' \item{Ta:}{ Air temperature = Ts-dT [K]}
#' \item{iter.coef:}{ Parameters from sensible heat iteration}
#' } 
#' @details 
#'  \describe{
#'  \item{\strong{Computing SEBAL or METRIC}}{\code{sebal} can compute either SEBAL(Bastiaanssen et al,1998)
#'  or METRIC (Allen et al 2007). In the case of METRIC, input data such as DEM,
#'  local lapse rate, and ETr must be provided. See Allen et al (2007) for more
#'  details.}
#'  \item{\strong{Using Landsat Folder as input data}}{Though this sebkc 
#'  function is not sensor dependent, one can point the
#'  folder parameter to a Landsat TM(5), ETM(7) or 8 folder that contains 
#'  landsat bands and metadata. One can also point the folder parameter 
#'  to object of  \code{\link{landsat578}} }
#'  \item{\strong{Automatic computation}}{When xycold and xyhot are set to "auto", about 80 per cent of the input
#'  image pixels are used to compute cold and hot pixels. This avoids using
#'  NAs in classification of pixels. }
#'  \item{\strong{Taking long time to Simulate?}}{If you point the folder to landsat files and 
#'  set xycold and xyhot to "auto", it will take about 2 hours for 
#'  the function to run. Without these "autos", it will take about 45
#'  minutes. It is advisable to perform 3 step analyses of using three 
#'  functions as follows: \code{\link{sebkcstack}}, \code{\link{landsat578}},
#'  \code{\link{hotTs}}, \code{\link{coldTs}}. You need not to go through this if you have your 
#'  albedo, Ts, NDVI, SAVI from MODIS, ASTER etc}
#'  }
#'  
#' @references 
#' \itemize{
#' \item Bastiaanssen, W. G. M., Menenti, M., Feddes, R. A., & Holtslag,
#'  A. A. M. 1998. A remote sensing surface energy balance algorithm for  
#'  land (SEBAL), part 1: formulation. Journal of 
#'  Hydrology: 212-213: 198-212.
#' \item Bastiaanssen, W. G. M., Pelgrum, H., Wang, J., Ma, Y., Moreno,
#'  J. F., Roerink, G. J., & van der Wal, T. 1998. A remote sensing 
#'  surface energy balance algorithm for land (SEBAL).: Part 2: 
#'  Validation. Journal of Hydrology, 212-213(0): 213-229.
#'  \item Allen, R., Tasumi, M., & Trezza, R. 2007. Satellite-Based 
#' Energy Balance for Mapping Evapotranspiration with Internalized 
#' Calibration (METRIC)-Model. Journal of Irrigation and Drainage 
#' Engineering, 133(4): 380-394.
#' \item Allen, R., Tasumi, M., Morse, A., Trezza, R., Wright, J., 
#' Bastiaanssen, W., Kramber, W., Lorite, I., & Robison, C. 2007. 
#' Satellite-Based Energy Balance for Mapping Evapotranspiration 
#' with Internalized Calibration (METRIC)-Applications. 
#' Journal of Irrigation and Drainage Engineering, 133(4): 395-406.
#' }
#' @import raster
#' @examples
#' \dontrun{
#' #use original landsat 8 data by specifying folder path
#' folder=system.file("extdata","stack",package="sebkc")
#' modauto=sebal(folder = folder,welev = 380,xycold="full",xyhot="full")
#' #plot ET fraction
#' plot(modauto$EF)
#' 
#' #Semi-auto
#' #load individual input files, for example from MODIS, ASTER etc
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
#' mod=sebal(albedo=albedo,Ts=Ts,NDVI=NDVI,SAVI=NULL,
#' iter.max=7,xyhot="full",xycold="full",
#' DOY=37,sunelev=50.71154048,welev=317.1,zx=10,
#' u=2,zomw=3,zom=NULL,LAI=LAI,DEM=NULL,
#' lapse=0.0065,Rn24=NULL,ETr=NULL,model="SEBAL")
#' #get latent heat flux
#' mod$LE
#' #evaporation fraction
#' mod$EF
#' 
#' #3 step simulation using landsat 7 with SLC OFF
#' ##step one: prepare the input data 
#' ### point to landsat 7 data with strips 
#' folder=system.file("extdata","slc",package="sebkc")
#' #####get the data stacked and remove strips
#' rawdata=sebkcstack(folder,remove_cloud = 2,gap_fill = TRUE)
#' ###### check if strips are removed
#' plot(rawdata$data)
#' #get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
#' welev=278
#'  data=landsat578(rawdata,welev=welev)
#'  #perform semi-auto simulation mod=sebal(data,welev=welev)
#' # Determine xyhot. Digitize polygon on the Ts map
#' modhot=hotTs(data,welev=welev,extent="auto",cluster=2)
#' #determine the cold. Digitize polygon on the Ts map
#' modcold=coldTs(data,welev=welev,extent="auto",cluster=2)
#' xyhot=modhot$xyhot
#' xycold=modcold$xycold
#' modsebal=sebal(folder=data,xyhot=xyhot,xycold=xycold,welev=welev)
#' }
#' @export 
#' @rdname sebal

sebal<-function(albedo,Ts,NDVI,SAVI,welev,xyhot="auto",xycold="auto",DOY=NULL,
sunelev,zx=10,u=2,zomw=2,zom=NULL,LAI=NULL,DEM=NULL,lapse=0.0065,Rn24=NULL,ETr=NULL,
ETr24=NULL,wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",
latitude=NULL,t1=1,time=NULL, Lz=NULL,Lm=NULL,
model="SEBAL",iter.max=7,clip=NULL,folder=NULL)UseMethod ("sebal")

#' @export 
#' @rdname sebal
sebal.default<-function(albedo=NULL,Ts=NULL,NDVI=NULL,SAVI=NULL,welev,xyhot="auto",
xycold="auto",DOY=NULL,sunelev=NULL,zx=10,u=2,zomw=2,zom=NULL,LAI=NULL,DEM=NULL,
lapse=0.0065,Rn24=NULL,ETr=NULL,ETr24=NULL,wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",
latitude=NULL,t1=1,time=NULL, Lz=NULL,Lm=NULL,model="SEBAL",iter.max=7,clip=NULL,folder=NULL)
{
  
  model=toupper(model)
  #if(!is.null(ETr)){
 #   model="METRIC"
 # }
  
  eNB=NULL
  if(is.null(folder)){
    folder="/nothing454782ghf7poi8r.hope"
  }
  
  if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(!inherits(albedo, "landsat578")){
    if(is.null(welev)){
        return(message("Please provide the parameter welev: 
                     the elavation of th weather station"))  
    }
    }
    if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
      if(inherits(albedo, "landsat578")){
        mod=albedo
        welev=mod$welev
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(folder=folder, welev=welev,clip=clip)
      }
      #mod=landsat578(data=folder, welev=welev)
      albedo=mod$albedo
      Ts=mod$Ts
      NDVI=mod$NDVI
      DOY=mod$DOY
      sunelev=mod$sunelev
      SAVI=mod$SAVI
      eNB=mod$eNB
      if(is.null(LAI)){
      LAI=mod$LAI
      }
    }
  
  if(is.null(ETr)||is.null(ETr24)||is.null(Rn24)){
  if(!is.null(wmo)||!is.null(airport)){
    mod=sebkc.tryCatch(mod)$value
    thisDOY=sebkc.tryCatch(mod$date)$value
    if(inherits(thisDOY, "simpleError")){
      thisDOY=DOY 
    }
    ETr24=sebkc.tryCatch(ETo(DOY=thisDOY,airport = airport,wmo=wmo,latitude=latitude,
                             z=zx,Krs =Krs,altitude=welev))$value
    if(inherits(ETr24, "simpleError")){
      return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo",ETr24))) 
    }
    Rn24=ETr24$Rn
    ETr24= ETr24$ETo
    
    thistime=sebkc.tryCatch(mod$time)$value
    if(inherits(thistime, "simpleError")){
      thistime=time 
    }
    #print(ETr24)
    if(is.null(ETr)){
    ETr=sebkc.tryCatch(ETohr(DOY=thisDOY,airport = airport,wmo=wmo,latitude=latitude,
    z=zx,Krs =Krs,altitude=welev,time= thistime,Lz=Lz,t1=t1,Lm=Lm))$value
    if(inherits(ETr, "simpleError")){
      return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo or time or Lz or Lm or t1",ETr))) 
    }
    #return(print(ETr$ETo))
    ETr=ETr$ETo
    }
  }
  }
 
  
  if(!is.numeric(DOY)&&!is.null(DOY)){
    DOY2=DOY
    DOY=strptime(DOY,"%d-%m-%Y")$yday+1
    if(is.na(DOY)){
      DOY=strptime(DOY2,"%Y-%m-%d")$yday+1
    }
  }
  if(!inherits(NDVI, "RasterLayer")){
    NDVI=raster(NDVI)
  }
  if(!inherits(Ts, "RasterLayer")){
    Ts=raster(Ts)
  }
  if(!inherits(albedo, "RasterLayer")){
    albedo=raster(albedo)
  }
  
  if(!inherits(DEM, "RasterLayer")&&!is.null(DEM)){
    DEM=raster(DEM)
  }
  if(!inherits(SAVI, "RasterLayer")&&!is.null(SAVI)){
    SAVI=raster(SAVI)
  }
  
  if(!inherits(zom, "RasterLayer")&&!is.null(zom)){
    zom=raster(zom)
  }
  
  if(!is.numeric(Rn24)){
  if(!inherits(Rn24, "RasterLayer")&&!is.null(Rn24)){
    Rn24=raster(Rn24)
  }
  }
  
  if(inherits(xyhot, "hotTs")){
    xyhot=xyhot$xyhot
  }
  
  if(inherits(xycold, "coldTs")){
    xycold=xycold$xycold
  }
  testcold=NULL
  
  if((xycold=="auto"||xycold=="Auto"||xycold=="AUTO"||xycold=="full"||xycold=="Full"||xycold=="FULL")&&!is.numeric(xycold)){
    xcold=tolower(xycold)
    ycold=tolower(xycold)
    testcold=TRUE
  }
  else{
    if(length(xycold)>2){
      testcold=TRUE 
    }else{
  xcold=xycold[1]
  ycold=xycold[2]
    }
  }
  testhot=NULL
  if((xyhot=="auto"||xyhot=="Auto"||xyhot=="AUTO"||xyhot=="FULL"||xyhot=="full"||xyhot=="Full")&&(!is.numeric(xyhot))){
  xhot=tolower(xyhot)
  yhot=tolower(xyhot)
  testhot=TRUE
  }else{
    if(length(xyhot)>2){
      testhot=TRUE 
    }else{
  xhot=xyhot[1]
  yhot=xyhot[2] 
    }
  }
  
  
  
  uw=u
  waterdensity=1000
  p=1.03547033
  cp=1004.14
  g=9.81
  k=0.41 #von Karman's constant
  z1=0.1 
  z2=2
  vaporisation=2430000
  dr=1+(0.033*cos(DOY*((2*pi)/365)))
  cos_teta=cos(((90-sunelev)/360)*2*pi)
  tsw=(0.75+2*0.00001*welev)^2
  ea=0.85*(-log(tsw))^0.09
  stefan=5.67E-08
  message("computing incoming shortwave solar radiation")
  Rs_in=1367*cos_teta*dr*tsw
  sunangle=sunelev
  if(is.null(DEM)){
    pressure=101.325
  }else{
    message("Computing air pressure with DEM")
    pressure=101.325*((293-(0.0065*DEM))/293)^5.26
    Ts=Ts+(lapse*DEM)
  }
  if(is.null(LAI)){
    LAI=(5.434*SAVI)+1.5416
    #LAI=-(log(0.69-SAVI)/0.59)/(0.91)
  }
  
  if(!is.null(testhot)){
    #print(xyhot)
    modhot=hotTs(Ts=Ts,NDVI=NDVI,albedo=albedo,DEM=DEM,
                 extent=xyhot,upper=0.80,lower=0.1)
      xhot=modhot$x
      yhot=modhot$y
    
  }
  if(!is.null(testcold)){
    modcold=coldTs(Ts=Ts,NDVI=NDVI,albedo=albedo,DEM=DEM,
                   extent=xycold,upper=0.95,lower=0.1,sunangle = sunangle)
    
      xcold=modcold$x
      ycold=modcold$y
    
    
  }
  if(is.null(eNB)){
    eNB=rastercon( NDVI<0 & albedo<0.47,0.99,rastercon(LAI>=3,0.98,0.97+(LAI*0.0033)))   
  }
 
  e0=rastercon( NDVI<0 & albedo<0.47,0.985,rastercon(LAI>=3,0.98,0.95+(LAI*0.01)))
  message("Computing Vaporaisation with Surface Temperature")
  vaporisation=(2.501-(0.00236*(Ts-273.15)))*10^6
  message("Computing outgoing longwave radiation")
  RL_out=e0*stefan*(Ts^4)
  Ts_cold=getValues(Ts)[cellFromXY(Ts,c(xcold,ycold))]
  message("Computing incoming longwave radiation")
  RL_in=ea*stefan*(Ts_cold^4)
  #Net radiation
  Rn=((1-albedo)*Rs_in)+RL_in-RL_out-((1-e0)*RL_in)
  #Soil Heat flux
  message("Computing Soil heat flux")
  G_by_Rn=(Ts-273.15)*(0.0038+0.007*albedo)*(1-0.98*NDVI^4)
  G_by_Rn[NDVI<0,]=0.5
  G_by_Rn[Ts-273.15<4&albedo>0.45,]=0.5
  G=G_by_Rn*Rn
  
  message("Computing surface roughness[zom]")
  #sensible Heat
  if(is.null(zom)){
    zom=0.018*LAI #surface roughness
  }
  u200=((uw*log(200))/zomw)/log(zx/zomw)
  u_star=(k*u200)/(log(200/zom))# friction velocity
  rah=log(z2/z1)/(u_star*k)#heat tranpsort
  df = data.frame(a= numeric(), b = numeric(), u_star = numeric(),
                  L = numeric(),pairhot=numeric(),paircold=numeric(),
                  rah_hot=numeric(),
                  rah_cold=numeric(),dThot=numeric(),dTcold=numeric(),
                  Hhot=numeric(),Hcold=numeric(),Ts_hot=numeric(),
                  Ts_cold=numeric(),
                  Rn_hot=numeric(),Rn_cold=numeric(),G_hot=numeric(),
                  G_cold=numeric(),
                  LEcold=numeric(),LEhot=numeric(),ETint_hot=numeric(),
                  ETint_cold=numeric())
  #df
  Ts_hot=getValues(Ts)[cellFromXY(Ts,c(xhot,yhot))]
  Rn_cold=getValues(Rn)[cellFromXY(Rn,c(xcold,ycold))]
  Rn_hot=getValues(Rn)[cellFromXY(Rn,c(xhot,yhot))]
  G_cold=getValues(G)[cellFromXY(G,c(xcold,ycold))]
  G_hot=getValues(G)[cellFromXY(G,c(xhot,yhot))]
  message("Computing sensible heat flux........................")
  
  i=1
  rah_hot_change=-10000
  coldDT=0
  while (i<=iter.max)
  {
    message(paste("Monin-Obukhov length iteration",i,"of",iter.max))
    
    #change in temerature at the hot pixel
    rah_hot=getValues(rah)[cellFromXY(rah,c(xhot,yhot))]
    rah_cold=getValues(rah)[cellFromXY(rah,c(xcold,ycold))]
    #print(paste("rah_cold",rah_cold))
    
    if(i>1){
      rah_hot_change=round((((rah_hot-rah2)/rah2)*100),2)
    }
    #rah_hot change
    hotDT=((Rn_hot-G_hot)*rah_hot)/(p*cp)
    
    #slope of the line
    a=(hotDT-coldDT)/(Ts_hot-Ts_cold)
    
    #b=hotDT-(a*Ts_hot)
    b=(hotDT-a)/Ts_hot
    #b=-a*Ts_cold
    b=hotDT-(a*Ts_hot)
    
    #the actual DT
    dT=(a*Ts)+b
    pair=(1000*pressure)/(1.01*((Ts-dT)*287))
    p=getValues(pair)[cellFromXY(pair,c(xhot,yhot))]
    
    #sensible heat
    H=(pair*cp*dT)/rah
    #LE=Rn-G-H
    H_hot=getValues(H)[cellFromXY(H,c(xhot,yhot))]
    H_cold=getValues(H)[cellFromXY(H,c(xcold,ycold))]
    
    if(model=="METRIC"){
      if(is.null(ETr)){
        return(message("ETr is needed"))
      }
      LE_cold=1.05*ETr*getValues(pair)[cellFromXY(pair,c(xcold,ycold))]
      H_cold=(Rn_cold-LE_cold-G_cold)/3600
      coldDT=(H_cold*rah_cold)/(getValues(pair)[cellFromXY(pair,
                        c(xcold,ycold))]*cp)
    }else{
      coldDT=(H_cold*rah_cold)/(getValues(pair)[cellFromXY(pair,
                                          c(xcold,ycold))]*cp)
    }
    u_star_hot=getValues(u_star)[cellFromXY(u_star,c(xhot,yhot))]
    
    L=-((p*cp*(u_star_hot^3)*Ts_hot)/(k*g*H_hot))
    x_200m=(1-(16*(200/L)))^0.25
    x_2m=(1-(16*(2/L)))^0.25
    x_01m=(1-(16*(0.1/L)))^0.25
    
    w_200m=ifelse(L<0,2*log((1+x_200m)/2)+log((1+x_200m^2)/2)-2*
                    atan(x_200m)+0.5*pi,(ifelse(L>0,-5*(200/L),0)))
    w_2m=ifelse(L<0,2*log((1+x_2m^2)/2),ifelse(L>0,-5*(2/L),0))
    w_01m=ifelse(L<0,2*log((1+x_01m^2)/2),ifelse(L>0,-5*(2/L),0))
    
    u_star=(u200*k)/(log(200/zom)-w_200m)
    rah=(log(z2/z1)-w_2m+w_01m)/(u_star*k)
    rah2=getValues(rah)[cellFromXY(rah,c(xhot,yhot))]
    
    df = rbind(df,data.frame(a= a, b = b, u_star = u_star_hot,L = L,
                             paircold=getValues(pair)[cellFromXY(pair,
                                  c(xcold,ycold))],pairhot=p,
                             rah_hot=rah_hot,rah_cold=rah_cold,
                             dThot=hotDT,dTcold=coldDT,
                             Hhot=H_hot,Hcold=H_cold,Ts_hot=Ts_hot,
                             Ts_cold=Ts_cold,
                             Rn_hot=Rn_hot,Rn_cold=Rn_cold,G_hot=G_hot,
                             G_cold=G_cold,
                             LEcold=Rn_cold-G_cold-H_cold,
                             LEhot=Rn_hot-G_hot-H_hot,ETint_hot=t1*3600*
                          ((Rn_hot-G_hot-H_hot)/(getValues(vaporisation)
                            [cellFromXY(vaporisation,c(xhot,yhot))])),
                             ETint_cold=t1*3600*((Rn_cold-G_cold-H_cold)/
                            (getValues(vaporisation)[cellFromXY
                              (vaporisation,c(xcold,ycold))]))))
    
    i=i+1
  }
  #H=Con(H<0,0,H)
  message("computing latent heat flux" )
  LE=Rn-G-H
  #LE=Con(LE<0,0,LE)
  ETrF=NULL
  ET24=NULL
  EF=NULL
  
  ETint=(t1*3600*LE)/vaporisation
  if(model=="METRIC"){
    EF=ETint/ETr
    EF[EF>1.1,]=1.1
    EF[EF<0,]=0
    
    if(!is.null(ETr24)){
     ET24=EF*ETr24 
    }
    
  }else{
    EF=1.1*(LE/(Rn-G))
    EF[EF<0,]=0
    EF[EF>1.1,]=1.1
    if(!is.null(Rn24)){   
      ET24=EF*Rn24*0.035
     }
  }

  factor=list(rah=rah, Rn=Rn, H=H, G=G,LE=LE,ETins=ETint,ETr=ETr,
       rah=rah,iter.coef=df,vaporisation=vaporisation,
       pair=pair,dT=dT,Ta=Ts-dT,wm=w_200m,EF=EF,ETrF=EF,ET24=ET24,
       folder=folder,DOY=DOY,G_by_Rn=G_by_Rn,Rs=Rs_in)
  
  factor$call<-match.call()
  
  class(factor)<-model
  factor
}

#generic function
#default function
#' @title  Automatic computation of cold pixel Radiometric Surface Temperature
#'
#' @description This function computes temperature of a cold pixel based on NDVI, 
#' Surface Temperature, DEM and albedo. It plots histograms indicating
#' the selected values. The main difference with \code{\link{coldTs2}} is that preference 
#' is given to temperature selection before considering NDVI and albedo selection.
#' @param sunangle sun angle (degree) above the horizon.
#' @param cluster numeric, a value indicating the number of 
#' clusters.
#' @param extent A subset of the raster file than include 
#' c(xmin, xmax, ymin, ymax). You can set it to "interactive"
#' where you can digitise polygon. You can set to "auto" where it will be
#' automatically generated. You can set it to "full" where all the image dimension
#' will be used. See layout parameter below for more details.
#' see \code{\link[raster]{extent}} 
#' @param upper The quantile probability at which NDVI should be included 
#' in the selection project. It ranges from 0-1. The default is 0.95
#' @param lower The quantile probability at which Ts should be included 
#' in the slection project.The default is 0.2
#' @param plot logical:TRUE or FALSE. To indicate if histograms or triangle 
#' should be plotted
#' @param layout character. It takes "portrait" or "landscape". If extent is set to 
#' "auto" the middle part of the image will be used so that NAs can be 
#' avoided. You indicate whether the automatic slection should indicate
#' west-east (landscape) or north-south direction (portraite)
#' @param draw interactive. The method of digitisation if extent is set "auto". 
#' It takes "poly" or "rect"
#' @param iter.max The number iterations that kmeans cluster should use.
#' @param seed optional single number. Passed to \code{\link[base]{set.seed}}
#'   before the k-means clustering so the pixel selection is reproducible.
#'   The default \code{NULL} leaves the random-number stream untouched.
#' @seealso \code{\link{hotTs}}
#' @inheritParams sebal
#' @inheritParams ETohr
#' @inheritParams weather
#' @inheritParams sebkcstack
#' @details The function first divides the selected area into a  
#' number of clusters. The mean and standard devaitions are
#' computed. The cluster means with the lowest standard devaitions
#' and qualifying for upper (NDVI) and lower (Ts) limits 
#' (see above) are slected. The provided upper and lower probabilities
#'  are also used to select the final Ts.
#' @return 
#' \describe{
#' \item{Ts:}{ The selected or input Ts [K]}
#' \item{Tscold:}{ The Temperature of cold pixe [K] }
#' \item{TC:}{ The average temperature of the cold pixels [K]}
#' \item{x:}{ The x cordinate of the cold pixel}
#' \item{y:}{ The y cordinate of the cold pixel}
#' \item{xycold: }{combined x and y coordinates}
#' \item{candidates:}{ The similiar candidates' pixels that can be used}
#' }
#' @examples
#' \dontrun{
#' #using landsat folder
#' folder=system.file("extdata","stack",package="sebkc")
#' modcold=coldTs(folder=folder,welev=170,extent="auto")
#'
#' #use object of  
#' folder=system.file("extdata","stack",package="sebkc")
#' data=landsat578(data=folder, welev=362)
#' modcold=coldTs(folder=data,welev=170,extent="auto",cluster=3)
#' 
#' modcoldTs=coldTs(data,welev=170,extent="auto",cluster=3)
#' 
#' #using input different input data
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' #simulate with default parameters without albedo
#' modcold=coldTs(Ts=Ts,NDVI=NDVI,sunangle=50.7)
#' #simulate with albedo but set extent to "auto" and cluster to 7
#' modcold=coldTs(Ts=Ts,NDVI=NDVI,albedo=albedo,sunangle=50,cluster=7,
#' extent="auto")
#' }
#' @author George Owusu
#' @references Allen, R. G., Burnett, B., Kramber, W., Huntington, J., 
#' Kjaersgaard, J., Kilic, A., Kelly, C., & Trezza, R. 2013. Automated 
#' Calibration of the METRIC-Landsat Evapotranspiration Process. 
#' JAWRA Journal of the American Water Resources Association, 49(3): 563-576.
#' @export
#' @rdname coldTs
 
coldTs<-function(Ts,NDVI,albedo=NULL,sunangle=NULL,DEM=NULL,
                cluster=8,extent="interactive",upper=0.95,
lower=0.2,plot=TRUE,layout="portrait",draw="poly",
folder=NULL,welev=NULL,clip=NULL,
iter.max=100,seed=NULL) UseMethod ("coldTs")
#' @export
#' @rdname coldTs
coldTs.default=function(Ts=NULL,NDVI,albedo=NULL,sunangle=NULL,DEM=NULL,
cluster=8,extent="interactive",upper=0.95,
lower=0.2,plot=TRUE,layout="portrait",draw="poly",folder=NULL,welev=NULL
,clip=NULL,
iter.max=100,seed=NULL){
  message("Selecting cold pixel")
  
  
  if(is.null(folder)){
    folder="nothing454782ghf7poi8r.hope"
  }
  
  
  if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
      if(inherits(Ts, "landsat578")){
        mod=Ts
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(data=folder, welev=welev)
    }
    
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    sunangle=mod$sunelev
    }
  
  if(is.null(sunangle)){
    return (message("sunangle is needed"))
  }
  
ext=extent
albedothre=0.02
if(!inherits(NDVI, "RasterLayer")){
NDVI=raster(NDVI)
}
if(!inherits(Ts, "RasterLayer")){
Ts=raster(Ts)
}
if(!inherits(albedo, "RasterLayer")&&!is.null(albedo)){
albedo=raster(albedo)
}

if(is.numeric(ext[1])){
Ts <-crop(Ts, ext)
}else{
if(ext=="auto"||ext=="full"||extent=="Full"||extent=="FULL"){
  if(ext=="full"||extent=="Full"||extent=="FULL"){
    ext=extent(Ts)
  }else{
extents=extent(Ts)
xrange=extents[2]-extents[1]
yrange=extents[4]-extents[3]
H=0.2
V=0.4
if(layout=="H"||layout=="horinzontal"||layout=="landscape"){
H=V
V=H
}
x40=V*xrange
xmin=extents[1]+x40
xmax=extents[2]-x40
y20=H*yrange
ymin=extents[3]+y20
ymax=extents[4]-y20
extents2=c(xmin,xmax,ymin,ymax)
Ts=crop(Ts,extents2)
ext=extent(Ts)
}
}else{
if(draw=="poly"||draw=="line"){
plot(Ts,main="click and draw a line or polygon on a plotand right-click and
     select 'stop'. If you are using RStudio click on Finish to stop")
ext=drawPoly()
}else{
plot(Ts,main="click on two places on this map to select Area of Interest.
     If you are using RStudio click on Finish to stop")
ext <- drawExtent() #draw a box by clicking
}
Ts <-crop(Ts, ext)
}
}
if(!is.null(DEM)){
DEM = crop(DEM, ext)
Ts=Ts+(0.0065*DEM)
}
NDVI2 <- crop(NDVI, ext)
if(!is.null(albedo)){
albedo2=crop(albedo, ext)
}


upper1=quantile(Ts,probs=0.3)
lower1=quantile(Ts,probs=0.05)
#Ts[Ts>upper1[[1]],]=NA
#Ts[Ts<lower1[[1]],]=NA

v <- na.omit(getValues(Ts))
if(exists(".Random.seed", envir = .GlobalEnv)){
oldseed <- get(".Random.seed", envir = .GlobalEnv)
on.exit(assign(".Random.seed", oldseed, envir = .GlobalEnv), add = TRUE)
}
if(!is.null(seed)) set.seed(seed)
Ts.kmeans <- kmeans(v, centers=cluster, iter.max = iter.max, nstart = 3)
  
kmeansraster <- raster(Ts)
i <- attr(v, "na.action")
#when Ts has no NA cells, na.omit() does not set "na.action" so i is NULL;
#(-NULL) raises "invalid argument to unary operator". Guard against it.
if(is.null(i)){
j <- 1:ncell(Ts)
}else{
j <- (1:ncell(Ts))[-i]
}
kmeansraster[j] <- Ts.kmeans$cluster
#Ts.kmeans <- kmeans(na.omit(Ts[]), cluster, iter.max = iter.max, nstart = 3)
#kmeansraster<-raster(Ts)
#kmeansraster[]<-Ts.kmeans$cluster
Tsmean=zonal(Ts, kmeansraster, 'mean') 
TsSD=zonal(Ts, kmeansraster, 'sd') 


NDVImean=zonal(NDVI2, kmeansraster, 'mean') 
NDVISD=zonal(NDVI2, kmeansraster, 'sd') 


#substract sd from the mean
NDVImean_sd=NDVImean
NDVImean_sd[,2]=NDVImean[,2]-NDVISD[,2]

#cold pixels AOI selection
NDVImax=NDVImean_sd[NDVImean_sd[,2]==max(NDVImean_sd[,2]),]
NDVImin=NDVImean_sd[NDVImean_sd[,2]==min(NDVImean_sd[,2]),]
NDVI95=quantile(NDVImean_sd[,2],  probs = upper,na.rm=TRUE)
NDVI95b=NDVImean_sd[NDVImean_sd[,2]>NDVI95,]
if(length(NDVI95b[1])==1){
Tsmean95=Tsmean[Tsmean[,1] %in%(NDVI95b[1]),]
Tsmean95SD=TsSD[TsSD[,1] %in%(NDVI95b[1]),]
Tsmean95_sd=Tsmean95
Tsmean95_sd[2]=Tsmean95[2]-Tsmean95SD[2]
Tsmean2_sda=quantile(Tsmean95_sd[2],  probs = lower)
Tsmean2b=Tsmean95_sd

}else
{
Tsmean95=Tsmean[Tsmean[,1] %in%(NDVI95b[,1]),]
Tsmean95SD=TsSD[TsSD[,1] %in%(NDVI95b[,1]),]
Tsmean95_sd=Tsmean95
Tsmean95_sd[,2]=Tsmean95[,2]-Tsmean95SD[,2]
Tsmean2_sda=quantile(Tsmean95_sd[,2],  probs = lower)
Tsmean2b=Tsmean95_sd[Tsmean95_sd[,2]<Tsmean2_sda[[1]],]
}


#cellStats(coldAOI,"mean")
##actual cold pixel calculation
coldAOIbin=kmeansraster==Tsmean2b[[1]]
###remove kmeansraster
coldAOINDVI=rastercon(coldAOIbin>0,NDVI2,0)
coldAOITs=rastercon(coldAOIbin>0,Ts,0)
coldAOINDVI[coldAOINDVI<=0]=NA
coldAOITs[coldAOITs<=0]=NA
coldAOITs0=quantile(coldAOITs,probs=lower)
coldAOINDVI95=quantile(coldAOINDVI,probs=upper)
coldAOINDVI95b=rastercon(coldAOINDVI>coldAOINDVI95[[1]],coldAOINDVI,0)
coldAOINDVI95c=rastercon(coldAOITs<coldAOITs0[[1]],coldAOINDVI95b,0)
coldAOINDVI95c[coldAOINDVI95c<=0]=NA
if(is.na(minValue(coldAOINDVI95c))&&is.na(maxValue(coldAOINDVI95c))){
coldAOINDVI95c=coldAOINDVI95b
coldAOINDVI95c[coldAOINDVI95c<=0]=NA
}
#correseponding Temperature
coldAOINDVI95cTs=rastercon(coldAOINDVI95c>=minValue(coldAOINDVI95c),coldAOITs,0)
coldAOINDVI95cTs[coldAOINDVI95cTs<=0]=NA
if(!is.null(albedo)){
abeldo_thresh=(0.001343*sunangle) +(0.3281* exp(-0.0188*sunangle))
albedo3=rastercon(Ts==coldAOINDVI95cTs,albedo2,0)
albedo3[albedo3<=0]=NA
#rm(albedo)
coldAOINDVI95cTsalbedo=rastercon(albedo3>=(abeldo_thresh-albedothre),coldAOINDVI95c,0)
coldAOINDVI95cTsalbedo2=rastercon(albedo3<=(abeldo_thresh+albedothre),coldAOINDVI95cTsalbedo,0)
coldAOINDVI95cTsalbedo2[coldAOINDVI95cTsalbedo2<=0]=NA
coldTs=rastercon(coldAOINDVI95cTsalbedo2>=minValue(coldAOINDVI95cTsalbedo2),coldAOINDVI95cTs,0)
coldTs[coldTs<=0]=NA
if(is.na(minValue(coldTs))&&is.na(maxValue(coldTs))){
coldTs=rastercon(coldAOINDVI95cTsalbedo>0,coldAOINDVI95cTs,0)
coldTs[coldTs<=0]=NA
}
if(is.na(minValue(coldTs))&&is.na(maxValue(coldTs))){
#print("yes")
coldTs=coldAOINDVI95cTs
}
coldTs[coldTs<=0]=NA
}else{
coldTs=coldAOINDVI95cTs
coldTs[coldTs<=0]=NA
}
p=data.frame(rasterToPoints(coldTs))
coldTsxy<-p[p$layer>0,]
#coldTsxy=p[,c("x","y")]
p3=p
if(nrow(p)>1){
p2= p[order(p$layer),]
p3=p2[round(nrow(p2)/2),] 
}
x=p3$x
y=p3$y
ndvi=getValues(NDVI2)[cellFromXY(NDVI2,c(x,y))]

if(plot==TRUE||plot==TRUE){
oldpar <- par(no.readonly = TRUE)
on.exit(par(oldpar), add = TRUE)
par(mfrow=c(1,2))
hist(Ts,main="Ts[Cold/Wet]",xlab="Temperature")
abline(v=p3,col="red")
hist(NDVI2,main="NDVI",xlab="NDVI")
abline(v=ndvi,col="red")
}
coldTs=p3$layer

factor<-list(x=x,y=y,xycold=c(x,y),Ts=Ts,NDVI=ndvi,Tscold=coldTs,
             p=p,candidates=p,TC=mean(p$layer),cluster=kmeansraster)

factor$call<-match.call()

class(factor)<-"coldTs"
factor
}

#' @title Computing Two Source Surface Energy Balance (TSEB) in R
#'
#' @description This function estimates evaporation (E), transpiration (T), 
#' and evapotranspiration(ET) of land surface. The computation is based on
#' Norman et al (1995) and Colaizzi et al(2012) models.   
#' @inheritParams sebal
#' @inheritParams ETo 
#' @inheritParams coldTs
#' @inheritParams ETohr
#' @inheritParams sebkcstack
#' @param TA Air temperature[K]. It can take "sebal" or 'Tmax' or 'Tmin' or numeric value.
#' When it is set to sebal, TA = Ts-dT. The 'Tmax' and 'Tmin' correspond to maximum and 
#' minimum temperature.
#' @param viewangle angle of view of the sensor in degrees
#' @param s numeric. Mean leaf size; it is four times the leaf 
#' area divided by the perimeter. The default is 2 (Grace et al., 1981).
#' @param C coefficient. Weighting a coefficient in the equation for leaf 
#' boundary layer. A value of 90 s^0.5/m (Norman et al, 1995).
#' @param xPT coefficient(s). The Priestley-Taylor equation coefficient (Norman et al, 2005).
#' It can take a single coefficient or in the form c(initial, minimum, increment). 
#' Example is xPT=c(1.26,1,0.1). It must be noted this is a decreasing increment 
#' from the initial value .i.e 1.26. The subsequent increment is only meant for 
#' a cell where LEs is negative. 
#' @param clump coefficient. A canopy clumping coefficient used for 
#' calculation of directional  fc (fractional cover of vegetation)
#' @param fg numeric. The fraction of LA1 that is green. If no information is
#'  available on fg, then it is assumed to be unity (1).
#' @param fc fraction (0-1).Directional vegetation cover. 
#' The default uses Norman et al(1995) fc equation 
#' @param hc numeric. it is canopy height[m]
#' @param network character. It takes either "parallel" (Norman et al, 1995)  or 
#' "series" (Colaizzi et al, 2012; Norman et al, 1995).
#' @param series character.The type of series. It takes "xPT" for 
#' Priestley-Taylor equation(Norman, 1995) or 
#' "PM" for Penman-Monteith equation (Colaizzi et al, 2012)
#' @param rc coefficient. Penman-Monteith equation 
#' coefficient (Colaizzi et al,2012). It can take a single coefficient 
#' or in the form c(initial, maximum, increment). 
#' Example is rc=c(0,1000,100). Unlike xPT, this is positively incremented.
#' @author George Owusu
#' @references 
#' \itemize{
#' \item Norman JM, Kustas WP, Humes KS (1995). A two-source approach for estimating 
#' soil and vegetation energy fluxes from observations of directional 
#' radiometric surface temperature. Agric For Meteorol 1995;77:263-93.
#' \item Colaizzi, Paul; Kustas, William P.; Anderson, Martha C.; 
#' Agam, Nurit; Tolk, Judy A.; Evett, Steven R.; Howell, Terry A.; 
#' Gowda, Prasanna H.; and O'Shaughnessy, Susan A., 
#' "Two-source energy balance model estimates of evapotranspiration using 
#' component and composite surface temperatures" (2012). Publications 
#' from USDA-ARS / UNL Faculty. Paper 1147.
#' http://digitalcommons.unl.edu/usdaarsfacpub/1147
#' \item Grace, J., 1981. Some effects of wind on plants. 
#' In: J. Grace, E.D. Ford and P.G. Jarvis (Editors), Plants and
#' their Atmospheric Environment. Blackwell Scientific, London, pp. 31-56.
#' \item Ronglin Tang, Zhao-Liang Li, Yuanyuan Jia, Chuanrong Li, 
#' Kun-Shan Chen, Xiaomin Sun & Jinyong Lou (2012): 
#' Evaluating one- and two-source energy balance models in
#' estimating surface evapotranspiration from Landsat-derived 
#' surface temperature and field
#' measurements, International Journal of Remote Sensing.
#' http://dx.doi.org/10.1080/01431161.2012.716529
#' \item KUSTAS, W. & ANDERSON, M. 2009. Advances in thermal infrared remote 
#' sensing for land surface modeling. Agricultural and Forest Meteorology, 149,
#'  2071-2081.
#'}
#' @details 
#' \describe{
#'  \item{\strong{Theory}}{This function separately computes LEs, LEs using
#'  Net radiation (Rn), sensible heat flux (H) and soil heat flux (G).
#'  The Rn was first computed with \code{sebal}. The soil net radiation computed
#'  as  \code{Rns=(sebal$Rn*exp(0.9*log(1-fc)))} . The canopy radiation was computed 
#'  as  \code{Rnc=sebal$Rn-Rns} . The G was computed from \code{sebal}. 
#'  The canopy sensible heat (Hc) and soil sensible heat (Hs) were computed
#'  based on corresponding canopy Temperature (Tc) and soil temperature (Tsoil).
#'  Two different approaches were employed to sensible heat fluxes. The first
#'   is Norman et al(1995) parallel network. The second is Norman (1995) 
#'   secant series but uses Priestley-Taylor equation (xPT).   }
#'  \item{\strong{Default Parameter}}{These parameters have been 
#'  set based on literature }
#'  }
#' @return 
#' \describe{
#' \item{LEc:}{ canopy latent heat flux [W/m2]}
#' \item{LEs:}{ Soil latent heat flux[W/m2]}
#' \item{T24:}{ 24 hour Transpiration [mm/day]}
#' \item{E24:}{ 24 hour Evaporation [mm/day]}
#' \item{ET24:}{ 24 hour Evapotranspiration [mm/day]}
#' \item{EF:}{ ET24 (daily Evapotranspiration) Evaporative Fraction. 
#' EF corresponds to the type of scaling method used: see \code{ETohr} for details}
#' \item{EFs:}{ E(Evaporation) Evaporative Fraction}
#' \item{EFc: }{ T (Transpiration) Evaporative Fraction}
#' } 
#' @examples
#' \dontrun{
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
#' Parallel=tseb(Ts=Ts,LAI=LAI,DOY=37,xyhot="full",
#' xycold="full",albedo=albedo,Tmax=31,
#' Tmin=28,RHmax=NULL,RHmin=NULL,zom=NULL,NDVI=NDVI,SAVI=NULL,hc=20,
#' DEM=NULL,viewangle=0,sunelev=50,welev=345,u=2,
#' s=2,C=90,lapse=0.0065,Rn24=NULL,zx=200,zomw=2,
#' xPT=1.3,clump=1,fg=1,fc="auto",network="parallel",
#' latitude=5.6,n=6.5)
#' 
#' series=tseb(Ts=Ts,LAI=LAI,DOY=37,xyhot="full",
#' xycold="full",albedo=albedo,Tmax=31,
#' Tmin=28,RHmax=NULL,RHmin=NULL,zom=NULL,NDVI=NDVI,SAVI=NULL,hc=20,
#' DEM=NULL,viewangle=0,sunelev=50,welev=345,u=2,
#' s=2,C=90,lapse=0.0065,Rn24=NULL,zx=200,zomw=2,
#' xPT=1.3,clump=1,fg=1,fc="auto",network="series",
#' latitude=5.6,n=6.5)
#' 
#' #using landsat folder parameters
#' folder=system.file("extdata","stack",package="sebkc")
#' tsebautoseries=tseb(folder=folder,welev=317,Tmax=31,Tmin=27,
#' latitude=5.6,n=6.5)
#' tsebautoparallel=tseb(folder=folder,welev=317,Tmax=31,Tmin=27,
#' latitude=5.6,n=6.5,network="parallel")
#' 
#' #3 step simulation using landsat 7 with SLC OFF
#' ##step one: prepare the input data 
#' ### point to landsat 7 data with strips 
#'
#' #####get the data stacked and remove strips
#' rawdata=sebkcstack(folder)
#' ###### check if strips are removed
#' plot(rawdata$data)
#' #get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
#' welev=278
#'  data=landsat578(rawdata,welev=welev)
#'  #perform semi-auto simulation 
#' # Determine xyhot. Digitize polygon on the Ts map
#' modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#' #determine the cold. Digitize polygon on the Ts map
#' modcold=coldTs(data,welev=275,extent="auto",cluster=2)
#' xyhot=modhot$xyhot
#' xycold=modcold$xycold
#' modTSEB2=tseb(data,Tmax=31,Tmin=28,
#' latitude=5.6,n=6.5)
#' #the same
#' modTSEB2=tseb(data,Tmax=33,Tmin=28,welev=345,xyhot=xyhot,xycold=xycold,
#' latitude=5.6,n=6.5)
#' #the same
#' modTSEB2=tseb(data,Tmax=33,Tmin=28,welev=345,xyhot=modhot,xycold=modcold,
#' latitude=5.6,n=6.5)

#' }
#' 

#' @export 
#' @rdname tseb
tseb=function(albedo,Ts,LAI,DOY,xyhot="auto",xycold="auto",TA='sebal'
,Tmax,Tmin,RHmax=NULL,RHmin=NULL,n=NULL,latitude=NULL,zom=NULL,NDVI,SAVI,hc=2,DEM=NULL,
viewangle=0,sunelev,welev,u=2,s=2,C=90,lapse=0.0065,Rn24=NULL,ETr=NULL,ETr24=NULL,wmo=NULL, 
airport=NULL,Krs=0.16,surface = "grass",time=NULL, Lz=NULL,Lm=NULL,t1=1,zx,zomw,
xPT=1.3,rc=c(70,1000,100),series="xPT",clump=1,fg=1,fc="auto",
network="parallel",clip=NULL,folder=NULL,iter.max = 7) UseMethod ("tseb")
#' @export 
#' @rdname tseb
tseb.default=function(albedo=NULL,Ts=NULL,LAI=NULL,DOY=NULL,
                      xyhot="auto",xycold="auto",TA='sebal',
Tmax,Tmin,RHmax=NULL,RHmin=NULL,n=NULL,latitude=NULL,zom=NULL,
NDVI,SAVI,hc=2,DEM=NULL,viewangle=0,sunelev,welev,u=2,
s=2,C=90,lapse=0.0065,Rn24=NULL,ETr=NULL,ETr24=NULL,wmo=NULL, 
airport=NULL,Krs=0.16,surface = "grass",
time=NULL, Lz=NULL,Lm=NULL,t1=1,zx=200,zomw=2,
xPT=c(1.26,0,0.1),rc=c(50,1000,100),series="PM",clump=1,fg=1,
fc="auto",network="series",clip=NULL,folder=NULL,iter.max = 7){
  PET=NULL
  thisTA=TA
  
  if(is.null(folder)){
    folder="/nothing454782ghf7poi8r.hope"
  }
  
  if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(!inherits(albedo, "landsat578")){
      
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
  }
    
    if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
      if(inherits(albedo, "landsat578")){
        mod=albedo
        welev=mod$welev
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(folder=folder, welev=welev,clip=clip)
      folder=mod
    }
    #mod=landsat578(data=folder, welev=welev)
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    DOY=mod$DOY
    sunelev=mod$sunelev
    SAVI=mod$SAVI
    if(is.null(LAI)){
      LAI=mod$LAI
    }
  }
  
  if(is.null(ETr)||is.null(ETr24)||is.null(Rn24)){
    
    if(!is.null(wmo)||!is.null(airport)){
      mod=sebkc.tryCatch(mod)$value
      thisDOY=sebkc.tryCatch(mod$date)$value
      if(inherits(thisDOY, "simpleError")){
        thisDOY=DOY 
      }
      ETr24=sebkc.tryCatch(ETo(DOY=thisDOY,airport = airport,wmo=wmo,latitude=latitude,
                               z=zx,Krs =Krs,altitude=welev))$value
      if(inherits(ETr24, "simpleError")){
        return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo",ETr24))) 
      }
      Rn24=ETr24$Rn
      Tmin=ETr24$data$Tmin
      Tmax=ETr24$data$Tmax
      PET=ETr24
      ETr24= ETr24$ETo
     
      thistime=sebkc.tryCatch(mod$time)$value
      if(inherits(thistime, "simpleError")){
        thistime=time 
      }
      #print(ETr24)
      if(is.null(ETr)){
        ETr=sebkc.tryCatch(ETohr(DOY=thisDOY,airport = airport,wmo=wmo,latitude=latitude,
                                 z=zx,Krs =Krs,altitude=welev,time= thistime,Lz=Lz,t1=t1,Lm=Lm))$value
        if(inherits(ETr, "simpleError")){
          return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo or time or Lz or Lm or t1",ETr))) 
        }
        #return(print(ETr$ETo))
        if(thisTA=="Tmean"){
        TA=273.15+ETr$data$Tmean
        #print(TA)
        }
        ETr=ETr$ETo
      }
    }
  }
  
#print(ETr24)
zu=zx  
U=u

a1=0.004
b1=0.012
d=0.65*hc
ZM=hc/8
ZM=0.018*LAI
cp=1004.14
iteration=FALSE
mod=sebal(albedo=albedo,Ts=Ts,NDVI=NDVI,SAVI=SAVI,iter.max=iter.max,
xyhot=xyhot,xycold=xycold,DOY=DOY,sunelev=sunelev,welev=welev,
zx=zx,u=u,zomw=zomw,zom=zom,LAI=LAI,DEM=DEM,lapse=lapse,
Rn24=Rn24,ETr=ETr,model="SEBAL",folder=folder)

if(is.null(PET)){
PET=ETo(Tmax=Tmax,Tmin=Tmin,z=zx,uz=u,altitude=welev,
RHmax=RHmax,RHmin=RHmin,n=n,DOY=mod$DOY,latitude=latitude)
}
Rn24=PET$Rn
y=PET$y
vpd=PET$vpd
slope=PET$slope
pair=mod$pair
G=mod$G
vaporisation=mod$vaporisation
#G=0
RA=mod$rah

if(thisTA=='sebal'||thisTA=='SEBAL'){
  TA=mod$Ta
}

if(thisTA=='Tmax'||thisTA=='TMAX'){
  TA=273.15+Tmax
}
if(thisTA=='Tmin'||thisTA=='TMIN'){
  TA=273.15+Tmin
}
#Ts=mod$Ts

pair=mod$pair
wm=mod$wm
ladt=0.5
angle=viewangle
message("Computing fc based on LAI, angle, clump, etc")

if(fc=="auto"||!is.numeric(fc)){
fc=min(1-exp((-ladt*clump*LAI)/(cos(angle*(pi/180)))),1)
}
message("Computing Soil surface net radiation")

Rns=(mod$Rn*exp(0.9*log(1-fc)))

G=mod$G_by_Rn*Rns
message("Computing canopy net radiation")
Rnc=mod$Rn-Rns
message("Computing transport resistance (Rs)and  wind speed above surface (Us)")

Uc=U*(log((hc-d)/ZM)/(log((zu-d)/ZM)-wm))
a=0.28*LAI^(2/3)*hc^(1/3)*s^(-1/3)
Us=Uc*exp(-1*a*(1-(0.05/hc)))
Rs=1/(a1+(b1*Us))
Udzm=Uc*exp(-1*a*(1-((d+ZM)/hc)))
Rx=(C/LAI)*((s/Udzm)^0.5)
steady=NULL
if(network=="series"){
  message("Network series started")
if(series=="PT"||series=="xPT"){
  message("series = xPT or PT:Priestley-Taylor")
  
  thisxPT=xPT
if(is.na(xPT[2])){
  thisxPT=c(xPT,xPT,xPT)
}
  H2=NULL
  i=thisxPT[1]
  while(i>=thisxPT[2]){
    message(paste("Trying PT=",i,"to correct soil surface latent heat"))
    
TClin=((TA/RA)+(Ts/(Rs*(1-fc)))+((Rnc*Rx)/(pair*cp))*
(1-(i*fg*(slope/(slope+y))))*((1/RA)+(1/Rs)+(1/Rx)))/
  ((1/RA)+(1/Rs)+(fc/(Rs*(1-fc))))
Tslin=TClin*(1+(Rs/RA))-((Rnc*Rx)/(pair*cp))*
  (1-(i*fg*(slope/(slope+y))))*(1+(Rs/Rx)+(Rs/RA))-(TA*(Rs/RA))
dTC=((Ts^4)-(fc*TClin^4)-((1-fc)*Tslin^4))/((4*(1-fc)*Tslin^3)*
    (1+(Rs/RA))+(4*fc*TClin^3))
Tc=TClin+dTC
Tsoil=(((Ts^4)-(fc*Tc^4))/(1-fc))^0.25
TAC=((TA/RA)+(Tsoil/Rs)+(Tc/Rx))/((1/RA)+(1/Rs)+(1/Rx))
Hs=pair*cp*((Tsoil-TAC)/Rs)
Hc=pair*cp*((Tc-TAC)/Rx)
H=pair*cp*((TAC-TA)/RA)
if(i==thisxPT[1]){
LEs=Rns-G-Hs
LEc=Rnc-Hc
#H2=H
}else{
  LEc=rastercon(LEs<=0,Rnc-Hc,LEc)
  #H2==rastercon(LEs<=0,H,H2)
  LEs=rastercon(LEs<=0,Rns-G-Hs,LEs)
}
#print(i)
i=i-thisxPT[3]

if(minValue(LEs)>0){
  i=thisxPT[2]-i 
  steady=TRUE
}
}
#LEs2=LEs
#LEs2[LEs2>0,]=NA
#H=H2
##if LEs<0
  if(is.null(steady)){
HsEso=Rns-G

TAClin=((TA/RA)+(Ts/(fc*Rx))-((1-fc)/(fc*Rx))*((HsEso*Rs)/(pair*cp))+
          (HsEso/(pair*cp)))/((1/RA)+(1/Rx)+((1-fc)/(fc*Rx)))
TE=TAClin*(1+(Rx/RA))-((HsEso*Rx)/(pair*cp))-((TA*Rx)/RA)
dTAC=((Ts^4)-(1-fc)*(((HsEso*Rs)/(pair*cp))+(TAClin))^4-(fc*(TE^4)))/
  ((4*fc*(TE^3))*(1+(Rx/RA))+(4*(1-fc))*(((HsEso*Rs)/(pair*cp))+(TAClin))^3)
TAC=TAClin+dTAC
Tsoil2=TAC+((HsEso*Rs)/(pair*cp))
Hs2=pair*cp*((Tsoil2-TAC)/Rs)
LEc=rastercon(LEs<=0,Rnc-Hc,LEc)
LEs=rastercon(LEs<=0,Rns-G-Hs2,LEs)

if(minValue(LEs)<0){
  LEs=rastercon(LEs<=0,0,LEs)
}
}
}
if(series=="PM"||series=="penman-monteith"||series=="pm"){
  thisrc=rc
  if(is.na(rc[2])){
    thisrc=c(rc,rc,rc)
  }
  #H2=NULL
  message("series = PM:Penman-Monteith")
  
  i=thisrc[1]
  while(i<=thisrc[2]){
    message(paste("Trying rc=",i,"to correct soil surface latent heat"))
    
y_star=y*(1+(i/RA))
#print(i)
TClin=((TA/RA)+(Ts/(Rs*(1-fc)))+(((Rx*y_star*Rnc)/(pair*cp*(slope+y_star)))-
((Rx/RA)*(vpd/(slope+y_star))))*((1/RA)+(1/Rs)+(1/Rx)))/((1/RA)+(1/Rs)+(fc/
(Rs*(1-fc))))
Tslin=TClin*(1+(Rs/RA))-(TA*(Rs/RA))-(((Rx*y_star*Rnc)/(pair*cp*(slope+y_star)))-
  ((Rx/RA)*(vpd/(slope+y_star))))*(1+(Rs/Rx)+(Rs/RA))

dTC=((Ts^4)-(fc*TClin^4)-((1-fc)*Tslin^4))/((4*(1-fc)*Tslin^3)*(1+(Rs/RA))+
                                              (4*fc*TClin^3))
Tc=TClin+dTC
Tsoil=(((Ts^4)-(fc*Tc^4))/(1-fc))^0.25
TAC=((TA/RA)+(Tsoil/Rs)+(Tc/Rx))/((1/RA)+(1/Rs)+(1/Rx))

Hs=pair*cp*((Tsoil-TAC)/Rs)
Hc=pair*cp*((Tc-TAC)/Rx)
H=pair*cp*((TAC-TA)/RA)
#LEc=Rnc-Hc

if(i==thisrc[1]){
  LEs=Rns-G-Hs
  LEc=Rnc-Hc
  #H2=H
}else{
  LEc=rastercon(LEs<=0,Rnc-Hc,LEc)
  #H2=rastercon(LEs<=0,H,H2)
  LEs=rastercon(LEs<=0,Rns-G-Hs,LEs)
  #LEc[LEs<=0,]=Rns-G-Hs
  #LEc[LEs<=0,]=Rnc-Hc
}

i=i+thisrc[3]
if(minValue(LEs)>0){
i=i+thisrc[2]  
steady=TRUE
}
  }
  
 
  
#LEs2[LEs2>0,]=NA
#H=H2
##if LEs<0
  if(is.null(steady)){
    
HsEso=Rns-G
          
TAClin=((TA/RA)+(Ts/(fc*Rx))+(HsEso/(pair*cp))-((HsEso*Rs)/(pair*cp))*((1-fc)/(fc*Rx)))/
  ((1/RA)+(1/Rx)+((1-fc)/(fc*Rx)))
TE=TAClin*(1+(Rx/RA))-(TA*(Rx/RA))-((HsEso*Rx)/(pair*cp))
dTAC=((Ts^4)-(1-fc)*(((HsEso*Rs)/(pair*cp))+(TAClin))^4-(fc*(TE^4)))/
  ((4*fc*(TE^3))*(1+(Rx/RA))+(4*(1-fc))*(((HsEso*Rs)/(pair*cp))+(TAClin))^3)
TAC=TAClin+dTAC
Tsoil2=TAC+((HsEso*Rs)/(pair*cp))
Hs2=pair*cp*((Tsoil2-TAC)/Rs)
LEs=rastercon(LEs<=0,Rns-G-Hs2,LEs)
if(minValue(LEs)<0){
LEs=rastercon(LEs<0,0,LEs)
}
}
}
}else{
  thisxPT=xPT
  if(is.na(xPT[2])){
    thisxPT=c(xPT,xPT,xPT)
  }
  #thisxPT=c(xPT[1],xPT[1],xPT[1])
 LEc2=NULL
 LEs2=NULL
 #H2=NULL
 #G2=NULL
  i=thisxPT[1]
  while(i>=thisxPT[2]){
    message(paste("Trying PT=",i,"to correct soil surface latent heat"))
    
LEc=i*fg*(slope/(slope+y))*Rnc
Hc=Rnc*(1-(i*fg*(slope/(slope+y))))
Tc=((Hc*RA)/(pair*cp))+TA
Tsoil=(((Ts^4)-(fc*Tc^4))/(1-fc))^0.25
Hs=pair*cp*((Tsoil-TA)/(Rs+RA))
if(i==thisxPT[1]){
LEs=Rns-Hs-G
}else{
LEs=rastercon(LEs<=0,Rns-G-Hs,LEs)
}
Hs=rastercon(LEs<0,Rns-G,Hs)
#Tsoil2=(Hs2/(pair*cp))*(Rs+RA)+TA
Tsoil=rastercon(LEs<0,(Hs/(pair*cp))*(Rs+RA)+TA,Tsoil)
#Tc2=(((Ts^4)-(1-fc)*Tsoil2^4)/fc)^0.25
Tc=rastercon(LEs<0,(((Ts^4)-(1-fc)*Tsoil^4)/fc)^0.25,Tc)
#Hc2=pair*cp*((Tc2-TA)/RA)
Hc=rastercon(LEs<0,pair*cp*((Tc-TA)/RA),Hc)
#LEc2=Rnc-Hc2
LEc=rastercon(LEs<0,Rnc-(pair*cp*((Tc-TA)/RA)),LEc)
Hc=rastercon(Hc>Rnc,Rnc,Hc)
Tc=rastercon(Hc>Rnc,((Hc*RA)/(pair*cp))+TA,Tc)
Tsoil=rastercon(Hc>Rnc,(((Ts^4)-(fc*Tc^4))/(1-fc))^0.25,Tsoil)
Hs=rastercon(Hc>Rnc,pair*cp*((Tsoil-TA)/(Rs+RA)),Hs)
G=rastercon(Hc>Rnc,Rns-Hs,G)
Rstar=(Ts-TA)/(((Tc-TA)/RA)+((Tsoil-TA)/(RA+Rs)))
#H=pair*cp*((Ts-TA)/Rstar)
H=Hs+Hc
if(i==thisxPT[1]){
  LEs2=Rns-G-Hs
  LEc2=Rnc-Hc
  #H2=H
  #G2=G
  #H3=H
}else{
  LEc2=rastercon(LEs2<=0,Rnc-Hc,LEc2)
  LEs2=rastercon(LEs2<=0,Rns-G-Hs,LEs2)
  
  #G2=rastercon(LEs2<=0,G,G2)
  #H2=rastercon(LEs2<=0,H,H2)
}


i=i-thisxPT[3]

  }
  #H=H2
  #G=G2
  LEc=LEc2
  LEs=LEs2
  if(minValue(LEs)<0){
    LEs=rastercon(LEs<=0,0,LEs)
  }
}
#print(LEs)
#print(spplot(stack(LEc,LEs)))
EFc=EFs=EF=NULL
#LE=mod$Rn-G-H
LE=LEc+LEs
#print(spplot(LE-LE2))
message("computing E, T and ET" )

if(is.null(ETr)){
EFc=1.1*(LEc/Rnc)
EFs=1.1*(LEs/(Rns-G))
#EF=1.1*((LEc+LEs)/((Rns+Rnc)-G))
EF=1.1*(LE/(mod$Rn-G))

#EF=mean(EFc)
EFc[EFc<0,]=0
EFc[EFc>1.1,]=1.1
EFs[EFs<0,]=0
EFs[EFs>1.1,]=1.1

EF[EF<0,]=0
EF[EF>1.1,]=1.1
ET24=NULL
T24=NULL
E24=NULL
if(!is.null(Rn24)){
  T24=max(EFc*Rn24*0.035,0)
  E24=max(EFs*Rn24*0.035,0)
  ET24=max(EF*Rn24*0.035,0)
}
}else{
  T24=(t1*3600*LEc)/vaporisation
  EFc=T24/ETr
  E24=(t1*3600*LEs)/vaporisation
  EFs=E24/ETr
  ET24=(t1*3600*LE)/vaporisation
  EF=ET24/ETr
  if(minValue(EF)<0){
    EF[EF<0,]=0
  }
  if(maxValue(EF)>1.1){
    EF[EF>1.1,]=1.1
  }
  
  if(minValue(EFs)<0){
    EFs[EFs<0,]=0
  }
  if(maxValue(EFs)>1.1){
    EFs[EFs>1.1,]=1.1
  }
  
  if(minValue(EFc)<0){
    EFc[EFc<0,]=0
  }
  if(maxValue(EFc)>1.1){
    EFc[EFc>1.1,]=1.1
  }
  
  if(!is.null(ETr24)){
    E24=EFs*ETr24
    T24=EFc*ETr24
    ET24=EF*ETr24
    }
}
factor<-list(LE=LE,LEc=LEc,LEs=LEs,T24=T24,E24=E24,ET24=ET24,EF=EF,
EFc=EFc,EFs=EFs,ETr=ETr,fc=fc,vaporisation=vaporisation,folder=folder,t1=t1,Rs=mod$Rs)
factor$call<-match.call()

class(factor)<-"tseb"
factor
}

###########################################
#rooting depth
#' @title Estimating FAO56 Single and Dual Crop coefficients with a water balance model in R
#' 
#' @description This function computes FAO56 crop coefficients and water balance. 
#' The output of the model includes crop evapotranspiration, water stress coefficient,
#' irrigation requirements, soil available water, etc. In addition to point estimation,
#' spatial modelling is integrated using Evaporation fractions from 
#' \code{\link{sebal}}, \code{\link{sebi}}, \code{\link{ETo}},  \code{\link{sseb}},  
#' \code{\link{sebs}}. \code{cal.kc} can be used to calibrate  the model.
#'
#' @param ETo Numeric. Reference evapotranspiration [mm]. This can be estimated with \code{\link{ETo}}
#' @param P numeric. Precipitation [mm] 
#' @param RHmin numeric. Minimum Relative Humidity [per cent] 
#' @param soil character. Name of soil texture as written in FAO56 TABLE 19
#' @param crop character. Name of the crop as written on 
#' FAO56 Table 11, TABLE 12 and TABLE 17 
#' @param I numeric. Irrigation [mm]
#' @param CNII numeric. The initial Curve Number of the study site. 
#' This can be calibrated with \code{cal.kc}.
#' @param u2 numeric. Wind speed of the nearby weather station [m/s] measured at 2m
#' @param FC numeric. soil water content at field capacity [-][m3 m-3]. 
#' This can be calibrated with \code{cal.kc}.
#' @param WP numeric. soil water content at wilting point [-][m3 m-3]. 
#' This can be calibrated with \code{cal.kc}.
#' @param h numeric. Maximum crop height [m]. See details below.
#' This can be calibrated with \code{cal.kc}
#' @param Zr numeric. Rooting depth.  See details below.  
#' This can be calibrated with \code{cal.kc}
#' @param Ze numeric. The depth of the surface soil layer that is subject to drying 
#' by way of evaporation [0.10-0.15 m]. This can be calibrated with \code{\link{cal.kc}}
#' @param kc list. The tabulated Kc values for single crop c(kcini,kcmid,kcend)
#' or  dual crop model c(kcbini,kcbmid,kcbend). To use with \code{\link{sebal}} or 
#' any of the Surface Energy Balance models, it should be linked as list(model1,model2,model3) or
#'  c(model1$EF,model2$EF,model3$EF) but not a mixture of both. 
#' This can be calibrated with \code{\link{cal.kc}}
#' @param p The depletion coefficient. This can be calibrated with \code{\link{cal.kc}}
#' @param lengths list. The lengths of the crop growth stages. It can take dates in the form of 
#' c(start of planting ('YYYY-mm-dd), start of rapid growth, 
#' start of mid season, start of maturity, harvesting) 
#' or in the form number of days: c(initial days, dev days, mid days, late days). 
#' See FAO56 TABLE 11. If performing time series simulation, different dates or 
#' one season dates can be set; for instance two seasons simulation can be of the form 
#' c(initial days, dev days, mid days, late days, initial days, dev days, mid days, late days) 
#' or just c(initial days, dev days, mid days, late days)
#' @param fw list or numeric or character. This can take different fw values
#' throughout the growing season. It can also take one numeric value and this will
#' be transform depending on the type of wetting. If the wetting event is only Precipitation,
#' it is set to 1; if it is only Irrigation it is set to the provided fw value. If it is
#' both Precipitation and Irrigation weights are assigned for calculation of fw. 
#' fw can also take a character such as "Drip", "Sprinkler", "Basin", "Border", 
#' "alternated furrows", "Trickle", "narrow bed" and "wide bed". 
#' In this case fw values from FAO56 are assigned. 
#' @param fc numeric. fraction of ground covered by tree canopy
#' @param kctype character. The type of model either "single" or "double". In spatial modelling involving
#' Evaporation Fraction, kctype can be set to "METRIC" or "SSEB" so that 
#' Actual Evapotranspiration will be estimated as ETo*EF else it will be
#' estimated with Rn as ETa=(EF*Rn)/2.45. Note that Rn is in MJm-2 day-1.
#' @param region character. The region of the crop as on FAO56 TABLE 11
#' @param regexpr character. Extra term that can be used to search crop names on FAO56 Tables
#' @param initgw numeric. Initial ground water level [m]
#' @param LAI numeric. Leaf Area Index
#' @param rc numeric Runoff coefficient [0-1]. 
#' This must be used if there is no information on Curve Number.
#' This can be calibrated with \code{\link{cal.kc}}
#' @param Kb ground water (base flow) recession parameter. 
#' The default is zero where no baseflow occurs
#' @param Rn Net daily radiation [w/m2]. Estimated this with  \code{\link{ETo}}
#' @param hmodel character. The type of crop growth model during the simulation. 
#' It takes "gaussian","linear", "exponential"
#' @param Zrmodel character. The type of root growth model during the simulation. 
#' It takes "gaussian","linear", "exponential"
#' @param xydata list c(longitude, latitude). The observation points of the model. 
#' This should be set only if the model is spatial.
#' @param REW numeric Readily Evaporable Water. It can be calibrated with \code{\link{cal.kc}}
#' @param a1 soil water storage to maximum root depth (Zr) at field capacity
#' @param b1 Liu et al. (2006) parameter for computing capillary rise [-0.17]. It can calibrated with
#' \code{\link{cal.kc}}
#' @param a2 Liu et al. (2006) parameter for computing capillary rise [240]. It can be calibrated with
#' \code{\link{cal.kc}}
#' @param b2 Liu et al. (2006) parameter for computing capillary rise [-0.27]. It can be calibrated with
#' \code{\link{cal.kc}}
#' @param a3 Liu et al. (2006) parameter for computing capillary rise [-1.3]. It can be calibrated with
#' \code{\link{cal.kc}}
#' @param b3 Liu et al. (2006) parameter for computing capillary rise [6.6]. It can be calibrated with
#' \code{\link{cal.kc}}
#' @param a4 Liu et al. (2006) parameter for computing capillary rise [4.6]. It can be calibrated with
#' \code{\link{cal.kc}}
#' @param b4 Liu et al. (2006) parameter for computing capillary rise [-0.65]. It can be calibrated with
#' \code{\link{cal.kc}}
#' @param report list. In case of spatial modelling, which days' map should be reported. 
#' The default is "stages" where the periods of stages maps are reported as values. 
#' In an ascending order it can take values such as 1:12 days or c(1,3,6:12) days.
#' @param time character. Time of the occurrence Irrigation or Precipitation. 
#' @param HWR numeric. Default is 1. Height to width ratio of individual plants or groups of 
#' plants when viewed from the east or from the west [-]. 
#' @param slope numeric. slope of saturation vapour pressure curve at air temperature T [kPa oC-1].
#' @param y numeric. psychometric constant  [kPa oC-1]
#' @param nongrowing character. The type of surface during non-growing season. 
#' It can take "weeds", "bare", or "mulch"
#' @param profile numeric or character. The profile depth to perform water balance.
#' It takes "variable" when updating the depth with root growth (default). Or taking "fixed" when
#' the maximum root zone is used. It can also take numeric value and this can be longer than
#' maximum root length. 
#' @param theta numeric. The measured soil water content [m3/m3] in the 
#' profile. Leave those days without measured data blank.
#' @inheritParams ETo
#' @param density character. density factor for the trees, 
#' vines or shrubs on how they are arranged.
#' It can take "full", "row" or "random" or "round". 
#' @param r1 leaf resistance FOR STOMATAL CONTROL [s/m]
#' @param Kcmin Minimum  Kc value 
#' @param Ky list or array. yield response factor [-] in the form of 
#' c(initial Ky,crop dev't Ky, mid season Ky, late season Ky, total season Ky). 
#' For instance Maize Ky is c(0.4,0.4,1.3,0.5,1.25). Ky values for other crops
#' are tabulated in FAO Irrigation and Drainage Paper 33 (Doorenbos and Kassam,
#' 1979) and Paper 56 (Allen et al., 1998).
#' If only one Ky value is supplied it will be repeated for all the seasons.
#' @param x a return object of the function.
#' @param main Title of the plot
#' @param output The kind of plot that should be displayed. It takes 
#' 'AW_measured' for plotting Available and modelled soil moisture;
#' 'TS_measured' for Total Available and modelled soil moisture;
#' 'Kc' for displaying Kcb and Kc
#' 'Evaporation' for displaying of Evaporation and Transpiration
#' 'balance' for displaying Irrigation, precipitation, ETa and critical water deficit
#' @param legend logical. TRUE or FALSE. Whether a legend should be displayed or not.
#' @param pos the position of legend. It takes one of the following 'top', 'topright',
#' 'topleft', 'bottom', 'bottomright' 'bottomleft', 'center'. This must be set so that
#' the legend should not obscure the main plot.
#' @param cex the relative font size of the texts. 
#' Vlaues: "horiz";	logical: if TRUE, set the legend horizontally rather than vertically
#' 
#' @seealso \code{\link{cal.kc}}
#' @details 
#'  \describe{
#'  \item{\strong{Using Existing Tables}}{The function include backend database of FAO Tables.
#'  The user can use the parameter values such as Zr, h, p, FC, WP, etc from FAO56 tables. 
#'  Most of the parameters with NULL values can take default values from FAO56 Tables. When a crop name results in 3 crops,
#'   the user can differentiate  that with  regexpr parameter. For example, 
#'   a crop parameter with a name "maize" can match
#'   two crops on FAO Table 12. Including  regexpr parameter with "sweet" with match only one. 
#'   }
#'  \item{\strong{Modelling crop height and Rooting Depth}}{ 
#'  At each time step a crop height or root depth is estimated. 
#'  The crop  or rooting depth growth model can take  exponential model of the form
#'  hday=0+0.75*max(h)*(1-exp(-day/0.75*max(days))) or    Zrday=0+0.75*Zr*(1-exp(-day/0.75*max(days)))
#'   The gaussian model is of the form hday=0+0.75*max(h)*(1-exp(-day^2/(0.75*max(days)^2))) or Zrday=0+0.75*(1-exp(-day^2/(0.75*max(day))^2))
#'   The linear model is of the form hday=(day*max(h))/max(days), Zrday=(day*Zr)/max(days)
#'     }
#'  \item{\strong{Point modeling with Single and dual crop coefficients}}{ 
#'  The function can estimate kc curve for the growing period by using the traditional
#'  kcini, kcmid and kcend and crop lengths growth stages, in case of single crop coefficients.
#'  Just like single crop coefficients, the dual crop coefficients kcbini, kcbmid, kcbend are
#'  embedded in kc parameter. The "kc" parameter must take 3 
#'  values as c(initial, mid, end). The values of "lengths" parameter must also be specified as
#'  c(initial, dev, mid, late). See example section for more details.}
#'   \item{\strong{Spatial modeling With Evaporation Fraction}}{ 
#'   Instead of supplying kcs values as described above, Evaporation Fraction (EF) from
#'   can be used. The corresponding EF at initital, mid amd end seasons can be supplied.
#'   The EF curve will be developed for each location at each time step and ETa derived 
#'   from (EF*Rn)/2.45.
#'   }
#'   \item{\strong{Interpolation of Input data}}{ Once EF is used instead of Kcs, one can
#'   provide  longitudes and latitudes in xydata parameter data frame. A 2D column data of P,
#'   I,ETo,RHmin, u2, Rn for each time step for each location can be provided. At each time step 
#'   an inverse distance interpolation will be perform to get continuous surface. In case of initial
#'   and static input column data such as CNII, FC, WP, initgw, Zr, h can also be interpolated.
#'   }
#'   \item{\strong{Calibration}}{A calibration of unmeasured variables can be done with \code{cal.kc}}
#'  }
#' @references 
#' \itemize{
#' \item Allen, R. G., PEREIRA, L. S., RAES, D., & SMITH, M. (1998). 
#' Crop Evapotranspiration (guidelines for computing crop water requirements) 
#' FAO Irrigation and Drainage Paper No. 56: FAO.
#' \item Allen, R.G., Wright, J.L., Pruitt, W.O., Pereira, L.S., Jensen, M.E., 2007. Water requirements.
#' In: Hoffman, G.J., Evans, R.G., Jensen, M.E., Martin, D.L., Elliot, R.L. (Eds.),
#' Design and Operation of Farm Irrigation Systems. , 2nd edition. ASABE, St. Joseph,
#' MI, pp. 208-288.
#' \item Liu, Y., Pereira, L.S., Fernando, R.M., 2006. Fluxes through the bottom boundary of 
#' the root zone in silty soils: parametric approaches to estimate groundwater 
#' contribution and percolation. Agric. Water Manage. 84, 27-40.
#'  
#' }
#'  
#'
#' @return The output of the function depends of the kctype and kc data type.
#' For single and dual crop coefficient, an "output" dataframe contains variables such as
#' TAW,RAW,Runoff,Ks,Kcb,h,Kc,ETc,DP,CR,Drend,Kr,De,AW,E,Ke,CN. In case kc is set to EF the function returns
#' Kc,Kcxy,ETc,ETcxy,Drend,Drendxy,TAW,TAWxy,RAW,RAWxy,AW,Awcxy,Ineed,Ineedxy
#'  \describe{
#' \item{output:}{  dataframe of the return values}
#' \item{ETc_adj:}{ The soil water stress adjusted ETc  [mm]}
#' \item{Ya_by_Ym:}{actual yield by potential yield}
#' \item{yield_decrease:}{The relative yield decrease: 1-Ya/Ym}
#' \item{ETdecrease:}{The relative evapotranspiration deficit :1- ETc_adj/ETc}
#' \item{yield_by_ET_decrease:}{he relative yield decrease: 1-Ya/Ym and 
#' the relative evapotranspiration deficit in one dataframe}
#' \item{Awcxy:}{ Available Soil water for each location at each time step [mm]}
#' \item{CR:}{ capillary rise [mm/day]}
#' \item{GW:}{ Updated ground water depth}
#' \item{CN:}{ Curve Number}
#' \item{De:}{ cumulative depth of evaporation (depletion) 
#' from the soil surface layer [mm] at the end of the day [mm/day]}
#' \item{DP:}{ Deep Percolation [mm/day]}
#' \item{Drend:}{ cumulative depth of evapotranspiration (depletion) 
#' from the root zone at the end of the day [mm]. 
#' This is equal to the amount of water needed to bring soil water to field capacity}
#' \item{Drendxy:}{ cumulative depth of evapotranspiration (depletion) 
#' from the root zone at the end of the day [mm] for each xydata location at each time step. 
#' This is equal to the amount of water needed to bring soil water to field capacity}
#' \item{E:}{ evaporation [mm/day]}
#' \item{ETc:}{ crop evapotranspiration [mm/day]. 
#' If water stress coefficient (Ks) is less than 1, 
#' ETc is crop evapotranspiration under non-standard conditions else ETc is
#' crop evapotranspiration under standard conditions}
#' \item{ETcxy:}{ crop evapotranspiration [mm/day] for each location at each time step.
#' crop evapotranspiration under standard conditions}
#' \item{h:}{ crop height [m]}
#' \item{Ineed:}{ Amount of soil water that is below maximum allowed depletion [mm]}
#' \item{Ineedxy:}{ Amount of soil water that is below maximum allowed depletion 
#' for each xydata location at each time step [mm] }
#' \item{Kc:}{ standard crop coefficient [-]}
#' \item{Kc_adj:}{ adjusted crop coefficient [-]}
#' \item{EF:}{ Evaporation Fraction [-]}
#' \item{EFxy:}{ Evaporation Fraction for each monitored location [-]}
#' \item{kcb:}{ basal crop coefficient for the growing season [-]}
#' \item{Kcxy:}{ crop coefficient for each location at each time step [-]}
#' \item{Ke:}{ soil evaporation coefficient}
#' \item{Kr: }{ soil evaporation reduction coefficient [-]}
#' \item{Ks:}{ Water stress coefficient [-]}
#' \item{RAW: }{ Readily Available Water [mm]}
#' \item{RAwcxy:}{ Readily Available Soil water for each xydata location at each time step [mm]}
#' \item{Runoff:}{ Runoff [mm/day]}
#' \item{TAW:}{ Total Available Water [mm] }
#' \item{TAwcxy:}{ Total Available Soil water for each location at each time step [mm]}
#' }
#' 
#' 
#' @author George Owusu
#' @export
#'
#' @examples
#' \dontrun{
#' #FAO56 Example 32: Calculation of the crop coefficient (Kcb + Ke) under sprinkler irrigation
#' FAO56Example32=kc(ETo=7,P=0,RHmin=20,soil="sandy loam",crop="Broccoli",I=10,
#' kctype = "dual",u2=3,kc=c(0.9,0,0),h=1,Zr=0.1)
#' FAO56Example32$output$fc
#' FAO56Example32$output$Kc_adj
#' #FAO56 Example 33: Calculation of the crop coefficient (Kcb + Ke) under furrow irrigation
#' FAO56Example33=kc(ETo=7,P=0,RHmin=20,soil="sandy loam",crop="Broccoli",I=10,
#'          kctype = "dual",u2=3,kc=c(0.9,0,0),h=1,Zr=0.1,fw=0.3)
#' #FAO56 Example 34: Calculation of the crop coefficient (Kcb + Ke) under drip irrigation
#' FAO56Example34=kc(ETo=7,P=0,RHmin=20,soil="sandy loam",crop="Broccoli",I=10,
#'            kctype = "dual",u2=3,kc=c(0.9,0,0),h=1,Zr=0.1,fw="drip")
#'          
#' file=system.file("extdata","sys","irrigation.txt",package="sebkc")
#' data=read.table(file,header=TRUE)  
#' P=data$P
#' rc=0
#' I=data$I
#' ETo=data$ETo
#' Zr=data$Zr
#' p=0.6
#' FC=0.23
#' WP=0.10
#' u2=1.6  
#' RHmin=35
#' soil="sandy loam"
#' crop="Broccoli"
#' ############Single crop coefficient model###########
#' modsingle=kc(ETo,P=P,RHmin,soil,crop,I)
#' #dataframe of the return values
#' modsingle$output 
#' modsingle$output$TAW #Total Avalaible water [mm]
#' #Single with runoff coefficient
#  modsinglerc=kc(ETo,P=P,RHmin,soil,crop,I,rc=0.2)

#' ############Dual crop coefficient model###########
#' moddual=kc(ETo,P=P,RHmin,soil,crop,I,kctype = "dual",FC=FC,p=p,WP=WP,u2=u2,kc=c(0.3,0,0),Zr=Zr)
#' #Reproducing FAO56 Example 36
#' fc=(0.00667*1:12)+0.07333
#' h=1:12/1:12*0.3
#'  moddual=kc(ETo,P=P,RHmin,soil,crop,I,kctype = "dual",FC=FC,p=p,WP=WP,u2=u2,rc=0,
#'  kc=data$Kcb,Zr=Zr,lengths = c(4,4,2,2),h=h,hmodel="linear",fc=fc,fw=0.8)
#'  plot(moddual$output$Day,moddual$output$Kc_adj,type="l",ylim=c(0,1.2))
#'  lines(moddual$output$Day,moddual$output$Kcb,type="l", lty=2)
#' ###########Spatial model I: Evaporation Fraction (EF) Model###################
#' #landsat folder with original files but a subset
#' folder=system.file("extdata","stack",package="sebkc") 
#' 
#' sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28) #sebi model
#' kc2=stack(sebiauto$EF/2,sebiauto$EF,sebiauto$EF/3) #assign EF to kc
#' modEF=kc(ETo,P=P,RHmin,soil,crop,I,kc=kc2,Rn=8,lengths = c(4,4,2,2))
#' spplot(modEF$EF)
#'
#'#############Spatial model II: interpolating precipitation ######################
#' h=1:12
#' Zr=1:12/12
#' xydata=data.frame(cbind(longitude=data$longitude,latitude=data$latitude))
#' print(xydata) #take a look at xydata
#' P2=data.frame(matrix(rep(P,12),ncol=12))
#' print(P2) #take a look at the format of precipitation data
#' modEFh=kc(ETo,P=P2,RHmin,soil,crop,I,kc=kc2,Rn=8,lengths = c(4,4,2,2),h=h,Zr=Zr,xydata=xydata)
#' #Amount of irrigation water needed 
#' spplot(modEFh$Drend)
#' #Amount of water Irrigation that is critically needed to reach maximum allowed depletion level
#' modEFh$Ineedxy
#' ETcxy=modEFh$ETcxy ##get ETc for each location at each time step
#' TAW=modEFh$TAW ## 
#' 
#' 
#' ############FAO56 Example 41: Estimation of mid-season crop coefficient for Trees or weeds ########
#' FAOExample41 =kc(fc=0.3,DOY=200,latitude=40,u2=1.5,h=2,density="row",lengths=c(0,0,1,0),
#' ETo=0,P=0,RHmin=55,soil="sand",crop="Broccoli",kctype="dual")
#' FAOExample41$output$Kcb
#' 
#' #FAO56 Example 43: Estimation of Kcb mid from ground cover with reduction for stomatal control
#' FAOExample43 =kc(fc=0.196,DOY=180,latitude=30,u2=2,h=5,density="round",y=0.0676,slope=0.189,
#' r1=420,RHmin=25,lengths=c(0,0,1,0),ETo=0,P=0,soil="sand",crop="Broccoli",kctype="dual")
#' FAOExample43$output$Kcb
#' }
# generic
kc<-function(ETo,P,RHmin,soil="Sandy Loam",crop,I=0,CNII=67,u2=2,FC=NULL,WP=NULL,h=NULL,
             Zr=NULL,Ze=0.1,kc=NULL,p=NULL,lengths=NULL,fw=0.8,fc=NULL,
             kctype="dual",region=NULL,regexpr="sweet",initgw=50,
             LAI=3,rc=NULL,Kb=0,Rn=NULL,hmodel="gaussian",Zrmodel="linear",
             xydata=NULL,REW=NULL,a1=360,b1=-0.17,a2=240,b2=-0.27,
             a3=-1.3,b3=6.6,a4=4.6,b4=-0.65,report="stages",time="06:00",
             HWR=1,latitude=NULL,density="full",DOY=NULL,slope=NULL,
             y=NULL,r1=100,nongrowing="weeds",theta=NULL,profile="variable",
        Kcmin=0.15,Ky=NULL
             ) UseMethod ("kc")

#' @rdname kc
#' @export
#default function
kc.default<-function(ETo,P,RHmin,soil="Sandy Loam",crop,I=0,CNII=67,u2=2,FC=NULL,WP=NULL,h=NULL,
                     Zr=NULL,Ze=0.1,kc=NULL,p=NULL,lengths=NULL,fw=1,fc=NULL,
                     kctype="dual",region=NULL,regexpr="sweet",initgw=50,
                     LAI=3,rc=NULL,Kb=0,Rn=NULL,hmodel="gaussian",Zrmodel="linear",
                     xydata=NULL,REW=NULL,a1=360,b1=-0.17,a2=240,b2=-0.27,
                     a3=-1.3,b3=6.6,a4=4.6,b4=-0.65,report="stages",time="06:00",
                     HWR=1,latitude=NULL,density="full",DOY=NULL,slope=NULL,
                     y=NULL,r1=100,nongrowing="weeds",theta=NULL,profile="variable",
        Kcmin=0.15,Ky=NULL)
{
  invisible(NULL)
  #get weather data
  
  if(!inherits(kc[1], "numeric")){
  if(length(kc)==3){
  if(!is.null(kc[[1]]$EF)){
    kctype=class(kc[[1]])
    kc[[1]]=kc[[1]]$EF
  }
  
  if(!is.null(kc[[2]]$EF)){
    kctype=class(kc[[2]])
    kc[[2]]=kc[[2]]$EF
  }
  
  if(!is.null(kc[[3]]$EF)){
    kctype=class(kc[[3]])
    if(!is.null(kc[[3]]$ETr)){
      kctype="METRIC" 
    }
    kc[[3]]=kc[[3]]$EF
  }
    kc=stack(kc[[1]],kc[[2]],kc[[3]])
  }
  }
  
    #print(kctype)
  lengths2=lengths
  #lengths5=lengths
  #lengths=lengths2
  if(!is.null(lengths)){
    if(!is.numeric(lengths)){
      init=as.numeric(as.Date(lengths[2], "%Y%m/%d")-as.Date(lengths[1], "%Y%m/%d"))
      if(is.na(init)){
        init=as.numeric(as.Date(lengths[2], "%Y-%m-%d")-as.Date(lengths[1], "%Y-%m-%d"))
      }
      
      dev=as.numeric(as.Date(lengths[3], "%Y%m/%d")-as.Date(lengths[2], "%Y%m/%d"))
      if(is.na(dev)){
        dev=as.numeric(as.Date(lengths[3], "%Y-%m-%d")-as.Date(lengths[2], "%Y-%m-%d"))
      }
      
      mid=as.numeric(as.Date(lengths[4], "%Y%m/%d")-as.Date(lengths[3], "%Y%m/%d"))
      if(is.na(mid)){
        mid=as.numeric(as.Date(lengths[4], "%Y-%m-%d")-as.Date(lengths[3], "%Y-%m-%d"))
      }
      
      late=as.numeric(as.Date(lengths[5], "%Y%m/%d")-as.Date(lengths[5], "%Y%m/%d"))
      if(is.na(late)){
        late=as.numeric(as.Date(lengths[5], "%Y-%m-%d")-as.Date(lengths[4], "%Y-%m-%d"))
      }
      
      lengths=c(init,dev,mid,late)
      lengthscum=cumsum(lengths)
    }
  
  if(!is.null(theta)&&length(theta)==2&&is.character(lengths2)&&!is.numeric(theta[1])){
    thisinit=(as.Date(lengths2[1], "%Y-%m-%d"))
   # print("yes")
    
    if(is.na(thisinit)){
      thisinit=(as.Date(lengths2[1], "%Y/%m/%d"))
    }
    thisend=(as.Date(lengths2[5], "%Y-%m-%d"))
    
    if(is.na(thisend)){
      thisend=(as.Date(lengths2[5], "%Y-%m-%d"))
    }
    
    #print("yes")
        
    modlen=as.data.frame(seq(thisinit,length=as.numeric(thisend-thisinit)+1,by=1))
    names(modlen)="date"
    
    #theta=data.frame(date=c("1960-04-01", "1960-04-06","1960-4-10","1960-5-4"),theta=c(0.2,0.3,0.4,0.5))
    names(theta)=c("date","theta")
    thisdata2 <- merge(modlen, theta,by=c("date"),all=TRUE)
    theta=NULL
   # date=NULL
    for(i in 1:nrow(thisdata2)){
      if(thisdata2$date[i]!=thisdata2$date[i+1]&&i<nrow(thisdata2)){
        theta=rbind(theta,thisdata2$theta[i])
       # date=rbind(date,thisdata2$date[i])
        
        #print(c(thisdata2$date[i]))
      }
      
    }
    theta=as.data.frame(rbind(theta,thisdata2$theta[i]))
    theta=theta[[1]]
    #print (thisdata2)
    #date=as.data.frame(rbind(date,thisdata2$date[i]))
    #print(theta)
    #names(theta)="date"
  }
  }
  
  if(!is.null(theta)&&length(theta)==2&&is.numeric(lengths2)){
    theta2=NULL
  for (i in 1:max(cumsum(lengths2))) {
    testtheta=theta[which(theta[1]==i),]
    if(nrow(testtheta)>0){
      theta2=rbind(theta2,testtheta[[2]])
      
    }else{
      theta2=rbind(theta2,NA)
      
    }
  } 
    theta=theta2
    
  }
  #print(lengths)
  if(inherits(ETo, "ETo")||inherits(ETo, "ETo2")){
    if(!is.null(ETo$ETo)){
      latitude=ETo$latitude
      u2=ETo$u2
      if(!is.null(y)){
        y=ETo$y
      }
      #y=ETo$y
      DOY=ETo$DOY
      if(!is.null(ETo$data)){
        thisETo=ETo$data
      }else{
        thisETo=ETo
      }
      
      if(!is.null(thisETo$u2)){
        u2=thisETo$u2
      }
      if(!is.null(thisETo$theta)){
        theta=thisETo$theta
      }
      RHmin=thisETo$RHmin
      
      if(is.null(slope)){
        thisETo$slope=NULL
      }
      Rn=thisETo$Rn
      if(!is.null(thisETo$DOY)){
        DOY=thisETo$DOY
      }
      ETo=thisETo$ETo
      
    }
    
  }
  #print(thisETo)
  daylenth=nrow(data.frame(ETo))-1
  day=1:daylenth
  
  #day=1:1522
  if(!is.null(lengths)&&!is.numeric(lengths2)&&day!=lengthscum[4]){
    #DOY=as.Date(DOY)
    #loopdates=as.Date(DOY)-100
    #thisdata=datasub(startdate=DOY[1],thisETo,substart=lengths2[1],subend=lengths2[5])$moddata
    #class(thisdata)="ETo2"
    years=unique( format(as.Date(DOY),'%Y'))
    lengths3=as.Date(lengths2)
    thisYear=format(lengths3,'%Y')
    thisMonth=( format(lengths3,'%m'))
    thisday=( format(lengths3,'%d'))
    startdate=DOY[1]
    enddate=DOY[length(DOY)]

    output=data.frame()
    yield_decrease=data.frame()
    water.balance=data.frame()
    DL=data.frame(Tday=numeric(),T24=numeric(),Tn=numeric(),Rs=numeric(),DL=numeric(),
                  plantdate=character(),harvestdate=character())
    lenmin=0
    lenmax=0
    if((length(lengths2)/5)==length(years)){
      lenmin=1
      lenmax=5
      #print("lenmin")
    }
    for(i in 1:length(years)){
      thislengths1=paste(years[i],thisMonth[1],thisday[1],sep="-")
      thislengths2=paste(years[i],thisMonth[2],thisday[2],sep="-")
      thislengths3=paste(years[i],thisMonth[3],thisday[3],sep="-")
      thislengths4=paste(years[i],thisMonth[4],thisday[4],sep="-")
      thislengths5=paste(years[i],thisMonth[5],thisday[5],sep="-")
      thislengths=c(thislengths1,thislengths2,thislengths3,
                        thislengths4,thislengths5)
      
      substart=thislengths[1]
      subend=thislengths[5]
      substart=as.Date(substart)
      subend=as.Date(subend)
      
      startdate=as.Date(startdate)
      enddate=as.Date(enddate)
      
      modlen=seq(startdate,length=as.numeric(enddate-startdate)+1,by=1)
      #as.numeric(format(modlen,"%m"))
      #print(thislengths)
      gcmdata=data.frame(cbind(modlen,thisETo))
      thisdata <- gcmdata[ which(gcmdata$modlen>=substart&gcmdata$modlen<= subend), ]
      #thisdata=datasub(s,thisETo,substart=lengths2[1],subend=lengths2[5])$moddata
      #class(thisdata)="ETo2"
      #print(lengths)
      #thislengthsminmax=lengths
      if(lenmin>0){
      thislengthsminmax=lengths2[lenmin:lenmax]

      init=as.numeric(as.Date(thislengthsminmax[2], "%Y%m/%d")-as.Date(thislengthsminmax[1], "%Y%m/%d"))
      if(is.na(init)){
        init=as.numeric(as.Date(thislengthsminmax[2], "%Y-%m-%d")-as.Date(thislengthsminmax[1], "%Y-%m-%d"))
      }
      
      dev=as.numeric(as.Date(thislengthsminmax[3], "%Y%m/%d")-as.Date(thislengthsminmax[2], "%Y%m/%d"))
      if(is.na(dev)){
        dev=as.numeric(as.Date(thislengthsminmax[3], "%Y-%m-%d")-as.Date(thislengthsminmax[2], "%Y-%m-%d"))
      }
      
      mid=as.numeric(as.Date(thislengthsminmax[4], "%Y%m/%d")-as.Date(thislengthsminmax[3], "%Y%m/%d"))
      if(is.na(mid)){
        mid=as.numeric(as.Date(thislengthsminmax[4], "%Y-%m-%d")-as.Date(thislengthsminmax[3], "%Y-%m-%d"))
      }
      
      late=as.numeric(as.Date(thislengthsminmax[5], "%Y%m/%d")-as.Date(thislengthsminmax[5], "%Y%m/%d"))
      if(is.na(late)){
        late=as.numeric(as.Date(thislengthsminmax[5], "%Y-%m-%d")-as.Date(thislengthsminmax[4], "%Y-%m-%d"))
      }
        
      lengths=c(init,dev,mid,late)

      
      lenmin=lenmin+5
      lenmax=lenmax+5
      }
      #print(theta)
      model=kc(ETo=thisdata$ETo,P=thisdata$P,RHmin=thisdata$RHmin,soil=soil,crop=crop,
               I=I,CNII=CNII,u2=thisdata$u2,
             FC=FC,WP=WP,h=h,Zr=Zr,Ze=Ze,kc=kc,p=p,fw=fw,fc=fc,lengths=lengths,
             kctype=kctype,region=region,regexpr=regexpr,initgw=initgw,LAI=LAI,rc=rc,
             Rn=thisdata$Rn,hmodel=hmodel,Zrmodel=Zrmodel,a1=a1,b1=b1,a2=a2,b2=b2,
             a3=a3,b3=b3,a4=a4,b4=b4,xydata=xydata,REW=REW
             ,report=report,time=time,HWR=HWR,latitude=latitude,
             density=density,DOY=thisdata$DOY,slope=thisdata$slope,y=y,r1=r1,
             nongrowing=nongrowing,theta=theta,profile=profile,Kcmin=Kcmin,Ky=Ky)
      #output2=rbind(output2,model$output)
      #print(length(model$output$P))
      
      thisyear=as.numeric(years[i])
      outputs=cbind(Year=years[i],model$output)
      output=rbind(output,outputs)
      Ya_by_Yms=cbind(Year=thisyear,model$yield_by_ET_decrease)
      yield_decrease=rbind(yield_decrease,Ya_by_Yms)
      thisDL=dl(latitude,DOY=thisdata$DOY,model= "CBM",Tmax=thisdata$Tmax,Tmin=thisdata$Tmin)
      DL=rbind(DL,data.frame(Tday=mean(thisDL$Tday),
      T24=mean(thisDL$T24, na.rm = TRUE),Tn=mean(thisDL$Tn, na.rm = TRUE),Rs=mean(thisDL$Rs, na.rm = TRUE),latitude=latitude,
      DL=mean(thisDL$DL, na.rm = TRUE),
      plantdate=substart,harvestdate=subend))
      water.balances=cbind(Year=years[i],model$water.balance.stages)
      water.balance=rbind(water.balance,water.balances)
     # print(substart)
      #print(average(thisDL))
      #print(years[i])
      #print(cbind(ETo=thisdata$ETo,P=thisdata$P,RHmin=thisdata$RHmin,u2=thisdata$u2,
           #  Rn=thisdata$Rn,DOY=thisdata$DOY,slope=thisdata$slope))
      #print(model$output)
      
#print (years[i])    
    }
    if(inherits(kc[[1]], "RasterLayer")||inherits(kc[[2]], "RasterLayer")
       ||inherits(kc[[3]], "RasterLayer")){
      factor=model
    }else{
      factor=list(output=output,yield_by_ET_decrease=yield_decrease,DL=DL,water.balance.stages=water.balance,kctype=kctype)
    }#message(thisdata$Tmax)
    
  }else{
  ##The main model starts here
  
  parameters=c(ETo=ETo,P=P,RHmin=RHmin,soil=soil,crop=crop,I=I,CNII=CNII,u2=u2,
               FC=FC,WP=WP,h=h,Zr=Zr,Ze=Ze,kc=kc,p=p,lengths=lengths,fc=fc,
               kctype=kctype,region=region,regexpr=regexpr,initgw=initgw,LAI=LAI,rc=rc,
               Rn=Rn,hmodel=hmodel,Zrmodel=Zrmodel,a1=a1,b1=b1,a2=a2,b2=b2,
               a3=a3,b3=b3,a4=a4,b4=b4,xydata=xydata,REW=REW
               ,report=report,time=time,HWR=HWR,latitude=latitude,
               density=density,DOY=DOY,slope=slope,y=y,r1=r1,
               nongrowing=nongrowing,theta=theta,profile=profile,Ky=Ky)
 
   
  
  latitude2=latitude
  density2=density
  DOY2=DOY
  slope2=slope
  y2=y
  r12=r1
  
  
  if(!is.null(DOY)){
  if(!is.numeric(DOY)){
    DOY2=DOY
    DOY=strptime(DOY,"%Y-%m-%d")$yday+1
    if(is.na(DOY[1])){
      DOY=strptime(DOY2,"%Y%m/%d")$yday+1
    }
  }
  }
  Zr3=Zr
  h3=h
  kc3=kc
  
  EF=NULL
  allEF=NULL
  if(inherits(kc[[1]], "RasterLayer")||inherits(kc[[2]], "RasterLayer")
     ||inherits(kc[[3]], "RasterLayer")){
    if(inherits(kc[[1]], "RasterLayer")){
      map=kc[[1]]
    }else{
      if(inherits(kc[[2]], "RasterLayer")){
        map=kc[[2]]
      }else{
        map=kc[[3]]
      }
    }
    EF=TRUE 
  }
  
  
  if(!is.null(EF)&&is.null(Rn)){
    return(message("Daily net radiation (Rn) is needed"))
  }
  ##preparing interpolation files
  Pinter=NULL
  if(!is.null(EF)&&!is.null(xydata)){
  if(!is.null(nrow(P))){
    Pinter=TRUE
  }
  }
  Rninter=NULL
  if(!is.null(EF)&&!is.null(xydata)){
    if(!is.null(nrow(Rn))){
      Rninter=TRUE
    }
  }
  RHinter=NULL
  if(!is.null(EF)&&!is.null(xydata)){
    if(!is.null(nrow(RHmin))){
      RHinter=TRUE
    }
  }
  
  u2inter=NULL
  if(!is.null(EF)&&!is.null(xydata)){
    if(!is.null(nrow(u2))){
      u2inter=TRUE
    }
  }
  ETointer=NULL
  if(!is.null(EF)&&!is.null(xydata)){
    if(!is.null(nrow(ETo))){
      ETointer=TRUE
    }
  }
  Iinter=NULL
  if(!is.null(EF)&&!is.null(xydata)){
    if(!is.null(nrow(I))){
      Iinter=TRUE
    }
  }
  thetainter=NULL
  if(!is.null(EF)&&!is.null(xydata)){
    if(!is.null(nrow(theta))){
      thetainter=TRUE
    }
  }
  folder=system.file("extdata","sys",package="sebkc")
  
 
  
  soilwater=read.csv(system.file("extdata","sys","soilwater2.csv",package="sebkc"),header=TRUE,stringsAsFactors=FALSE)
  stages=(read.csv(system.file("extdata","sys","stages.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
  crop_H_Zr=na.omit(read.csv(system.file("extdata","sys","crop_H_Zr.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
  singlekc=na.omit(read.csv(system.file("extdata","sys","singlekc.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
  dualkc=na.omit(read.csv(system.file("extdata","sys","dualkc.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
  ky=na.omit(read.csv(system.file("extdata","sys","ky.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
  wetting=na.omit(read.csv(system.file("extdata","sys","fw.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
   #for crop heigh and roots
  soilwater$ID=1:9
  range=0.75
  soil2=soil
  crop2=crop
  CN=CNII
  report2=NULL
  daylenths=nrow(data.frame(P))
  day=1:daylenths
  #print(day)
  ##all kc is EF
  if(!is.null(EF)){
    if(nlayers(kc)==max(day)){
    allEF=TRUE
    }
  }
  ETcxy=NULL
  ones=day/day
  if(I==0&&length(I)==1){
    I=day*0
  }else{
    if(length(I)==1){
      I=ones*I
    }
  }
  
  if(length(Rn)==1){
    Rn=ones*Rn
  }
  
  if(length(u2)==1){
    u2=u2*ones
  }
  if(length(RHmin)==1){
    RHmin=RHmin*ones
  }
  if(length(DOY)==1){
    DOY=ones*DOY
  }
  if(length(HWR)==1){
    HWR=ones*HWR
  }
  if(length(latitude)==1){
    latitude=ones*latitude
  }
  
  if(length(slope)==1){
    slope=ones*slope
  }
  
  if(length(y)==1){
    y=ones*y
  }
  
  if(length(r1)==1){
    r1=ones*r1
  }
  
  fc2=fc
  if(length(fc2)==1){
    fc2=ones*fc2
  }
  data=data.frame(cbind(ETo,P,I))
  
  #WP and FC interpolation
  if(!is.null(xydata)&&length(WP)>1&&!is.null(EF)){
    wpdata=data.frame(cbind(xydata,WP))
    longitude=wpdata[1]
    latitude=wpdata[2]
    var=wpdata[3]
    #print(wpdata[3])
    WP= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
    
  }
  
  if(!is.null(xydata)&&length(FC)>1&&!is.null(EF)){
    FCdata=data.frame(cbind(xydata,FC))
    longitude=FCdata[1]
    latitude=FCdata[2]
    var=FCdata[3]
    FC= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
  }
  if(!is.null(xydata)&&length(initgw)>1&&!is.null(EF)){
    wpdata=data.frame(cbind(xydata,initgw))
    longitude=wpdata[1]
    latitude=wpdata[2]
    var=wpdata[3]
    initgw= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
  }
  
  if(!is.null(xydata)&&length(CNII)>1&&!is.null(EF)){
    wpdata=data.frame(cbind(xydata,CNII))
    longitude=wpdata[1]
    latitude=wpdata[2]
    var=wpdata[3]
    CNII= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
    message(CNII)
  }
  if(is.null(Zr)||is.null(p)){
    crop= crop_H_Zr[(grep(crop2,crop_H_Zr$crop,ignore.case=TRUE)),] 
    
    if(nrow(crop)>1){
      crop= crop[(grep((regexpr),(crop),ignore.case=TRUE)),] 
    }
    if(nrow(crop)==0){
      return (message(paste(crop2," not found [for crop type name]"))) 
    }
    if(is.null(Zr)){
      Zr=mean(c(as.numeric(strsplit(crop$Zr, "-")[[1]][1]),as.numeric(strsplit(crop$Zr, "-")[[1]][2])))
    }
    
    if(is.null(p)){
      p=crop$p[1]
    }
  }
  
  Zrinter=NULL
  co=0
  c1=range*Zr
  a=range*max(day)
  if(length(Zr)==1){
    if(Zrmodel=="exponential"||Zrmodel=="exponent"||Zrmodel=="exp"){
      Zr=co+c1*(1-exp(-day/a))
      # Zr=(co+c1)%o%(1-exp(-day/a))
    }else{
      
      if(Zrmodel=="linear"||Zrmodel=="lin"||Zrmodel=="line"){
        
        Zr=(day*Zr)/max(day)
        
      }else{
        Zr=co+c1*(1-exp(-day^2/a^2))
      }
    }
  }else{
    Zrinter=TRUE
    if(Zrmodel=="exponential"||Zrmodel=="exponent"||Zrmodel=="exp"){
      
      Zr=(co+c1)%o%(1-exp(-day/a))
      Zr=t(Zr)
    }else{
      
      if(Zrmodel=="linear"||Zrmodel=="lin"||Zrmodel=="line"){
        
        Zr=(day%o%Zr)/max(day)
        Zr=t(Zr)
      }else{
        Zr=co+c1%o%(1-exp(-day^2/a^2))
        Zr=t(Zr)
      }
    } 
  }
  
  #soil characteristics
  soil=toupper(soil)
  soil=soilwater[(grep(soil2,soilwater$type,ignore.case=TRUE)),]
  if(nrow(soil)==0){
    return (message(paste(soil2," not found: check soil"))) 
  }
  
  #lengths of crop growth stages
  if(is.null(lengths)){
    #stages2= stages[(grep(toupper(crop2),toupper(stages$crop))),] 
    stages2= stages[(grep((crop2),(stages$crop),ignore.case=TRUE)),] 
    if(nrow(stages2)>1){
      stages2= stages2[(grep((region),(stages2$region),ignore.case=TRUE)),] 
    }
    
    if(nrow(stages2)==0){
      return (message(paste(crop2," not found crop growth stages"))) 
    }
    init=mean(as.numeric(stages2$init))
    dev=mean(as.numeric(stages2$dev))
    mid=mean(as.numeric(stages2$mid))
    late=mean(as.numeric(stages2$late))
    
  }else{
    init=lengths[1]
    dev=lengths[2]
    mid=lengths[3]
    late=lengths[4]
  }
  stages2=c(init,dev,mid,late)
  stages2cum=cumsum(stages2)
  ##dual crop coefficient
  if(is.null(kc)){
    kcs= dualkc[(grep((crop2),(dualkc$crop),ignore.case=TRUE)),] 
    if(nrow(kcs)>1){
      kcs= kcs[(grep((regexpr),(kcs$crop),ignore.case=TRUE)),] 
    }
    
    if(nrow(kcs)==0){
      return (message(paste(" Kc not found for", crop2))) 
    }
    kcbini=mean(as.numeric(kcs$Kcbini))
    kcbmid=mean(as.numeric(kcs$Kcbmid))
    kcbend=mean(as.numeric(kcs$Kcbend))
    if(is.na(kcbend)){
      kcbend=mean(as.numeric(c(strsplit(kcs$Kcbend, "-")[[1]][1],strsplit(kcs$Kcbend, "-")[[1]][2])))
      if(is.na(kcbend)){
        kcbend=mean(as.numeric(c(strsplit(kcs$Kcbend, ",")[[1]][1],strsplit(kcs$Kcbend, ",")[[1]][2])))
        
      }
    }
    
    if(is.na(kcbmid)){
      kcbmid=mean(as.numeric(c(strsplit(kcs$Kcmid, "-")[[1]][1],strsplit(kcs$Kcmid, "-")[[1]][2])))
      if(is.na(kcbmid)){
        kcbmid=mean(as.numeric(c(strsplit(kcs$Kcmid, ",")[[1]][1],strsplit(kcs$Kcmid, ",")[[1]][2])))
        
      }
    }
    
    if(is.na(kcbini)){
      kcbini=mean(as.numeric(c(strsplit(kcs$Kcbini, "-")[[1]][1],strsplit(kcs$Kcbini, "-")[[1]][2])))
      if(is.na(kcbini)){
        kcbini=mean(as.numeric(c(strsplit(kcs$Kcbini, ",")[[1]][1],strsplit(kcs$Kcbini, ",")[[1]][2])))
        
      }
    }
    
  }else{
    kcbini=kc[1]
    kcbmid=kc[2]
    kcbend=kc[3]  
  }
  
  #kcb=(day*kcb)/max(day)
  
  if(is.null(h)){
    h= singlekc[(grep((crop2),(singlekc$crop),ignore.case=TRUE)),] 
    if(nrow(h)==0){
      return (message(paste(" h not found for", crop2))) 
    }
    if(nrow(h)>1){
      h= h[(grep((regexpr),(h$crop),ignore.case=TRUE)),] 
    }
    h=mean(as.numeric(h$h))
  }
  ksingle=FALSE
  if(kctype=="single"){
    ksingle=TRUE
  }
  
  if(is.null(EF)){
    if(ksingle==TRUE){
      kcss= singlekc[(grep((crop2),(singlekc$crop),ignore.case=TRUE)),] 
      if(nrow(kcss)==0){
        #return (print(paste(" Kc not found for", crop2))) 
      }
      if(nrow(kcss)>1){
        kcss= kcss[(grep((regexpr),(kcss$crop),ignore.case=TRUE)),] 
      }
      
      if(is.null(kc)){
        
        kcini=mean(as.numeric(kcss$Kcini))
        kcmid=mean(as.numeric(kcss$Kcmid))
        kcend=mean(as.numeric(kcss$Kcend))
        if(is.na(kcend)){
          kcend=mean(as.numeric(c(strsplit(kcss$Kcend, "-")[[1]][1],strsplit(kcss$Kcend, "-")[[1]][2])))
          if(is.na(kcend)){
            kcend=mean(as.numeric(c(strsplit(kcss$Kcend, ",")[[1]][1],strsplit(kcss$Kcend, ",")[[1]][2])))
            
          }
        }
        
        if(is.na(kcmid)){
          kcmid=mean(as.numeric(c(strsplit(kcss$Kcmid, "-")[[1]][1],strsplit(kcss$Kcmid, "-")[[1]][2])))
          if(is.na(kcmid)){
            kcmid=mean(as.numeric(c(strsplit(kcss$Kcmid, ",")[[1]][1],strsplit(kcss$Kcmid, ",")[[1]][2])))
            
          }
        }
        
        if(is.na(kcini)){
          kcini=mean(as.numeric(c(strsplit(kcss$Kcini, "-")[[1]][1],strsplit(kcss$Kcini, "-")[[1]][2])))
          if(is.na(kcini)){
            kcini=mean(as.numeric(c(strsplit(kcss$Kcini, ",")[[1]][1],strsplit(kcss$Kcini, ",")[[1]][2])))
            
          }
        }
        #if(kcmid>0.45){
          #kcmid = kcmid + (0.04*(u2-2) - 0.004*(RHmin-45))* (h/3)^0.3
          #}
        
        #if(kcend>0.45){
          # kcend =kcend + (0.04*(u2-2) - 0.004*(RHmin-45))* (h/3)^0.3
          # }
        kcss=c(kcini,kcmid,kcend)
      }else{
        kcini=kc[1]
        kcmid=kc[2]
        kcend=kc[3]
        kcss=c(kcini,kcmid,kcend)
      }
      #kcsscum=cumsum(kcss)
    }
  }else{
    kcss=kc
  }

  if(!is.null(kcbmid)&&!is.null(kcbend)&&is.null(EF)){
    
    #if(kcbmid>0.45){
    # kcbmid = kcbmid + (0.04*(u2-2) - 0.004*(RHmin-45))* (h/3)^0.3
    # }
    
    #if(kcbend>0.45){
    # kcbend =kcbend + (0.04*(u2-2) - 0.004*(RHmin-45))* (h/3)^0.3
    # }
    kcss=c(kcbini,kcbmid,kcbend)
  }
  
  if(ksingle==FALSE&&is.null(EF)){
    
    kcss=c(kcbini,kcbmid,kcbend)
  }
  #print(kcss)
  
  #ky
  if(is.null(Ky)){ 
   ky1= ky[(grep((crop2),(ky$crop),ignore.case=TRUE)),]
    if(nrow(ky1)>0){
      ky1=ky1$ky
          Ky=mean(as.numeric(ky1))
      if(is.na(Ky)){
        Ky=mean(as.numeric(c(strsplit(ky1, "-")[[1]][1],strsplit(ky1, "-")[[1]][2])))

      }
          
    }    
  }
    #kcsscum=cumsum(kcss)
  
  ############Dual Kc#############
  Kcmin=Kcmin
  if(is.null(REW)){
  REW=mean(c(as.numeric(strsplit(soil$REW, "-")[[1]][1]),as.numeric(strsplit(soil$REW, "-")[[1]][2])))
  }
  #Maximum depth of water that can be evaporated from the topsoil change to 0.125
  if(length(Ze)==1){
    Ze=(1:max(day)/1:max(day))*Ze
  }
  
  
  if(is.null(FC)||is.null(WP)){
    if(is.null(FC)){
      FC=mean(c(as.numeric(strsplit(soil$FC, "-")[[1]][1]),as.numeric(strsplit(soil$FC, "-")[[1]][2])))
    }
    if(is.null(WP)){
      WP=mean(c(as.numeric(strsplit(soil$WP, "-")[[1]][1]),as.numeric(strsplit(soil$WP, "-")[[1]][2])))
    }
    
  }
  #print(kcss)
  hinter=NULL
  co=0
  c1=range*h
  a=range*max(day)
  
  if(length(h)==1){
  if(hmodel=="exponential"||hmodel=="exponent"||hmodel=="exp"){
    h=co+c1*(1-exp(-day/a))
    # h=(co+c1)%o%(1-exp(-day/a))
  }else{
  
  if(hmodel=="linear"||hmodel=="lin"||hmodel=="line"){
    
  h=(day*h)/max(day)
  
  }else{
    h=co+c1*(1-exp(-day^2/a^2))
  }
  }
  }else{
    hinter=TRUE
    if(hmodel=="exponential"||hmodel=="exponent"||hmodel=="exp"){
      
    h=(co+c1)%o%(1-exp(-day/a))
    h=t(h)
    }else{
      
      if(hmodel=="linear"||hmodel=="lin"||hmodel=="line"){
        
        h=(day%o%h)/max(day)
        h=t(h)
      }else{
        h=co+c1%o%(1-exp(-day^2/a^2))
        h=t(h)
      }
    } 
  }
  
  #FC=0.23
  #WP=0.10
  #TAW=1000*(FC-WP)*Zr
  #RAW=p*TAW

  outputdata=data.frame()
  
  outputdual=data.frame()
  
  
  REWcor=2.5
  TEWcor=10
  if(report=="stages"){
    report=stages2cum
  }
  if(report=="all"||report=="All"){
    report=day
  }
    report3=report
    Awcxy=NULL
    ETcxy=NULL
    TAWxy=NULL
    RAWxy=NULL
    Ineedxy=NULL
    time2=time
    if(length(time)!=max(day)){
      time2=rep(time[1],max(day))
    }
    fwproj=NULL
    fw2=fw
    fwp=NULL
    if(length(fw)==1){
      fw2=rep(fw,max(day))
      fwproj=TRUE
    }
    
    ETc_adjsum=0
    ETc_sum=0
    TSW_measured=NA
    AW_measured2=NA
    if(is.null(Zr)||is.null(p)||is.null(h)){
      return(message("These input parameters are needed Zr, p, and h "))
      
    }
    #check kcini
    thiskcini=NULL
  i=1
  #Adjust Kc for weather
  if(is.null(EF)){
  u2mid=mean(u2[stages2cum[2]:stages2cum[3]])
  RHminmid=mean(RHmin[stages2cum[2]:stages2cum[3]])
  hmid=mean(h[stages2cum[2]:stages2cum[3]])
  kcss[[2]]=kcss[[2]] + (0.04*(u2mid-2) - 0.004*(RHminmid-45))* (hmid/3)^0.3
  u2end=mean(u2[stages2cum[3]:stages2cum[4]])
  RHminend=mean(RHmin[stages2cum[3]:stages2cum[4]])
  hend=mean(h[stages2cum[3]:stages2cum[4]])
  kcss[[3]]=rastercon(kcss[[3]]>0.45,kcss[[3]] + (0.04*(u2end-2) - 0.004*(RHminend-45))* (hend/3)^0.3,kcss[[3]])
  #print(kcss)
  }
  #hmid=mean(h[stages2cum[2]:stages2cum[3]])
  #kcss[[2]]=kcss[[2]] + (0.04*(u2mid-2) - 0.004*(RHminmid-45))* (hmid/3)^0.3
  
 
  DEEP=FALSE
  timeDP=0
  #kcb2=data$Kcb
  if(is.null(EF)){
  if(!is.null(Zr3)){
    if(length(Zr3)==length(day)){
      Zr=Zr3
    }
  }
  
  if(!is.null(h3)){
    if(length(h3)==length(day)){
      h=h3
    }
  }
  }
  if(is.numeric(profile)){
    Zri=profile
  }else{
    if(profile=="fixed"){
      Zri=max(Zr)
    }else{
      Zri=Zr[1]
    }
  }
  
  if(!is.null(theta)){
    if(length(theta)==1){
     theta=c(theta,rep(NA,max(day)-1)) 
    }
    inittheta=theta[[1]]
  }else{
    inittheta=WP
  }
  if(!is.null(theta)){
  if(is.na(inittheta)){
    inittheta=WP 
  }
  }
  Drend=(1000*(FC-inittheta)*Zri*p[[1]])
  
  CR=0
  #fw=0.0001
 
  Dei=(1000*(FC-(0.5*WP))*Ze[1])
  AW_measured=NA
  AW_measuredxy=NULL
  pi=p
  #print(P)
  while(i<=length(day)){
    #if(!is.null(EF)){
    if(is.character(fw2[i])){
      fw= wetting[(grep((fw2[i]),(wetting$Wetting),ignore.case=TRUE)),]$fw[1]
    }else{
      fw=fw2[i]
    }
      if(!is.null(fwproj)){
        
      if(P[i]>0&&I[i]==0){
        fw=1 
        fwp=fw
      } 
        if(P[[i]]>0&&I[i]>0){
          total=P[[i]]+I[[i]]
          fw=((P[[i]]/total)*1)+((I[[i]]/total)*fw)
          fwp=fw
          
        }
      
      if(P[[i]]==0&&I[i]>0){
       fwp=NULL 
      }
      }
      fw=ifelse(!is.null(fwp),fwp,fw)
      #print(fw)
      
    #}
    #print(i)
    
    TEW=(1000*(FC-(0.5*WP))*Ze[i])
    #limits of h
    if(is.null(EF)){
    if(h[[i]]<0.1||h[[i]]>10){
      if(h[[i]]<0.1){
        h[[i]]=0.1 
      }
      if(h[[i]]>10){
        h[[i]]=10
      }
    }
    #limits of u2
    if(u2[[i]]<1||u2[[i]]>6){
      if(u2[[i]]<1){
        u2[[i]]=1 
      }
      if(u2[[i]]>6){
        u2[[i]]=6
      }
    }
    
    #limits of RHmin
    if((RHmin[[i]]<20||RHmin[[i]]>80)){
      if(RHmin[[i]]<20){
        RHmin[[i]]=20 
      }
      if(RHmin[[i]]>80){
        RHmin[[i]]=80
      }
    }
    }
    if(!is.null(hinter)&&!is.null(EF)&&!is.null(xydata)){
      hdata=cbind(xydata,h[i,])
      longitude=hdata[,1]
      latitude=hdata[,2]
      var=hdata[,3]
    hi= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
    #plot(hi)
    hi=rastercon(hi<0.1,0.1,rastercon(hi>10,10,hi))
    }else{
      hi=h[[i]]
    }
    
    if(!is.null(Zrinter)&&!is.null(EF)&&!is.null(xydata)){
      Zrdata=cbind(xydata,Zr[i,])
      longitude=Zrdata[,1]
      latitude=Zrdata[,2]
      var=Zrdata[,3]
      Zri= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
      #Zri=rastercon(Zri<0.1,0.1,rastercon(Zri>10,10,Zri))
    }else{
      if(is.numeric(profile)){
        Zri=profile
      }else{
        if(profile=="fixed"){
          Zri=max(Zr)
        }else{
          Zri=Zr[[i]]
        }
      }
     # Zri=Zr[[i]]
    }
    #do interpolation
    if(!is.null(EF)&&!is.null(RHinter)){
      if(nrow(xydata)==length(RHmin[i,])){
        Pdata=cbind(xydata,RHmin[i,])
        longitude=Pdata[,1]
        latitude=Pdata[,2]
        var=Pdata[,3]  
        RHmini= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
      }else{
        RHmini=mapmatrix(RHmin,i,map) 
      }
      
    }else{
      RHmini=RHmin[[i]]
    }
    
    if(!is.null(EF)&&!is.null(u2inter)){
      if(nrow(xydata)==length(u2[i,])){
        Pdata=cbind(xydata,u2[i,])
        longitude=Pdata[,1]
        latitude=Pdata[,2]
        var=Pdata[,3]  
        u2i= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
      }else{
        u2i=mapmatrix(u2,i,map)  
      }
    }else{
      u2i=u2[[i]]
    }
    
    if(!is.null(EF)&&!is.null(ETointer)){
      if(nrow(xydata)==length(ETo[i,])){
        Pdata=cbind(xydata,ETo[i,])
        longitude=Pdata[,1]
        latitude=Pdata[,2]
        var=Pdata[,3]  
        EToi= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
      }else{
        EToi=mapmatrix(ETo,i,map)  
      }
    }else{
      EToi=ETo[[i]]
    }
    if(!is.null(EF)&&!is.null(thetainter)){
      if(nrow(xydata)==length(theta[i,])){
        Pdata=cbind(xydata,theta[i,])
        longitude=Pdata[,1]
        latitude=Pdata[,2]
        var=Pdata[,3]  
        thetai= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
      }else{
        thetai=mapmatrix(theta,i,map)  
      }
    }else{
      thetai=theta[[i]]
    }
    if(!is.null(EF)&&!is.null(Iinter)){
      if(nrow(xydata)==length(I[i,])){
        Pdata=cbind(xydata,I[i,])
        longitude=Pdata[,1]
        latitude=Pdata[,2]
        var=Pdata[,3]  
        I= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
      }else{
        Ii=mapmatrix(I,i,map)  
      }
    }else{
      Ii=I[[i]]
    }
    #this=as.numeric(lapply(lists,function(x)if(i<x){x*2}else{0}))
    if(!is.null(allEF)){
      Kci=kc[[i]]
    }else{
    if(i<=stages2cum[1]){
      #kcini
      #print(stages2[1])
      
      if(ksingle==TRUE&&is.null(EF)){
        if(nrow(data)>stages2[1]){
          initdata=data[1:stages2[1],]
        }else{
          initdata=data
         # print(length(data))
          
        }
       # print(initdata)
        
        initdataI=initdata[initdata$I>0.2*ETo[[i]],]
        if(is.null(initdata[["P"]])){
          initdataP=initdata[initdata$P.RO>0.2*ETo[[i]],]
          rain="P.RO"
        }else{
          initdataP=initdata[initdata[["P"]]>0.2*ETo[[i]],]
          rain="P"
        }
        nw=nrow(initdataP)+nrow(initdataI)
        #print(nw)
        thisETo=mean(initdata$ETo)
        Eso=1.15*thisETo
        Pmean=mean(initdataP[[rain]])
        Imean=mean(initdataI$I)
        infiltration=Pmean+Imean
        if(is.na(Imean)&&!is.na(Pmean)){
          infiltration=Pmean
        }
        if(is.na(Pmean)&&!is.na(Imean)){
          infiltration=Imean
        }
        if(!is.na(Pmean)&&!is.na(Imean)){
          infiltration=mean(initdata$I)+mean(initdata$P)
          #print("YES")
        }
       if(is.na(infiltration)){
         infiltration=9
       }
        if(infiltration<=10){
          TEWcor=10
          REWcor=min(max(2.5,6/(thisETo^0.5)),7)
        }else{
          soil3=toupper(soil2)
          if(infiltration>=40&&(soil3=="SAND"||soil3=="LOAMY SAND")){
            TEWcor = min(15, 7 *(thisETo^0.5))
            REWcor = min(6, TEWcor - 0.01)
          }else{
            if(infiltration>=40&&(soil3=="LOAM"||soil3=="SILT"||soil3=="SANDY LOAM"
              ||soil3=="SILT LOAM"||soil3=="CLAY LOAM"||soil3=="SILTY CLAY"||soil3=="CLAY")){
              TEWcor = min(28, 13 *(thisETo^0.5))
              REWcor = min(9, TEWcor - 0.01)
            }
            
            
          }
        }
        
        if(infiltration>10&&infiltration<40){
          TEWcor=10
          REWcor=min(max(2.5,6/(thisETo^0.5)),7)
          t1 = REWcor / Eso
          tw=stages2[1]/(nw+0.5)
          if(tw>t1){
            kcss10=(TEWcor-(TEWcor-REWcor)*exp((-(tw-t1)*Eso*(1+((REWcor)/(TEWcor-REWcor))))/(TEWcor)))/(tw*thisETo)
            #kcss[1]=(TEWcor-(TEW-REW)*exp((-(tw-t1)*Eso*(1+((REW)/(TEW-REW))))/(TEW)))/(tw*ETo[[i]])
          }else{
            kcss10=Eso/thisETo
            
          }
          if((infiltration>10&&infiltration<40)&&(soil3=="SAND"||soil3=="LOAMY SAND")){
            TEWcor = min(15, 7 *(thisETo^0.5))
            REWcor = min(6, TEWcor - 0.01)
          }
          if((infiltration>10&&infiltration<40)&&(soil3=="LOAM"||soil3=="SILT"||soil3=="SANDY LOAM"
          ||soil3=="SILT LOAM"||soil3=="CLAY LOAM"||soil3=="SILTY CLAY"||soil3=="CLAY")){
            TEWcor = min(28, 13 *(thisETo^0.5))
            REWcor = min(9, TEWcor - 0.01)
          }
          t1 = REWcor / Eso
          if(tw>t1){
            kcss40=(TEWcor-(TEWcor-REWcor)*exp((-(tw-t1)*Eso*(1+((REWcor)/(TEWcor-REWcor))))/(TEWcor)))/(tw*thisETo)
            #kcss[1]=(TEWcor-(TEW-REW)*exp((-(tw-t1)*Eso*(1+((REW)/(TEW-REW))))/(TEW)))/(tw*ETo[[i]])
            
          }else{
            kcss40=Eso/thisETo
          }
          
          kcss[1]=kcss10+((infiltration-10)/(40-10))*(kcss40-kcss10)
          #print(cbind(day=i,kcss10=kcss10,kcss40=kcss40,kcss=kcss[1]))
          
        }else{
        t1 = REWcor / Eso
        tw=stages2[1]/(nw+0.5)
        if(tw>t1){
          kcss[1]=(TEWcor-(TEWcor-REWcor)*exp((-(tw-t1)*Eso*(1+((REWcor)/(TEWcor-REWcor))))/(TEWcor)))/(tw*thisETo)
          #kcss[1]=(TEWcor-(TEW-REW)*exp((-(tw-t1)*Eso*(1+((REW)/(TEW-REW))))/(TEW)))/(tw*ETo[[i]])
          
        }else{
          kcss[1]=Eso/thisETo
          
          }
        }
        
      }
     # print(cbind(REWcor=REWcor,TEWcor=TEWcor,t1=t1,tw=tw,Eso=Eso,Kc=kcss[[1]]))
      Kci=  kcss[[1]]*fw
      
     # print(fw)
      
      #print(c(nw=nw,t1=t1,tw=tw,REWcor=REWcor,TEWcor=TEWcor,REW=REW,Kc=kcss[1]))
      
    }else{
      if(i>stages2cum[1]&&i<=stages2cum[2]){
        Kci=kcss[[1]]+((i-stages2[1])/stages2[2])*(kcss[[2]]-kcss[[1]])
      }else{
        if(i>stages2cum[2]&&i<=stages2cum[3]){
          #mid
          #Kci=kcss[2]+((i-stages2cum[2])/stages2[3])*(kcss[4]-kcss[2])
          #if(is.null(EF)){
            #kcss[[2]]=kcss[[2]] + (0.04*(u2mid-2) - 0.004*(RHminmid-45))* (hi/3)^0.3
           # }
          Kci=kcss[[2]]
          #Kci=rastercon(Kci>0.45,Kci + (0.04*(u2i-2) - 0.004*(RHmini-45))* (hi/3)^0.3,Kci)
        }else{
          #late
          #if(is.null(EF)){
          #kcss[[3]]=rastercon(kcss[[3]]>0.45,kcss[[3]] + (0.04*(u2end-2) - 0.004*(RHminend-45))* (hi/3)^0.3,kcss[[3]])
          #}
          Kci=kcss[[2]]+((i-stages2cum[3])/stages2[4])*(kcss[[3]]-kcss[[2]])
          #print("yes")
        }
      }
    }
    }
    #print(Kci)
    if(!is.null(kc3)&&is.null(EF)){
      if(length(kc3)==length(day)){
        Kci=kc3[i]
      }
    }
    
    if(is.null(EF)){
      kcb=max(Kcmin,Kci)
      #kcb=Kci
      Kcmax = max(1.2 + (0.04*(u2[i]-2) - 0.004*(RHmini-45))* (h[[i]]/3)^0.3,(kcb+0.05))
      
      if(is.null(fc2)){
        fc=((kcb-Kcmin)/(max(Kcmax-Kcmin,0.01)))^(1+(0.5*h[[i]]) )
        #print(fc)
      }else{
        fc=fc2[[i]]
      }
      if(!is.null(latitude2)&&density2!="full"&&!is.null(DOY2)){
        if((is.null(latitude2)||is.null(DOY2))&&(nongrowing=="weeds"||nongrowing=="grass")){
          return (message("latitude and DOY needed"))
        }
        
        
        
        
       
          
    if((i>=stages2cum[2]&&i<=stages2cum[4])||(nongrowing=="weeds"||nongrowing=="grass")){
      
      if(density!="full"){
        kcbh=min(1.0+(0.1*max(h)),1.20)
        kcbfull=kcbh + (0.04*(u2i-2) - 0.004*(RHmini-45))* (max(h)/3)^0.3
        latituderad=latitude[i]*(pi/180)
        declination=0.409*sin((((2*pi*DOY[i])/(365)))-1.39)
        sinethisBeta=sin(latituderad)*sin(declination)+cos(latituderad)*cos(declination)
        thisBeta=asin(sinethisBeta)
        if(density=="row"||density=="Row"||density=="ROW"){
          fceff=min(fc*(1+(HWR[i]/tan(thisBeta))),1)
        }
        if(density=="random"||density=="round"||density=="Random"||density=="RANDOM"){
          fceff=min(fc/sin(thisBeta),1)
        }
        Kd=min(1,2*fc,fceff^(1/(1+(max(h)))))
        #Kcmin=kcss[1]
        Kci=Kcmin+Kd*(kcbfull-Kcmin)
        #print(Kci)
        }
      
    }
      }
    }
    if(!is.null(slope2)&&!is.null(r12)&&!is.null(y2)){
      Fr=(slope[i]+y[i]*(1+0.34*(u2i)))/(slope[i]+y[i]*(1+0.34*(u2i)*(r1[i]/100)))
    Kci=Kci*Fr
    }
    if(!is.null(nongrowing)){
    if(nongrowing=="Bare"||nongrowing=="bare"||nongrowing=="Bare Soil"||nongrowing=="bare soil"||
       nongrowing=="mulch"||nongrowing=="Mulch"||nongrowing=="MULCH"){
    Kci=kcss[1]  
    if(nongrowing=="mulch"||nongrowing=="Mulch"||nongrowing=="MULCH"){
      Kci=Kci*0.95 
      #
    }
    
    }
    }
    if(!is.null(nongrowing)){
      if(kctype!="single"&&(nongrowing=="Bare"||nongrowing=="bare"||nongrowing=="Bare Soil"||nongrowing=="bare soil"
                          ||nongrowing=="mulch"||nongrowing=="Mulch"||nongrowing=="MULCH")){
        
       if(nongrowing=="mulch"||nongrowing=="Mulch"||nongrowing=="MULCH"){
         fc=0
         fw=1
         TEW=TEW*0.95
         
       }else{
         Kci=0
        Kcmin=0
        
       }
      }
      
    }
    kcb=Kci
    
    #print(kcb)
    
    
    if(!is.null(EF)&&!is.null(Pinter)){
      if(nrow(xydata)==length(P[i,])){
        #print("YES")
        Pdata=cbind(xydata,P[i,])
        longitude=Pdata[,1]
        latitude=Pdata[,2]
        var=Pdata[,3]  
        Prec= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
        #print(i)
      }else{
        Prec=mapmatrix(P,i,map)  
      }
    }else{
      Prec=P[[i]]
    }
    if(is.numeric(rc)){
      Runoff=rc*Prec
    }else{
      CNI=CNII/(2.281-(0.01281*CNII))
      #CNI=(4.2*CNII)/(10-(0.058*CNII))
      CNIII=CNII/(0.427+(0.00573*CNII))
      #CNIII=(23*CNII)/(10+(0.13*CNII))
      DeAWCIII=0.5*REW
      DeAWCI=(0.7*REW)+(0.3*TEW)
      if(is.null(EF)){
        if(Dei<=DeAWCIII){
          CN= CNIII 
        }else{
          if(Dei<=DeAWCI){
            CN= CNI
          }else{
            if(Dei<DeAWCI&&Dei>DeAWCIII){
              CN=(((Dei-(0.5*REW))*CNI)+(((0.7*REW)+(0.3*TEW)-Dei)*CNIII))/((0.2*REW)+(0.3*TEW))  
            }else{
              CN=CNII
            }  
          }  
        }
      }else{
        CN=rastercon(Dei<=DeAWCIII,CNIII,-1000)
        #CN=rastercon(Dei<DeAWCI&&Dei>DeAWCIII,(((Dei-(0.5*REW))*CNI)+(((0.7*REW)+(0.3*TEW)-Dei)*CNIII))/((0.2*REW)+(0.3*TEW)),CN)
        CN=rastercon(Dei<DeAWCI,(((Dei-(0.5*REW))*CNI)+(((0.7*REW)+(0.3*TEW)-Dei)*CNIII))/((0.2*REW)+(0.3*TEW)),CN)
        CN=rastercon(Dei>DeAWCIII,(((Dei-(0.5*REW))*CNI)+(((0.7*REW)+(0.3*TEW)-Dei)*CNIII))/((0.2*REW)+(0.3*TEW)),CN)
        
        CN=rastercon(CN==-1000,CNII,CN)
        
        #CN=rastercon(Dei<=DeAWCIII,CNIII,rastercon(Dei<=DeAWCI,CNI,rastercon(Dei<DeAWCI&&Dei>DeAWCIII,(((Dei-(0.5*REW))*CNI)+(((0.7*REW)+(0.3*TEW)-Dei)*CNIII))/((0.2*REW)+(0.3*TEW)))),CNII)
      }
      #}
      #print(CN)
      
      S=(1000/CN) -10
      #S05=1.33*S^1.15
      Ia=0.05*S
      #Ia=0.2*S
      Pin=0.0393701*Prec
      #Pin=0.0393701*Pin
      #Runoff=rastercon(Pin<Ia,0,((Pin-Ia)^2)/((Pin+(0.95*S05)))
      #Runoff=rastercon(Pin<Ia,0,((Pin-Ia))^2/((Pin+0.95*S05)))*25.4
      Runoff=rastercon(Pin<Ia,0,((Pin-Ia)^2)/((Pin-Ia)+S))*25.4
      CNII=CN
    }
    #kc###################
    
    
    #print((I[i]/fw) - (P[i]-Runoff))
    if(is.null(EF)){
      #print(fw)
      Deistart=ifelse(strptime(time2[i], "%H:%M") <= strptime("12:00", "%H:%M"), max(Dei - (I[i]/fw) - (P[i]-Runoff),  0), Dei)
      #thisDr=ifelse(strptime(time2[i], "%H:%M") <= strptime("12:00", "%H:%M"), Drend , Drstart)
    }
    
    
   # Deistart=ifelse(strptime(time, "%H:%M") <= strptime("12:00", "%H:%M"), max(Dei - I[i]/fw - (P[i]-Runoff),  0), Dei)
    
    #Deistart=data$Dstart[[i]]
    #Dei=Dei-(P-Run)
    
      #print(c(Kcmax,kcb,fc))
      #if(is.na(fc)&&kcb<=0.15){
       # kcb=0.2
       # fc=((kcb-Kcmin)/(Kcmax-Kcmin))^(1+(0.5*h[[i]]) ) 
        
        # return(print(paste("fc cannot be calculated. Please choose a higher kcbini")))
    #}#
    
    if(is.null(EF)){
      
      few=min(1-fc,fw)

      if(fw2[i]=="drip"||fw2[i]=="DRIP"||fw2[i]=="Drip"){
        few=min(1-fc,(1-((2/3)*fc))*fw)
      }
      if(Deistart<=REW){
        Kr=1
      }else{
        Kr=(TEW-Deistart)/(TEW-REW)
      }
      Ke=max(min(Kr*(Kcmax-kcb),few*Kcmax),0)
      
      E=max(Ke*ETo[[i]],0)
      
    }
    #Kr=max((TEW-Deistart)/(TEW-REW),0)
    
    #runoff
    #Runoff=(P[[i]]*rc)
    #if(P[[i]]>0){
    
    
    #water loss out of the root zone by deep percolation on day i
    #print(c(Kr,Kcmin,kcb,few,Kcmax,Ke))
    
    
    #Dei=max()
    
    #Total available soil water
    TAW=1000*(FC-WP)*Zri
    #print(Zri)
    
    #readily available soil water
    RAW=pi*TAW
    
    #DP=max(P.RO[[i]]+I[[i]]-Drend,0)
    #root zone depletion at the end of day i
    
    
    
    if(is.null(EF)){
      Drstart=ifelse(strptime(time2[i], "%H:%M") <= strptime("12:00", "%H:%M"), max(Drend - I[i] - (P[i]-Runoff),  0), Drend)
      #thisDr=ifelse(strptime(time2[i], "%H:%M") <= strptime("12:00", "%H:%M"), Drend , Drstart)
    }else{
      
      #Deistart=ifelse(strptime(time, "%H:%M") <= strptime("12:00", "%H:%M"), max(Dei - I[[i]]/fw - (P[[i]]-Runoff),  0), Dei)
      thisDrend=Drend - I[i] - (Prec-Runoff)
      thisDrend[thisDrend<0]=0
      #print(P[i])
      
      #print(thisDrend)
      if(i>1){
       # Drstart=ifelse(strptime(time2[i], "%H:%M") <= strptime("12:00", "%H:%M"), thisDrend, Drend)
      if(strptime(time2[i], "%H:%M") <= strptime("12:00", "%H:%M")){
        Drstart=thisDrend
      }else{
        Drstart=Drend 
      }
        }else{
        Drstart=Drend
        
      }
      
      
    }
    
   # print(cbind(Drstart,Drend))

    
    if(is.null(EF)){
      if(Drstart<RAW){
        Ks=1 
      }else{
        Ks=max((TAW-Drstart)/((1-pi)*TAW),0)
      }
    }
    
    if(!is.null(EF)&&!is.null(Rninter)){
      if(nrow(xydata)==length(Rn[i,])){
        Pdata=cbind(xydata,Rn[i,])
        longitude=Pdata[,1]
        latitude=Pdata[,2]
        var=Pdata[,3]  
        Rni= invdist(longitude=longitude,latitude=latitude,var=var,ext=map) 
        #print(i)
      }else{
        Rni=mapmatrix(Rn,i,map)  
      }
    }else{
      Rni=Rn[[i]]
    }
    #print(Ks)
    #Ks=1
    if(is.null(EF)){
      if(ksingle==FALSE){
        kcb=Kci
        Kc=((Ks*kcb)+Ke)
        Kci1=(kcb+Ke)
      }else{
        Kci1=Kci
        kcb=Kci
        Kc=kcb*Ks  
      }
      
      Kc=max(Kcmin,Kc)
      #print(kcss)
      ETc=Kci1*EToi
      ETc2=Kc*EToi
      #ETc_adjust=ETc2
      pi=p+0.04*(5-ETc)
      
    }else{
      #print(Kci)
      if(kctype=="METRIC"||kctype=="metric"||kctype=="SSEB"||kctype=="sseb"){
        ETc2=Kci*ETo[[i]]
      }else{
        ETc2=(Kci*Rni)*0.035
          }
      pi=p+0.04*(5-ETo[i])
    }
    
    if(is.null(EF)){
      pi=max(0.1,pi)
      pi=min(pi,0.8)
    }else{
     # pi=maxValue(0.1,pi)
      pi[pi<0.1]=0.1
      pi[pi>0.8]=0.8
      
      #pi=minValue(pi,0.8)
    }
    #pi=max(0.1,pi)
    #pi=min(pi,0.8)
    
    #pi=0.6
    thisDr=rastercon(strptime(time2[i], "%H:%M") <= strptime("12:00", "%H:%M"), Drend , Drstart)
    
    
    DP=max((Prec-Runoff)+Ii-ETc2-thisDr,0)
    #plot(ETc2)
    #Dei=data$Dei[[i]]
    #Dei=max(Dei-(P[[i]]-Runoff)-(I[[i]]/fw)+(E/few)+Tewi+DP,0)
    ##capillary rise
    #initgw=(DP/1000)+initgw
    if(is.null(a1)){
    a1=FC*Zri*1000
    }
    #b1=-0.17
    if(is.null(a2)){
    a2=1.1*((FC+WP)/2)*Zri
    }
    #b2=-0.27
    #a3=-1.3
    soil4=toupper(soil2)
    if(soil4=="CLAY"||soil4=="SILTY CLAY"||soil4=="LOAM"){
      if(is.null(b3)){
      b3=6.7
      }
    }else{
      if(is.null(b3)){
      b3=6.2
      }
    }
    if(soil4=="SILTY LOAM"||soil4=="SILTY CLAY"||soil4=="LOAM"){
      if(is.null(a4)){
      a4=4.6
      }
    }else{
      if(is.null(a4)){
        a4=4.6
      }
    }
    
    if(soil4=="SILTY"||soil4=="LOAM"){
      if(is.null(b4)){
        b4=-0.65
      }
    }else{
      if(is.null(b4)){
        b4=-2.5
      }
    }
    Wc=a1*initgw^b1
    if(is.null(EF)){
      if(initgw>=3){
        Ws= a2*initgw^b2
      }else{
        Ws=240  #mm
      }
    }else{
      Ws= rastercon(initgw>=3,a2*initgw^b2,240)
    }
    ETm=ETc2
    if(is.null(EF)){
      if(ETm<=4){
        Dwc=(a3*ETm)+b3 
        k=1-exp(-0.61*LAI)
      }else{
        Dwc=1.4 #m
        k=3.8*ETm
      }
    }else{
      Dwc=rastercon(ETm<=4,(a3*ETm)+b3,1.4)  
      k=rastercon(ETm<=4,1-exp(-0.61*LAI),3.8/ETm) 
    }
    if(is.null(EF)){
      if(initgw<=Dwc){
        CRmax=k*ETm
      }else{
        CRmax=a4*initgw^b4
      }
    }else{
      CRmax=rastercon(initgw<=Dwc,k*ETm,a4*initgw^b4)
    }
    a=((Ws+FC)/2)*Zri*1000
    if(soil4=="SAND"||soil4=="SANDY LOAM"||soil4=="LOAMY SAND")
    {
      b=-0.0174
    }else{
      b=-0.0172
    }
    if(is.null(EF)){
      if(DP>0){
        timeDP=1
        DEEP=TRUE
      }else{
        if(DEEP==TRUE){
          timeDP=timeDP+1
        }
      }
    }else{
      timeDP=rastercon(DP>0,1,timeDP+1) 
      DEEP=rastercon(DP>0,TRUE,FALSE)  
      
    }
    #if(timeDP>0){
    Wa=a*timeDP^b
    #print(timeDP)
    #}
    if(is.null(EF)){
      if(Wa<Ws){
        CR=CRmax
      }else{
        if(Wa>=Ws&&Wa<=Wc){
          CR=CRmax*((Wc-Wa)/(Wc-Ws)) 
        }else{
          CR=0
        }
      }
    }else{
      CR=rastercon(Wa<Ws,CRmax,CRmax*((Wc-Wa)/(Wc-Ws)))
      CR=rastercon(Wa<Wc,CRmax*((Wc-Wa)/(Wc-Ws)),CR)
      
      #CR=rastercon(Wa>=Ws&Wa<=Wc,CRmax*((Wc-Wa)/(Wc-Ws)),0)
      #CR[Wa>=Ws&Wa<=Wc,]=CRmax*((Wc-Wa)/(Wc-Ws))
    }
    CR=max(CR,0)
    DPei=(Prec-Runoff)+(Ii/fw)-Dei
    if(is.null(EF)){
      DPei=max(DPei,0)
      kti=((1-(Dei/TEW))/(1-(Drend/TAW)))*(Ze[[i]]/Zr[[i]])^0.6
      Tewi=kti*kcb*Ks*ETo[[i]]
      Dei=max(Dei-((Prec-Runoff))-(Ii/fw)+(E/few)+(Tewi)+DPei,0)
    }else{
      DPei[DPei<0]=0
      Dei=max(Dei-((Prec-Runoff))-(Ii/fw)+ETc2+DPei,0)
    }
    
    #Drend=Drend-(Prec-Runoff)-Ii-CR+ETc2+DP
    if(!is.null(EF)){
      Drend=Drend-(Prec-Runoff)-Ii-CR+ETc2+DP
      Drend[Drend<0,]=0
      Drend[Drend>TAW,]=TAW
    }else{
      Drend=max(Drend-(Prec-Runoff)-Ii-CR+ETc2+DP,0)
      Drend=min(Drend,TAW)
      
    }
    
    #print(c(P[i],Runoff,I[i]))
    #print(paste("P",Prec))
    #print(paste("Runoff",Runoff))
    #print(paste("I",Ii))
    
    
    
    #print(CR)
    #print(ETc2)
    #print(DP)
    
    initgw=initgw-(DP/1000)+(CR/1000)+((DP*Kb)/1000)
    #AW=TAW-Drend
    mod_theta=FC-(Drstart/(Zri*1000))
    if(!is.null(EF)){
      mod_theta=FC-(Drend/(Zri*1000))
    }
    if(!is.null(EF)&&!is.null(theta)){
    mod_theta[mod_theta<0,]=0
    mod_theta[mod_theta>1,]=1
    }
    
    TSW=1000*(mod_theta)*Zri
    #print(mod_theta)
    AW=1000*(mod_theta-WP)*Zri
    #print(AW)
    
    if(!is.null(theta)){
    AW_measured=1000*(thetai-WP)*Zri
    TSW_measured=1000*(thetai)*Zri
    
    }
    #print(TSW)
    Ineed=rastercon(Drstart>=RAW,Drstart,0)
    if(!is.null(EF)){
      Ineed=rastercon(Drend>=RAW,Drend,0)
    }
    ETc_adjsum=ETc_adjsum+ETc2
    ETc_sum=ETc_sum+ETo[i]
    #print(EF)
    
    #print(RAW)
    #print(AW_measured)
    message(paste("Day", i,"of", max(day)))
    Ineed[Ineed<=0]=0
    if(!is.null(EF)){
     # print(day)
      
      #i2= format(i, nsmall = 200000000)
      #reportF=format(report, nsmall = 200000000)
      if(length(report[report==i])>0&&is.null(report2)){
        thisETc2=NULL
        ETa1=ETc2
        if(!inherits(ETa1, "RasterLayer")){
         thisETc2=kcss[[1]] 
         thisETc2[thisETc2>-1000,]=ETa1
         ETa1=thisETc2
         #rm(thisETc2)
        }
        Kc1=Kci
        Drend2= Drend 
        if(!inherits(Drend2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=Drend2
          Drend2=thisETc2
        }
        AW2=AW
        if(!inherits(AW2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=AW2
          AW2=thisETc2
          }
        TAW2=TAW
        if(!inherits(TAW2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=TAW2
          TAW2=thisETc2
        }
        RAW2=RAW
        if(!inherits(RAW2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=RAW2
          RAW2=thisETc2
        }
        Ineed2=Ineed
        if(!inherits(Ineed2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=Ineed2
          Ineed2=thisETc2
        }
        Runoff2=Runoff
        if(!inherits(Runoff2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=Runoff2
          Runoff2=thisETc2
        }
        CR2=CR
        if(!inherits(CR2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=CR2
          CR2=thisETc2
        }
        DP2=DP
        if(!inherits(DP2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=DP2
          DP2=thisETc2
        }
        gw=initgw
        if(!inherits(gw, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=gw
          gw=thisETc2
        }
        report2=TRUE
        #print(Runoff2)
        
        #report[i]=NA
        if(!is.null(theta)){
        AW_measured2=AW_measured
        if(!inherits(AW_measured2, "RasterLayer")){
          thisETc2=Kci 
          thisETc2[thisETc2>-1000,]=AW_measured2
          AW_measured2=thisETc2
        }
        }
        
        rm(thisETc2)
      }else{
      if(length(report[report==i])>0&&!is.null(report2)){
        #print(report3)
        #print(Ineed2)
        
        ETa1=stack(ETa1,ETc2)
        Kc1=stack(Kc1,Kci)
        Drend2= stack(Drend2,Drend)
        AW2=stack(AW2,AW)
        
        #TAW2=stack(TAW2,TAW)
        #RAW2=stack(RAW2,RAW)
        
        Ineed2=stack(Ineed2,Ineed)
        
        if(inherits(Runoff, "RasterLayer")){
          Runoff2=stack(Runoff2,Runoff)
        }else{
          Runoff2=rbind(Runoff2,Runoff)
        }
        
        if(inherits(CR, "RasterLayer")){
          CR2=stack(CR2,CR)
        }else{
          CR2=rbind(CR2,CR)
        }
        
        if(inherits(DP, "RasterLayer")){
          DP2=stack(DP2,DP)
        }else{
          DP2=rbind(DP2,DP)
        }
        #print(report[i])
        
        if(inherits(initgw, "RasterLayer")){
          #if(!inherits(gw, "RasterLayer")){
            #gw=initgw
          #}
          #print(initgw)
          gw=stack(gw,initgw)
        }else{
          gw=rbind(gw,initgw)
        }
        #print(AW_measured)
        if(!is.null(theta)){
          
        if(inherits(AW_measured, "RasterLayer")){
          AW_measured2=stack(AW_measured2,AW_measured)
        
        }else{
          #print("yess")
          
          AW_measured2=rbind(AW_measured2,AW_measured)
        }
        }
        if(!is.null(theta)&&length(report3)==daylenths){
          report[i]=NA
          
        }
        
      }
        #print(i)
       # print(report[i])
        
      }
      #print("KKKKKKKKKKKKKKKKKKKKK")
      runofftest=NULL
    if(i==1){
      if(!is.null(xydata)){
        #print("KKKKKKKKKKKKKKKKKKKKK")
        
          ETcxy=xyextract(ETc2,xydata[,1],xydata[,2])
          #print(Kcxy)
          
          colnames(ETcxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          Kcxy=xyextract(Kci,xydata[,1],xydata[,2])
          
          #print("kci......")
          colnames(Kcxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          Drendxy=xyextract(Drend,xydata[,1],xydata[,2])
          #print("Drend......")
          colnames(Drendxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          
          if(inherits(AW, "RasterLayer")){
          Awcxy=xyextract(AW,xydata[,1],xydata[,2])
          colnames(Awcxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }else{
            Awcxy= cbind(xydata[,1],xydata[,2],AW)
            colnames(Awcxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }
          
          #TAWxy=xyextract(TAW,xydata[,1],xydata[,2])
          #colnames(TAWcxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          #RAWxy=xyextract(RAW,xydata[,1],xydata[,2])
          #colnames(RAWcxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          if(inherits(Ineed, "RasterLayer")){
            Ineedxy=xyextract(Ineed,xydata[,1],xydata[,2])
          colnames(Ineedxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }else{
            Ineedxy= cbind(xydata[,1],xydata[,2],Ineed)
            colnames(Ineedxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }
          if(inherits(Runoff, "RasterLayer")){
            Runoff2xy=xyextract(Runoff,xydata[,1],xydata[,2])
            colnames(Runoff2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))}else{
            Runoff2xy= cbind(xydata[,1],xydata[,2],Runoff)
            colnames(Runoff2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }
          if(inherits(CR, "RasterLayer")){
            CR2xy=xyextract(CR,xydata[,1],xydata[,2])
            colnames(CR2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))}else{
              CR2xy= cbind(xydata[,1],xydata[,2],CR)
              colnames(CR2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))
            }
          
          if(inherits(DP, "RasterLayer")){
            DP2xy=xyextract(DP,xydata[,1],xydata[,2])
            colnames(DP2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))}else{
              DP2xy= cbind(xydata[,1],xydata[,2],DP)
              colnames(DP2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))
            }
          
          if(inherits(initgw, "RasterLayer")){
            gwxy=xyextract(initgw,xydata[,1],xydata[,2])
            colnames(gwxy)=c("longitude", "latitude",paste("day",day[i],sep=""))}else{
              gwxy= cbind(xydata[,1],xydata[,2],initgw)
              colnames(gwxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
            }
          
          if(!is.null(theta)){
            
          if(inherits(AW_measured, "RasterLayer")){
            AW_measuredxy=xyextract(AW_measured,xydata[,1],xydata[,2])
            colnames(AW_measuredxy)=c("longitude", "latitude",paste("day",day[i],sep=""))}else{
              AW_measuredxy= cbind(xydata[,1],xydata[,2],AW_measured)
              colnames(AW_measuredxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
            }
          }
          
      }
      
      #RO=Runoff
    }else{
      
      if(!is.null(xydata)){
        thisextract=xyextract(ETc2,xydata[,1],xydata[,2])
        thisextract=thisextract[3]
        colnames(thisextract)=paste("day",day[i],sep="")
        ETcxy=cbind(ETcxy,thisextract)
        thisextract=xyextract(AW,xydata[,1],xydata[,2])
        thisextract=thisextract[3]
        colnames(thisextract)=paste("day",day[i],sep="")
        Awcxy=cbind(Awcxy,thisextract)
        #print(Awcxy)
        thisextract=xyextract(Kci,xydata[,1],xydata[,2])
        thisextract=thisextract[3]
        colnames(thisextract)=paste("day",day[i],sep="")
        Kcxy=cbind(Kcxy,thisextract)
        
        thisextract=xyextract(Drend,xydata[,1],xydata[,2])
        thisextract=thisextract[3]
        colnames(thisextract)=paste("day",day[i],sep="")
        Drendxy=cbind(Drendxy,thisextract)
        
        
        thisextract=xyextract(Ineed,xydata[,1],xydata[,2])
        thisextract=thisextract[3]
        colnames(thisextract)=paste("day",day[i],sep="")
        Ineedxy=cbind(Ineedxy,thisextract)
        #print("okkkkkkkkkkkk2")
        
        
        
        if(inherits(initgw, "RasterLayer")){
          thisextract=xyextract(initgw,xydata[,1],xydata[,2])
          thisextract=thisextract[3]
          colnames(thisextract)=paste("day",day[i],sep="")
          gwxy=cbind(gwxy,thisextract)}else{
            gwxy=cbind(gwxy,initgw)
            #colnames(gwxy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }
        
        if(inherits(Runoff, "RasterLayer")){
          thisextract=xyextract(Runoff,xydata[,1],xydata[,2])
          thisextract=thisextract[3]
          colnames(thisextract)=paste("day",day[i],sep="")
          runofftest=NULL
          Runoff2xy=cbind(Runoff2xy,thisextract)}else{
            runofftest=TRUE
            Runoff2xy=cbind(Runoff2xy,Runoff)
            #colnames(Runoff2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }
        if(inherits(CR, "RasterLayer")){
          thisextract=xyextract(CR,xydata[,1],xydata[,2])
          thisextract=thisextract[3]
          colnames(thisextract)=paste("day",day[i],sep="")
          CR2xy=cbind(CR2xy,thisextract)}else{
            CR2xy=cbind(CR2xy,CR)            #colnames(CR2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }
        
        if(inherits(DP, "RasterLayer")){
          thisextract=xyextract(DP,xydata[,1],xydata[,2])
          thisextract=thisextract[3]
          colnames(thisextract)=paste("day",day[i],sep="")
          DP2xy=cbind(DP2xy,thisextract) }else{
            DP2xy=cbind(DP2xy,DP)            #colnames(DP2xy)=c("longitude", "latitude",paste("day",day[i],sep=""))
          }
        if(!is.null(theta)){
        if(inherits(AW_measured, "RasterLayer")){
          thisextract=xyextract(AW_measured,xydata[,1],xydata[,2])
          thisextract=thisextract[3]
          colnames(thisextract)=paste("day",day[i],sep="")
          AW_measuredxy=cbind(AW_measuredxy,thisextract) 
          }else{
            AW_measuredxy=cbind(AW_measuredxy,AW_measured) 
          }
        }
        #print("yess")
      
      }
      
    }
    }
    #print(kcb)
    
    #plot(Drend)
    if(!is.null(DOY2)){
      today=DOY2[i]
    }else{
      today=NA
    }
    if(is.null(thetai)){
      thetai=NA
    }
    if(is.null(EF)){ 
      
      if(kctype!="single"){
        
        outputdual=rbind(outputdual,data.frame(Date=today,Day=i,ETo=ETo[[i]],Ineed=max(0,Ineed),TAW=TAW,
                                               RAW=RAW,Drstart=Drstart,Runoff=Runoff,
                                             Ks=Ks,Zr=Zr[i],h=h[i],
                                             Kc=kcb,Kcb=kcb,De=Dei,Drend=Drend,
                                             Kr=Kr,Ke=Ke,p=pi,
                                             DP=DP,CR=CR,GW=initgw,Evaporation=E,Transpiration=ETc2-E,Kc_adj=Kc,
                                             ETc_adj=ETc2,ETc=ETc,CN=CN,fw=fw,few=few,fc=fc,P=P[i],I=I[i],
                                             AW_modelled=max(AW,0),
                                             AW_measured=AW_measured,TSW_measured=TSW_measured,TSW_modelled=TSW,
                                             theta_measured=thetai,theta_modelled=mod_theta,crop=crop2,kctype=kctype))
                                             
      }else{
      
        outputdual=rbind(outputdual,data.frame(Date=today,Day=i,ETo=ETo[[i]],Zr=Zr[[i]],
                  TAW=TAW,RAW=RAW,Drstart=Drstart,
                                             Ineed=max(0,Ineed),Runoff=Runoff,p=pi,
                                             Ks=Ks,Kcb=kcb,h=h[[i]],Kc_adj=Kc,
                                             ETc_adj=ETc2,ETc=ETc,DP=DP,CR=CR,GW=initgw,
                                             Drend=Drend,CN=CN,fc=fc,P=P[i],I=I[i]
                                             ,AW_modelled=max(AW,0),AW_measured=AW_measured,
                                             TSW_measured=TSW_measured,TSW_modelled=TSW,
                                             theta_measured=thetai,theta_modelled=mod_theta,crop=crop2,kctype=kctype))
      #print(c(today=today,Day=i,ETo=ETo[[i]],Zr[i],TAW=TAW,RAW=RAW,Drstart=Drstart, Ks=Ks,fc=fc,lengths=lengths))
      }
    }
    
    #print(ETc2)
    #print(ETa1)
    #print(c(RAW,DP,Drstart,Drend))
    #print(cbind(Date=today,Day=i,ETo=ETo[[i]],Zr=Zr[[i]],TAW=TAW,Drstart=Drstart))
    i=i+1
  }
  #print(output)
  if(is.null(Ky)){ 
    Ky=1
  }
  if(is.null(EF)){
   
    if(length(Ky)==1){
      Ky=c(Ky,Ky,Ky,Ky,Ky)
    }
    if(length(Ky)>1){
      initoutput<- outputdual[ which(outputdual$Day <=stages2cum[1]),]
      ETdecrease_init=(1-(sum(initoutput$ETc_adj)/sum(initoutput$ETc)))
      Ya_by_Ym_init=(1-(Ky[1]*ETdecrease_init))
      yield_decrease_init=1-Ya_by_Ym_init
      cropoutput<- outputdual[ which(outputdual$Day>stages2cum[1]&outputdual$Day <=stages2cum[2]),]
      ETdecrease_crop=(1-(sum(cropoutput$ETc_adj)/sum(cropoutput$ETc)))
      Ya_by_Ym_crop=(1-(Ky[2]*ETdecrease_crop))
      yield_decrease_crop=1-Ya_by_Ym_crop
      midoutput<- outputdual[ which(outputdual$Day>stages2cum[2]&outputdual$Day <=stages2cum[3]),]
      ETdecrease_mid=(1-(sum(midoutput$ETc_adj)/sum(midoutput$ETc)))
      Ya_by_Ym_mid=(1-(Ky[3]*ETdecrease_mid))
      yield_decrease_mid=1-Ya_by_Ym_mid
      lateoutput<- outputdual[ which(outputdual$Day>stages2cum[3]&outputdual$Day <=stages2cum[4]),]
      ETdecrease_late=(1-(sum(lateoutput$ETc_adj)/sum(lateoutput$ETc)))
      Ya_by_Ym_late=(1-(Ky[4]*ETdecrease_late))
      yield_decrease_late=1-Ya_by_Ym_late
     # print(lateoutput)
      sumvariables=c("ETo","TAW","RAW","Drstart","Ineed","Runoff", "ETc_adj",
                     "ETc","DP","CR","GW","Drend","CN",          
                     "P" ,"I","AW_modelled","AW_measured","TSW_measured" ,"TSW_modelled")
      meanvariables=c("p","Ks","Kcb","Kc_adj","h","Zr","fc","theta_measured", "theta_modelled")
      initsum=(colSums(initoutput[sumvariables]))
      initmean=(colMeans(initoutput[meanvariables]))
      thisinit=c(initsum,initmean)
      cropsum=(colSums(cropoutput[sumvariables]))
      cropmean=(colMeans(cropoutput[meanvariables]))
      thiscrop=c(cropsum,cropmean)
      midsum=(colSums(midoutput[sumvariables]))
      midmean=(colMeans(midoutput[meanvariables]))
      thismid=c(midsum,midmean)
      latesum=(colSums(lateoutput[sumvariables]))
      latemean=(colMeans(lateoutput[meanvariables]))
      thislate=c(latesum,latemean)
      totalsum=(colSums(outputdual[sumvariables]))
      totalmean=(colMeans(outputdual[meanvariables]))
      thistotal=c(totalsum,totalmean)
    #names(thislate)=paste("late",names(thislate),sep="_")
    #print(names(thislate))
    }
    water.balance=data.frame(total=thistotal, initial=thisinit,crop=thiscrop,
                          mid=thismid,late=thislate)
    Ky=Ky[5]
    ETdecrease=(1-(sum(outputdual$ETc_adj)/sum(outputdual$ETc)))
    Ya_by_Ym=(1-(Ky*ETdecrease))
    yield_decrease=1-Ya_by_Ym
    yield_decrease=data.frame(total=yield_decrease, initial=yield_decrease_init,crop=yield_decrease_crop,
                              mid=yield_decrease_mid,late=yield_decrease_late)
    ETdecrease=data.frame(total=ETdecrease, initial=ETdecrease_init,crop=ETdecrease_crop,
                          mid=ETdecrease_mid,late=ETdecrease_late)
    
    Ya_by_Ym=data.frame(Ya_by_Ym.total=Ya_by_Ym, Ya_by_Ym.initial=Ya_by_Ym_init,Ya_by_Ym.crop=Ya_by_Ym_crop,
                        Ya_by_Ym.mid=Ya_by_Ym_mid,Ya_by_Ym.late=Ya_by_Ym_late)
    
    Ya_by_Yms=cbind(ETdecrease=ETdecrease,yield_decrease=yield_decrease,Ya_by_Ym)
    #Kc=c(Kcini=kcss[1],Kcmid=kcss[2],Kcend=kcss[2])

    factor=list(output=outputdual,yield_by_ET_decrease=Ya_by_Yms,
         parameters=parameters,water.balance.stages=water.balance,kctype=kctype)
    
  }else{
    
      names(AW2) =paste("Day",report3,sep="")
      names(Kc1)=paste("Day",report3,sep="")
      names(ETa1)=paste("Day",report3,sep="")
      names(Drend2)=paste("Day",report3,sep="")
     names(Ineed2)=paste("Day",report3,sep="")
     names(DP2)=paste("Day",report3,sep="")
     names(Runoff2)=paste("Day",report3,sep="")
     names(gw)=paste("Day",report3,sep="")
     names(CR2)=paste("Day",report3,sep="")
     if(!is.null(theta)){
       #print("yes")
     names(AW_measured2)=paste("Day",report3,sep="")
     #print("yes")
     
     }
     
     TAW=AW2+Drend2
     
     RAW=Drend2-Ineed2
     if(!is.null(xydata)){
     TAWxy=Awcxy+Drendxy
     TAWxy$longitude=Awcxy$longitude
     TAWxy$latitude=Awcxy$latitude
     RAWxy=Drendxy-Ineedxy
     RAWxy$longitude=Ineedxy$longitude
     RAWxy$latitude=Ineedxy$latitude
     
     }else{
       Drendxy=NULL
       TAWxy=NULL
       RAWxy=NULL
       Kcxy=NULL
       ETcxy=NULL
       Ineedxy=NULL
       AWxy=NULL
       Runoff2xy=NULL
       DP2xy=NULL
       CR2xy=NULL
       gwxy=NULL
       AW_measuredxy=NULL
     }
     if(!is.null(runofftest)&&!is.null(xydata)){
       names(Runoffxy)=paste("Day",day,sep="")
       Runoffxy$longitude=xydata$longitude
       Runoffxy$latitude=xydata$latitude
     }
      #names(TAW2)=paste("Day",report3,sep="")
      #names(RAW2)=paste("Day",report3,sep="")
    #}
      #TAW=TAW2,TAWxy=TAWxy,RAW=RAW2,RAWxy=RAWxy,Ineed=Ineed2,Ineedxy=Ineedxy,
      #print("okkkkkkkkkkkk2")
     ETdecrease=(1-((ETc_adjsum)/(ETc_sum)))
     Ya_by_Ym=(1-(Ky*ETdecrease))
     
     ETdecreasexy=NULL
     yield_decreasexy=NULL
     if(!is.null(xydata)){
     ETdecreasexy=xyextract(ETdecrease,xydata[,1],xydata[,2])
     colnames(ETdecreasexy)=c("longitude", "latitude","ETdecrease")
     yield_decreasexy=xyextract(1-Ya_by_Ym,xydata[,1],xydata[,2])
     colnames(yield_decreasexy)=c("longitude", "latitude","yield_decrease")
     }
     factor= list(EF=Kc1,EFxy=Kcxy,ETc_adj=ETa1,ETc_adj_xy=ETcxy,Drend=Drend2,
         Drendxy=Drendxy,TAW=TAW,TAWxy=TAWxy,RAW=RAW,RAWxy=RAWxy,
         AW_modelledxy=Awcxy,AW_modelled=AW2,AW_measured=AW_measured2,AW_measuredxy=AW_measuredxy,
         Ineed=Ineed2,Ineedxy=Ineedxy,Runoff=Runoff2,Runoffxy=Runoff2xy,
         DP=DP2,DPxy=DP2xy,CR=CR2,CRxy=CR2xy,gw=gw, gwxy=gwxy,xydata=xydata,map=map,
         ETdecrease=ETdecrease,ETdecreasexy=ETdecreasexy,
         yield_decrease=1-Ya_by_Ym,yield_decreasexy=yield_decreasexy)
  }
}
  factor$call<-match.call()
  
  class(factor)<-"kc"
  factor
}

#' @param horiz logical; if TRUE the legend is drawn horizontally rather than vertically.
#' @param ... further arguments passed to the plotting functions.
#' @method plot kc
#' @export
#' @rdname kc
plot.kc<-function(x,output="AW_measured",main=NULL,cex=1,legend=TRUE,pos="top",horiz=FALSE,...)
{
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  #print(x$output$)
  if(is.null(x$EFxy)){
    #par(xpd = TRUE, mar = par()$mar + c(0,0,0,7))
    refine="black"
    dots="p"
    label1="Measured"
    label2="modelled"
    
thismeasure=output
thismodel="AW_modelled"
thisylab="Available Soil Moisture[mm]"
  if(output=="TSW_measured"||output=="TSW_modelled"||output=="total"){
    thismeasure="TSW_measured"
    thismodel="TSW_modelled"
    thisylab="Total Available Soil Moisture[mm]"
    
  }
if(output=="theta"||output=="theta_modelled"||output=="theta_measured"){
  thismeasure="theta_measured"
  thismodel="theta_modelled"
  thisylab=" Soil Moisture Content[-]"
}

if(output=="ETc"||output=="ETc_adj"){
  thisylab="Evaporation and Transpiration[mm]"
  
  thismodel="ETc"
  thismeasure="ETc_adj"
  label1="ETc"
  label2="ETc_adj"  
}
if(output=="Evaporation"||output=="Transpiration"||output=="T"||output=="E"){
   thismodel="Evaporation"
   thismeasure="Transpiration"
   
  thisylab="Evaporation and Transpiration[mm]"
  refine="black"
  label1="E"
  label2="T"
  if(x$kctype=="single"){
    thismodel="ETc"
    thismeasure="ETc_adj"
    label1="ETc"
    label2="ETc_adj"
  }
 }
if(output=="Kcb"||output=="kcb"||output=="Kc"||output=="KC"){
  thismodel="Kc"
  thismeasure="Kc_adj"
  thisylab=" Crop Coefficient[-]"
  refine="black"
  label1="Kc_adj"
  label2="Kcb"
  if(x$kctype=="single"){
    thismodel="Kcb"
    label2="Kc"
  }
  }
all=NULL
if(output=="balance"){
  all=TRUE
  thismodel="ETc_adj"
  thismeasure="DP"
  
  thismeasure="Transpiration"
  dots="l"
  thisylab="P, DP [mm]"
  
  barplot(x$output$P+x$output$I,main=main,...)
  par(new = TRUE)
  #lines(x$output$Drend,type="l")
  lines(x$output$ETc_adj,col="green",lty=2,lwd=2)
  lines(x$output$Ineed, col="red",lty=3,lwd=2)
  #lines(x$output$Runoff, col="red",lty=4,lwd=2)
  if(length(pos)>1){
    legend(pos[1], pos[2],(c("P+I","ETc_adj","Deficit")), cex=cex, col=c("gray","green","red"),lty=1:3, lwd=2, bty="n",horiz=horiz)
  }
  else{
  legend(pos, (c("P+I","ETc_adj","Deficit")), cex=cex, col=c("gray","green","red"),lty=1:3, lwd=2, bty="n",horiz=horiz)
  }
  
  thisdate=as.character(x$output$Date)
  thisdate<-strftime(as.Date(thisdate),"%d-%m-%Y")
  labelss=as.character(thisdate)
  thisday=max(x$output$Day)
  
  thislist=round(c(1,thisday*0.1,thisday*0.2,thisday*0.30,
                   thisday*0.40,thisday*0.50,
                   thisday*0.6,thisday*0.7,
                   thisday*0.8,thisday*0.90,1*thisday))
  thislabel=labelss[thislist]
  if(is.na(x$output$Date[1])){
    thislabel=x$output$Day[[thislist]]
  }
  text(thislist, par("usr")[3], labels = thislabel, srt = 45, pos = 1,offset=2, xpd = TRUE,cex=cex)
  
  
 
}else{
  #if(type=="all"||type=="available"||type=="soil water"||type=="TSW_measured"
    # ||type=="theta"||type=="AW_measured"||type=="AW_modelled"
     #||type=="TSW_modelled"||type=="theta_modelled"){
  day=x$output$Day[which(!is.na(x$output[thismeasure]))]
  measured=x$output[[thismeasure]][which(!is.na(x$output[[thismeasure]]))]
  date=x$output$Date[which(!is.na(x$output[[thismeasure]]))]
  if(is.na(x$output$Date[1])){
    plot(x$output$Day,x$output[[thismodel]],type="l",ylab=thisylab,main=main,cex=cex,...)
  }else{
    plot(x$output$Day,x$output[[thismodel]],type="l",xaxt = "n", xlab="",ylab=thisylab,main=main,cex=cex,...)
    thisdate=as.character(x$output$Date)
    
    thisdate<-strftime(as.Date(thisdate),"%d-%m-%Y")
    labelss=as.character(thisdate)
    thisday=max(x$output$Day)
    thislist=round(c(1,thisday*0.1,thisday*0.15,thisday*0.2,thisday*0.25,thisday*0.30,
                     thisday*0.35,thisday*0.40,thisday*0.45,thisday*0.50,
                     thisday*0.55,thisday*0.6,thisday*0.65,thisday*0.7,thisday*0.75,
                     thisday*0.8,thisday*0.85,thisday*0.90,thisday*0.95,1*thisday))
    thislist=round(c(1,thisday*0.1,thisday*0.2,thisday*0.30,
                     thisday*0.40,thisday*0.50,
                     thisday*0.6,thisday*0.7,
                     thisday*0.8,thisday*0.90,1*thisday))
    #at=c(1,50,100,120)
    axis(1, at=thislist, labels=FALSE)
    #axis(1, at=seq(1, 10, by=1), labels = FALSE)
    text(thislist, par("usr")[3], labels = labelss[thislist], srt = 45, pos = 1,offset=2, xpd = TRUE,cex=cex)
    lines(day,measured ,xlab="Date",type=dots,col=refine,lty=1)
    if(length(pos)>1){
      legend(pos[1],pos[2],(c(label1,label2)), cex=cex,lty=0:1, bty="n",horiz=horiz,pch=c("o",""))
      
    }else{
      legend(pos,(c(label1,label2)), cex=cex,lty=0:1, bty="n",horiz=horiz,pch=c("o",""))
    }
    
  }
  }
  
}
}
#' @export
#' @rdname kc
fit.kc<-function(x,output="AW_measured"){
  thismeasure=output
  thismodel="AW_modelled"
  thisylab="Available Soil Moisture[mm]"
  mod=NULL
  if(is.null(x$EFxy)){
  obs=x$output$AW_measured
  est=(x$output$AW_modelled)
  if(output=="TSW_measured"||output=="TSW_modelled"||output=="total"){
    obs=x$output$TSW_measured
    est=x$output$TSW_measured
  }
  if(output=="theta"||output=="theta_modelled"||output=="theta_measured"){
    obs=x$output$theta_measured
    est=x$output$theta_modelled
  }
  mod=cbind(crop=as.character(x$output$crop[1]),fitting(obs,est))
  }else{
    #print("YESSSS")
    obs= data.frame(t((x$AW_measuredxy)))
    est= data.frame(t((x$AW_modelledxy)))
    mod=NULL
    i=1
    while(i<=length(est)){
      
      mod=rbind(mod,cbind(longitude=est[1,][[i]],latidute=est[2,][[i]],fitting(obs[3:nrow(obs),][[i]],est[3:nrow(est),][[i]])))
      
      i=i+1
      
    }

  }
  mod
}

#' @title Reprojecting and extracting Raster values 
#' 
#' @description The function reprojects longitudes and latitudes into map
#' coordinates and extract raster values
#'
#' @param map a raster object.
#' @inheritParams ETo
#' @return data
#' @author George Owusu
#' @export 
#'
#' @examples
#' \dontrun{
#' lonlat=read.table(system.file("extdata","sys","xyvalues.txt",package="sebkc"),header=TRUE)
#' longitude=lonlat$longitude
#' latitude=lonlat$latitude
#' folder=system.file("extdata","stack",package="sebkc")
#' data=landsat578(data=folder, welev=362)
#' Ts=data$Ts
#' Tsdata=xyextract(Ts,longitude,latitude)
#' 
#' }

xyextract=function(map,longitude,latitude){
  
  #if(!is.null(longitude)){
  
  # if(!is.null(map$xydata)){
  #  longitude=map$xydata$longitude
  #  latitude=map$xydata$latitude
  #}
  
  #if(!is.null(map$map$ETa)){
  #  map=map$map$ETa
  #}
  d <- data.frame(x=longitude, y=latitude)
  d2=d
  coordinates(d) <- c("x", "y")
  if(longitude[[1]]<300||latitude[[1]]<300){
  projection(d) <- CRS("+init=epsg:4326") # WGS 84
  CRS.new=CRS( proj4string(map))
  dtrans <- spTransform(d, CRS.new)
  }else{
    dtrans= cbind(longitude, latitude)
    #projection(dtrans) <- proj4string(map)
  }
  
  data=extract(map, dtrans,df=TRUE)
  data=cbind(d2,data)
  data$ID=NULL
  thisname=paste("value",1:(length(data)-2),sep="")
  names(data)=c("longitude","latitude",thisname)
  data
}
#xydata=read.table("C:/Users/GeoKings/Documents/George Owusu/UG PhD/lectures/remote sensing/model/sebkc/inst/extdata/sys/xyvalues.txt",header=TRUE)

#Tsdata2[5]=rev(Tsdata2[5])
#Ts=data$Ts
#Px=read.table("C:/Users/GeoKings/Documents/George Owusu/UG PhD/lectures/remote sensing/model/sebkc/inst/extdata/sys/P.txt",header=TRUE)
#ETOI=read.table("C:/Users/GeoKings/Documents/George Owusu/UG PhD/lectures/remote sensing/model/sebkc/inst/extdata/sys/ETOI.txt",header=TRUE)
# I=ETOI$I
#ETo=ETOI$Eto
#mapmatrix(Px,1,map)
#

#' @title Internal function for doing interpolation
#' 
#' @description Internal function for doing interpolation
#'
#' @param xydata x and y data
#' @param i index
#' @param map geographical map
#'
#' @return map
#' @author George Owusu
#' @export 
#' @examples
#' \dontrun{
#' folder=system.file("extdata","stack",package="sebkc")
#' sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28)
#' points=sampleRandom(sebiauto$EF,100,sp=TRUE)
#' pt=cbind(points@@coords,points@@data)
#' longitude=pt$x
#' latitude=pt$y
#' value=pt$layer
#' mapmatrix=mapmatrix(xydata,2)
#' map=invdist(longitude,latitude,var=value,ext=sebiauto$EF)
#' }
mapmatrix=function(xydata,i,map){
  
rclmat <- matrix(xydata[i,], ncol=3, byrow=TRUE)
longitude=apply(rclmat, 1,as.numeric)[1,]
latitude=apply(rclmat, 1,as.numeric)[2,]
var=apply(rclmat, 1,as.numeric)[3,]

#rclmat=data.frame(cbind(longitude,latitude,var))
map=invdist(longitude=longitude,latitude=latitude,var=var,ext=map)
map$var
}
#mapmatrix=mapmatrix(xydata,2)
#map=invdist(rclmat,ext=Ts)

#' @title  Automatic computation of hot pixel Radiometric Surface Temperature (Ts)

#' @description This function computes temperature of a hot pixel based on NDVI, 
#' Surface Temperature, DEM and albedo. It plots histograms indicating
#' the selected values
#' @param upper The quantile probability at which Ts should be included 
#' in the selection project. It ranges from 0-1. The default is 0.8
#' @param lower The quantile probability at which NDVI should be included 
#' in the slection project.The default is 0.2
#' @inheritParams sebal
#' @inheritParams coldTs2
#' @inheritParams coldTs
#' @details The function first divides the selected area into a 
#' number of clusters. The mean and standard deviations are
#' computed. The cluster means with the lowest standard deviations
#' and qualifying for upper (Ts) and lower (NDVI) limits 
#' (see above) are selected. The provided upper and lower probabilities
#'  are also used to select the final Ts.
#' @seealso \code{\link{coldTs}},  \code{\link{hotTs}}, \code{\link{coldTs2}} 
#' @return 
#' \describe{
#' \item{Ts:}{ The selected or input Ts}
#' \item{Tshot:}{ The Temperature of hot pixel}
#' \item{TX:}{ The average temperature of the hot pixels}
#' \item{x:}{ The x cordinate of the hot pixel}
#' \item{y:}{ The y cordinate of the hot pixel}
#' \item{xyhot: }{combined x and y coordinates}
#' \item{candidates:}{ The similar candidates' pixels that can be used}
#' }
#' @examples
#' \dontrun{
#'  folder=system.file("extdata","stack",package="sebkc")
#'   modhot=hotTs2(folder=folder,welev=170,extent="auto",cluster=3)
#'   
#'    #use object of  landsat578
#' folder=system.file("extdata","stack",package="sebkc")
#' data=landsat578(data=folder, welev=170)
#' modfhot=hotTs2(folder=data,welev=170,extent="auto",cluster=3)
#' 
#' #Or Ts = data
#' modhotTs2=hotTs2(data,welev=170,extent="auto",cluster=3)

#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' #simulate  with default parameters without albedo
#' modhot=hotTs2(Ts=Ts,NDVI=NDVI,extent="auto",cluster=3)
#' #simulate with albedo but set extent to "auto" and cluster to 7
#' modhot=hotTs2(Ts=Ts,NDVI=NDVI,albedo=albedo,cluster=7,
#' extent="auto")
#' }
#' @author George Owusu
#' @references Allen, R. G., Burnett, B., Kramber, W., Huntington, J., 
#' Kjaersgaard, J., Kilic, A., Kelly, C., & Trezza, R. 2013. Automated 
#' Calibration of the METRIC-Landsat Evapotranspiration Process. 
#' JAWRA Journal of the American Water Resources Association, 49(3): 563-576.
#' @export
#' @rdname hotTs2
#'
 
hotTs2<-function(Ts,NDVI,albedo=NULL,sunangle=NULL,DEM=NULL,cluster=10,
                extent="interactive",upper=0.95,
lower=0.8,NDVI.probs=0.1,plot=TRUE,layout="portrait",draw="poly",
folder=NULL,welev=NULL) UseMethod ("hotTs2")
#' @export
#' @rdname hotTs2
hotTs2.default=function(Ts=NULL,NDVI,albedo=NULL,sunangle=NULL,DEM=NULL,
cluster=10,extent="interactive",upper=0.95,
lower=0.8,NDVI.probs=0.2,plot=TRUE,layout="portrait",draw="poly",folder=NULL,welev=NULL){
  
  if(is.null(folder)){
    folder="nothing454782ghf7poi8r.hope"
  }
  
  
  if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
      if(inherits(Ts, "landsat578")){
        mod=Ts
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(data=folder, welev=welev)
    }
    
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    sunangle=mod$sunelev
    }
  
  if(is.null(sunangle)){
    return (message("sunangle is needed"))
  }
  
ext=extent
albedothre=0.02
if(!inherits(NDVI, "RasterLayer")){
NDVI=raster(NDVI)
}
if(!inherits(Ts, "RasterLayer")){
Ts=raster(Ts)
}
if(!inherits(albedo, "RasterLayer")&&!is.null(albedo)){
albedo=raster(albedo)
}

if(is.numeric(ext[1])){
Ts <-crop(Ts, ext)
}else{
if(ext=="auto"||ext=="full"||extent=="Full"||extent=="FULL"){
  if(ext=="full"||extent=="Full"||extent=="FULL"){
    ext=extent(Ts)
  }else{
extents=extent(Ts)
xrange=extents[2]-extents[1]
yrange=extents[4]-extents[3]
H=0.2
V=0.4
if(layout=="H"||layout=="horinzontal"||layout=="landscape"){
H=V
V=H
}
x40=V*xrange
xmin=extents[1]+x40
xmax=extents[2]-x40
y20=H*yrange
ymin=extents[3]+y20
ymax=extents[4]-y20
extents2=c(xmin,xmax,ymin,ymax)
Ts=crop(Ts,extents2)
ext=extent(Ts)
}
}else{
if(draw=="poly"||draw=="line"){
plot(Ts,main="click and draw a line or polygon on a plotand right-click and
     select 'stop'. If you are using RStudio click on Finish to stop")
ext=drawPoly()
}else{
plot(Ts,main="click on two places on this map to select Area of Interest.
     If you are using RStudio click on Finish to stop")
ext <- drawExtent() #draw a box by clicking
}
Ts <-crop(Ts, ext)
}
}
if(!is.null(DEM)){
DEM = crop(DEM, ext)
Ts=Ts+(0.0065*DEM)
}
NDVI <- crop(NDVI, ext)
NDVI2=NDVI
if(!is.null(albedo)){
albedo2=crop(albedo, ext)
}
hotTs2=Ts
upper1=quantile(hotTs2,probs=upper)
lower1=quantile(hotTs2,probs=lower)
hotTs2[hotTs2>upper1[[1]],]=NA
hotTs2[hotTs2<lower1[[1]],]=NA
#NDVI2[is.na(hotTs2),]=NA
hotTs2[NDVI2<quantile(NDVI2,probs=NDVI.probs)[[1]],]=NA
p=data.frame(rasterToPoints(hotTs2))
hotTs2xy<-p[p$layer>0,]
#hotTs2xy=p[,c("x","y")]
p3=p
if(nrow(p)>1){
p2= p[order(p$layer),]
p3=p2[round(nrow(p2)/2),] 
}
x=p3$x
y=p3$y
ndvi=getValues(NDVI2)[cellFromXY(NDVI2,c(x,y))]

if(plot==TRUE||plot==TRUE){
oldpar <- par(no.readonly = TRUE)
on.exit(par(oldpar), add = TRUE)
par(mfrow=c(1,2))
hist(Ts,main="Ts[Hot/Dry]",xlab="Temperature")
abline(v=p3,col="red")
hist(NDVI,main="NDVI",xlab="NDVI")
abline(v=ndvi,col="red")
}
hotTs2=p3$layer

factor<-list(x=x,y=y,xyhot=c(x,y),Ts=Ts,NDVI=ndvi,Tshot=hotTs2,
             p=p,candidates=p,TC=mean(p$layer))

factor$call<-match.call()

class(factor)<-"hotTs"
factor
}

#' @title Inverse distance interpolation
#' 
#' @description This function performs inverse distance interpolation based 
#' on projection and geometry of input spatial data 'ext'. 
#' @inheritParams ETo
#'
#' @param var numeric variable. The length should be same as the length of 
#' latitude and longitude
#' @param ext RasterLayer object. 
#' @param idp Neighbourhood parameter
#'
#' @return var: interpolated values
#' @author George Owusu
#' @export
#'
#' @examples
#'  \dontrun{
#' #Get ext data
#' folder=system.file("extdata","stack",package="sebkc")
#' data=landsat578(data=folder, welev=362)
#' temp=data$Ts
#' latitude=seq(7.544,7.590,0.001)
#' longitude=seq(-1.211,-1.187,0.001)
#' var=seq(20,30,0.2)
#' latitude=latitude[1:length(longitude)]
#' var=var[1:length(longitude)]
#' map=invdist(longitude,latitude,var,ext=temp)
#' }
invdist=function(longitude,latitude,var,ext,idp = .5){
  
 
#if(!is.null(longitude)){
  d <- data.frame(longitude=longitude, latitude=latitude,var=var)
 names(d)=c("longitude","latitude","var")
  coordinates(d) <- c("longitude", "latitude")
  lon1=ifelse(length(longitude[[1]])>1,longitude[[1]][1],longitude[[1]])
  lat1=ifelse(length(latitude[[1]])>1,latitude[[1]][1],latitude[[1]])
  
  if(lon1<300||lat1<300){
  projection(d) <- CRS("+init=epsg:4326") # WGS 84
  CRS.new=CRS( proj4string(ext))
  d <- spTransform(d, CRS.new)
  #data=cbind(dtrans@coords,dtrans@data)
  #print(coordinates(data))
  }
    projection(d) <- proj4string(ext)
    #data=d
    mg <- gstat(id = "var", formula = var~1, data=d, set=list(idp = idp))
    
    var <- interpolate(ext, mg)
  
  ## inverse distance weighted (IDW)
  var
  #list(var=var)
}

#' Reproject WGS 84 longitude and latitude  into a new raster coordinates and vice versa 
#'
#' @details If x is a list of longitudes then new  horizontal  coordinates based on map
#' coordinates. If x is a raster object with  horizontal  coordinates then it will be projected to
#' WGS 84.
#' @param x a longitude or raster or vector object
#' @param y a latitude or raster or vector object
#' @param map Raster* object that will be used to define the new coordinates.
#' @param epsg coordinate reference number. The default is WGS 84: 4326
#' @inheritParams ETo
#' @inheritParams invdist
#' @return New x, y and map
#' @export 
#' @author George Owusu
#' @examples
#' folder=system.file("extdata","stack",package="sebkc")
#' data=landsat578(data=folder, welev=362)
#' temp=data$Ts
#' latitude=seq(7.544,7.590,0.001)
#' longitude=seq(-1.211,-1.187,0.001)
#' var=seq(20,30,0.2)
#' latitude=latitude[1:length(longitude)]
#' var=var[1:length(longitude)]
#' newlonlat=lonlatreproject(longitude,latitude,var=var,map=temp)
#' longitude=newlonlat$longitude
#' map=newlonlat$map
#' 
#' 
lonlatreproject=function(x,y=NULL,var=NA,map,epsg=4326){
  longitude=x
  latitude=y
  #ext=map
  if(is.null(var)){
    var=NA
  }
  if(is.null(latitude)){
    d=longitude
  }else{
  d <- data.frame(longitude=longitude, latitude=latitude,var=var)
  coordinates(d) <- c("longitude", "latitude")
  }
  if(extent(d)[1]>180){
    data= projectRaster(d, crs=paste("+init=epsg:",epsg,sep=""))
  }else{
  projection(d) <- CRS(paste("+init=epsg:",epsg,sep="")) # WGS 84
  CRS.new=CRS( proj4string(map))
  data <- spTransform(d, CRS.new)
  }
  #data=cbind(dtrans@coords,dtrans@data)
  
  list(x=coordinates(data)[,1],y=coordinates(data)[,2],map=data)
}

#' @title Biomass and yield estimation of crops and plants
#' 
#' @description This function uses	Wageningen procedure 
#' (De Wit, 1968; Slabbers et al., 1979) to compute the 
#' point or spatial possible maximum yield of a standard crop based on a 
#' Rs data. This function can also perform internal interpolation of 
#' data if longitude and latitude are provided. See datails and examples below.
#'
#' @param Tday numeric. Average Day time temperature in degrees Celsius 
#' for the growing period. A Remote sensing surface Temperature can also be used
#' 
#' @param T24 numeric. Average 24-hour (including night hours) temperature 
#' in degrees Celsius for the growing period. 
#' @param Rs A shortwave radiation data [ca^-2 day^-1]. 
#' RasterLayer data can be supplied.
#' @inheritParams ETo
#' @inheritParams sebal
#' @param HI numeric. Harvest Index. It can be list or spatial
#' @param plantdate character. A date of planting in the form "YYYY-mm-dd"
#' @param harvestdate character. A date of harvest in the form "YYYY-mm-dd"
#' @param c30 numeric. A maintenance respiration coefficient (c30) at 30 oC. 
#' it takes either a value of 0.0108 for non-legume crop or 0.0283 for legumes
#' @param adaptability numeric. The  climate adaptability group  number. 
#' It takes values of 1,2,3,or 4. it depends on the temperature profile of 
#' study area. See references (eg. FAO (1981)) for more explanation. 
#' @param crop character. A crop type. For example "maize"
#' @author George Owusu
#' @details 
#'  \describe{
#'  The function can be used in several ways:
#'  \item{\strong{Point Estimation}}{\code{biomass} can compute biomass 
#'  and yield for  one location. Each input data must be given only one value}
#'  \item{\strong{Point Estimation for several locations }}{The function can 
#'  compute biomass and yields for more than one location. Input data must 
#'  be a list or array.}
#'  \item{\strong{Spatial Estimation}}{Input data such as Rs, Tday, T24,
#'  HI or LAI can be spatial.}
#'  \item{\strong{Spatial Estimation with internal interpolation}}{It takes Tday,
#'   for example from satellite image, and do interpolation for 
#'  the rest of the data. Both longitude, latitude, and the variables must be of
#'  the same length. If longitude is not provided no interpolation will be done.}
#'  }
#'  
#'  @references 
#' \itemize{
#' \item De Wit, C. T. (1968). Plant production Miscellaneous Papers: 
#' Landbouw Hogeschool, Wageningen.
#' \item Slabbers, P. J., Herrendorf, V. S., & Stapper, M. (1979). 
#' Evaluation of simplified water-crop yield models. 
#' Agricultural Water Management, 2(2), 95-129. 
#' doi: http://dx.doi.org/10.1016/0378-3774(79)90026-X
#' \item FAO (1981). Report on the Agro-ecological Zones Project: 
#' Food and Agriculture Organization of the United Nations.
#' }
#' @return yield and biomass in t/ha
#' @export
#'
#' @examples
#' \dontrun{
#' ################# Point Estimation ###################
#' # Identify the climatic input data on 
#' latitude=11.18 #in decimal degrees
#' Rs= 452 #Average shortwave radiation over the growing period (cal-2day-1)
#' Tday = 26.9 #	Average day time difference (oC) over the growing period
#' T24 =25.3 #Average 24 hour mean temperature over the growing period
#' # Gather crop Information
#' crop="maize"
#' plantdate="2015-05-1" #Start date
#' harvestdate="2015-08-29" #end date 120 days
#' HI=0.4 #Harvest index for maize
#' adaptability=3 #crop climate adaptability group III
#' LAI = 4 #Leaf Area Index
#' pointmod=biomass(Tday,T24,Rs,latitude,longitude=NULL,HI,
#' plantdate,harvestdate,c30=0.0108,adaptability=3,LAI=5,crop=NULL)
#' 
#' ################# Point Estimation for several locations#################
#' latitude=c(10,11.18,12,9)
#' longitude=c(-1.4,-2,-1,-2.6)
#' Rs= c(450,452,400,500)
#' Tday = c(25,26.9,24,20,30)
#' T24=c(24,25.3,23,28)
#' HI=c(0.4,0.4,0.5,0.25)
#' plantdate=c("1-03-2015","1-04-2015","1-05-2015")
#' harvestdate=c("29-08-2015","29-08-2015","29-08-2015")
#' 
#' # computing several points
#' pointSmod=biomass(Tday,T24,Rs,latitude,longitude=longitude,HI,
#' plantdate,harvestdate,c30=0.0108,adaptability=3,LAI=5,crop=NULL)
#' yield=pointSmod$yield #tabular data
#' biomass=pointSmod$output #tabular data
#' #access on biomass
#' pointSmod$output$yield
#' }
#' \dontrun{
#' ############## Spatial Estimation with internal interpolation ###########################
#' #generate spatial data
#' folder=system.file("extdata","stack",package="sebkc")
#' modauto=landsat578(data=folder, welev=362)
#' Tday=modauto$Ts-273.15 #temperature in degree celcius
#' LAI=modauto$LAI
#' latitude=seq(7.544,7.590,0.001) #for interpolation
#' longitude=seq(-1.211,-1.187,0.001) #for interpolation
#' latitude=latitude[1:length(longitude)] #to make sure they are of the same length
#' spatialmodint=biomass(Tday,T24,Rs,latitude,longitude=longitude,HI,
#' plantdate,harvestdate,c30=0.0108,adaptability=3,LAI=LAI,crop=NULL)
#' # Rs, T24, and HI are interpolated
#' plot(spatialmodint$yield)
#' ############## Spatial Estimation  ###########################
#' Rs= 452
#' T24 =25.3
#' HI=0.4 
#' spatialmod=biomass(Tday,T24,Rs,latitude,longitude=longitude,HI,
#' plantdate,harvestdate,c30=0.0108,adaptability=3,
#' LAI=LAI,crop=NULL)
#' plot(spatialmod$yield)
#' }

biomass=function(Tday=NULL,T24,Rs,latitude,longitude=NULL,HI=NULL,
plantdate,harvestdate,c30=0.0108,adaptability=3,LAI=5,crop=NULL){
  if(inherits(Tday, "kc")){
    Tday=Tday$DL
    #print(Tday)
  if(!is.null(Tday$Tday)){
    data1=Tday
    T24=data1$T24
    Rs=data1$Rs
    latitude=data1$latitude
    plantdate=data1$plantdate
    harvestdate=data1$harvestdate
    Tday=data1$Tday
    
  }
  }
  inter=NULL
  if(inherits(Tday, "RasterLayer")){
    inter=Tday  
  }else{
    if(inherits(Rs, "RasterLayer")){
      inter=Rs  
    }else{
      if(inherits(T24, "RasterLayer")){
        inter=T24
      }else{
        if(inherits(LAI, "RasterLayer")) 
          inter=LAI
      }
    }
  }
  
  
  if(is.null(inter)&&(length(plantdate)>1||length(harvestdate)>1||length(Tday)>1
     ||length(T24)>1||length(Rs)>1||length(c30)>1||length(adaptability)>1
     ||length(LAI)>1||length(crop)>1)){
    
outputdata=data.frame(Tday=numeric(),T24=numeric(),Rs=numeric(),
                      latitude=numeric(),longitude=numeric(),HI=numeric(),
  plantdate=numeric(),harvestdate=numeric(),c30=numeric(),
  adaptability=numeric(),LAI=numeric(),yield=numeric(),biomass=numeric())

outputdata=data.frame()
                         
    i=1
    mod=NULL
    while(i<=length(plantdate)){
      
      Tday2=Tday
      if(length(Tday)>1){
        Tday2=Tday[i]
      }
      T242=T24
      if(length(T24)>1){
        T242=T24[i]
      }
      rs2=Rs
      if(length(T24)>1){
        rs2=Rs[i]
      }
      longitude2=longitude
      if(length(longitude)>1){
        longitude2=longitude[i]
      }
      
      latitude2=latitude
      if(length(latitude)>1){
        latitude2=latitude[i]
      }
      HI2=HI
      if(length(HI)>1){
        HI2=HI[i]
      }
      c302=c30
      if(length(c30)>1){
        c302=c30[i]
      }
      adaptability2=adaptability
      if(length(adaptability)>1){
        adaptability2=adaptability[i]
      }
      LAI2=LAI
      if(length(LAI)>1){
        LAI2=LAI[i]
      }
      plantdate2=plantdate
      if(length(plantdate)>1){
        plantdate2=plantdate[i]
      }
      harvestdate2=harvestdate
      if(length(harvestdate)>1){
        harvestdate2=harvestdate[i]
      }
      crop2=crop
      if(length(crop)>1){
        crop2=crop[i]
      }
      pointmod=biomass(Tday=Tday2,T24=T242,Rs=rs2,latitude=latitude2,
        longitude=longitude2,HI=HI2,plantdate=plantdate2,harvestdate=harvestdate2,
        c30=c302,adaptability=adaptability2,
                        LAI=LAI2,crop=crop2)
      if(is.null(longitude)){longitude2=NA}
      outputdata=rbind(outputdata,data.frame(Tday=Tday2,T24=T242,Rs=rs2,
                                             latitude=latitude2,longitude=longitude2,HI=HI2,plantdate=plantdate2,
                                             harvestdate=harvestdate2,
              c30=c302,adaptability=adaptability2,LAI=LAI2,yield=pointmod$yield,
             biomass=pointmod$biomass))
      #print (c(latitude2,longitude2))
      #mod=rbind(mod,pointmod)
      #print (cbind(output,Tday=Tday2,T24=T242,Rs=rs2,latitude=latitude2,longitude=longitude2,HI=HI2,plantdate=plantdate2,harvestdate=harvestdate2,c30=c302,adaptability=adaptability2,LAI=LAI2,crop=crop2))
      
      
      i=i+1
    }
    
    #colnames(mod)=paste(plantdate,"to",harvestdate,sep="")
    
    return(list(output=outputdata,yield=outputdata,biomass=outputdata))
    
  }else{
  
Ndays=as.numeric(as.Date(harvestdate, "%Y/%m/%d")-as.Date(plantdate, "%Y/%m/%d"))
if(any(is.na(Ndays))){
  Ndays=as.numeric(as.Date(harvestdate, "%d-%m-%Y")-as.Date(plantdate, "%d-%m-%Y"))
  
}
if(any(is.na(Ndays))){
  Ndays=as.numeric(as.Date(harvestdate, "%d/%m/%Y")-as.Date(plantdate, "%d/%m/%Y"))
  
}
if(any(is.na(Ndays))){
  Ndays=as.numeric(as.Date(harvestdate, "%m/%d/%Y")-as.Date(plantdate, "%m/%d/%Y"))
}

if(any(is.na(Ndays))){
  Ndays=as.numeric(as.Date(harvestdate, "%m-%d-%Y")-as.Date(plantdate, "%m-%d-%Y"))
}
if(any(is.na(Ndays))){
  Ndays=as.numeric(as.Date(harvestdate, "%Y-%m-%d")-as.Date(plantdate, "%Y-%m-%d"))
}
#print(Ndays)
Pm20=20
inter=NULL


if(latitude<0){
  Pmfile=read.table(system.file("extdata","sys","pmwrite2.txt",package="sebkc"),header=TRUE)
  latrad=read.table(system.file("extdata","sys","latradiancesouth.txt",package="sebkc"),header=TRUE)
  boin=read.table(system.file("extdata","sys","bowrite2south.txt",package="sebkc"),header=TRUE)
  bcin=read.table(system.file("extdata","sys","bcwrite2south.txt",package="sebkc"),header=TRUE)  
}else{Pmfile=read.table(system.file("extdata","sys","pmwrite2.txt",package="sebkc"),header=TRUE)
latrad=read.table(system.file("extdata","sys","latradiance.txt",package="sebkc"),header=TRUE)
boin=read.table(system.file("extdata","sys","bowrite2.txt",package="sebkc"),header=TRUE)
bcin=read.table(system.file("extdata","sys","bcwrite2.txt",package="sebkc"),header=TRUE)
}


start_date=as.numeric(format.Date(as.Date(plantdate), "%m"))+1
end_date=as.numeric(format.Date(as.Date(harvestdate), "%m"))+1
latrad=latrad[c(1,start_date:end_date)]
boin=boin[c(1,start_date:end_date)]
bcin=bcin[c(1,start_date:end_date)]

rdata=FALSE

if(inherits(Tday, "RasterLayer")){
inter=Tday  
}else{
  if(inherits(Rs, "RasterLayer")){
    inter=Rs  
  }else{
    if(inherits(T24, "RasterLayer")){
      inter=T24
    }else{
    if(inherits(LAI, "RasterLayer")) 
      inter=LAI
    }
  }
}
if(inherits(Tday, "RasterLayer")){
  rdata=TRUE
  
  if(is.null(longitude)&&length(latitude>1)){
  latitude=mean(latitude)  
  }
}

Pm=NULL

if(length(latitude)>1){
  latrad2 <- NULL
  boin2 <-NULL
  bcin2 <- NULL
  i=1
  while(i<=length(latitude)){
    boin2b <- boin[boin$id==round(latitude[i],1), ]
    boin2=rbind(boin2,boin2b)
    
    latrad2b <- latrad[latrad$id==round(latitude[i],1), ]
    latrad2=rbind(latrad2,latrad2b)
    
    bcin2b <- bcin[bcin$id==round(latitude[i],1), ]
    bcin2=rbind(bcin2,bcin2b)
    
    #latrad2   
 i=i+1
 }
  
  #Ac
  Ac=rowMeans((latrad2[-1]))
  #bo
  bo=rowMeans((boin2[-1]))
  #bc
  bc=rowMeans((bcin2[-1]))
  
  if(!is.null(inter)&&!is.null(longitude)){
    Ac=invdist(longitude,latitude,Ac,inter)
    #Ac=Ac$var
    bc=invdist(longitude,latitude,bc,inter)
    #bc=bc$var
    bo=invdist(longitude,latitude,bo,inter)
    #bo=bo$var
    
    if(length(latitude)==length(T24)){
      T24=invdist(longitude,latitude,T24,inter)
      #T24=T24$var
    }
    if(length(latitude)==length(LAI)){
      LAI=invdist(longitude,latitude,LAI,inter)
      #LAI=LAI$var
      #plot(LAI)
    }
    
    if(length(latitude)==length(HI)){
      HI=invdist(longitude,latitude,HI,inter)
      #HI=HI$var
    }
    
    if(length(latitude)==length(Rs)){
      Rs=invdist(longitude,latitude,Rs,inter)
      #Rs=Rs$var
    }
    
  }
  }else{
  latrad2 <- latrad[latrad$id==round(latitude,1), ]
  boin2 <- boin[boin$id==round(latitude,1), ]
  bcin2 <- bcin[bcin$id==round(latitude,1), ]
  
  #Ac
  Ac=mean(t(latrad2[-1]))
  #bo
  bo=mean(t(boin2[-1]))
  #bc
  
  bc=mean(t(bcin2[-1]))
  
  
  
  }

##########################################
if(length(Tday)>1&&rdata!=TRUE){
  Pm=NULL
  i=1
  while(i<=length(Tday)){

    if(length(adaptability)==length(Tday)){
      adaptability=adaptability[i]
    }
    
      Tday2=Tday[i]
      if(adaptability==1){
        Pm2 <- Pmfile[Pmfile$id==round(Tday2,1), ][[2]]
      }
      
      if(adaptability==2){
        Pm2 <- Pmfile[Pmfile$id==round(Tday2,1), ][[3]]
      }
      
      if(adaptability==3){
        Pm2 <- Pmfile[Pmfile$id==round(Tday2,1), ][[4]]
      }
      
      if(adaptability==4){
        Pm2 <- Pmfile[Pmfile$id==round(Tday2,1), ][[5]]
        
      }
      Pm=rbind(Pm,Pm2)
    #print(Pm)
    #latrad2   
    i=i+1
  }
}




ct=c30*(0.044+(0.0019*T24)+(0.0010*(T24^2)))

#Ac=361
#bo=427
#bc=228
growth_rate=-0.0307*LAI^2 + 0.3549*LAI - 0.0111
fraction=(Ac-(0.5*Rs))/(0.8*Ac)
bgm=(fraction*bo)+((1-fraction)*bc)
#if(LAI!=5){
  bgm=bgm* growth_rate
#}
  
  if(rdata==TRUE){
    #Pm1=Pmfile[1:2]
    #Pm2=Pmfile[c(1,3)]
    #Pm3=Pmfile[c(1,4)]
    #Pm4=Pmfile[c(1,5)]
    if(adaptability==1){
    Pm=subs(round(Tday,1),Pmfile[1:2])
    }
    
    if(adaptability==2){
      Pm=subs(round(Tday,1),Pmfile[c(1,3)])
    }
    if(adaptability==3){
      Pm=subs(round(Tday,1),Pmfile[c(1,4)])
    }
    if(adaptability==4){
      Pm=subs(round(Tday,1),Pmfile[c(1,5)])
    }
    
  }else{

    if(!is.numeric(Pm)){
      
if(adaptability==1){
  
  Pm <- Pmfile[Pmfile$id==round(Tday,1), ][[2]]
  }


if(adaptability==2){
  Pm <- Pmfile[Pmfile$id==round(Tday,1), ][[3]]
}

if(adaptability==3){
  Pm <- Pmfile[Pmfile$id==round(Tday,1), ][[4]]
}


if(adaptability==4){
  Pm <- Pmfile[Pmfile$id==round(Tday,1), ][[5]]
  
}
     # print(Pm)
}
}

  
#Pm=max(Pm,0)

y=((Pm-Pm20)/Pm20)*100

if(rdata==FALSE){
if(bgm>Pm20){
 bgm=bgm+(((y/5)/100)*(fraction*bo)+((y/2)/100)*(1-fraction)*bc) 
}else{
  bgm=bgm-(((y/2)/100)*(fraction*bo)+(y/100)*(1-fraction)*bc) 
}
  
}else{
 bgm= rastercon(bgm>Pm20,bgm+(((y/5)/100)*(fraction*bo)+((y/2)/100)*(1-fraction)*bc),
                bgm-(((y/2)/100)*(fraction*bo)+(y/100)*(1-fraction)*bc))
  }
  
  
Bn=(0.36*bgm)/(1/Ndays+0.25*ct)/907.18474

  By=Bn*HI
  

list(biomass=Bn,yield=By)
}
}

########################



   
   

#' @title Goodness of fit Test
#'
#' @param obs Observed data
#' @param est Estimated data
#' @param P Number of parameters
#'
#' @return
#' \describe{
#' \item{r2:}{  R-square [-]}
#'  \item{r2_adj:}{ An adjusted R-square [-]}
#' \item{d:}{ Index of agreement[-]}
#' \item{p_value:}{ p value}
#' \item{sse:}{ Sum of square error}
#' \item{sst:}{ Sum of square total}
#' \item{AIC:}{ AIC}
#' \item{BIC:}{ BIC}
#' \item{RMSE:}{ Root Mean Square Error}
#' \item{RMSE_adj:}{ adjusted Root Mean Square Error}
#' \item{NRMSE}{ Normalised Root Mean Square Error}
#' \item{NRMSE_adj:}{ adjusted Normalised Root Mean Square Error}
#' \item{E:}{ Nash Sutcliffe Coefficient}
#' \item{b:}{ coefficients of regression}
#' \item{AAE:}{ average absolute error}
#' \item{ARE:}{ average relative error}
#' \item{MBE:}{ Mean Bias  error}
#' \item{MPE:}{ Mean Percentage error}
#' }
#' @export
#'
#' @examples
#' file=system.file("extdata","sys","irrigation.txt",package="sebkc")
#' data=read.table(file,header=TRUE)  
#' obs=data$ETo
#' est=data$ETc
#' P=4
#' mod=fitting(obs,est,P)
#' r2=mod$r2
fitting<-function(obs,est,P=1)
{
  
  data=data.frame(obs=obs,est=est)
  data=na.omit(data)
  obs=data$obs
  est=data$est
  N=length(obs)
 # b=sum(obs*est)/sum(obs^2)
  AAE=sum(abs(obs-est))/N
  ARE=sum(abs((obs-est)/obs))*(100/N)
  MBE=sum((est-obs))/N
  #MPE=sum(((obs-est)/obs))*(100/N)
  MPE=sum(((est-obs)/obs)*100)/N
  
  mod=lm(est~obs)
  #print(summary(mod))
  #print("p-value")
  x=mod
  F_value=summary(mod)$fstatistic[[1]]
  df=summary(mod)$fstatistic[[3]]
  r2=summary(mod)[[8]]
  f <- summary(mod)$fstatistic
  p <- pf(f[1],f[2],f[3],lower.tail=FALSE)
  attributes(p) <- NULL
  #Nash-Sutcliffe efficiency rating for model estiamtes https://github.com/USGS-R/rloadest/blob/master/R/nashSutcliffe.R
  #nashSutcliffe <- function(obs, est, na.rm=TRUE) {
  na.rm=TRUE
  
  E <- 1 - sum((obs - est)^2, na.rm=na.rm)/sum((obs-mean(obs,na.rm=na.rm))^2, na.rm=na.rm)
  #root mean square error
  #RMSE=sqrt(mean(mod$residuals^2))
  sse=sum((obs-est)^2)
  RMSE=sqrt(sse/N)
  NRMSE=RMSE/(max(obs)-min(obs))
  #AIC
  AIC=AIC(mod)
  #print(P)
  rmse_adj=sqrt((sse/(N-P)))
  NRMSE_adj=rmse_adj/(max(obs)-min(obs))
  
  adj=N-P
  sst=sum((obs-mean(obs))^2)
 # r2_adj=1-((sse/(adj-1))/(sst/(N-1)))
  r2_adj=r2-(1-r2)*(P/(N-P-1))
  #AIC=N*log(sse)+(2*P)
  
  AIC=(N*(log(2*pi)+log(sse/adj)+1))+P
  #Priestley, M.B. (1981), Spectral Analysis and Time Series, Academic Press. ISBN 0-12-564922-3 (p. 375)
  BIC=(N*log(sse/N))+P*log(N)
  #parameters=as.data.frame(list(r2=r2,r2_adj=r2_adj,N=N, p=P,F_value=F_value, df=df, p_value=p,AIC=AIC,BIC=BIC,RMSE=RMSE,RMSE_adj=rmse_adj,NRMSE=NRMSE,NRMSE_adj=NRMSE_adj,nashSutcliffe=E))
  Psquare=sum((est-obs)^2)
  obsmean=(abs(obs-mean(obs)))
  estmean=(abs(est-mean(obs)))
  obs.est=(obsmean+estmean)^2
  #d=1-((sum(est-obs)^2)/sum(((abs(est-mean(obs)))+(abs(obs-mean(obs))))^2))
  d=1-(Psquare/sum(obs.est))
  parameters=as.data.frame(list(r2=r2,r2_adj=r2_adj,d=d,NSE=E, p_value=p,sse=sse,sst=sst,AIC=AIC,BIC=BIC,RMSE=RMSE,RMSE_adj=rmse_adj,NRMSE=NRMSE,NRMSE_adj=NRMSE_adj,nashSutcliffe=E,
                                b=coef(mod)[[2]],AAE=AAE,ARE=ARE,MBE=MBE,MPE=MPE,r=cor(obs,est), intercept=coef(mod)[[1]],N=N, parameters=P,F_value=F_value, df=df,E=E))
  #  #print(parameters)
  
}

#' @title Automatic computation of hot pixel Radiometric Surface Temperature (Ts)

#' @description This function computes temperature of a hot pixel based on NDVI, 
#' Surface Temperature, DEM and albedo. The main difference with \code{\link{hotTs2}} is that preference 
#' is given to temperature selection before considering NDVI and albedo selection. Moreover, this
#' is a simple end member selection where no clustering is done.
#' @param upper The quantile probability at which Ts should be included 
#' in the selection project. It ranges from 0-1. The default is 0.8
#' @param lower The quantile probability at which NDVI should be included 
#' in the selection project. The default is 0.2
#' @inheritParams sebal
#' @inheritParams coldTs2
#' @inheritParams ETohr
#' @inheritParams weather
#' @inheritParams sebkcstack
#' @details The function first divides the selected area into a 
#' number of clusters. The mean and standard deviations are
#' computed. The cluster means with the lowest standard deviations
#' and qualifying for upper (Ts) and lower (NDVI) limits 
#' (see above) are selected. The provided upper and lower probabilities
#'  are also used to select the final Ts.
#' @param seed optional single number. Passed to \code{\link[base]{set.seed}}
#'   before the k-means clustering so the pixel selection is reproducible.
#'   The default \code{NULL} leaves the random-number stream untouched.
#' @seealso \code{\link{coldTs}}
#' @return
#' \describe{
#' \item{Ts:}{ The selected or input Ts}
#' \item{Tshot:}{ The Temperature of hot pixel}
#' \item{TX:}{ The average temperature of the hot pixels}
#' \item{x:}{ The x coordinate  of the hot pixel}
#' \item{y:}{ The y coordinate  of the hot pixel}
#' \item{xyhot: }{combined x and y coordinates}
#' \item{candidates:}{ The similar candidates' pixels that can be used}
#' }
#' @import raster
#' @examples
#' \dontrun{
#'  folder=system.file("extdata","stack",package="sebkc")
#'   modhot=hotTs(folder=folder,welev=170,extent="auto",cluster=3)
#'   
#'    #use object of  landsat578
#' folder=system.file("extdata","stack",package="sebkc")
#' data=landsat578(data=folder, welev=170)
#' modfhot=hotTs(folder=data,welev=170,extent="auto",cluster=3)
#' 
#' #Or Ts = data
#' modhotTs=hotTs(data,welev=170,extent="auto",cluster=3)

#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' #simulatte with default parameters without albedo
#' modhot=hotTs(Ts=Ts,NDVI=NDVI,extent="auto",cluster=3)
#' #simulate with albedo but set extent to "auto" and cluster to 7
#' modhot=hotTs(Ts=Ts,NDVI=NDVI,albedo=albedo,cluster=7,
#' extent="auto")
#' }
#' @author George Owusu
#' @references Allen, R. G., Burnett, B., Kramber, W., Huntington, J., 
#' Kjaersgaard, J., Kilic, A., Kelly, C., & Trezza, R. 2013. Automated 
#' Calibration of the METRIC-Landsat Evapotranspiration Process. 
#' JAWRA Journal of the American Water Resources Association, 49(3): 563-576.
#' @export
#' @rdname hotTs
#'
hotTs=function(Ts,NDVI,albedo=NULL,DEM=NULL,
      cluster=8,extent="interactive",upper=0.80,
    lower=0.2,plot=TRUE,layout="portrait",draw="poly",folder=NULL,welev=NULL,clip=NULL,iter.max = 500,seed=NULL)
  UseMethod ("hotTs")
#' @export
#' @rdname hotTs
hotTs.default=function(Ts=NULL,NDVI,albedo=NULL,DEM=NULL,
cluster=8,extent="interactive",upper=0.80,
lower=0.2,plot=TRUE,layout="portrait",draw="poly",folder=NULL,welev=NULL,clip=NULL,iter.max = 500,seed=NULL){
  message("Selecting hot pixel")
  if(is.null(Ts)&&!is.null(folder)){
    #Ts=folder 
  }
  
  if(is.null(folder)){
    folder="nothing454782ghf7poi8r.hope"
  }
  if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
      if(inherits(Ts, "landsat578")){
        mod=Ts
      }else{
      mod=folder
      }
    }else{
      mod=landsat578(data=folder, welev=welev)
    }
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    sunangle=mod$sunelev
  }
  
albedothre=0.02
ext=extent

if(!inherits(NDVI, "RasterLayer")){
NDVI=raster(NDVI)
}
if(!inherits(Ts, "RasterLayer")){
Ts=raster(Ts)
}
if(!inherits(albedo, "RasterLayer")&&!is.null(albedo)){
albedo=raster(albedo)
}

if(is.numeric(ext[1])){
Ts <-crop(Ts, ext)
}else{
  if(ext=="auto"||ext=="full"||extent=="Full"||extent=="FULL"){
    if(ext=="full"||extent=="Full"||extent=="FULL"){
      ext=extent(Ts)
      #print("yes")
    }else{
extents=extent(Ts)
xrange=extents[2]-extents[1]
yrange=extents[4]-extents[3]
H=0.2
V=0.4
if(layout=="H"||layout=="horinzontal"||layout=="landscape"){
H=V
V=H
}
x40=V*xrange
xmin=extents[1]+x40
xmax=extents[2]-x40
y20=H*yrange
ymin=extents[3]+y20
ymax=extents[4]-y20
extents2=c(xmin,xmax,ymin,ymax)
Ts=crop(Ts,extents2)
ext=extent(Ts)
}
}else{
if(draw=="poly"||draw=="line"){
plot(Ts,main="Click and draw a line or polygon on a plot and right-click and 
     select 'stop' If you are using RStudio click on Finish to stop")
ext=drawPoly()
}else{
plot(Ts,main="click on two places on this map to select Area of Interest.
     If you are using RStudio click on Finish to stop")
ext <- drawExtent() #draw a box by clicking
}
Ts <-crop(Ts, ext)
}
}

if(!is.null(DEM)){
DEM = crop(DEM, ext)
Ts=Ts+(0.0065*DEM)
#Ts[Ts<0]=NA
}
NDVI2 <- crop(NDVI, ext)
if(!is.null(albedo)){
albedo2=crop(albedo, ext)
}
v <- na.omit(getValues(Ts))
if(exists(".Random.seed", envir = .GlobalEnv)){
oldseed <- get(".Random.seed", envir = .GlobalEnv)
on.exit(assign(".Random.seed", oldseed, envir = .GlobalEnv), add = TRUE)
}
if(!is.null(seed)) set.seed(seed)
Ts.kmeans <- kmeans(v, centers=cluster, iter.max = iter.max, nstart = 3)
  
kmeansraster <- raster(Ts)
i <- attr(v, "na.action")
#when Ts has no NA cells, na.omit() does not set "na.action" so i is NULL;
#(-NULL) raises "invalid argument to unary operator". Guard against it.
if(is.null(i)){
j <- 1:ncell(Ts)
}else{
j <- (1:ncell(Ts))[-i]
}
kmeansraster[j] <- Ts.kmeans$cluster

#Ts.kmeans <- kmeans(na.omit(Ts[]), cluster, iter.max = 100, nstart = 3)
#kmeansraster<-raster(Ts)
#kmeansraster[]<-Ts.kmeans$cluster
Tsmean=zonal(Ts, kmeansraster, 'mean') 
TsSD=zonal(Ts, kmeansraster, 'sd') 
#TsCV=zonal(Ts, kmeansraster, fun='cv') 
#TsCV15=TsCV
#TsCV15[,2]=TsCV15[,2]*0.15

NDVImean=zonal(NDVI2, kmeansraster, 'mean') 
NDVISD=zonal(NDVI2, kmeansraster, 'sd') 
#NDVICV=zonal(NDVI2, kmeansraster, fun='cv') 
#NDVICV15=NDVICV
#NDVICV15[,2]=NDVICV15[,2]*0.15

#substract sd from the mean
NDVImean_sd=NDVImean
NDVImean_sd[,2]=NDVImean[,2]-NDVISD[,2]

#hot pixels AOI selection
NDVImax=NDVImean_sd[NDVImean_sd[,2]==max(NDVImean_sd[,2]),]
NDVImin=NDVImean_sd[NDVImean_sd[,2]==min(NDVImean_sd[,2]),]
NDVI95=quantile(NDVImean_sd[,2],  probs = lower)
NDVI95b=NDVImean_sd[NDVImean_sd[,2]<NDVI95,]
if(length(NDVI95b[1])==1){
Tsmean95=Tsmean[Tsmean[,1] %in%(NDVI95b[1]),]
Tsmean95SD=TsSD[TsSD[,1] %in%(NDVI95b[1]),]
Tsmean95_sd=Tsmean95
Tsmean95_sd[2]=Tsmean95[2]-Tsmean95SD[2]
Tsmean2_sda=quantile(Tsmean95_sd[2],  probs = upper)
Tsmean2b=Tsmean95_sd

}else
{
Tsmean95=Tsmean[Tsmean[,1] %in%(NDVI95b[,1]),]
Tsmean95SD=TsSD[TsSD[,1] %in%(NDVI95b[,1]),]
Tsmean95_sd=Tsmean95
Tsmean95_sd[,2]=Tsmean95[,2]-Tsmean95SD[,2]
Tsmean2_sda=quantile(Tsmean95_sd[,2],  probs = upper)
Tsmean2b=Tsmean95_sd[Tsmean95_sd[,2]>Tsmean2_sda[[1]],]
}


#cellStats(hotAOI,"mean")
##actual hot pixel calculation
hotAOIbin=kmeansraster==Tsmean2b[[1]]
###remove kmeansraster
hotAOINDVI=rastercon(hotAOIbin>0,NDVI2,0)
hotAOITs=rastercon(hotAOIbin>0,Ts,0)
hotAOINDVI[hotAOINDVI<=0]=NA
hotAOITs[hotAOITs<=0]=NA
hotAOITs0=quantile(hotAOITs,probs=upper)
hotAOINDVI95=quantile(hotAOINDVI,probs=lower)
hotAOINDVI95b=rastercon(hotAOINDVI<=hotAOINDVI95[[1]],hotAOINDVI,0)
hotAOINDVI95c=rastercon(hotAOITs>=hotAOITs0[[1]],hotAOINDVI95b,0)
hotAOINDVI95c[hotAOINDVI95c<=0]=NA
if(is.na(minValue(hotAOINDVI95c))&&is.na(maxValue(hotAOINDVI95c))){
hotAOINDVI95c=hotAOINDVI95b
hotAOINDVI95c[hotAOINDVI95c<=0]=NA
}
#correseponding Temperature
hotAOINDVI95cTs=rastercon(hotAOINDVI95c>=minValue(hotAOINDVI95c),hotAOITs,0)
hotAOINDVI95cTs[hotAOINDVI95cTs<=0]=NA
hotTs=hotAOINDVI95cTs
hotTs[hotTs<=0]=NA
p=data.frame(rasterToPoints(hotTs))
p3=p
if(nrow(p)>1){
p2= p[order(-p$layer),]
p3=p2[round(nrow(p2)/2),] 
}
x=p3$x
y=p3$y
ndvi=getValues(NDVI2)[cellFromXY(NDVI2,c(x,y))]

if(plot==TRUE||plot==TRUE){
oldpar <- par(no.readonly = TRUE)
on.exit(par(oldpar), add = TRUE)
par(mfrow=c(1,2))
hist(Ts,main="Ts[Hot]",xlab="Temperature")
abline(v=p3,col="red")
hist(NDVI2,main="NDVI",xlab="NDVI")
abline(v=ndvi,col="red")
}
hotTs=p3$layer

factor<-list(x=x,y=y,xyhot=c(x,y),Tshot=hotTs,NDVI=ndvi,Ts=Ts,p=p,
             candidates=p,TH=mean(p$layer),cluster=kmeansraster)

factor$call<-match.call()

class(factor)<-"hotTs"
factor
}

#' @title Writing sebkc object to a folder
#' 
#' @description This function writes spatial output of the following functions
#' \code{\link{sebal}}, \code{\link{sebi}},\code{\link{ssebi}},
#' \code{\link{wdi}},\code{\link{sebs}},\code{\link{sseb}},
#' \code{\link{tseb}} and \code{\link{landsat578}}  
#'
#' @param object sebkc object 
#' @param folder Folder the files should be written to.If it is set to NULL,
#' it is written to the input folder of sebkc object.
#' @param xy A dataframe of of xy coordinates in in decimal degrees or 
#' meters in the order of c(x,y). If it is not set NULL,
#'  the corresponding values are extracted and written to the folder
#' @param overwrite logical whether the file should be over written
#' @return Writes output to a folder
#' @export
#' @examples
#' \dontrun{
#' folder=system.file("extdata","stack",package="sebkc")
#' stack=landsat578(data=folder, welev=362)
#' writesebkc(stack)
#' sebaloutput=sebal(folder = folder,welev = 317)
#' writesebkc(sebaloutput)
#' 
#' }
writesebkc<-function(object,folder=NULL,xy=NULL,overwrite=TRUE){
  #longitude=c(-1.62,-1.61,-1.59,-1.59,-1.61,-1.62,-1.58,-1.57)
  #latitude=c(6.7,6.5,6.3,7.0,6.7,6.5,6.3,7.0)
  #xy1 <- data.frame(x=longitude, y=latitude)
  #xy2=NULL
  if(!is.null(xy)){
    xy=data.frame(x=xy[1],y=xy[2])
    if(xy[[1]][1]<200){
      coordinates(xy) <- c("x", "y")
      projection(xy) <- CRS("+init=epsg:4326") # WGS 84
      CRS.new=CRS( proj4string(object[[1]]))
      dtrans <- spTransform(xy, CRS.new)
      xy=dtrans@coords  
    }
    
  }
  
  
  if(is.null(folder)){
    folder=object$folder
  }
  
  if(inherits(object, "sebal")){
    writeRaster(object$EF, filename=paste(folder,"EF.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$LE, filename=paste(folder,"LE.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$H, filename=paste(folder,"H.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$Rn, filename=paste(folder,"Rn.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$G, filename=paste(folder,"G.tif",sep="/"), format="GTiff",overwrite=overwrite)
    if(!is.null(object$ET24)){
      writeRaster(object$ET24, filename=paste(folder,"ET24.tif",sep="/"), format="GTiff",overwrite=overwrite)
      if(!is.null(xy)){
      xydata=stack(object$ET24, object$EF,object$LE,object$H,object$Rn,object$G)
      varname=c("ET24","EF","LE","H","Rn","G")
      names(xydata)=varname
      }
    }else{
      if(!is.null(xy)){
      xydata=stack(object$EF,object$LE,object$H,object$Rn,object$G)
      varname=c("EF","LE","H","Rn","G")
      names(xydata)=varname
      }
      
    }
    #xy=modcold$p[1:2]
  }  
  
  if(inherits(object, "sebi")||inherits(object, "ssebi")||inherits(object, "wdi")||
     inherits(object, "sebs")||inherits(object, "sseb")){
    writeRaster(object$EF, filename=paste(folder,"EF.tif",sep="/"), format="GTiff",overwrite=overwrite)
    if(!is.null(object$ET24)){
      writeRaster(object$ET24, filename=paste(folder,"ET24.tif",sep="/"), format="GTiff",overwrite=overwrite)
    }
    if(!is.null(xy)&&!is.null(object$ET24)){
      xydata=stack(object$ET24, object$EF)
      varname=c("ET24","EF")
      names(xydata)=varname
    }else{
      xydata= object$EF
      varname=c("EF")
      names(xydata)=varname
    }
  }
  
  
  if(inherits(object, "tseb")){
    writeRaster(object$EFc, filename=paste(folder,"EFc.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$EFs, filename=paste(folder,"EFs.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$LEc, filename=paste(folder,"LEc.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$LEs, filename=paste(folder,"Les.tif",sep="/"), format="GTiff",overwrite=overwrite)
    if(!is.null(object$ET24)){
      writeRaster(object$T24, filename=paste(folder,"T24.tif",sep="/"), format="GTiff",overwrite=overwrite)
      writeRaster(object$E24, filename=paste(folder,"E24.tif",sep="/"), format="GTiff",overwrite=overwrite)
      writeRaster(object$ET24, filename=paste(folder,"ET24.tif",sep="/"), format="GTiff",overwrite=overwrite)
    }
    
    if(!is.null(xy)&&!is.null(object$ET24)){
      xydata=stack(object$EFc,object$EFs,object$LEc,object$LEs,object$ET24,
                   object$T24, object$E24)
      varname=c("EFcanopy","EFsoil","LEcanopy","LEsoil","ET24","Transpiration24hr", "Evaporation24hr")
      names(xydata)=varname
    }else{
      xydata=stack(object$EFc,object$EFs,object$LEc,object$LEs)
      varname=c("EFcanopy","EFsoil","LEcanopy","LEsoil")
      names(xydata)=varname
    }
  }
  
  if(inherits(object, "landsat578")){
    writeRaster(object$albedo, filename=paste(folder,"albedo.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$Ts, filename=paste(folder,"Ts.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$NDVI, filename=paste(folder,"NDVI.tif",sep="/"), format="GTiff",overwrite=overwrite)
    writeRaster(object$SAVI, filename=paste(folder,"SAVI.tif",sep="/"), format="GTiff",overwrite=overwrite)
      writeRaster(object$fc, filename=paste(folder,"fc.tif",sep="/"), format="GTiff",overwrite=overwrite)
      writeRaster(object$radiance, filename=paste(folder,"radiance.tif",sep="/"), format="GTiff",overwrite=overwrite)
      writeRaster(object$reflectance, filename=paste(folder,"reflectance.tif",sep="/"), format="GTiff",overwrite=overwrite)
      writeRaster(object$LAI, filename=paste(folder,"LAI.tif",sep="/"), format="GTiff",overwrite=overwrite)
      if(!is.null(xy)){
        xydata=stack(object$albedo,object$Ts,object$NDVI,object$SAVI,object$LAI,object$fc,
                     object$radiance, object$reflectance)
        varname=c("albedo","Ts","NDVI","SAVI","LAI","fc","radiance", "reflectance")
        names(xydata)=varname
      } 
  }
  
  if(inherits(object, "sebkcstack")){
    writeRaster(object$data, filename=paste(folder,"sebkcstack.tif",sep="/"), 
                format="GTiff",bandorder='BIL',overwrite=overwrite)
    if(!is.null(object$band8)){
      writeRaster(object$band8, filename=paste(folder,"band8pan.tif",sep="/"), 
                  format="GTiff",overwrite=overwrite)
    }
    
    if(!is.null(xy)){
      xydata=object$data
      #names(xydata)=varname
    }
  }
  
  if(inherits(object, "ETo")){
    writeRaster(object$ETa, filename=paste(folder,"ETa.tif",sep="/"), 
                format="GTiff",overwrite=overwrite)
    
    if(!is.null(xy)){
      if(!is.null(object$xy)){
        xydata=stack(object$ETa,object$EF,object$Rn)
        names(xydata)=c("ETa","EF","Rn")
      }else{
        if(!is.null(object$EF)){
        xydata=stack(object$ETa,object$EF)
        names(xydata)=c("ETa","EF")  
        }else{
          xydata=object$ETa
          names(xydata)=c("ETa") 
        }
      }
     
    }
  }
  
  if(!is.null(xy)){
    xytable=cbind(xy,extract(xydata,xy,df=TRUE))
    filename=paste(class(object),"_xydata.txt",sep="")
    write.table(data.frame(xytable), file=paste(folder,filename,sep="/"),row.names = FALSE)
  }
  
}

#' Historical Weather data of Kumasi from 1960 to 2015
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity.
#' 
#' @name kumasi1960_2015
#' @docType data
#' @author George Owusu
#' @format A data frame with 20454 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL

#' A 2.6 RCP downscaled Weather data of Kumasi from 2016 to 2050
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity. The RCPs 2.6 assumes that green house 
#' gas will peak in between 2010-2020 and declines throughout 21st Century. 
#' 
#' @name kumasircp262016_2050
#' @docType data
#' @author George Owusu
#' @format A data frame with 12784 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL

#' A 4.5 RCP downscaled Weather data of Kumasi from 2016 to 2050
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity. The RCP 4.5 assumes that green house 
#' gas will peak in 2040 and declines throughout 21st Century. 
#' 
#' @name kumasircp452016_2050
#' @docType data
#' @author George Owusu
#' @format A data frame with 12784 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL

#' A 8.5 RCP downscaled Weather data of Kumasi from 2016 to 2050
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity. The RCP 8.5 assumes that green house 
#' gas will rise throughout 21st Century. 
#' 
#' @name kumasircp852016_2050
#' @docType data
#' @author George Owusu
#' @format A data frame with 12784 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL

  
#' @title Calibration of crop coefficients
#' 
#' @description This function can estimate up to 23 parameters of FAO single and dual crop
#' parameters. The parameters include soil parameters: WP,FC, REW; 
#' crop parameters: kcini,kcmid,kcend,kcbini,kcbmid,kcbend,h,Zr; 
#' the evaporation parameters: p and Ze, 
#' runoff coefficients: CNII or rc, as well as groundwater and capillary rise 
#' parameters: initgw,a1,b1,a2,b2,a3,b3,a4,b4. For each parameter initial, maximum and range 
#' values must be provided if one wants to optimise it.
#' There are also 12 goodness of fit tests for the evaluation of the model with
#' \code{\link{fitting}} :r-square, index of agreement, F-value, p-value, sse,sst, AIC,
#' BIC, RMSE, NRMSE and nashSutcliffe coefficients. A calibrated model that uses more 
#' parameters gets punished by AIC and BIC as well as adjusted R2, RMSE, NRMSE.
#' 
#' Both point and spatial model can be calibrated. The actual model is available at \code{\link{kc}}
#'
#' @inheritParams kc
#' @inheritParams ETo
#' @param ETa Measured Actual Evapotranspiration [mm]
#' @author George Owusu
#' @details  
#' \describe{
#' There are several ways  to use cal.kc:
#'  \item{\strong{Measured Data}}{The kc model can be calibrated against either Actual Evapotranspiration (ETa) or
#'  Available Soil Water; the user must specify theta. If the calibration is done on a point data then time series for
#'  that location must be provided. If multiple locations are provided with xydata and corresponding 
#'  multiple locations as input data, then ETa or theta must be provided for all locations. See also \code{kc} 
#'  for more details on locations.     }
#'  \item{\strong{Number of optimised parameters and Goodness of fit}}{ 
#'  There is a need to limit the number of calibrated parameters.
#'   One can compare models by observing the adjusted goodness of fit tests. 
#'   The more parameters that one uses the less are the values of R-square but
#'   RMSE, BIC, AIC increases. Unadjusted and adjusted goodness of fit tests
#'    are provided with this function.
#'     }
#'   }
#' @seealso \code{\link{kc}}
#'
#' @return The function returns optimised parameters as "parameters", goodness of fit 
#' as "fit", and calibrated model as "mod". It also prints candidates of parameters and 
#' goodness of fit of the local optimum values. For the explanation of calibrated model 
#' output and optimised parameters see the return values of \code{\link{kc}}. Here, only
#' goodness of fit tests explanations are given.
#' \describe{
#' \item{mod:}{ The optimised model. See return of \code{\link{kc}} for more details}
#' \item{parameters:}{ parameters of the optimised model}
#' \item{fit:}{  Goodness of fit tests. See \code{\link{fitting}} for explanation of variables}
#' }
#' @export
#'
#' @examples
#' \dontrun{
#' #fewer iteration
#' file=system.file("extdata","sys","irrigation.txt",package="sebkc")
#' data=read.table(file,header=TRUE)  
#' P=data$P
#' rc=0
#' I=data$I
#' ETo=data$ETo
#' Zr=data$Zr
#' p=0.6
#' FC=0.23
#' WP=0.10
#' u2=1.6  
#' RHmin=35
#' soil="sandy loam"
#' crop="Broccoli"
#' #single kc calibration with fewer iteration
#' cal=cal.kc(ETo,P,RHmin,soil,crop,I=0,kc=NULL,CNII=67,
#' u2=2,FC=NULL,WP=NULL,h=NULL,Zr=NULL,lengths=NULL,fc=NULL,
#' kctype="single",region=NULL,regexpr=NULL,initgw=50,LAI=3,
#' rc=NULL,Rn=NULL,hmodel="gaussian",Zrmodel="linear",
#' xydata=NULL,ETa=ETo,a1=360,b1=-0.17,a2=240,b2=-0.27,
#' a3=-1.3,b3=6.6,a4=4.6,b4=-0.65)
#' parameters=cal$parameters
#' output=cal$mod$output
#' gof=cal$fit
#' ETc=cal$mod$output$ETc
#' TAW=cal$mod$output$TAW
#' 
#' #Single kc with different kcs. kc uses default values
#' #RMSE_adj and NRMSE_adj equal to NAN because the number of parameters more than time steps(days)
#' cal=cal.kc(ETo,P,RHmin,soil,crop,I=0,CNII=67,u2=2,FC=NULL,WP=NULL,
#' h=NULL,Zr=NULL,lengths=NULL,fc=NULL,kctype="single",region=NULL,
#' regexpr=NULL,initgw=50,a1=c(100,400,10),b1=c(-1,-0.1,0.1),
#' a2=c(100,400,10),b2=c(-1,1,0.1),a3=c(-2,1,0.1),b3=c(1,10,1),
#' a4=c(1,10,1),b4=c(-1.5,1,0.1),LAI=3,rc=NULL,
#' Rn=NULL,hmodel="gaussian",Zrmodel="linear",xydata=NULL,ETa=ETo)
#' 
#' #Example with fixed kcs
#' cal=cal.kc(ETo,P,RHmin,soil,crop,I=0,CNII=67,kc=c(0.1,1.15,1),
#' u2=2,FC=NULL,WP=NULL,h=NULL,Zr=NULL,lengths=NULL,
#' fc=NULL,kctype="single",region=NULL,regexpr=NULL,initgw=50,
#' b1=c(-1,-0.1,0.1),b2=c(-1,1,0.1),LAI=3,rc=NULL,Rn=NULL,
#' hmodel="gaussian",Zrmodel="linear",xydata=NULL,ETa=ETo)
#' 
#' #Example with spatial kcs
#' #landsat folder with original files but a subset
#' folder=system.file("extdata","stack",package="sebkc") 
#' sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28) #sebi model
#' kc2=stack(sebiauto$EF/2,sebiauto$EF,sebiauto$EF/3) #assign EF to kc
#' xydata=data.frame(cbind(longitude=data$longitude,latitude=data$latitude))
#' modEF=kc(ETo,P=P,RHmin,soil,crop,I,kc=kc2,Rn=8,lengths = c(4,4,2,2),xydata=xydata)
#' ETa=modEF$ETcxy[3:14]

#' 
#' calsp=cal.kc(ETo,P,RHmin,soil,crop,I=0,CNII=67,kc=kc2,u2=2,FC=NULL,
#'  WP=WP,h=NULL,Zr=NULL,lengths = c(4,4,2,2),fc=NULL,kctype="single",
#'  region=NULL,regexpr=NULL,initgw=50,b1=c(-1,-0.1,0.1),b2=c(-1,1,0.1),
#'  LAI=3,rc=NULL,Rn=7,hmodel="gaussian",Zrmodel="linear",xydata=xydata,ETa=ETa)
#'  gof=cal$fit
#' r2=cal$fit$r2
#' AIC=cal$fit$AIC
#' ETcxy=cal$mod$ETcxy
#' TAW=cal$mod$TAW
#' }
  cal.kc=function(ETo,P,RHmin,soil,crop,I=0,CNII=c(30,100,1),u2=2,FC=NULL,WP=NULL,h=NULL,
                   Zr=NULL,Ze=c(0.1,0.15,0.01),kc=c(c(0.1,0.3,0.05),c(0.1,1,0.1),
    c(0.1,1,0.1)),p=c(0.1,1,0.1),lengths=NULL,fw=0.8,fc=NULL,
    kctype="dual",region=NULL,regexpr="sweet",initgw=50,
    LAI=3,rc=NULL,Kb=0,Rn=NULL,hmodel="gaussian",Zrmodel="linear",xydata=NULL,
    REW=c(2,12,0.5),a1=360,b1=c(-1,1,0.1),a2=240,b2=c(-1,1,0.1),
    a3=c(-2,1,0.1),b3=6.6,a4=4.6,b4=c(-1.5,1,0.1),ETa=NULL,report="stages"
    ,time="06:00",HWR=1,latitude=NULL,density="full",DOY=NULL,slope=NULL,
    y=NULL,r1=100,nongrowing="weeds",theta=NULL,profile="variable",Kcmin=0.15,Ky=NULL){
    
    if(is.null(ETa)&&is.null(theta)){
      return  (message("Observation data ETa or theta is needed"))
    }
    
    soilwater=read.csv(system.file("extdata","sys","soilwater2.csv",package="sebkc"),header=TRUE,stringsAsFactors=FALSE)
    stages=(read.csv(system.file("extdata","sys","stages.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
    crop_H_Zr=na.omit(read.csv(system.file("extdata","sys","crop_H_Zr.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
    singlekc=na.omit(read.csv(system.file("extdata","sys","singlekc.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
    dualkc=na.omit(read.csv(system.file("extdata","sys","dualkc.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
    ky=na.omit(read.csv(system.file("extdata","sys","ky.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
    wetting=na.omit(read.csv(system.file("extdata","sys","fw.csv",package="sebkc"),stringsAsFactors=FALSE,header=TRUE))
    soil2=soil
    crop2=crop
    
    EF=NULL
    para=NULL
    count=0
    if(inherits(kc[[1]], "RasterLayer")||inherits(kc[[2]], "RasterLayer")
       ||inherits(kc[[3]], "RasterLayer")){
      EF=TRUE 
    }
    
    soil=toupper(soil)
    soil=soilwater[(grep(soil2,soilwater$type,ignore.case=TRUE)),]
    if(is.null(FC)||is.null(WP)){
      if(is.null(FC)){
        FC=mean(c(as.numeric(strsplit(soil$FC, "-")[[1]][1]),as.numeric(strsplit(soil$FC, "-")[[1]][2])))
      }
      if(is.null(WP)){
        WP=mean(c(as.numeric(strsplit(soil$WP, "-")[[1]][1]),as.numeric(strsplit(soil$WP, "-")[[1]][2])))
      }
      
    }
    if(is.null(Zr)||is.null(p)){
      crop= crop_H_Zr[(grep(crop2,crop_H_Zr$crop,ignore.case=TRUE)),] 
      if(nrow(crop)>1){
        crop= crop[(grep((regexpr),(crop),ignore.case=TRUE)),] 
      }
      if(nrow(crop)==0){
        return (message(paste(crop2," not found [for crop type name]"))) 
      }
      if(is.null(Zr)){
        Zr=mean(c(as.numeric(strsplit(crop$Zr, "-")[[1]][1]),as.numeric(strsplit(crop$Zr, "-")[[1]][2])))
      }
      
      if(is.null(p)){
        p=crop$p
      }
    }
    
    
    ##dual crop coefficient
    if(is.null(kc)){
      kcs= dualkc[(grep((crop2),(dualkc$crop),ignore.case=TRUE)),] 
      if(nrow(kcs)>1){
        kcs= kcs[(grep((regexpr),(kcs$crop),ignore.case=TRUE)),] 
      }
      
      if(nrow(kcs)==0){
        return (message(paste(" Kcs not found for", crop2))) 
      }
      kcbini=mean(as.numeric(kcs$Kcbini))
      kcbmid=mean(as.numeric(kcs$Kcbmid))
      kcbend=mean(as.numeric(kcs$Kcbend))
      kc=c(kcbini,kcbmid,kcbend)
    }
    
    #kcb=(day*kcb)/max(day)
    
    if(is.null(h)){
      h= singlekc[(grep((crop2),(singlekc$crop),ignore.case=TRUE)),] 
      if(nrow(h)==0){
        return (message(paste(" h not found for", crop2))) 
      }
      if(nrow(h)>1){
        h= h[(grep((regexpr),(h$crop),ignore.case=TRUE)),] 
      }
      h=mean(as.numeric(h$h))
    }
    if(is.null(REW)){
      REW=mean(c(as.numeric(strsplit(soil$REW, "-")[[1]][1]),as.numeric(strsplit(soil$REW, "-")[[1]][2])))
    }
  #cal.kc=function(object,kcini=c(0.1,0.7,0.05),kcmid=c(0.1,1,0.05),
  #kcend=c(0.1,10,0.05),p=c(0.1,1,0.05),Ze=c(0.1,0.15,0.005),
  #REW=c(2,12,,0.05),ETc,AW=NULL)
    if(is.null(EF)){
    kcmat <- matrix(kc, ncol=3, byrow=TRUE)
    
    
    if(nrow(kcmat)==1){
      kcini=c(kcmat[1],kcmat[1],kcmat[1])
      kcmid=c(kcmat[2],kcmat[2],kcmat[2])
      kcend=c(kcmat[3],kcmat[3],kcmat[3])
    }else{
      kcini=kcmat[1,]
      kcmid=kcmat[2,]
      kcend=kcmat[3,]
      para=cbind(para,kcini=kcini,kcmid=kcmid,kcend=kcend)
      count=count+1
    }
    }else{
      kcini=c(1,1,1)
      kcmid=c(1,1,1)
      kcend=c(1,1,1)
      kc2=kc
    }
    
    if(length(REW)==1){
      REW=c(REW,REW,REW)
    }else{
      para=cbind(para,REW=REW)
      count=count+1
    }
    
    CNIIlist=NULL
    if(!is.null(CNII)){
      if(!is.null(xydata)&&length(CNII)==nrow(xydata)&&!is.null(EF)){
        CNIIlist=CNII
        CNII=c(1,1,1)
        #print("CNIIPPPPPP")
      }else{
        if(length(CNII)==1){
          CNII=c(CNII,CNII,CNII)
        }else{
          para=cbind(para,CNII=CNII)
          count=count+1
        }
      }
    }
    
    hlist=NULL
    if(!is.null(h)){
      if(!is.null(xydata)&&length(h)==nrow(xydata)&&!is.null(EF)){
        hlist=h
        h=c(1,1,1)
        #print("hPPPPPP")
      }else{
        if(length(h)==1){
          h=c(h,h,h)
        }else{
          para=cbind(para,h=h)
          count=count+1
        }
      }
    }
    
    Zrlist=NULL
    if(!is.null(Zr)){
      if(!is.null(xydata)&&length(Zr)==nrow(xydata)&&!is.null(EF)){
        Zrlist=Zr
        Zr=c(1,1,1)
        #print("ZrPPPPPP")
      }else{
        if(length(Zr)==1){
          Zr=c(Zr,Zr,Zr)
        }else{
          para=cbind(para,Zr=Zr)
          count=count+1
        }
      }
    }
    rc21=NULL
    if(!is.null(rc)){
      if(length(rc)==1){
        rc=c(rc,rc,rc)
      }else{
        para=cbind(para,rc=rc)
        count=count+1
      }
      rc21=NULL
    }else{
      rc21=TRUE
      rc=c(1,1,1)
    }
    
    FClist=NULL
    if(!is.null(FC)){
      if(!is.null(xydata)&&length(FC)==nrow(xydata)&&!is.null(EF)){
        FClist=FC
        FC=c(1,1,1)
        #print("FCPPPPPP")
      }else{
        if(length(FC)==1){
          FC=c(FC,FC,FC)
        }else{
          para=cbind(para,FC=FC)
          count=count+1
        }
      }
    }
    WPlist=NULL
    if(!is.null(WP)){
      if(!is.null(xydata)&&length(WP)==nrow(xydata)&&!is.null(EF)){
        WPlist=WP
        WP=c(1,1,1)
      }else{
        if(length(WP)==1){
          WP=c(WP,WP,WP)
        }else{
          para=cbind(para,WP=WP)
          count=count+1
        }
      }
    }
    if(!is.null(p)){
      if(length(p)==1){
        p=c(p,p,p)
      }else{
        para=cbind(para,p=p)
        count=count+1
      }
    }
    
    if(!is.null(Ze)){
      if(length(Ze)==1){
        Ze=c(Ze,Ze,Ze)
      }else{
        para=cbind(para,Ze=Ze)
        count=count+1
      }
    }
    
    initgwlist=NULL
    if(!is.null(initgw)){
      if(!is.null(xydata)&&length(initgw)==nrow(xydata)&&!is.null(EF)){
        initgwlist=initgw
        initgw=c(1,1,1)
        #print("initgwPPPPPP")
      }else{
        if(length(initgw)==1){
          initgw=c(initgw,initgw,initgw)
        }else{
          para=cbind(para,initgw=initgw)
          count=count+1
          }
      }
    }
   
    
    
    if(!is.null(a1)){
      if(length(a1)==1){
        a1=c(a1,a1,a1)
      }else{
        para=cbind(para,a1=a1)
        count=count+1
      }
    }
    
    if(!is.null(b1)){
      if(length(b1)==1){
        b1=c(b1,b1,b1)
      }else{
        para=cbind(para,b1=b1)
        count=count+1
      }
    }
    #print(b1)
    
    if(!is.null(a2)){
      if(length(a2)==1){
        a2=c(a2,a2,a2)
      }else{
        para=cbind(para,a2=a2)
        count=count+1
      }
    }
    
    if(!is.null(b2)){
      if(length(b2)==1){
        b2=c(b2,b2,b2)
      }else{
        para=cbind(para,b2=b2)
        count=count+1
      }
    }
    
    if(!is.null(a3)){
      if(length(a3)==1){
        a3=c(a3,a3,a3)
      }else{
        para=cbind(para,a3=a3)
        count=count+1
      }
    }
    
    if(!is.null(b3)){
      if(length(b3)==1){
        b3=c(b3,b3,b3)
      }else{
        para=cbind(para,b3=b3)
        count=count+1
      }
    }
    if(!is.null(a4)){
      if(length(a4)==1){
        a4=c(a4,a4,a4)
      }else{
        para=cbind(para,a4=a4)
        count=count+1
      }
    }
    
    if(!is.null(b4)){
      if(length(b4)==1){
        b4=c(b4,b4,b4)
      }else{
        para=cbind(para,b4=b4)
        count=count+1
      }
    }
    candidate=NULL
    message("Iteration Started. Kindly wait.................")
    pnames=names(data.frame(para))
    count=length(pnames)
    #return(list(para=para))
    #print(count)
    rmse=100
    r2=0
    kcini1=kcini[1]
    while(kcini1<=kcini[2]){
      kcmid1=kcmid[1]
      while(kcmid1<=kcmid[2]){
        kcend1=kcend[1]
        while(kcend1<=kcend[2]){
          p1=p[1]
          while(p1<=p[2]){
            Ze1=Ze[1]
            while(Ze1<=Ze[2]){
              REW1=REW[1]
              while(REW1<=REW[2]){
                CN1=CNII[1]
                while(CN1<=CNII[2]){
                  FC1=FC[1]
                  while(FC1<=FC[2]){
                    WP1=WP[1]
                    while(WP1<=WP[2]){
                      initgw1=initgw[1]
                      while(initgw1<=initgw[2]){
                        a11=a1[1]
                        while(a11<=a1[2]){
                          b11=b1[1]                      
                          while(b11<=b1[2]){
                            a21=a2[1]
                            while(a21<=a2[2]){
                              
                              b21=b2[1]
                              while(b21<=b2[2]){
                                a31=a3[1]
                                while(a31<=a3[2]){
                                  b31=b3[1]
                                  while(b31<=b3[2]){
                                    a41=a4[1]
                                    while(a41<=a4[2]){
                                      b41=b4[1]
                                      while(b41<=b4[2]){
                                        if(!is.null(EF)){
                                          thiskc2=kc2
                                        }else{
                                          thiskc2=c(kcini1,kcmid1,kcend1)
                                        }
                                          rc1=rc[1]
                                          while(rc1<=rc[2]){
                                            h1=h[1]
                                            while(h1<=h[2]){
                                              Zr1=Zr[1]
                                             while(Zr1<=Zr[2]){
                                                
                                            if(!is.null(rc21)){
                                              thisrc=NULL
                                            }else{
                                              thisrc=rc1
                                            }
                                            
                                            if(!is.null(WPlist)){
                                              WP2=WPlist
                                            }else{
                                              WP2=WP1
                                            }
                                            if(!is.null(FClist)){
                                              FC2=FClist
                                            }else{
                                              FC2=FC1
                                            }
                                            
                                            if(!is.null(initgwlist)){
                                              initgw2=initgwlist
                                            }else{
                                              initgw2=initgw1
                                            }
                                            
                                            if(!is.null(CNIIlist)){
                                              CNII2=CNIIlist
                                            }else{
                                              CNII2=CN1
                                            }
                                              if(!is.null(hlist)){
                                                h2=hlist
                                              }else{
                                                h2=h1
                                              }
                                               if(!is.null(Zrlist)){
                                                 Zr2=Zrlist
                                               }else{
                                                 Zr2=Zr1
                                               }
                                              #print(crop2)
                                          mod=kc(ETo=ETo,P=P,RHmin=RHmin,soil=soil,crop=crop2,I=I,CNII=CNII2,
                                                 u2=u2,FC=FC2,WP=WP2,h=h2,
                                                 Zr=Zr2,Ze=Ze1,kc=thiskc2,p=p1,lengths=lengths,fw=fw,fc=fc,
                                                 kctype=kctype,region=region,regexpr=regexpr,initgw=initgw2,
                                                 LAI=LAI,rc=thisrc,Kb=Kb,Rn=Rn,hmodel=hmodel,Zrmodel=Zrmodel,xydata=xydata,
                                                 REW=REW1,a1=a11,b1=b11,a2=a21,b2=b21,a3=a31,b3=b31,a4=a41,b4=b41,report=report
                                                 ,time=time,HWR=HWR,latitude=latitude,
                                                 density=density,DOY=DOY,slope=slope,y=y,r1=r1,
                                                 nongrowing=nongrowing,theta=theta,profile=profile,Ky=Ky)
                                          #print(nrow(mod$output))
                                          #if(!is.null(names(ETo)))
                                          #{
                                           # obs=mod$output$theta_measured[(length(mod$output$Day)/2)+1:(length(mod$output$Day)*2)]
                                            #first=((length(mod$output$Day)/2)+1)
                                            #second=length(mod$output$Day)
                                            #print(c(first,second))
                                            #mod$output= mod$output[seq(1,second,2),]
                                            #print(mod$output[seq(1,second,2),])
                                         # }
                                          #print(cbind(WP1,FC1,kcini1,kcmid1,kcend1,p1,Ze1,REW1,CN1,FC1,initgw1,a11,b11,a21,b21,a31,b31,a41,b41,rc1))   
                                          #print(length(ETo$ETo))
                                          if(!is.null(EF)){
                                            if(!is.null(ETa)){
                                            obs=rowMeans(ETa)
                                            est=rowMeans(mod$ETc_adj_xy[3:length(mod$ETc_adj_xy)])
                                            }
                                            if(!is.null(theta)){
                                              obs=rowMeans(mod$AW_measuredxy)
                                              est=rowMeans(mod$AW_modelledxy[3:length(mod$AW_modelledxy)])  
                                            }
                                            }else{
                                              if(!is.null(ETa)){
                                              obs=ETa
                                              est=mod$output$ETc_adj
                                              }else{
                                                if(!is.null(mod$output$AW_modelled)){
                                                  obs=mod$output$AW_measured 
                                                  est=mod$output$AW_modelled
                                                  }
                                              }
                                              
                                              }
                                          fit=fitting(obs,est,count)
                                          if(fit$r2>r2){
                                           #print(cbind(WP1,FC1,kcini1,kcmid1,kcend1,p1,Ze1,REW1,CN1,FC1,initgw1,a11,b11,a21,b21,a31,b31,a41,b41,rc1))   
                                        parameters=(list(WP=WP2,FC=FC2,kc=c(kcini1,kcmid1,kcend1),p=p1,Ze=Ze1,REW=REW1,
                                                          CNII=CNII2,initgw=initgw2,h=h2,Zr=Zr2,a1=a11,b1=b11,a2=a21,
                                                         b2=b21,a3=a31,b3=b31,a4=a41,b4=b41,rc=rc1,h=h2,Zr=Zr2,kcini=kcini1,
                                                         kcmid=kcmid1,kcend=kcend1)) 
                                        #print()
                                        
                                        para2=parameters[pnames]
                                            message(data.frame(para2))
                                            if(is.null(candidate)){
                                              candidate=cbind(para2,fit)
                                            }else{
                                              candidate=rbind(crop=crop2,candidate,cbind(para2,fit))
                                          }
                                          rmse=fit$RMSE
                                          r2=fit$r2
                                          thismod2=mod
                                          thisfit=cbind(crop=crop2,para2,fit)
                                          message(thisfit)
                                          
                                          }
                                          Zr1=Zr1+Zr[3]
                                            }
                                          h1=h1+h[3]
                                          }
                                          rc1=rc1+rc[3]
                                          }
                                        if(b4[1]==b4[2]){
                                          b41=b41-b4[3] 
                                        }else{
                                          b41=b41+b4[3] 
                                        }  
                                      }
                                      a41=a41+a4[3]
                                    }
                                    b31=b31+b3[3]
                                  }
                                  if(a3[1]==a3[2]){
                                    a31=a31-a3[3] 
                                  }else{
                                    a31=a31+a3[3] 
                                  }
                                }                              
                                if(b2[1]==b2[2]){
                                  b21=b21-b2[3] 
                                }else{
                                  b21=b21+b2[3] 
                                }
                              }                      
                              a21=a21+a2[3]
                            }
                            
                            if(b1[1]==b1[2]){
                            b11=b11-b1[3] 
                            }else{
                              b11=b11+b1[3] 
                            }
                          }
                          
                          a11=a11+a1[3]  
                        }
                        
                       
                      
                        initgw1=initgw1+initgw[3]  
                      }
                    
                      WP1=WP1+WP[3]
                      
                    }
                  
                  FC1=FC1+FC[3]
                  }
                  CN1=CN1+CNII[3]
                  
                  
                }
                
                REW1=REW1+REW[3]
              }
              Ze1=Ze1+Ze[3]
              
            }
      
          p1=p1+p[3]
          }
          
          kcend1=kcend1+kcend[3] 
        }
        
        kcmid1=kcmid1+kcmid[3] 
      }
      kcini1=kcini1+kcini[3] 
    }
    
    list(fit=thisfit,parameters=para2,mod=thismod2,candidate=candidate)
  }
  
  
  #b1=c(-1,-0.7,0.1)
#kc=c(0.1,1.15,1)

#' @title Water Deficit Index (WDI) 
#' @description Moran et. al (1994)  
#' Estimation of crop water deficit using surface-air 
#' temperature and spectral vegetation index.
#' @param Ta numeric. Air temperature. If not available you can use 
#' @param Tsmax numeric. Maximum sensor surface temperature [K]
#' @param Tsmin numeric. Minimum sensor surface temperature [K]
#' \code{\link{sebal}} to get pixel based Ta.
#' @inheritParams sebal
#' @inheritParams ETo 
#' @inheritParams coldTs
#' @author George Owusu
#' @details Note that surface temperature (Tsmin and Tsmax)
#'  are different from air temperature (Tmin and Tmax) in
#' \code{\link{ETo}}, \code{\link{sseb}}, \code{\link{sebs}}
#' 
#' You can get Ta from  \code{\link{sebal}} Ta
#' @seealso \code{\link{sseb}} and \code{\link{sebi}}
#' @references Moran, M. S., Clarke, T. R., Inoue, Y., & Vidal, A. 1994.
#' Estimating crop water deficit using the relation between surface-air 
#' temperature and spectral vegetation index. Remote Sensing of Environment,
#'  49(3): 246-263.
#' @return
#'  \describe{
#' \item{EF:}{ standardised wdi excluding negatives and one plus}
#' \item{index:}{ raw wdi including negatives and more than one}
#' }
#' @examples 
#'  \dontrun{
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' modWDI=wdi(Ts=Ts,Ta=299,NDVI=NDVI,albedo=NULL,Tsmax="auto",
#' Tsmin="auto",sunelev=50)
#' plot(modWDI$EF)
#' 
#' #Using folder parameter
#' folder=system.file("extdata","stack",package="sebkc")
#' wdiauto=wdi(folder=folder,welev=317,Tsmax=31,Tsmin=28,Ta=290)
#' 
#' #' #Another interactive example
#' #get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
#' welev=278
#' data=landsat578(rawdata,welev=welev)
#' #perform semi-auto simulation 
#' #Determine xyhot. Digitize polygon on the Ts map
#' modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#' #determine the cold. Digitize polygon on the Ts map
#' modcold=coldTs(data,welev=275,extent="auto",cluster=2)
#' Tsmax=modhot$Tshot
#' Tsmin=modcold$Tscold
#' #use object of \code{\link{coldTs}} or \code{\link{hotTs}} in different ways
#' modWDI2=wdi(data,Ta=299,Tsmax=Tsmax,Tsmin=modcold,sunelev=50)
#' plot(modWDI2$EF)
#' }
#' @export
#' @rdname wdi
wdi=function(Ts,Ta,NDVI,albedo=NULL,Tsmax="auto",Tsmin="auto",
             sunelev=NULL,folder=NULL,welev=NULL,Rn24=NULL)
  UseMethod ("wdi")
#' @export
#' @rdname wdi
wdi=function(Ts=NULL,Ta,NDVI,albedo=NULL,Tsmax="auto",Tsmin="auto",
             sunelev=NULL,folder=NULL,welev=NULL,Rn24=NULL){
  if(is.null(folder)){
    folder="/nothing454782ghf7poi8r.hope"
  }
  
  if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)&&!inherits(Ts, "landsat578")){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    
    if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
      if(inherits(Ts, "landsat578")){
        mod=Ts
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(folder=folder, welev=welev)
    }
    #mod=landsat578(data=folder, welev=welev)
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    DOY=mod$DOY
    sunelev=mod$sunelev
    SAVI=mod$SAVI
    welev=mod$welev
    }
  NDVI[NDVI<0,]=0
  NDVI[NDVI>1,]=1
  
  #maximun Ts
  
  if(inherits(Tsmax, "hotTs")){
    Tsmax=Tsmax$Tshot
  }
  
  if(inherits(Tsmin, "coldTs")){
    Tsmin=Tsmin$Tscold
  }
  
  if(Tsmax=="auto"&&!is.numeric(Tsmax)){
    modhot=hotTs(Ts=Ts,NDVI=NDVI,albedo=albedo,DEM=NULL,cluster=10,
                 extent="auto")
    Tsmax=modhot$Tshot 
  }
  
  if(Tsmin=="auto"&&!is.numeric(Tsmin)){
    modcold=coldTs(Ts=Ts,NDVI=NDVI,sunangle=sunelev,albedo=albedo,
                   DEM=NULL,cluster=10, extent="auto",upper=0.95,lower=0.1)
    Tsmin=modcold$Tscold 
  }
  #minimun Ts
  
  Tacold=Tahot=Ta
  if(inherits(Ta, "RasterLayer")){
    Tacold=getValues(Ta)[cellFromXY(Ta,c(modcold$x,modcold$y))]
    Tahot=getValues(Ta)[cellFromXY(Ta,c(modhot$x,modhot$y))]
  }
  
  
  wdi=((Tsmin-Tacold)-(Ts-Ta))/((Tsmin-Tacold)-(Tsmax-Tahot))
  index=wdi
  wdi[wdi>1,]=1
  wdi[wdi<0,]=0
  
  ET24=NULL
  if(!is.null(Rn24)){
    ET24=(wdi*Rn24)/2.45
  }
  factor<-list(EF=wdi, index=index,ET24=ET24,folder=folder)
  factor$call<-match.call()
  class(factor)<-"wdi"
  factor
}

#' Internal function to Catch  errors and warnings
#'
#' @param expr an  R expression to evaluate
#'
#' @return value a list with 'value' and 'warning', where value may be an error caught
#' @export
#'
#' @examples
#' thiserror=str( sebkc.tryCatch( log( "a" ) ) )$value
sebkc.tryCatch = function(expr){
  
  W <- NULL
  w.handler <- function(w){ # warning handler
    W <<- w
    invokeRestart("muffleWarning")
  }
  list(value = withCallingHandlers(tryCatch(expr, error = function(e) e),
                                   warning = w.handler),
       warning = W)
}

#' @title S-SEBI Evapotranspiration Fraction
#' 
#' @description \code{ssebi} computes Roerink et. al (2000) S-SEBI: A simple
#'  remote sensing algorithm to estimate the surface energy balance.
#' @inheritParams sebal
#' @inheritParams coldTs
#' @param threshold numeric. Albedo inflection point threshold
#' @author George Owusu
#'
#' @return EF, amax,bmax,amin,bmin
#' @references 
#' Roerink, G. J., Su, Z., & Menenti, M. 2000. S-SEBI: 
#' A simple remote sensing algorithm to estimate the surface energy balance.
#' Physics and Chemistry of the Earth, Part B: Hydrology, Oceans and Atmosphere
#' , 25(2): 147-157.
#' @examples
#' \dontrun{
#' #Manual data specification
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' mod=ssebi(Ts=Ts,albedo=albedo,threshold=0,plot=TRUE)
#' 
#' #Using landsat folder
#' folder=system.file("extdata","stack",package="sebkc")
#' ssebiauto=ssebi(folder=folder,welev=278)
#' }
#' @export
#' @rdname ssebi 
ssebi=function(Ts,albedo=NULL,
               threshold=0.2,plot=TRUE,Rn24=NULL,folder=NULL,welev=NULL)
  UseMethod ("ssebi")
#' @export
#' @rdname ssebi 
  
ssebi.default=function(Ts=NULL,albedo=NULL,
            threshold=0.2,plot=TRUE,Rn24=NULL,folder=NULL,welev=NULL){
  
  
  if(is.null(folder)){
    folder="/nothing454782ghf7poi8r.hope"
  }
  
  if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    
    if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
      if(inherits(Ts, "landsat578")){
        mod=Ts
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(folder=folder, welev=welev)
    }
    #mod=landsat578(data=folder, welev=welev)
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    DOY=mod$DOY
    sunelev=mod$sunelev
    SAVI=mod$SAVI
    
    }
  
if(!inherits(Ts, "RasterLayer")){
Ts=raster(Ts)
}
if(!inherits(albedo, "RasterLayer")&&!is.null(albedo)){
albedo=raster(albedo)
}
#max=maxValue(albedo)
#min=minValue(albedo)
#range=max-min
#ave=range/cluster
#seq=(seq(min,max,ave))
seq = unique(round(albedo, 2))
albedoclass=cut(albedo,breaks=seq)
albedomin=zonal(albedo, albedoclass, fun='min') 
Tsmin=zonal(Ts, albedoclass, fun='min')
Tsmax=zonal(Ts, albedoclass, fun='max') 
albedomax=zonal(albedo, albedoclass, fun='max') 

alTsmindata=merge(albedomin,Tsmin,by="zone")
alTsmaxdata=merge(albedomax,Tsmax,by="zone")
alTsmindata2<- alTsmindata[ which(alTsmindata[2]>0), ]
alTsmaxdata2<- alTsmaxdata[ which(alTsmaxdata[2]>0), ]
if(threshold=="auto"){
inflection=alTsmaxdata2[ which(alTsmaxdata2[3]==max(alTsmaxdata2[[3]])), ]
alTsmaxdata2<- alTsmaxdata2[ which(alTsmaxdata2[1]>=inflection[[1]]), ]
}
if(is.numeric(threshold)){
alTsmaxdata2<- alTsmaxdata2[ which(alTsmaxdata2[2]>threshold), ]
}
modTmax=lm(alTsmaxdata2[[3]]~alTsmaxdata2[[2]])
modTmin=lm(alTsmindata2[[3]]~alTsmindata2[[2]])
amax=coef(modTmax)[[1]]
bmax=coef(modTmax)[[2]]
amin=coef(modTmin)[[1]]
bmin=coef(modTmin)[[2]]
EF=(((bmax*albedo)+amax)-Ts)/(((bmax*albedo)+amax)-((bmin*albedo)+amin))
EF[EF<0]=0
EF[EF>1]=1

if(plot==TRUE||plot==TRUE){
#plot(albedo,Ts, maxpixels=5000000,xlab="Albedo",ylab="Surface Tempearture[Ts]")
plot(albedo,Ts,maxpixels=ncell(albedo),xlab="Albedo",ylab="Surface Tempearture[Ts]")

lines(alTsmindata2[[2]],alTsmindata2[[3]],type="p",col="forestgreen", pch = 19)
lines(alTsmaxdata2[[2]],alTsmaxdata2[[3]],type="p",col="red", pch = 19)
abline(modTmin, col = "forestgreen")
abline(modTmax, col = "red")
dev.new(width=15, height=10)

plot(EF)
}
ET24=NULL
if(!is.null(Rn24)){
  ET24=EF*Rn24*0.035
}

factor<-list(EF=EF,ET24=ET24,amax=amax,bmax=bmax,amin=amin,bmin=bmin,folder=folder)
factor$call<-match.call()
class(factor)<-"ssebi"
factor
}

#DOY="1-8-2001"
#latitude=0
#model="CBM"
#Tmax=31
#Tmin=26

#' Day length calculation
#' @inheritParams ETo
#'
#' @param model character Type of model:"CBM" (Schoolfield, 1982),"BGC" (Running & Coughlan, 1988),
#'  "CERES" (Ritchie, 1991) or "FAO56"
#' @param p numeric. CMB parameter
#'
#' @return DL
#' @author George Owusu
#' @export
#' 
#' @references 
#' \itemize{
#' \item Schoolfield R. (1982). Expressing daylength as a function of latitude and Julian date.
#' \item Running S. W. and Coughlan J. C. (1988). A general model of forest ecosystem processes 
#' for regional applications I. Hydrologic balance, canopy gas exchange and primary production 
#' processes. Ecological Modelling, 42(2), 125-154 http://dx.doi.org/10.1016/0304-3800(88)90112-3.
#' \item Ritchie J. T. (1991). Wheat Phasic Development. Modeling Plant and Soil Systems. 
#'  Agronomy Monograph, 31.
#' \item Allen R. G., Pereira L. S., Raes D. and Smith M. (1998). Crop evapotranspiration: 
#' Guidelines for computing crop water requirements. FAO Irrigation and Drainage Paper, 56, 300.
#' }
#' 
#'
#' @examples
#' DOY="2001-8-1"
#' latitude=0
#' model="CBM"
#' Tmax=31
#' Tmin=26
#' mod=dl(latitude,DOY,model,Tmax,Tmin=Tmin)

dl=function(latitude,DOY,model="CBM",Tmax=NULL,Tmin=NULL,p=0.5){
  
  
if(!is.numeric(DOY)){
  DOY2=DOY
  DOY=strptime(DOY,"%Y-%m-%d")$yday+1
  if(is.na(DOY)){
    DOY=strptime(DOY2,"%d-%m-%Y")$yday+1
  }
}


#better
if(model=="CBM"||model=="cbm"){

phi=0.2163108+2*atan(0.9671396*tan(0.00860)*(DOY-186))
lat2=asin(0.39795*cos(phi))
DL=24-(24/pi)*(acos(((sin((p*pi)/180))+(sin((latitude*pi/180)*sin(lat2)))/(cos((latitude/180)*cos(phi))))))
}

if(model=="Brock"||model=="brock"||model=="BROCK"){
  phi=23.45*sin(360*((283+DOY)/(365)))
  hourangle=acos((-1*tan(latitude)*tan(phi)))
  DL=2*(hourangle/15)
  
}

if(model=="BGC"||model=="bgc"||model=="Bgc"){
  ampl=exp((7.42+0.045*latitude)/3600)
  DL=ampl*sin((DOY-79)*0.01721)+12
  
}

if(model=="CERES"||model=="ceres"||model=="Ceres"){
 phi=0.4093*sin(0.0172*(DOY-82.2))
#DL=7.639*acos(max(-0.87,((-sin((latitude*pi)/180)*sin(phi)-0.1047))/cos((latitude*pi)/180)*cos(phi)))
#DL=7.639*acos(max(-0.87,(-sin((latitude*pi)/180)*sin(phi)-0.1047)/(cos((latitude*pi)/180)*cos(phi))))
DL=7.639*acos(max(-0.87,((-sin((latitude*pi)/180)*sin(phi)-(0.1047))/(cos((latitude*pi)/180)*(cos(phi))))))
}
if(model=="FAO56"){
  declination=0.409*sin((((2*pi*DOY)/(365)))-1.39)
  ws=acos(-tan(latitude)*tan(declination))
  DL=(24/pi)*ws  
}  
  
  
  
  T24=NULL
  Tn=NULL
  Rs=NULL
  Tday=NULL
  if(!is.null(Tmin)&&!is.null(Tmax)){
Ta=0.5*(Tmax+Tmin)
h=12-(0.5*DL)
Tday=Ta+((Tmax-Tmin)/(4*pi))*((11+h)/(12-h))*sin(pi*((11-h)/(11+h)))
Tn=Ta+((Tmax-Tmin)/(4*pi))*((11+h)/(h))*sin(pi*((11-h)/(11+h)))
#T24=(Tn+Tday)/2

Ta2=0.5*(Tmax-Tmin)
T24=Ta+Ta2*((24-Tmax)/(pi*(DL-Tmax)))*sin((pi*(DL-Tmax))/(24-Tmax))
if(is.na(T24)){
 # print("yes")
  T24=(Tday+Tn)*0.5  
}
PET=ETo(Tmax=Tmax,Tmin=Tmin,latitude=latitude,DOY=DOY)
Rs= PET$Rs*23.9
  }
  

list(Tday=Tday, T24=T24, Tn=Tn,Rs=Rs,DL=DL)
}

#' @title Simplified Surface Energy Balance
#' 
#' @description Senay etal (2011) Simplified Surface Energy Balance (SSEB) 
#' approach for estimating landscape ET
#'
#' @param TH the average of the representative 
#' 3 hot pixels selected for hot "bare" areas. 
#' If it is set to "auto" TH will be estimated from \code{\link{hotTs}} 
#' @param TC the average of representative 3 cold 
#' pixels selected from the irrigated fields. 
#' When it is set to "auto" TC will be estimated from \code{\link{coldTs}} 
#' @inheritParams sebal
#' @inheritParams ETo 
#' @inheritParams coldTs 
#' @inheritParams sebkcstack
#' @inheritParams ETohr
#' @inheritParams weather
#' @param NDVImax numeric, NDVI value for heavily green-vegetated area in the image
#' @param cc numeric, the correction coefficient if the NDVI approaches 0.0
#' @param KL numeric, the assumed lapse rate of air moving along the terrain
#' @param ETo numeric, a standardized clipped grass reference ET. when it is set to NULL,
#' it is automatically estimated from \code{\link{ETo}}
#' @param x multiplier needed  to estimate ET for tall, full cover crops 
#' such as alfalfa, corn and wheat. Default is 1.2.
#' @author George Owusu
#' @return
#'  \describe{
#' \item{ET24:}{ Actual 24 hour ET}
#' \item{EF:}{ ET fraction}
#' \item{ETo:}{ 24 hour reference ETo}
#' }
#'
#' @examples
#' \dontrun{
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' #minimal data
#'modsseb=sseb(Ts=Ts,TH="full",TC="full",sunelev=50,NDVI=NDVI,DEM=NULL,
#'albedo=albedo,NDVImax=0.7,cc=0.65,KL=0.0065,
#'ETo="auto",x=1.2,Tmax=31,Tmin=28,zx=10,
#'u=2,DOY=37,latitude=5.6,n=NULL,RHmax=NULL,RHmin=NULL)
#' 
#' #uaing the folder parameters
#' folder=system.file("extdata","stack",package="sebkc")
#' ssebauto=sseb(folder=folder,welev=317,Tmax=31,Tmin=28, latitude=5.6)
#' 
#' #Another interactive example
#' #get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
#' welev=278
#' data=landsat578(rawdata,welev=welev)
#' #perform semi-auto simulation 
#' #Determine xyhot. Digitize polygon on the Ts map
#' modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#' #determine the cold. Digitize polygon on the Ts map
#' modcold=coldTs(data,welev=275,extent="auto",cluster=2)
#' xyhot=modhot$Tshot
#' xycold=modcold$Tscold
#' #use object of \code{\link{coldTs}} or \code{\link{hotTs}} in different ways
#' modsseb=sseb(Ts=Ts,TH=xyhot,TC=modcold,sunelev=50,NDVI=NDVI,DEM=NULL,
#' albedo=albedo,NDVImax=0.7,cc=0.65,KL=0.0065,
#' ETo="auto",x=1.2,Tmax=31,Tmin=28,zx=10,
#' u=2,DOY=37,latitude=5.6,n=NULL,
#' RHmax=NULL,RHmin=NULL)
#' }
#' @references 
#' Senay, G. B., Budde, M. E., & Verdin, J. P. 2011. 
#' Enhancing the Simplified Surface Energy Balance (SSEB) approach 
#' for estimating landscape ET: Validation with the METRIC model. 
#' Agricultural Water Management, 98(4): 606-618.
#' @export
#' @rdname sseb 

sseb=function(Ts,welev=NULL,TH="auto",TC="auto",sunelev=NULL,NDVI=NULL,DEM=NULL,
              albedo=NULL,NDVImax=0.7,cc=0.65,KL=0.0065,
              ETo=NULL,wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",
              x=1.0,Tmax=NULL,Tmin=NULL,zx=NULL,
              u=2,DOY=NULL,latitude=NULL,n=NULL,
              RHmax=NULL,RHmin=NULL,clip=NULL,folder=NULL) UseMethod ("sseb")
#' @export
#' @rdname sseb
sseb.default=function(Ts=NULL,welev=NULL,TH="auto",TC="auto",sunelev=NULL,
  NDVI=NULL,DEM=NULL,albedo=NULL,NDVImax=0.7,cc=0.65,KL=0.0065,
ETo=NULL,wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",
x=1.2,Tmax=NULL,Tmin=NULL,zx=NULL,
u=2,DOY=NULL,latitude=NULL,n=NULL,
RHmax=NULL,RHmin=NULL,clip=NULL,folder=NULL){
  
  if(is.null(folder)){
    folder="/nothing454782ghf7poi8r.hope"
  }
  
  if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                     the elavation of th weather station"))  
    }
    
    if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
      if(inherits(Ts, "landsat578")){
        mod=Ts
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(folder=folder, welev=welev,clip=clip)
    }
    #mod=landsat578(data=folder, welev=welev)
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    DOY=mod$DOY
    sunelev=mod$sunelev
    SAVI=mod$SAVI
  }
  PET=NULL
  if(is.null(ETo)){
    
    if(!is.null(wmo)||!is.null(airport)){
      mod=sebkc.tryCatch(mod)$value
      thisDOY=sebkc.tryCatch(mod$date)$value
      if(inherits(thisDOY, "simpleError")){
        thisDOY=DOY 
      }
      ETr24=sebkc.tryCatch(ETo(DOY=thisDOY,airport = airport,wmo=wmo,latitude=latitude,
                               z=zx,Krs =Krs,altitude=welev))$value
      if(inherits(ETr24, "simpleError")){
        return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo",ETr24))) 
      }
      
      Tmin=ETr24$data$Tmin
      Tmax=ETr24$data$Tmax
      ETo=ETr24$ETo
    
    }
  }
  
Tx=Ts    
multiplier=x   
threshold=NDVImax  
  

if(!is.numeric(DOY)&&!is.null(DOY)){
  DOY2=DOY
  DOY=strptime(DOY,"%d-%m-%Y")$yday+1
  if(is.na(DOY)){
    DOY=strptime(DOY2,"%Y-%m-%d")$yday+1
  }
}
NDVI[NDVI<0,]=0
NDVI[NDVI>1,]=1

if(!is.null(DEM)){
Tx=Tx+(KL*DEM)
}

if(inherits(TH, "hotTs")){
  TH=TH$Tshot
}

if(inherits(TC, "coldTs")){
  TC=TC$Tscold
}

testcold=NULL

if((TC=="auto"||TC=="Auto"||TC=="AUTO"||TC=="full"||TC=="Full"||TC=="FULL")&&!is.numeric(TC)){
  TC=tolower(TC)
  testcold=TRUE
}
else{
  if(length(TC)>2){
    testcold=TRUE 
  }else{
    TC=TC
  }
}
testhot=NULL
if((TH=="auto"||TH=="Auto"||TH=="AUTO"||TH=="FULL"||TH=="full"||TH=="Full")&&(!is.numeric(TH))){
  TH=tolower(TH)
  testhot=TRUE
}else{
  if(length(TH)>2){
    testhot=TRUE 
  }else{
    TH=TH
  }
}

if(!is.null(testhot)){
  modhot=hotTs(Ts=Tx,NDVI=NDVI,albedo=albedo,DEM=DEM,
               extent=TH,upper=0.80,lower=0.1)
  TH=modhot$TH
}
if(!is.null(testcold)){
  modcold=coldTs(Ts=Tx,sunangle=sunelev,NDVI=NDVI,albedo=albedo,
                 DEM=DEM,extent=TC,upper=0.95,lower=0.1)
  TC=modcold$TC
}
message("computing ETf" )
ETf=(TH-Tx)/(TH-TC)

if(!is.null(NDVI)){
ETf=(((1-cc)*(NDVI/threshold)+(cc)))*ETf
}
if(is.null(ETo)||ETo=="auto"){
if(is.null(Tmax)||is.null(Tmin)||is.null(DOY)
||is.null(latitude)){
return(message("The following variables are needed:
Tmax,Tmin,DOY,latitude for estimation of ETo"))

}
  
PET=ETo(Tmax=Tmax,Tmin=Tmin,z=zx,uz=u,
altitude=welev,RHmax=RHmax,RHmin=RHmin,
n=n,DOY=DOY,latitude=latitude)

ETo=PET$ETo

}

ETf[ETf>1.1,]=1.1
ETf[ETf<0,]=0

message("computing ETm" )

ETm=multiplier*ETo
message("computing ETa" )

ETa=ETf*ETm
factor<-list(ET24=ETa, EF=ETf,ETo=ETo,folder=folder)
factor$call<-match.call()
class(factor)<-"ssebi"
factor
}

#' @title Stacking, gap-filling, cloud removal of Landsat image ET Model
#' 
#' @description This function prepares landsat 5, 7, and 8 data for  
#' \code{\link{landsat578}}. First, it reads meta data and returns many 
#' input parameters for \code{\link{landsat578}} and \code{\link{sebal}}.
#'  Second, it stacks 7 landsat layers: it stacks bands 1-7 for landsat 5 and 7; 
#' and bands 2-7,10 for landsat 8.. Third, it removes landsat clouds based 
#'  on Martinuzzi et al(2009) method. Fourth, it gap-fills landsat 7 strips 
#'  due to SLC-off as well as gap-filling NAs in other landsat sensors. 
#'  It can also process any selected bands.   
#' 
#' @param folder The folder or directory of landsat files
#' @param meta_pattern The regular expression pattern to identify 
#' metadata file. The default is "MTL.txt". The function works 
#' on the new landsat metadata only.
#' @param remove_cloud TRUE FALSE, 1, or 2. To indicate the cloud cover removal
#' from the image. Default is FALSE. If it is set to 1, 
#' the overlapping areas of  band 1 DN values between 120 and 255 and band 6
#' DN values of 102 to 128 are used. If it is set to 2 the union of the DN
#' ranges are used; and clouds as well as urban, barren, quarries, rocks, 
#' and sand are removed; otherwise only clouds are removed. 
#' See Martinuzzi et al(2009) for more details. 
#' At the moment only landsat 5 and 7 clouds are removed.
#' @param gap_fill Logical (TRUE or FALSE).It replaces strips in 
#' landsat 7 or NAs in other data set. It uses  \code{\link[raster]{focal}} 
#' neighbourhood approach. It is an automated Version of USGS "Gap-Filling Landsat
#'  7 SLC-off Single Scenes Using Erdas Imagine TM". Like USGS, it also takes 
#'  approximately one hour to complete.
#'  see details at http://landsat.usgs.gov/ERDAS_Approach.php
#' @param gap_loop numeric. The number of loops or repetitions that gap-fill 
#'  should be run. The default is 8 for one landsat scene. If the landsat image
#'  is smaller, it can be reduced. 
#' @param clip extent object or raster object or polygon from which an Extent 
#'  object can be extracted. A polygon or raster will be reprojected to 
#'  conform the data to be cropped. for example clip can take the form of c(xmin, xmax, ymin, ymax)
#' @details The function can simulate any of the activities described in the 
#' description section. It takes several minutes to replace strips (1 hour) 
#' and remove clouds (30 minutes) on one landsat scene. But stacking of bands 
#' and reading of metadata take few seconds. One can save the stacked and 
#' processed data in the same "folder" with \code{\link{writesebkc}}. 
#' The output file name is "sebkcstack.tif". 
# Whenever the function detects 
# a stacked file ("sebkcstack.tif") in the folder, it skips binding the layers 
# but reads meta data and performs other functions. 
# This is useful especailly after spending time refilling gap and masking 
# clouds. Therefore you can import stacked map from other applications 
# and name it "sebkcstack.tif" and gap-fill strips and/or remove clouds.
# Note that the imported file contains 7 layers. 
# You may delete this file if you want to stack from afresh.       
#' @author George Owusu
#' @references 
#' Martinuzzi, S., Gould, W.A., Ramos Gonzales, O.M. 2007.
#' Creating Cloud-Free Landsat ETM+ Data Sets in Tropical Landscapes: 
#' Cloud and Cloud-Shadow Removal. USDA Forest Service General Technical 
#' Report IITF-GTR-32.
#' 
#' USGS Using Landsat 7 Data http://landsat.usgs.gov/ERDAS_Approach.php
#'
#' @return 
#' \describe{
#' \item{data:}{ raster stacked data. If crop is not NULL then  crop Extent object is returned}
#' \item{brescale:}{ brescale of the stacked data}
#' \item{grescale:}{ grescale of the stacked data}
#' \item{sunelev:}{ sun elevation for stacked data}
#' \item{date:}{ Acquired date of the raster stacked data}
#' \item{viewangle:}{ view angle of landsat 8}
#' \item{azimuth:}{ azimuth angle of the image}
#' \item{sensor:}{ Type of satellite}
#' \item{K1:}{ K1 of landsat 8 thermal band 10}
#' \item{K2:}{ K2 of landsat 8 thermal band 10}
#' \item{cc:}{ cloud cover percentage}
#' \item{quality:}{ Image quality}
#' \item{row:}{ WRS_ROW of the image }
#' \item{path:}{ WRS_PATH of the image}
#' }
#'
#' @examples
#' \dontrun{
#' folder=system.file("extdata","stack",package="sebkc")
#' stack=sebkcstack(folder=folder)
#' #returns data and grescale
#' stack$data
#' stack$grescale
#' }
#' @export
#' @rdname sebkcstack
sebkcstack<-function(folder,meta_pattern="MTL.txt",remove_cloud=FALSE,
    gap_fill=FALSE,gap_loop=8,clip=NULL) UseMethod ("sebkcstack")
#' @export
#' @rdname sebkcstack
sebkcstack.default<-function(folder,meta_pattern="MTL.txt",remove_cloud=FALSE,
                            gap_fill=FALSE,gap_loop=8,clip=NULL){
  
crop=clip
metafile=list.files(folder,meta_pattern,full.names = TRUE)
metadata=read.delim(metafile,sep="=",stringsAsFactors = FALSE,skipNul = TRUE)
SPACECRAFT_ID=(metadata[grep("SPACECRAFT_ID",metadata$GROUP),][[2]])
sensor=as.numeric(strsplit(SPACECRAFT_ID, "_")[[1]][2])
raster1crop=NULL
Mp=NA
Ap=NA



sunelev=as.numeric(metadata[grep("SUN_ELEVATION",metadata$GROUP),][[2]])
viewangle=as.numeric(metadata[grep("ROLL_ANGLE",metadata$GROUP),][[2]])
date=as.Date(metadata[grep("DATE_ACQUIRED",metadata$GROUP),][[2]])
RADIANCE_MULT_BAND_1=as.numeric(metadata[grep("RADIANCE_MULT_BAND_1.",metadata$GROUP),][[2]])[1]
RADIANCE_MULT_BAND_2=as.numeric(metadata[grep("RADIANCE_MULT_BAND_2.",metadata$GROUP),][[2]])[1]

RADIANCE_MULT_BAND_3=as.numeric(metadata[grep("RADIANCE_MULT_BAND_3.",metadata$GROUP),][[2]])[1]
RADIANCE_MULT_BAND_4=as.numeric(metadata[grep("RADIANCE_MULT_BAND_4.",metadata$GROUP),][[2]])[1]

RADIANCE_MULT_BAND_5=as.numeric(metadata[grep("RADIANCE_MULT_BAND_5.",metadata$GROUP),][[2]])[1]
RADIANCE_MULT_BAND_6=as.numeric(metadata[grep("RADIANCE_MULT_BAND_6.",metadata$GROUP),][[2]])[1]

RADIANCE_MULT_BAND_7=as.numeric(metadata[grep("RADIANCE_MULT_BAND_7.",metadata$GROUP),][[2]])[1]
if(sensor==8){
RADIANCE_MULT_BAND_9=as.numeric(metadata[grep("RADIANCE_MULT_BAND_9.",metadata$GROUP),][[2]])[1]
RADIANCE_MULT_BAND_10=as.numeric(metadata[grep("RADIANCE_MULT_BAND_10.",metadata$GROUP),][[2]])[1]
grescale=c(RADIANCE_MULT_BAND_2,RADIANCE_MULT_BAND_3,RADIANCE_MULT_BAND_4,
           RADIANCE_MULT_BAND_5,RADIANCE_MULT_BAND_6,RADIANCE_MULT_BAND_7,RADIANCE_MULT_BAND_10)
}else{
  grescale=c(RADIANCE_MULT_BAND_1,RADIANCE_MULT_BAND_2,RADIANCE_MULT_BAND_3,RADIANCE_MULT_BAND_4,
             RADIANCE_MULT_BAND_5,RADIANCE_MULT_BAND_6,RADIANCE_MULT_BAND_7)
}

RADIANCE_ADD_BAND_1=as.numeric(metadata[grep("RADIANCE_ADD_BAND_1.",metadata$GROUP),][[2]])[1]
RADIANCE_ADD_BAND_2=as.numeric(metadata[grep("RADIANCE_ADD_BAND_2.",metadata$GROUP),][[2]])[1]

RADIANCE_ADD_BAND_3=as.numeric(metadata[grep("RADIANCE_ADD_BAND_3.",metadata$GROUP),][[2]])[1]
RADIANCE_ADD_BAND_4=as.numeric(metadata[grep("RADIANCE_ADD_BAND_4.",metadata$GROUP),][[2]])[1]

RADIANCE_ADD_BAND_5=as.numeric(metadata[grep("RADIANCE_ADD_BAND_5.",metadata$GROUP),][[2]])[1]
RADIANCE_ADD_BAND_6=as.numeric(metadata[grep("RADIANCE_ADD_BAND_6.",metadata$GROUP),][[2]])[1]

RADIANCE_ADD_BAND_7=as.numeric(metadata[grep("RADIANCE_ADD_BAND_7.",metadata$GROUP),][[2]])[1]
if(sensor==8){
  RADIANCE_ADD_BAND_9=as.numeric(metadata[grep("RADIANCE_ADD_BAND_9.",metadata$GROUP),][[2]])[1]
  RADIANCE_ADD_BAND_10=as.numeric(metadata[grep("RADIANCE_ADD_BAND_10.",metadata$GROUP),][[2]])[1]
  brescale=c(RADIANCE_ADD_BAND_2,RADIANCE_ADD_BAND_3,RADIANCE_ADD_BAND_4,
             RADIANCE_ADD_BAND_5,RADIANCE_ADD_BAND_6,RADIANCE_ADD_BAND_7,RADIANCE_ADD_BAND_10)
}else{
  brescale=c(RADIANCE_ADD_BAND_1,RADIANCE_ADD_BAND_2,RADIANCE_ADD_BAND_3,RADIANCE_ADD_BAND_4,
             RADIANCE_ADD_BAND_5,RADIANCE_ADD_BAND_6,RADIANCE_ADD_BAND_7)
}


#if(sensor==8){
REFLECTANCE_MULT_BAND_1=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_1.",metadata$GROUP),][[2]])[1]
REFLECTANCE_MULT_BAND_2=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_2.",metadata$GROUP),][[2]])[1]

REFLECTANCE_MULT_BAND_3=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_3.",metadata$GROUP),][[2]])[1]
REFLECTANCE_MULT_BAND_4=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_4.",metadata$GROUP),][[2]])[1]

REFLECTANCE_MULT_BAND_5=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_5.",metadata$GROUP),][[2]])[1]
REFLECTANCE_MULT_BAND_6=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_6.",metadata$GROUP),][[2]])[1]

REFLECTANCE_MULT_BAND_7=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_7.",metadata$GROUP),][[2]])[1]
if(sensor==8){
  REFLECTANCE_MULT_BAND_9=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_9.",metadata$GROUP),][[2]])[1]
  REFLECTANCE_MULT_BAND_10=as.numeric(metadata[grep("REFLECTANCE_MULT_BAND_10.",metadata$GROUP),][[2]])[1]
  Mp=c(REFLECTANCE_MULT_BAND_2,REFLECTANCE_MULT_BAND_3,REFLECTANCE_MULT_BAND_4,
       REFLECTANCE_MULT_BAND_5,REFLECTANCE_MULT_BAND_6,REFLECTANCE_MULT_BAND_7,
       REFLECTANCE_MULT_BAND_10)
}else{
  Mp=c(REFLECTANCE_MULT_BAND_1,REFLECTANCE_MULT_BAND_2,REFLECTANCE_MULT_BAND_3,REFLECTANCE_MULT_BAND_4,
       REFLECTANCE_MULT_BAND_5,REFLECTANCE_MULT_BAND_6,REFLECTANCE_MULT_BAND_7)
}

REFLECTANCE_ADD_BAND_1=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_1.",metadata$GROUP),][[2]])[1]
REFLECTANCE_ADD_BAND_2=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_2.",metadata$GROUP),][[2]])[1]

REFLECTANCE_ADD_BAND_3=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_3.",metadata$GROUP),][[2]])[1]
REFLECTANCE_ADD_BAND_4=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_4.",metadata$GROUP),][[2]])[1]

REFLECTANCE_ADD_BAND_5=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_5.",metadata$GROUP),][[2]])[1]
REFLECTANCE_ADD_BAND_6=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_6.",metadata$GROUP),][[2]])[1]

REFLECTANCE_ADD_BAND_7=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_7.",metadata$GROUP),][[2]])[1]
if(sensor==8){
  REFLECTANCE_ADD_BAND_9=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_9.",metadata$GROUP),][[2]])[1]
  REFLECTANCE_ADD_BAND_10=as.numeric(metadata[grep("REFLECTANCE_ADD_BAND_10.",metadata$GROUP),][[2]])[1]
  Ap=c(REFLECTANCE_ADD_BAND_2,REFLECTANCE_ADD_BAND_3,REFLECTANCE_ADD_BAND_4,
       REFLECTANCE_ADD_BAND_5,REFLECTANCE_ADD_BAND_6,REFLECTANCE_ADD_BAND_7,REFLECTANCE_ADD_BAND_10)
}else{
  Ap=c(REFLECTANCE_ADD_BAND_1,REFLECTANCE_ADD_BAND_2,REFLECTANCE_ADD_BAND_3,REFLECTANCE_ADD_BAND_4,
       REFLECTANCE_ADD_BAND_5,REFLECTANCE_ADD_BAND_6,REFLECTANCE_ADD_BAND_7)
}


#}


K1_CONSTANT_BAND_10=as.numeric(metadata[grep("K1_CONSTANT_BAND_10",metadata$GROUP),][[2]])
K2_CONSTANT_BAND_10=as.numeric(metadata[grep("K2_CONSTANT_BAND_10",metadata$GROUP),][[2]])

if(length(K1_CONSTANT_BAND_10)==0){
  K1_CONSTANT_BAND_10=NULL  
}
if(length(K2_CONSTANT_BAND_10)==0){
  K2_CONSTANT_BAND_10=NULL  
}
time=strsplit(
  gsub("Z","",
  (metadata[grep("SCENE_CENTER_TIME",metadata$GROUP),])[2]
  ),":")
#time=round(
#  as.numeric(format(strptime(time, "%I:%M %p"), "%H")) +         # hours component
#    (as.numeric(format(strptime(time, "%I:%M %p"), "%M")) / 60),2  # rounded minutes component
#)
time=as.numeric(time[[1]][1])+(as.numeric(time[[1]][2])/60)
SUN_AZIMUTH=as.numeric(metadata[grep("SUN_AZIMUTH",metadata$GROUP),][[2]])
IMAGE_QUALITY=as.numeric(metadata[grep("IMAGE_QUALITY",metadata$GROUP),][[2]])
CLOUD_COVER=as.numeric(metadata[grep("CLOUD_COVER",metadata$GROUP),][[2]])
row=as.numeric(metadata[grep("WRS_ROW",metadata$GROUP),][[2]])[1]
path=as.numeric(metadata[grep("WRS_PATH",metadata$GROUP),][[2]])[1]
#image files
#chesck first if sebkcstack exists
if(file.exists(paste(folder,"sebkcstack.tif",sep="/"))){
data=brick(paste(folder,"sebkcstack.tif",sep="/"))  
band1=data[[1]]
band2=data[[2]]
band3=data[[3]]
band4=data[[4]]
band5=data[[5]]
band6=data[[6]]
band7=data[[7]]
if(sensor==8){
  band10=data[[10]]
  band11=data[[11]]
}

}else{
imagefiles=list.files(folder,NULL,full.names = FALSE)
band1=raster(paste(folder,grep("_B1.",imagefiles,value = TRUE)[1],sep="/"))
band2=raster(paste(folder,grep("_B2.",imagefiles,value = TRUE)[1],sep="/"))
band3=raster(paste(folder,grep("_B3.",imagefiles,value = TRUE)[1],sep="/"))
band4=raster(paste(folder,grep("_B4.",imagefiles,value = TRUE)[1],sep="/"))
band5=raster(paste(folder,grep("_B5.",imagefiles,value = TRUE)[1],sep="/"))
band6=raster(paste(folder,grep("_B6.",imagefiles,value = TRUE)[1],sep="/"))
band7=raster(paste(folder,grep("_B7.",imagefiles,value = TRUE)[1],sep="/"))
if(sensor==8){
  band9=raster(paste(folder,grep("_B9.",imagefiles,value = TRUE)[1],sep="/"))
  band10=raster(paste(folder,grep("_B10.",imagefiles,value = TRUE)[1],sep="/"))
  data=stack(band2,band3,band4,band5,band6,band7,band10)
  
}else{
  data=stack(band1,band2,band3,band4,band5,band6,band7)
}
}
name1=names(data)

#clousd masking
if((remove_cloud!=FALSE||remove_cloud!=FALSE)&sensor!=8){
  
  if(remove_cloud==2){
  mask<-(data[[1]]>120&data[[1]]<255)|(data[[6]]>=102&data[[6]]<128)
  }else{
  mask<-(data[[1]]>120&data[[1]]<255)&(data[[6]]>=102&data[[6]]<128)
  }
  #band1=data[[1]]
  data[mask==1]<-NA
  names(data)=name1
}

#strips replacing
if(gap_fill==TRUE||gap_fill==TRUE){
  #system.time()
  
  band1[band1==0]=NA
  band2[band2==0]=NA
  band3[band3==0]=NA
  band4[band4==0]=NA
  band5[band5==0]=NA
  band6[band6==0]=NA
  band7[band7==0]=NA
  if(sensor==8){
  band10[band10==0]=NA
  band11[band11==0]=NA
  }
  
  i=1
  while(i<=gap_loop){
    band1<- focal(band1, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    band2<- focal(band2, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    band3<- focal(band3, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    band4<- focal(band4, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    band5<- focal(band5, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    band6<- focal(band6, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    band7<- focal(band7, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    if(sensor==8){
    band10<- focal(band10, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    band11<- focal(band11, w=matrix(1,3,3), "mean",na.rm=TRUE,NAonly=TRUE) 
    }
    
    i=i+1
  }
  if(sensor==8){
  data=stack(band1,band2,band3,band4,band5,band6,band7,band10)
  }else{
    data=stack(band1,band2,band3,band4,band5,band6,band7)
  }
  names(data)=name1
  
  
  #system.time()
}
##cropping the image

if(!is.null(crop)){
  data=cropsebkc(x=data,y=crop)$x
  
}
#class(extent(crop))=="Extent"

factor<-list(data=data,brescale=brescale,grescale=grescale,Mp=Mp,Ap=Ap,sunelev=sunelev,
     date=date,viewangle=viewangle,sensor=sensor,path=path,row=row,
     K1=K1_CONSTANT_BAND_10,K2=K2_CONSTANT_BAND_10,cc=CLOUD_COVER,
     quality=IMAGE_QUALITY,azimuth=SUN_AZIMUTH,folder=folder,time=time)


factor$call<-match.call()

class(factor)<-"sebkcstack"
factor
}

#' @title SEBS Estimation of Evapotranspiration Fraction
#'
#' @description \code{sebs} computes Su (2002) 
#' Surface Energy Balance System (SEBS) for estimation of turbulent heat fluxes
#' @inheritParams sebal
#' @inheritParams ETo
#' @inheritParams ETohr
#' @inheritParams weather
#' @inheritParams sebkcstack
#' @details Function \code{\link{ETo}} will be automatically used to 
#' estimate  parameters y (psychrometric constant), 
#' slope (slope of saturation vapour pressure curve) and 
#' vpd (Vapour pressure deficit). For information on data input procedure
#' see 'details' at \code{\link{sebal}}
#' @seealso \code{\link{sebal}} , \code{\link{ETo}}, \code{\link{sebi}}  
#' @references 
#' Su, Z. 2002. The Surface Energy Balance System (SEBS) for estimation 
#' of turbulent heat fluxes. Hydrol. Earth Syst. Sci., 6(1): 85-100.
#' @return EF Evapotranspiration Fraction
#' @author George Owusu
#'
#' @examples
#' \dontrun{
#' #define Input data
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
#' Tmax=31
#' Tmin=28
#' RHmax=84
#' RHmin=63
#' #perform simulation
#' modsebs=sebs(albedo=albedo,Ts=Ts,Tmax=Tmax,Tmin=Tmin,RHmax=RHmax,
#' RHmin=RHmin,NDVI=NDVI,SAVI=NULL,iter.max=7,xyhot="full",
#' xycold="full",DOY=37,sunelev=50.71154048,welev=317.1,zx=10,
#' u=2,zomw=2,zom=NULL,LAI=LAI,Rn24=NULL,model="SEBS")
#'
#' 
#' #use original landsat 7 data by specifying folder path
#' folder=system.file("extdata","stack",package="sebkc")
#' modauto=sebs(folder = folder,welev = 380,Tmax=Tmax,
#' Tmin=Tmin,RHmax=RHmax,RHmin=RHmin)
#' #plot ET fraction
#' plot(modauto$EF)
#' 
#' #get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
#' welev=278
#' data=landsat578(rawdata,welev=welev)
#'  #perform semi-auto simulation 
#' #Determine xyhot. Digitize polygon on the Ts map
#' modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#' #determine the cold. Digitize polygon on the Ts map
#' modcold=coldTs(data,welev=275,extent="auto",cluster=2)
#' xyhot=modhot$xyhot
#' xycold=modcold$xycold
#' #use object of \code{\link{coldTs}} or \code{\link{hotTs}} in different ways
#' modsebs=sebs(data,xyhot=modhot,xycold=xycold,welev=welev,
#' Tmax=Tmax,Tmin=Tmin,RHmax=RHmax,RHmin=RHmin)
#'  }
#' @export
#' @rdname sebs 

sebs=function(albedo,Ts,Tmax,Tmin,RHmax=NULL,RHmin=NULL,n=NULL,
      NDVI,SAVI,iter.max=7,xyhot,xycold,DOY,
      sunelev,welev,zx,u,zomw,zom=NULL,LAI=NULL,DEM=NULL,
      lapse=0.0065,Rn24=NULL,wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",
      latitude=NULL,model="SEBS",clip=NULL,folder=NULL) UseMethod ("sebs")
#' @export
#' @rdname sebs 
sebs.default=function(albedo=NULL,Ts,Tmax,Tmin,RHmax=NULL,RHmin=NULL,n=NULL,
NDVI,SAVI,iter.max=7,xyhot="auto",xycold="auto",DOY=NULL,
sunelev,welev,zx=10,u=2,zomw=2,zom=NULL,LAI=NULL,DEM=NULL,lapse=0.0065,Rn24=NULL,
wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",latitude=NULL,
model="SEBS",clip=NULL,folder=NULL){
  
  if(is.null(folder)){
    folder="/nothing454782ghf7poi8r.hope"
  }
  
  if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    
    if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
      if(inherits(albedo, "landsat578")){
        mod=albedo
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(folder=folder, welev=welev,clip=clip)
      folder=mod
    }
    #mod=landsat578(data=folder, welev=welev)
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    DOY=mod$DOY
    sunelev=mod$sunelev
    SAVI=mod$SAVI
    }
  PET=NULL

    if(!is.null(wmo)||!is.null(airport)){
      mod=sebkc.tryCatch(mod)$value
      thisDOY=sebkc.tryCatch(mod$date)$value
      if(inherits(thisDOY, "simpleError")){
        thisDOY=DOY 
      }
      ETr24=sebkc.tryCatch(ETo(DOY=thisDOY,airport = airport,wmo=wmo,latitude=latitude,
                               z=zx,Krs =Krs,altitude=welev))$value
      if(inherits(ETr24, "simpleError")){
        return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo",ETr24))) 
      }
      
      Tmin=ETr24$data$Tmin
      Tmax=ETr24$data$Tmax
      Rn24=ETr24$Rn
      PET=ETr24

    }
  
uw=u
waterdensity=1000
p=1.03547033
cp=1004.14
g=9.81
k=0.41 #von Karman's constant
z1=0.1 
z2=2
vaporisation=2430000

if(!is.numeric(DOY)&&!is.null(DOY)){
  DOY2=DOY
  DOY=strptime(DOY,"%d-%m-%Y")$yday+1
  if(is.na(DOY)){
    DOY=strptime(DOY2,"%Y-%m-%d")$yday+1
  }
}

mod=sebal(albedo=albedo,Ts=Ts,NDVI=NDVI,SAVI=SAVI,
iter.max=iter.max,xyhot=xyhot,xycold=xycold,
DOY=DOY,sunelev=sunelev,welev=welev,
zx=zx,u=uw,zomw=zomw,zom=zom,LAI=LAI,DEM=DEM,
lapse=lapse,Rn24=Rn24,model="SEBAL",folder=folder)

if(is.null(PET)){
PET=ETo(Tmax=Tmax,Tmin=Tmin,z=zx,uz=u,altitude=welev,
RHmax=RHmax,RHmin=RHmin,n=n,DOY=mod$DOY,latitude=1)
}
data=mod$iter.coef[nrow(mod$iter.coef),]
y=PET$y
vpd=PET$vpd
slope=PET$slope
Hdry=data$Rn_hot-data$G_hot
Hwet=((data$Rn_cold-data$G_cold)-((data$paircold*cp)/data$rah_cold)*(vpd/y))/((1+slope/y))
message("computing EF" )
EFr=1-((mod$H-Hwet)/(Hdry-Hwet))
EF=1.1*((EFr*data$LEcold)/(mod$Rn-mod$G))
EF[EF<0,]=0
EF[EF>1.1,]=1.1

ET24=NULL
#print(Rn24)
if(!is.null(Rn24)){
  message("computing ETa" )
  ET24=EF*Rn24*0.035
}
factor<-list(EF=EF,ET24=ET24,folder=folder)
factor$call<-match.call()
class(factor)<-"sebs"
factor
}

#' @title  Automatic computation of hot and cold pixels from Radiometric Surface Temperature (Ts)

#' @description This function computes temperature coordinates of a hot and cold pixels based on only
#' Surface Temperature. This function uses ranges of temperature.
#' @param Tmax Numeric. Maximum surface Temperature in map units (eg. Kilvin) that you want computation from. 
#' You can keep changing untill you get your desired values
#' @param Tmin Numeric. Minimum surface Temperature in  map units (eg. Kilvin) that you want computation from. 
#' You can keep changing untill you get your desired values
#' @inheritParams sebal
#' @inheritParams coldTs2
#' @inheritParams coldTs
#' @inheritParams landsat578
#' @details The function first divides the selected area into a 
#' number of clusters. The mean and standard deviations are
#' computed. The cluster means with the lowest standard deviations
#' and qualifying for upper (Ts) and lower (NDVI) limits 
#' (see above) are selected. The provided upper and lower probabilities
#'  are also used to select the final Ts.
#' @seealso \code{\link{coldTs}},  \code{\link{hotTs}}, \code{\link{coldTs2}} 
#' @return 
#' \describe{
#' \item{Ts:}{ The selected or input Ts}
#' \item{Tshot:}{ The Temperature of hot pixel}
#' \item{Tscold:}{ The Temperature of cold pixel}
#' \item{xyhot: }{combined x and y coordinates}
#' \item{candidates:}{ The similar candidates' pixels that can be used}
#' }
#' @examples
#' \dontrun{
#'  folder=system.file("extdata","stack",package="sebkc")
#'   model=coldhot(Tmin=300,Tmax=310,folder=folder,welev=170)
#'   spplot(model$Tshot)
#'   spplot(model$Tscold)
#'   modauto=sebal(folder = folder,welev = 380,xycold=model$xycold,xyhot=model$xyhot)}
#' @author George Owusu
#' @references Allen, R. G., Burnett, B., Kramber, W., Huntington, J., 
#' Kjaersgaard, J., Kilic, A., Kelly, C., & Trezza, R. 2013. Automated 
#' Calibration of the METRIC-Landsat Evapotranspiration Process. 
#' JAWRA Journal of the American Water Resources Association, 49(3): 563-576.
#' @export
#' @rdname coldhot
#'

coldhot<-function(Tmin,Tmax, Ts=NULL,folder=NULL,welev=NULL ) UseMethod ("coldhot")
#' @export
#' @rdname coldhot
coldhot.default=function(Tmin,Tmax,Ts=NULL,folder=NULL,welev=NULL){
  
  if(is.null(folder)){
    folder="nothing454782ghf7poi8r.hope"
  }
  
  
  if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    if(inherits(folder, "landsat578")||inherits(Ts, "landsat578")){
      if(inherits(Ts, "landsat578")){
        mod=Ts
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(data=folder, welev=welev)
    }
    
    
    Ts=mod$Ts

    }
  
 
  
  #get the hot atemperature values
  temphot=Ts
  #set a threshold for hot temperatures
  temphot[temphot<=Tmax]=NA
  phot=data.frame(rasterToPoints(temphot))
  
  #get the cold atemperature values
  tempcold=Ts
  #set a threshold for cold temperatures
  tempcold[tempcold>Tmin]=NA
  pcold=data.frame(rasterToPoints(tempcold))
  xyhot=c(phot[1,][[1]], phot[1,][[2]])
  xycold=c(pcold[1,][[1]], pcold[1,][[2]])
  
 
  
  factor<-list(xyhot=xyhot,xycold=xycold,Ts=Ts,Tshot=temphot,Tscold=tempcold)
  
  factor$call<-match.call()
  
  class(factor)<-"coldhot"
  factor
  }

#folder=system.file("extdata","stack",package="sebkc")
#model=coldhot(Tmin=307,Tmax=310,folder=folder,welev=170)
#spplot(model$Tshot)
#spplot(model$Tscold)

#' Spatial Conditional function on raster object
#' 
#' It computes equivalent of logical if function in R
#'
#' @param condition logical
#' @param trueValue logical
#' @param falseValue logical
#' @author George Owusu
#' @return Return True or False
#' @export
#' @examples
#' \dontrun{
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
#'   eNB=rastercon( NDVI<0 & albedo<0.47,0.99,rastercon(LAI>=3,0.98,0.97+(LAI*0.0033)))
#'   }

rastercon=function(condition, trueValue,falseValue){
    return (condition*trueValue+(!condition)*falseValue)
}

#' sebkc: Surface Energy Balance and Crop Coefficient Evapotranspiration
#' Estimation
#'
#' Computes and integrates surface energy balance components of
#' evapotranspiration into the FAO56 water balance model, providing the
#' SEBAL, METRIC, SEBI, SSEB, SEBS and TSEB models.
#'
#' Generics that the 'raster' package also provides as methods (such as plot,
#' hist, quantile, text, lines and points) are intentionally not imported
#' here, because they are made available through import(raster); importing
#' them again would trigger "replacing previous import" warnings.
#'
#' @keywords internal
#' @importFrom stats na.omit kmeans lm coef cor pf
#' @importFrom graphics abline axis legend par
#' @importFrom grDevices dev.new
#' @importFrom utils read.csv read.table read.delim write.csv write.table data
"_PACKAGE"

#' @title Reprojecting and cropping images
#' @description This function returns the subset of x object as defined by y extent object. 
#' See \code{\link[raster]{crop}} and \code{\link[raster]{extent}} for more details.
#' The difference between \code{\link[raster]{crop}}  and cropsebkc is the ability for the
#' latter to reproject y object with x object projection.
#'
#' @param x object to be extracted
#' @param y extent object or raster object or polygon from which an Extent 
#'  object can be extracted. A polygon or raster will be reprojected to 
#'  conform x
#' @author George Owusu
#'
#' @return cropped  x and reprojected y
#' @seealso  \code{\link[raster]{crop}} 
#' @export
#'
#' @examples
#' \dontrun{
#' folder=system.file("extdata","stack",package="sebkc")
#' stack=sebkcstack(folder=folder)
#' data=cropsebkc(stack$data,stack$data)
#' }
#' 
cropsebkc<-function(x,y){
if(is.numeric(y)){
  # extent=y
  raster1y=crop(x, y)
  yproj=y
  #print("yes")
}else{
  if(inherits(y, "RasterLayer")){
    xy=xyFromCell(y, 1) 
    if(xy[[1]]<300||xy[[2]]<300){
      projection(y) <- CRS("+init=epsg:4326") # WGS 84
      CRS.new=CRS( proj4string(x))
      yproj=projectRaster(y, crs=CRS.new)
      
    }
    else{
      CRS.new=CRS( proj4string(x))
      yproj=projectRaster(y, crs=CRS.new) 
    }
    raster1y=crop(x, yproj)
  }else{
    #extent
    if(inherits(y, "Extent")){
      # print("Yes")
      raster1y=crop(x, y)
      yproj=y
      
      
    }else{
      
      
      #polygons functions here
      xy=coordinates(y)
      if(xy[[1]]<300||xy[[2]]<300){
        projection(y) <- CRS("+init=epsg:4326") # WGS 84
        CRS.new=CRS( proj4string(x))
        yproj <- spTransform(y, CRS.new)
        
      }else{
        projection(y) <- proj4string(x)
        yproj=y 
        
      }
      
      raster1y=crop(x, yproj)
      
    }
  }
  
}
  list(x=raster1y,y=yproj)
}

#' @title SEBI Evapotranspiration Fraction Estimation in R
#'
#' @description \code{sebi} computes Evapotranspiration Fraction Estimation
#' based on Menenti& Choudhury (1993) parameterization of 
#' land surface evaporation by means of location dependent potential 
#' evaporation and surface temperature range. 
#' @inheritParams sebal
#' @inheritParams ETo
#' 
#' @details Function \code{\link{ETo}} will be automatically used to 
#' estimate  parameters y (psychrometric constant), 
#' slope (slope of saturation vapour pressure curve) and 
#' vpd (Vapour pressure deficit).
#' @references 
#' Menenti, M., & Choudhury, B. 1993. Parameterization of land surface 
#' evaporation by means of location dependent potential evaporation and 
#' surface temperature range. Paper presented at the Proceedings of 
#' IAHS conference on Land Surface Processes.
#' @return EF Evapotranspiration Fraction
#' @author George Owusu
#'
#' @examples
#' \dontrun{
#' #using folder parameter 
#' folder=system.file("extdata","stack",package="sebkc")
#' sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28)
#' 
#' #manual input of parameters
#' #data
#' albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
#' Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
#' NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#' LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
#' #model
#' modsebi=sebi(albedo=albedo,Ts=Ts,Tmax=31,Tmin=28,RHmax=84,
#' RHmin=63,NDVI=NDVI,SAVI=NULL,iter.max=7,xyhot="full",
#' xycold="full",DOY=37,sunelev=50.71154048,welev=317.1,zx=10,
#' u=2,zomw=2,zom=NULL,LAI=LAI,Rn24=NULL,model="sebi")
#' 
#' 
#' #Another interactive example
#' #get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
#' welev=278
#' data=landsat578(rawdata,welev=welev)
#'  #perform semi-auto simulation 
#' #Determine xyhot. Digitize polygon on the Ts map
#' modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#' #determine the cold. Digitize polygon on the Ts map
#' modcold=coldTs(data,welev=275,extent="auto",cluster=2)
#' xyhot=modhot$xyhot
#' xycold=modcold$xycold
#' #use object of \code{\link{coldTs}} or \code{\link{hotTs}} in different ways
#' modsebi=sebi(data,xyhot=modhot,xycold=xycold,welev=welev,
#' Tmax=Tmax,Tmin=Tmin,RHmax=RHmax,RHmin=RHmin)
#' }
#' @seealso \code{\link{sebal}} , \code{\link{ETo}}, \code{\link{sebs}}  
#' @export
#' @rdname sebi 

sebi=function(albedo,Ts,Tmax,Tmin,RHmax=NULL,RHmin=NULL,latitude=NULL,
      NDVI,SAVI,iter.max=7,xyhot="auto",xycold="auto",DOY,
      sunelev,welev,zx=10,u=2,zomw=2,zom=NULL,LAI=NULL,DEM=NULL,
lapse=0.0065,Rn24=NULL,wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",
model="sebi",clip=NULL,folder=NULL) UseMethod ("sebi")
#' @export
#' @rdname sebi 
sebi.default=function(albedo=NULL,Ts,Tmax,Tmin,RHmax=NULL,RHmin=NULL,
latitude=NULL,NDVI,SAVI,iter.max=7,xyhot="auto",xycold="auto",DOY,
sunelev,welev,zx=10,u=2,zomw=2,zom=NULL,
LAI=NULL,DEM=NULL,lapse=0.0065,Rn24=NULL,wmo=NULL, airport=NULL,Krs=0.16,surface = "grass",
model=model,clip=NULL,folder=NULL){

  if(is.null(folder)){
    folder="/nothing454782ghf7poi8r.hope"
  }
  
  if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
    file.info2=TRUE
  }else{
    file.info2=file.info(folder)[["isdir"]]
  }
  if(file.info2==TRUE&&!is.na(file.info2)){
    if(is.null(welev)){
      return(message("Please provide the parameter welev: 
                   the elavation of th weather station"))  
    }
    
    if(inherits(folder, "landsat578")||inherits(albedo, "landsat578")){
      if(inherits(albedo, "landsat578")){
        mod=albedo
      }else{
        mod=folder
      }
    }else{
      mod=landsat578(folder=folder, welev=welev,clip=clip)
      folder=mod
    }
    #mod=landsat578(data=folder, welev=welev)
    albedo=mod$albedo
    Ts=mod$Ts
    NDVI=mod$NDVI
    DOY=mod$DOY
    sunelev=mod$sunelev
    SAVI=mod$SAVI
    }
  PET=NULL
  
  if(!is.null(wmo)||!is.null(airport)){
    mod=sebkc.tryCatch(mod)$value
    thisDOY=sebkc.tryCatch(mod$date)$value
    if(inherits(thisDOY, "simpleError")){
      thisDOY=DOY 
    }
    ETr24=sebkc.tryCatch(ETo(DOY=thisDOY,airport = airport,wmo=wmo,latitude=latitude,
                             z=zx,Krs =Krs,altitude=welev))$value
    if(inherits(ETr24, "simpleError")){
      return (message(paste("Invalid DOY [YYYY-mm-dd] or airport or wmo",ETr24))) 
    }
    
    Tmin=ETr24$data$Tmin
    Tmax=ETr24$data$Tmax
    Rn24=ETr24$Rn
    PET=ETr24
    
  }
uw=u
waterdensity=1000
p=1.03547033
cp=1004.14
g=9.81
k=0.41 #von Karman's constant
z1=0.1 
z2=2
vaporisation=2430000

mod=sebal(albedo=albedo,Ts=Ts,NDVI=NDVI,SAVI=SAVI,
iter.max=iter.max,xyhot=xyhot,xycold=xycold,
DOY=DOY,sunelev=sunelev,welev=welev,
zx=zx,u=uw,zomw=zomw,zom=zom,LAI=LAI,DEM=DEM,
lapse=lapse,Rn24=Rn24,model="SEBAL",folder=folder)

if(is.null(PET)){
PET=ETo(Tmax=Tmax,Tmin=Tmin,z=zx,uz=u,altitude=welev,
RHmax=RHmax,RHmin=RHmin,n=9.2,DOY=mod$DOY,latitude=latitude)
}
data=mod$iter.coef[nrow(mod$iter.coef),]
y=PET$y
vpd=PET$vpd
slope=PET$slope
dTmax=(data$rah_hot/(data$pairhot*cp))*(data$Rn_hot+data$G_hot)
dTmin=((data$rah_cold/(data$paircold*cp))*(data$Rn_cold+data$G_cold)-(1/y)*vpd)/(1+(slope/y))
message("computing EF" )
dT=(mod$H*mod$rah)/((mod$pair*cp))
EF=1-(((dT/mod$rah)-(dTmin/data$rah_cold))/((dTmax/data$rah_hot)-(dTmin/data$rah_cold)))
EF[EF<0,]=0
EF[EF>1.1,]=1.1
ET24=NULL
if(!is.null(Rn24)){
  message("computing ETa" )
  
  ET24=EF*Rn24*0.035
}

factor<-list(EF=EF,ET24=ET24,folder=folder)
factor$call<-match.call()
class(factor)<-"sebi"
factor
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsebstack.R
\name{sebkcstack}
\alias{sebkcstack}
\alias{sebkcstack.default}
\title{Stacking, gap-filling, cloud removal of Landsat image ET Model}
\usage{
sebkcstack(
  folder,
  meta_pattern = "MTL.txt",
  remove_cloud = FALSE,
  gap_fill = FALSE,
  gap_loop = 8,
  clip = NULL
)

\method{sebkcstack}{default}(
  folder,
  meta_pattern = "MTL.txt",
  remove_cloud = FALSE,
  gap_fill = FALSE,
  gap_loop = 8,
  clip = NULL
)
}
\arguments{
\item{folder}{The folder or directory of landsat files}

\item{meta_pattern}{The regular expression pattern to identify 
metadata file. The default is "MTL.txt". The function works 
on the new landsat metadata only.}

\item{remove_cloud}{TRUE FALSE, 1, or 2. To indicate the cloud cover removal
from the image. Default is FALSE. If it is set to 1, 
the overlapping areas of  band 1 DN values between 120 and 255 and band 6
DN values of 102 to 128 are used. If it is set to 2 the union of the DN
ranges are used; and clouds as well as urban, barren, quarries, rocks, 
and sand are removed; otherwise only clouds are removed. 
See Martinuzzi et al(2009) for more details. 
At the moment only landsat 5 and 7 clouds are removed.}

\item{gap_fill}{Logical (TRUE or FALSE).It replaces strips in 
landsat 7 or NAs in other data set. It uses  \code{\link[raster]{focal}} 
neighbourhood approach. It is an automated Version of USGS "Gap-Filling Landsat
 7 SLC-off Single Scenes Using Erdas Imagine TM". Like USGS, it also takes 
 approximately one hour to complete.
 see details at http://landsat.usgs.gov/ERDAS_Approach.php}

\item{gap_loop}{numeric. The number of loops or repetitions that gap-fill 
should be run. The default is 8 for one landsat scene. If the landsat image
is smaller, it can be reduced.}

\item{clip}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform the data to be cropped. for example clip can take the form of c(xmin, xmax, ymin, ymax)}
}
\value{
\describe{
\item{data:}{ raster stacked data. If crop is not NULL then  crop Extent object is returned}
\item{brescale:}{ brescale of the stacked data}
\item{grescale:}{ grescale of the stacked data}
\item{sunelev:}{ sun elevation for stacked data}
\item{date:}{ Acquired date of the raster stacked data}
\item{viewangle:}{ view angle of landsat 8}
\item{azimuth:}{ azimuth angle of the image}
\item{sensor:}{ Type of satellite}
\item{K1:}{ K1 of landsat 8 thermal band 10}
\item{K2:}{ K2 of landsat 8 thermal band 10}
\item{cc:}{ cloud cover percentage}
\item{quality:}{ Image quality}
\item{row:}{ WRS_ROW of the image }
\item{path:}{ WRS_PATH of the image}
}
}
\description{
This function prepares landsat 5, 7, and 8 data for  
\code{\link{landsat578}}. First, it reads meta data and returns many 
input parameters for \code{\link{landsat578}} and \code{\link{sebal}}.
 Second, it stacks 7 landsat layers: it stacks bands 1-7 for landsat 5 and 7; 
and bands 2-7,10 for landsat 8.. Third, it removes landsat clouds based 
 on Martinuzzi et al(2009) method. Fourth, it gap-fills landsat 7 strips 
 due to SLC-off as well as gap-filling NAs in other landsat sensors. 
 It can also process any selected bands.
}
\details{
The function can simulate any of the activities described in the 
description section. It takes several minutes to replace strips (1 hour) 
and remove clouds (30 minutes) on one landsat scene. But stacking of bands 
and reading of metadata take few seconds. One can save the stacked and 
processed data in the same "folder" with \code{\link{writesebkc}}. 
The output file name is "sebkcstack.tif".
}
\examples{
\dontrun{
folder=system.file("extdata","stack",package="sebkc")
stack=sebkcstack(folder=folder)
#returns data and grescale
stack$data
stack$grescale
}
}
\references{
Martinuzzi, S., Gould, W.A., Ramos Gonzales, O.M. 2007.
Creating Cloud-Free Landsat ETM+ Data Sets in Tropical Landscapes: 
Cloud and Cloud-Shadow Removal. USDA Forest Service General Technical 
Report IITF-GTR-32.

USGS Using Landsat 7 Data http://landsat.usgs.gov/ERDAS_Approach.php
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kumasi.R
\docType{data}
\name{kumasircp262016_2050}
\alias{kumasircp262016_2050}
\title{A 2.6 RCP downscaled Weather data of Kumasi from 2016 to 2050}
\format{
A data frame with 12784 rows and 8 variables:
\describe{
  \item{DOY}{Day of the Year in dates}
  \item{Tmax}{Maximum Temperature [oC]}
  \item{Tmin}{Minimum Temperature [oC]}
  \item{RHmin}{Minimum Relative Humidity [per cent]}
  \item{RHmax}{Maximum Relative Humidity [per cent]}
  \item{n}{sunshine hours}
  \item{uz}{wind speed [m/s]}
  \item{P}{Daily precipitation [mm]}
}
}
\description{
A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
and minimum and maximum Relative Humidity. The RCPs 2.6 assumes that green house 
gas will peak in between 2010-2020 and declines throughout 21st Century.
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETohr.R
\name{ETohr}
\alias{ETohr}
\alias{ETohr.default}
\title{Hourly FAO-56 and ASCE-EWRI Reference Evapotranspiration}
\usage{
ETohr(
  Tmean,
  RH,
  DOY,
  Lz,
  t1,
  time,
  Lm,
  latitude,
  longitude = NULL,
  n = NULL,
  uz = 2,
  Rs = NULL,
  Rn = NULL,
  G = NULL,
  altitude = NULL,
  z = NULL,
  Krs = 0.16,
  Tmax = NULL,
  Tmin = NULL,
  albedo = 0.23,
  as = 0.25,
  bs = 0.5,
  ETins = NULL,
  ETr24 = NULL,
  wmo = NULL,
  airport = NULL,
  map = NULL,
  surface = "grass",
  period = "daytime"
)

\method{ETohr}{default}(
  Tmean,
  RH,
  DOY,
  Lz,
  t1 = 1,
  time,
  Lm,
  latitude,
  longitude = NULL,
  n = NULL,
  uz = 2,
  Rs = NULL,
  Rn = NULL,
  G = NULL,
  altitude = NULL,
  z = NULL,
  Krs = 0.16,
  Tmax = NULL,
  Tmin = NULL,
  albedo = 0.23,
  as = 0.25,
  bs = 0.5,
  ETins = NULL,
  ETr24 = NULL,
  wmo = NULL,
  airport = NULL,
  map = NULL,
  surface = "grass",
  period = "daytime"
)
}
\arguments{
\item{Tmean}{Numeric. Hourly Mean Air Temperature in degree Celsius}

\item{RH}{Hourly Mean Relative Humidity in percent}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{Lz}{longitude of the centre of the local time zone [degrees west of Greenwich].
MUST BE POSITIVE
Lz = 0 degrees for Greenwich.}

\item{t1}{The length of the calculation period in hour; 1 for hour, 0.5 for 30 minutes
0.25 for 15 minutes.}

\item{time}{The midpoint of the time of measurement[hour]; for example
time is 12.5 for a period between 12:00 and 13:00}

\item{Lm}{longitude of the measurement site [degrees west of Greenwich]. MUST BE POSITIVE}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{n}{Sunshine hours}

\item{uz}{The wind speed in m/s}

\item{Rs}{solar radiation [MJ/m2/hr]}

\item{Rn}{net radiation [MJ/m2/hr]}

\item{G}{Daily Soil heat flux in W/m2}

\item{altitude}{The elevation of the weather station, in metres}

\item{z}{The height above surface where wind speed was measured, in metres.}

\item{Krs}{Relationship between the fraction of extraterrestrial radiation that reaches the 
earth's surface, Rs/Ra, and the air temperature difference Tmax - Tmin for
interior (Krs = 0.16) and coastal (Krs = 0.19) regions}

\item{Tmax}{Numeric. Maximum air Temperature in degree Celsius}

\item{Tmin}{Numeric. Minimum air Temperature in degree Celsius}

\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{as}{Regression constant, intercept, expressing the fraction of extra-terrestrial 
radiation reaching the earth on overcast days (n = 0); 
either a calibrated value or as = 0.25 can be used.}

\item{bs}{regression constant, slope, expressing 
the fraction of extraterrestrial radiation
reaching the earth on overcast days (n = 0); either a calibrated value or bs = 0.5 can be used.}

\item{ETins}{model. It takes ET model from \code{\link{sebal}} or \code{\link{tseb}} and
retrieve instantaneous ET estimates. This helps to estimate ET24, T24, E24, EF, Kc, EFs, EFc}

\item{ETr24}{daily grass or alfalfa reference Evapotranspiration (mm/day)}

\item{wmo}{numeric. World Meteorological Organization weather station code.
You can use the  Worldwide Station List at 
\url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
\url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]}

\item{airport}{numeric. IATA or ICAO code. They are alphabetically  listed at 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]}

\item{map}{spatial raster data for inverse distance interpolation}

\item{surface}{character. It is either "grass" or "alfalfa"}

\item{period}{character. Time of day for the calculation: "daytime" or "nighttime". The default is "daytime".}
}
\value{
\describe{
\item{ETo:}{hour reference ETo [mm/hr]}
\item{ETa:}{hour actual ET. It is not NULL if EF has a numeric value}
\item{ETc:}{hour crop ET. It is not NULL if Kc has a numeric value}
\item{Rn:}{ Net  Radiation  [W/m2] }
\item{Rnwm_2:}{ Net Radiation [MJ/m2/hr] }
\item{y:}{ psychrometric constant [kPa/degree Celsius]}
\item{slope:}{ slope of saturation vapour pressure 
curve [kPa/degree Celsius]}
\item{vpd:}{ Vapour pressure deficit[kPa]}
}
}
\description{
The function calculates hourly or less than hour FAO-56 (grass) and 
ASCE-EWRI Reference (alfalfa) Evapotranspiration (ETo ETr). 
The "surface" parameter determines #' the type of ET. 
The function can also be used to scale hourly or less than hour actual evapotranspiration.
The daily ET is implemented in \code{\link{ETo}}.
}
\details{
\code{\link{sebal}}, \code{\link{sebi}},\code{\link{sebs}},\code{\link{sseb}},
\code{\link{tseb}}
}
\examples{
\donttest{
#FAO56 Example 19 14:00-15:00 h using Cd = 0.24 instead 0.34 in FAO56
ET1hr=ETohr(Tmean=38,RH=52,DOY=274,Lz=15,t1=1,time=14.5,Lm=16.25,latitude=16.22,uz=3.3,Rs=2.450)
ET1hr$ETo
}
}
\references{
\itemize{ 
\item ALLEN, R. G., PEREIRA, L. S., RAES, D., & SMITH, M. 1998. 
Crop Evapotranspiration (guidelines for computing crop water requirements) 
FAO Irrigation and Drainage Paper No. 56: FAO.

\item Jackson, R. D., Hatfield, J. L., Reginato, R. J., Idso, S. B., & Pinter Jr, P. J. (1983). 
Estimation of daily evapotranspiration from one time-of-day measurements. 
Agricultural Water Management, 7(1-3), 351-362. doi: http://dx.doi.org/10.1016/0378-3774(83)90095-1

\item French, A. N., Fitzgerald, G., Hunsaker, D., Barnes, E., Clarke, T., Lesch, S., . . .
 Pinter, P. (2005). 
Estimating spatially distributed cotton water use from thermal infrared aerial imagery. 
Paper presented at the World Water Congress 2005: 
Impacts of Global Climate Change - Proceedings of the 2005 World Water and 
Environmental Resources Congress, Reston, Va.

\item Colaizzi, P. D., Kustas, W. P., Anderson, M. C., Agam, N., Tolk, J. A., 
Evett, S. R., . . . O'Shaughnessy, S. A. (2012). Two-source energy balance model
 estimates of evapotranspiration using component and composite surface temperatures. 
 Advances in Water Resources, 50, 134-151. 
 doi: http://dx.doi.org/10.1016/j.advwatres.2012.06.004
 }
}
\seealso{
\code{\link{ETo}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sebkcerror.R
\name{sebkc.tryCatch}
\alias{sebkc.tryCatch}
\title{Internal function to Catch  errors and warnings}
\usage{
sebkc.tryCatch(expr)
}
\arguments{
\item{expr}{an  R expression to evaluate}
}
\value{
value a list with 'value' and 'warning', where value may be an error caught
}
\description{
Internal function to Catch  errors and warnings
}
\examples{
thiserror=str( sebkc.tryCatch( log( "a" ) ) )$value
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xyextract.R
\name{mapmatrix}
\alias{mapmatrix}
\title{Internal function for doing interpolation}
\usage{
mapmatrix(xydata, i, map)
}
\arguments{
\item{xydata}{x and y data}

\item{i}{index}

\item{map}{geographical map}
}
\value{
map
}
\description{
Internal function for doing interpolation
}
\examples{
\dontrun{
folder=system.file("extdata","stack",package="sebkc")
sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28)
points=sampleRandom(sebiauto$EF,100,sp=TRUE)
pt=cbind(points@coords,points@data)
longitude=pt$x
latitude=pt$y
value=pt$layer
mapmatrix=mapmatrix(xydata,2)
map=invdist(longitude,latitude,var=value,ext=sebiauto$EF)
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hotTs2.R
\name{hotTs2}
\alias{hotTs2}
\alias{hotTs2.default}
\title{Automatic computation of hot pixel Radiometric Surface Temperature (Ts)}
\usage{
hotTs2(
  Ts,
  NDVI,
  albedo = NULL,
  sunangle = NULL,
  DEM = NULL,
  cluster = 10,
  extent = "interactive",
  upper = 0.95,
  lower = 0.8,
  NDVI.probs = 0.1,
  plot = TRUE,
  layout = "portrait",
  draw = "poly",
  folder = NULL,
  welev = NULL
)

\method{hotTs2}{default}(
  Ts = NULL,
  NDVI,
  albedo = NULL,
  sunangle = NULL,
  DEM = NULL,
  cluster = 10,
  extent = "interactive",
  upper = 0.95,
  lower = 0.8,
  NDVI.probs = 0.2,
  plot = TRUE,
  layout = "portrait",
  draw = "poly",
  folder = NULL,
  welev = NULL
)
}
\arguments{
\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{NDVI}{A RasterLayer data that indicates NDVI 
(Normalized Difference Vegetation Index) values. You can also point to a 
file on your computer.}

\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{sunangle}{sun angle (degree) above the horizon.}

\item{DEM}{A digital elevation model[m]}

\item{cluster}{numeric, a value indicating the number of 
clusters.}

\item{extent}{A subset of the raster file than include 
c(xmin, xmax; serastercond row: ymin, ymax). You can set it to "interactive"
where you can digitise polygon. You can set to "auto" where it will be
automatically generated. See layout parameter below for more details.
see \code{\link[raster]{extent}}}

\item{upper}{The quantile probability at which Ts should be included 
in the selection project. It ranges from 0-1. The default is 0.8}

\item{lower}{The quantile probability at which NDVI should be included 
in the slection project.The default is 0.2}

\item{NDVI.probs}{The probability of selecting quantile NDVI that is above the threshold. 
The value of 0.6 means only NDVI values that is above 60% quantile will be included in
the analyses.}

\item{plot}{logical:TRUE or FALSE. To indicate if histograms or triangle 
should be plotted}

\item{layout}{character. It takes "portrait" or "landscape". If extent is set to 
"auto" the middle part of the image will be used so that NAs can be 
avoided. You indicate whether the automatic selection should indicate
west-east (landscape) or north-south direction (portrait)}

\item{draw}{interactive. The method of digitisation if extent is set "auto". 
It takes "poly" or "rect"}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}

\item{welev}{Weather station elevation in meters}
}
\value{
\describe{
\item{Ts:}{ The selected or input Ts}
\item{Tshot:}{ The Temperature of hot pixel}
\item{TX:}{ The average temperature of the hot pixels}
\item{x:}{ The x cordinate of the hot pixel}
\item{y:}{ The y cordinate of the hot pixel}
\item{xyhot: }{combined x and y coordinates}
\item{candidates:}{ The similar candidates' pixels that can be used}
}
}
\description{
This function computes temperature of a hot pixel based on NDVI, 
Surface Temperature, DEM and albedo. It plots histograms indicating
the selected values
}
\details{
The function first divides the selected area into a 
number of clusters. The mean and standard deviations are
computed. The cluster means with the lowest standard deviations
and qualifying for upper (Ts) and lower (NDVI) limits 
(see above) are selected. The provided upper and lower probabilities
 are also used to select the final Ts.
}
\examples{
\dontrun{
 folder=system.file("extdata","stack",package="sebkc")
  modhot=hotTs2(folder=folder,welev=170,extent="auto",cluster=3)
  
   #use object of  landsat578
folder=system.file("extdata","stack",package="sebkc")
data=landsat578(data=folder, welev=170)
modfhot=hotTs2(folder=data,welev=170,extent="auto",cluster=3)

#Or Ts = data
modhotTs2=hotTs2(data,welev=170,extent="auto",cluster=3)
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#simulate  with default parameters without albedo
modhot=hotTs2(Ts=Ts,NDVI=NDVI,extent="auto",cluster=3)
#simulate with albedo but set extent to "auto" and cluster to 7
modhot=hotTs2(Ts=Ts,NDVI=NDVI,albedo=albedo,cluster=7,
extent="auto")
}
}
\references{
Allen, R. G., Burnett, B., Kramber, W., Huntington, J., 
Kjaersgaard, J., Kilic, A., Kelly, C., & Trezza, R. 2013. Automated 
Calibration of the METRIC-Landsat Evapotranspiration Process. 
JAWRA Journal of the American Water Resources Association, 49(3): 563-576.
}
\seealso{
\code{\link{coldTs}},  \code{\link{hotTs}}, \code{\link{coldTs2}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coldTs2.R
\name{coldTs2}
\alias{coldTs2}
\alias{coldTs2.default}
\title{Optional Automatic computation of cold pixel Radiometric Surface Temperature}
\usage{
coldTs2(
  Ts,
  NDVI,
  albedo = NULL,
  sunangle = NULL,
  DEM = NULL,
  cluster = 10,
  extent = "interactive",
  upper = 0.2,
  lower = 0.1,
  NDVI.probs = 0.95,
  plot = TRUE,
  layout = "portrait",
  draw = "poly",
  folder = NULL,
  welev = NULL
)

\method{coldTs2}{default}(
  Ts = NULL,
  NDVI,
  albedo = NULL,
  sunangle = NULL,
  DEM = NULL,
  cluster = 10,
  extent = "interactive",
  upper = 0.2,
  lower = 0.1,
  NDVI.probs = 0.6,
  plot = TRUE,
  layout = "portrait",
  draw = "poly",
  folder = NULL,
  welev = NULL
)
}
\arguments{
\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{NDVI}{A RasterLayer data that indicates NDVI 
(Normalized Difference Vegetation Index) values. You can also point to a 
file on your computer.}

\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{sunangle}{sun angle (degree) above the horizon.}

\item{DEM}{A digital elevation model[m]}

\item{cluster}{numeric, a value indicating the number of 
clusters.}

\item{extent}{A subset of the raster file than include 
c(xmin, xmax; serastercond row: ymin, ymax). You can set it to "interactive"
where you can digitise polygon. You can set to "auto" where it will be
automatically generated. See layout parameter below for more details.
see \code{\link[raster]{extent}}}

\item{upper}{The quantile probability at which NDVI should be included 
in the selection project. It ranges from 0-1. The default is 0.95}

\item{lower}{The quantile probability at which Ts should be included 
in the selection project.The default is 0.2}

\item{NDVI.probs}{The probability of selecting quantile NDVI that is above the threshold. 
The value of 0.6 means only NDVI values that is above 60% quantile will be included in
the analyses.}

\item{plot}{logical:TRUE or FALSE. To indicate if histograms or triangle 
should be plotted}

\item{layout}{character. It takes "portrait" or "landscape". If extent is set to 
"auto" the middle part of the image will be used so that NAs can be 
avoided. You indicate whether the automatic selection should indicate
west-east (landscape) or north-south direction (portrait)}

\item{draw}{interactive. The method of digitisation if extent is set "auto". 
It takes "poly" or "rect"}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}

\item{welev}{Weather station elevation in meters}
}
\value{
\describe{
\item{Ts:}{ The selected or input Ts [K]}
\item{Tscold:}{ The Temperature of cold pixel [K] }
\item{TC:}{ The average temperature of the cold pixels [K]}
\item{x:}{ The x coordinate  of the cold pixel}
\item{y:}{ The y coordinate  of the cold pixel}
\item{xycold: }{combined x and y coordinates}
\item{candidates:}{ The similar candidates' pixels that can be used}
}
}
\description{
This function computes temperature of a cold pixel based on NDVI, 
Surface Temperature, DEM and albedo. The main difference with \code{\link{coldTs}} is that preference 
is given to temperature selection before considering NDVI and albedo selection. Moreover, this
is a simple end member selection where no clustering is done.
}
\details{
The function first divides the selected area into a  
number of clusters. The mean and standard deviations are
computed. The cluster means with the lowest standard devaitions 
and qualifying for upper (NDVI) and lower (Ts) limits 
(see above) are selected. The provided upper and lower probabilities
 are also used to select the final Ts.
}
\examples{
\dontrun{
#using landsat folder
folder=system.file("extdata","stack",package="sebkc")
modcold=coldTs2(folder=folder,welev=170,extent="auto")

#use object of  
folder=system.file("extdata","stack",package="sebkc")
data=landsat578(data=folder, welev=362)
modcold=coldTs2(folder=data,welev=170,extent="auto",cluster=3)

modcoldTs2=coldTs2(data,welev=170,extent="auto",cluster=3)

#using input different input data
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#simulate with default parameters without albedo
modcold=coldTs2(Ts=Ts,NDVI=NDVI,sunangle=50.7)
#simulate with albedo but set extent to "auto" and cluster to 7
modcold=coldTs2(Ts=Ts,NDVI=NDVI,albedo=albedo,sunangle=50,cluster=7,
extent="auto")
}
}
\references{
Allen, R. G., Burnett, B., Kramber, W., Huntington, J., 
Kjaersgaard, J., Kilic, A., Kelly, C., & Trezza, R. 2013. Automated 
Calibration of the METRIC-Landsat Evapotranspiration Process. 
JAWRA Journal of the American Water Resources Association, 49(3): 563-576.
}
\seealso{
\code{\link{coldTs}},  \code{\link{hotTs}}, \code{\link{hotTs2}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cropsebkc.R
\name{cropsebkc}
\alias{cropsebkc}
\title{Reprojecting and cropping images}
\usage{
cropsebkc(x, y)
}
\arguments{
\item{x}{object to be extracted}

\item{y}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform x}
}
\value{
cropped  x and reprojected y
}
\description{
This function returns the subset of x object as defined by y extent object. 
See \code{\link[raster]{crop}} and \code{\link[raster]{extent}} for more details.
The difference between \code{\link[raster]{crop}}  and cropsebkc is the ability for the
latter to reproject y object with x object projection.
}
\examples{
\dontrun{
folder=system.file("extdata","stack",package="sebkc")
stack=sebkcstack(folder=folder)
data=cropsebkc(stack$data,stack$data)
}

}
\seealso{
\code{\link[raster]{crop}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WDI.R
\name{wdi}
\alias{wdi}
\title{Water Deficit Index (WDI)}
\usage{
wdi(
  Ts = NULL,
  Ta,
  NDVI,
  albedo = NULL,
  Tsmax = "auto",
  Tsmin = "auto",
  sunelev = NULL,
  folder = NULL,
  welev = NULL,
  Rn24 = NULL
)

wdi(
  Ts = NULL,
  Ta,
  NDVI,
  albedo = NULL,
  Tsmax = "auto",
  Tsmin = "auto",
  sunelev = NULL,
  folder = NULL,
  welev = NULL,
  Rn24 = NULL
)
}
\arguments{
\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{Ta}{numeric. Air temperature. If not available you can use}

\item{NDVI}{A RasterLayer data that indicates NDVI 
(Normalized Difference Vegetation Index) values. You can also point to a 
file on your computer.}

\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{Tsmax}{numeric. Maximum sensor surface temperature [K]}

\item{Tsmin}{numeric. Minimum sensor surface temperature [K]
\code{\link{sebal}} to get pixel based Ta.}

\item{sunelev}{Angle of Sun elevation in degrees, 
as found in the meta data of the satellite image}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}

\item{welev}{Weather station elevation in meters}

\item{Rn24}{A 24 hour net radiation [W/m2]. It is needed to estimate daily ET. 
It can be computed with \code{\link{ETo}} with even a minimal data of 
Tmax and Tmin. 
 You can also point to raster file on your computer}
}
\value{
\describe{
\item{EF:}{ standardised wdi excluding negatives and one plus}
\item{index:}{ raw wdi including negatives and more than one}
}
}
\description{
Moran et. al (1994)  
Estimation of crop water deficit using surface-air 
temperature and spectral vegetation index.
}
\details{
Note that surface temperature (Tsmin and Tsmax)
 are different from air temperature (Tmin and Tmax) in
\code{\link{ETo}}, \code{\link{sseb}}, \code{\link{sebs}}

You can get Ta from  \code{\link{sebal}} Ta
}
\examples{
 \dontrun{
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
modWDI=wdi(Ts=Ts,Ta=299,NDVI=NDVI,albedo=NULL,Tsmax="auto",
Tsmin="auto",sunelev=50)
plot(modWDI$EF)

#Using folder parameter
folder=system.file("extdata","stack",package="sebkc")
wdiauto=wdi(folder=folder,welev=317,Tsmax=31,Tsmin=28,Ta=290)

#' #Another interactive example
#get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
welev=278
data=landsat578(rawdata,welev=welev)
#perform semi-auto simulation 
#Determine xyhot. Digitize polygon on the Ts map
modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#determine the cold. Digitize polygon on the Ts map
modcold=coldTs(data,welev=275,extent="auto",cluster=2)
Tsmax=modhot$Tshot
Tsmin=modcold$Tscold
#use object of \code{\link{coldTs}} or \code{\link{hotTs}} in different ways
modWDI2=wdi(data,Ta=299,Tsmax=Tsmax,Tsmin=modcold,sunelev=50)
plot(modWDI2$EF)
}
}
\references{
Moran, M. S., Clarke, T. R., Inoue, Y., & Vidal, A. 1994.
Estimating crop water deficit using the relation between surface-air 
temperature and spectral vegetation index. Remote Sensing of Environment,
 49(3): 246-263.
}
\seealso{
\code{\link{sseb}} and \code{\link{sebi}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kc.R
\name{kc}
\alias{kc}
\alias{kc.default}
\alias{plot.kc}
\alias{fit.kc}
\title{Estimating FAO56 Single and Dual Crop coefficients with a water balance model in R}
\usage{
kc(
  ETo,
  P,
  RHmin,
  soil = "Sandy Loam",
  crop,
  I = 0,
  CNII = 67,
  u2 = 2,
  FC = NULL,
  WP = NULL,
  h = NULL,
  Zr = NULL,
  Ze = 0.1,
  kc = NULL,
  p = NULL,
  lengths = NULL,
  fw = 0.8,
  fc = NULL,
  kctype = "dual",
  region = NULL,
  regexpr = "sweet",
  initgw = 50,
  LAI = 3,
  rc = NULL,
  Kb = 0,
  Rn = NULL,
  hmodel = "gaussian",
  Zrmodel = "linear",
  xydata = NULL,
  REW = NULL,
  a1 = 360,
  b1 = -0.17,
  a2 = 240,
  b2 = -0.27,
  a3 = -1.3,
  b3 = 6.6,
  a4 = 4.6,
  b4 = -0.65,
  report = "stages",
  time = "06:00",
  HWR = 1,
  latitude = NULL,
  density = "full",
  DOY = NULL,
  slope = NULL,
  y = NULL,
  r1 = 100,
  nongrowing = "weeds",
  theta = NULL,
  profile = "variable",
  Kcmin = 0.15,
  Ky = NULL
)

\method{kc}{default}(
  ETo,
  P,
  RHmin,
  soil = "Sandy Loam",
  crop,
  I = 0,
  CNII = 67,
  u2 = 2,
  FC = NULL,
  WP = NULL,
  h = NULL,
  Zr = NULL,
  Ze = 0.1,
  kc = NULL,
  p = NULL,
  lengths = NULL,
  fw = 1,
  fc = NULL,
  kctype = "dual",
  region = NULL,
  regexpr = "sweet",
  initgw = 50,
  LAI = 3,
  rc = NULL,
  Kb = 0,
  Rn = NULL,
  hmodel = "gaussian",
  Zrmodel = "linear",
  xydata = NULL,
  REW = NULL,
  a1 = 360,
  b1 = -0.17,
  a2 = 240,
  b2 = -0.27,
  a3 = -1.3,
  b3 = 6.6,
  a4 = 4.6,
  b4 = -0.65,
  report = "stages",
  time = "06:00",
  HWR = 1,
  latitude = NULL,
  density = "full",
  DOY = NULL,
  slope = NULL,
  y = NULL,
  r1 = 100,
  nongrowing = "weeds",
  theta = NULL,
  profile = "variable",
  Kcmin = 0.15,
  Ky = NULL
)

\method{plot}{kc}(
  x,
  output = "AW_measured",
  main = NULL,
  cex = 1,
  legend = TRUE,
  pos = "top",
  horiz = FALSE,
  ...
)

fit.kc(x, output = "AW_measured")
}
\arguments{
\item{ETo}{Numeric. Reference evapotranspiration [mm]. This can be estimated with \code{\link{ETo}}}

\item{P}{numeric. Precipitation [mm]}

\item{RHmin}{numeric. Minimum Relative Humidity [per cent]}

\item{soil}{character. Name of soil texture as written in FAO56 TABLE 19}

\item{crop}{character. Name of the crop as written on 
FAO56 Table 11, TABLE 12 and TABLE 17}

\item{I}{numeric. Irrigation [mm]}

\item{CNII}{numeric. The initial Curve Number of the study site. 
This can be calibrated with \code{cal.kc}.}

\item{u2}{numeric. Wind speed of the nearby weather station [m/s] measured at 2m}

\item{FC}{numeric. soil water content at field capacity [-][m3 m-3]. 
This can be calibrated with \code{cal.kc}.}

\item{WP}{numeric. soil water content at wilting point [-][m3 m-3]. 
This can be calibrated with \code{cal.kc}.}

\item{h}{numeric. Maximum crop height [m]. See details below.
This can be calibrated with \code{cal.kc}}

\item{Zr}{numeric. Rooting depth.  See details below.
This can be calibrated with \code{cal.kc}}

\item{Ze}{numeric. The depth of the surface soil layer that is subject to drying 
by way of evaporation [0.10-0.15 m]. This can be calibrated with \code{\link{cal.kc}}}

\item{kc}{list. The tabulated Kc values for single crop c(kcini,kcmid,kcend)
or  dual crop model c(kcbini,kcbmid,kcbend). To use with \code{\link{sebal}} or 
any of the Surface Energy Balance models, it should be linked as list(model1,model2,model3) or
 c(model1$EF,model2$EF,model3$EF) but not a mixture of both. 
This can be calibrated with \code{\link{cal.kc}}}

\item{p}{The depletion coefficient. This can be calibrated with \code{\link{cal.kc}}}

\item{lengths}{list. The lengths of the crop growth stages. It can take dates in the form of 
c(start of planting ('YYYY-mm-dd), start of rapid growth, 
start of mid season, start of maturity, harvesting) 
or in the form number of days: c(initial days, dev days, mid days, late days). 
See FAO56 TABLE 11. If performing time series simulation, different dates or 
one season dates can be set; for instance two seasons simulation can be of the form 
c(initial days, dev days, mid days, late days, initial days, dev days, mid days, late days) 
or just c(initial days, dev days, mid days, late days)}

\item{fw}{list or numeric or character. This can take different fw values
throughout the growing season. It can also take one numeric value and this will
be transform depending on the type of wetting. If the wetting event is only Precipitation,
it is set to 1; if it is only Irrigation it is set to the provided fw value. If it is
both Precipitation and Irrigation weights are assigned for calculation of fw. 
fw can also take a character such as "Drip", "Sprinkler", "Basin", "Border", 
"alternated furrows", "Trickle", "narrow bed" and "wide bed". 
In this case fw values from FAO56 are assigned.}

\item{fc}{numeric. fraction of ground covered by tree canopy}

\item{kctype}{character. The type of model either "single" or "double". In spatial modelling involving
Evaporation Fraction, kctype can be set to "METRIC" or "SSEB" so that 
Actual Evapotranspiration will be estimated as ETo*EF else it will be
estimated with Rn as ETa=(EF*Rn)/2.45. Note that Rn is in MJm-2 day-1.}

\item{region}{character. The region of the crop as on FAO56 TABLE 11}

\item{regexpr}{character. Extra term that can be used to search crop names on FAO56 Tables}

\item{initgw}{numeric. Initial ground water level [m]}

\item{LAI}{numeric. Leaf Area Index}

\item{rc}{numeric Runoff coefficient [0-1]. 
This must be used if there is no information on Curve Number.
This can be calibrated with \code{\link{cal.kc}}}

\item{Kb}{ground water (base flow) recession parameter. 
The default is zero where no baseflow occurs}

\item{Rn}{Net daily radiation [w/m2]. Estimated this with  \code{\link{ETo}}}

\item{hmodel}{character. The type of crop growth model during the simulation. 
It takes "gaussian","linear", "exponential"}

\item{Zrmodel}{character. The type of root growth model during the simulation. 
It takes "gaussian","linear", "exponential"}

\item{xydata}{list c(longitude, latitude). The observation points of the model. 
This should be set only if the model is spatial.}

\item{REW}{numeric Readily Evaporable Water. It can be calibrated with \code{\link{cal.kc}}}

\item{a1}{soil water storage to maximum root depth (Zr) at field capacity}

\item{b1}{Liu et al. (2006) parameter for computing capillary rise [-0.17]. It can calibrated with
\code{\link{cal.kc}}}

\item{a2}{Liu et al. (2006) parameter for computing capillary rise [240]. It can be calibrated with
\code{\link{cal.kc}}}

\item{b2}{Liu et al. (2006) parameter for computing capillary rise [-0.27]. It can be calibrated with
\code{\link{cal.kc}}}

\item{a3}{Liu et al. (2006) parameter for computing capillary rise [-1.3]. It can be calibrated with
\code{\link{cal.kc}}}

\item{b3}{Liu et al. (2006) parameter for computing capillary rise [6.6]. It can be calibrated with
\code{\link{cal.kc}}}

\item{a4}{Liu et al. (2006) parameter for computing capillary rise [4.6]. It can be calibrated with
\code{\link{cal.kc}}}

\item{b4}{Liu et al. (2006) parameter for computing capillary rise [-0.65]. It can be calibrated with
\code{\link{cal.kc}}}

\item{report}{list. In case of spatial modelling, which days' map should be reported. 
The default is "stages" where the periods of stages maps are reported as values. 
In an ascending order it can take values such as 1:12 days or c(1,3,6:12) days.}

\item{time}{character. Time of the occurrence Irrigation or Precipitation.}

\item{HWR}{numeric. Default is 1. Height to width ratio of individual plants or groups of 
plants when viewed from the east or from the west [-].}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{density}{character. density factor for the trees, 
vines or shrubs on how they are arranged.
It can take "full", "row" or "random" or "round".}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{slope}{numeric. slope of saturation vapour pressure curve at air temperature T [kPa oC-1].}

\item{y}{numeric. psychometric constant  [kPa oC-1]}

\item{r1}{leaf resistance FOR STOMATAL CONTROL [s/m]}

\item{nongrowing}{character. The type of surface during non-growing season. 
It can take "weeds", "bare", or "mulch"}

\item{theta}{numeric. The measured soil water content [m3/m3] in the 
profile. Leave those days without measured data blank.}

\item{profile}{numeric or character. The profile depth to perform water balance.
It takes "variable" when updating the depth with root growth (default). Or taking "fixed" when
the maximum root zone is used. It can also take numeric value and this can be longer than
maximum root length.}

\item{Kcmin}{Minimum  Kc value}

\item{Ky}{list or array. yield response factor [-] in the form of 
c(initial Ky,crop dev't Ky, mid season Ky, late season Ky, total season Ky). 
For instance Maize Ky is c(0.4,0.4,1.3,0.5,1.25). Ky values for other crops
are tabulated in FAO Irrigation and Drainage Paper 33 (Doorenbos and Kassam,
1979) and Paper 56 (Allen et al., 1998).
If only one Ky value is supplied it will be repeated for all the seasons.}

\item{x}{a return object of the function.}

\item{output}{The kind of plot that should be displayed. It takes 
'AW_measured' for plotting Available and modelled soil moisture;
'TS_measured' for Total Available and modelled soil moisture;
'Kc' for displaying Kcb and Kc
'Evaporation' for displaying of Evaporation and Transpiration
'balance' for displaying Irrigation, precipitation, ETa and critical water deficit}

\item{main}{Title of the plot}

\item{cex}{the relative font size of the texts. 
Vlaues: "horiz";    logical: if TRUE, set the legend horizontally rather than vertically}

\item{legend}{logical. TRUE or FALSE. Whether a legend should be displayed or not.}

\item{pos}{the position of legend. It takes one of the following 'top', 'topright',
'topleft', 'bottom', 'bottomright' 'bottomleft', 'center'. This must be set so that
the legend should not obscure the main plot.}

\item{horiz}{logical; if TRUE the legend is drawn horizontally rather than vertically.}

\item{...}{further arguments passed to the plotting functions.}
}
\value{
The output of the function depends of the kctype and kc data type.
For single and dual crop coefficient, an "output" dataframe contains variables such as
TAW,RAW,Runoff,Ks,Kcb,h,Kc,ETc,DP,CR,Drend,Kr,De,AW,E,Ke,CN. In case kc is set to EF the function returns
Kc,Kcxy,ETc,ETcxy,Drend,Drendxy,TAW,TAWxy,RAW,RAWxy,AW,Awcxy,Ineed,Ineedxy
 \describe{
\item{output:}{  dataframe of the return values}
\item{ETc_adj:}{ The soil water stress adjusted ETc  [mm]}
\item{Ya_by_Ym:}{actual yield by potential yield}
\item{yield_decrease:}{The relative yield decrease: 1-Ya/Ym}
\item{ETdecrease:}{The relative evapotranspiration deficit :1- ETc_adj/ETc}
\item{yield_by_ET_decrease:}{he relative yield decrease: 1-Ya/Ym and 
the relative evapotranspiration deficit in one dataframe}
\item{Awcxy:}{ Available Soil water for each location at each time step [mm]}
\item{CR:}{ capillary rise [mm/day]}
\item{GW:}{ Updated ground water depth}
\item{CN:}{ Curve Number}
\item{De:}{ cumulative depth of evaporation (depletion) 
from the soil surface layer [mm] at the end of the day [mm/day]}
\item{DP:}{ Deep Percolation [mm/day]}
\item{Drend:}{ cumulative depth of evapotranspiration (depletion) 
from the root zone at the end of the day [mm]. 
This is equal to the amount of water needed to bring soil water to field capacity}
\item{Drendxy:}{ cumulative depth of evapotranspiration (depletion) 
from the root zone at the end of the day [mm] for each xydata location at each time step. 
This is equal to the amount of water needed to bring soil water to field capacity}
\item{E:}{ evaporation [mm/day]}
\item{ETc:}{ crop evapotranspiration [mm/day]. 
If water stress coefficient (Ks) is less than 1, 
ETc is crop evapotranspiration under non-standard conditions else ETc is
crop evapotranspiration under standard conditions}
\item{ETcxy:}{ crop evapotranspiration [mm/day] for each location at each time step.
crop evapotranspiration under standard conditions}
\item{h:}{ crop height [m]}
\item{Ineed:}{ Amount of soil water that is below maximum allowed depletion [mm]}
\item{Ineedxy:}{ Amount of soil water that is below maximum allowed depletion 
for each xydata location at each time step [mm] }
\item{Kc:}{ standard crop coefficient [-]}
\item{Kc_adj:}{ adjusted crop coefficient [-]}
\item{EF:}{ Evaporation Fraction [-]}
\item{EFxy:}{ Evaporation Fraction for each monitored location [-]}
\item{kcb:}{ basal crop coefficient for the growing season [-]}
\item{Kcxy:}{ crop coefficient for each location at each time step [-]}
\item{Ke:}{ soil evaporation coefficient}
\item{Kr: }{ soil evaporation reduction coefficient [-]}
\item{Ks:}{ Water stress coefficient [-]}
\item{RAW: }{ Readily Available Water [mm]}
\item{RAwcxy:}{ Readily Available Soil water for each xydata location at each time step [mm]}
\item{Runoff:}{ Runoff [mm/day]}
\item{TAW:}{ Total Available Water [mm] }
\item{TAwcxy:}{ Total Available Soil water for each location at each time step [mm]}
}
}
\description{
This function computes FAO56 crop coefficients and water balance. 
The output of the model includes crop evapotranspiration, water stress coefficient,
irrigation requirements, soil available water, etc. In addition to point estimation,
spatial modelling is integrated using Evaporation fractions from 
\code{\link{sebal}}, \code{\link{sebi}}, \code{\link{ETo}},  \code{\link{sseb}},  
\code{\link{sebs}}. \code{cal.kc} can be used to calibrate  the model.
}
\details{
\describe{
 \item{\strong{Using Existing Tables}}{The function include backend database of FAO Tables.
 The user can use the parameter values such as Zr, h, p, FC, WP, etc from FAO56 tables. 
 Most of the parameters with NULL values can take default values from FAO56 Tables. When a crop name results in 3 crops,
  the user can differentiate  that with  regexpr parameter. For example, 
  a crop parameter with a name "maize" can match
  two crops on FAO Table 12. Including  regexpr parameter with "sweet" with match only one. 
  }
 \item{\strong{Modelling crop height and Rooting Depth}}{ 
 At each time step a crop height or root depth is estimated. 
 The crop  or rooting depth growth model can take  exponential model of the form
 hday=0+0.75*max(h)*(1-exp(-day/0.75*max(days))) or    Zrday=0+0.75*Zr*(1-exp(-day/0.75*max(days)))
  The gaussian model is of the form hday=0+0.75*max(h)*(1-exp(-day^2/(0.75*max(days)^2))) or Zrday=0+0.75*(1-exp(-day^2/(0.75*max(day))^2))
  The linear model is of the form hday=(day*max(h))/max(days), Zrday=(day*Zr)/max(days)
    }
 \item{\strong{Point modeling with Single and dual crop coefficients}}{ 
 The function can estimate kc curve for the growing period by using the traditional
 kcini, kcmid and kcend and crop lengths growth stages, in case of single crop coefficients.
 Just like single crop coefficients, the dual crop coefficients kcbini, kcbmid, kcbend are
 embedded in kc parameter. The "kc" parameter must take 3 
 values as c(initial, mid, end). The values of "lengths" parameter must also be specified as
 c(initial, dev, mid, late). See example section for more details.}
  \item{\strong{Spatial modeling With Evaporation Fraction}}{ 
  Instead of supplying kcs values as described above, Evaporation Fraction (EF) from
  can be used. The corresponding EF at initital, mid amd end seasons can be supplied.
  The EF curve will be developed for each location at each time step and ETa derived 
  from (EF*Rn)/2.45.
  }
  \item{\strong{Interpolation of Input data}}{ Once EF is used instead of Kcs, one can
  provide  longitudes and latitudes in xydata parameter data frame. A 2D column data of P,
  I,ETo,RHmin, u2, Rn for each time step for each location can be provided. At each time step 
  an inverse distance interpolation will be perform to get continuous surface. In case of initial
  and static input column data such as CNII, FC, WP, initgw, Zr, h can also be interpolated.
  }
  \item{\strong{Calibration}}{A calibration of unmeasured variables can be done with \code{cal.kc}}
 }
}
\examples{
\dontrun{
#FAO56 Example 32: Calculation of the crop coefficient (Kcb + Ke) under sprinkler irrigation
FAO56Example32=kc(ETo=7,P=0,RHmin=20,soil="sandy loam",crop="Broccoli",I=10,
kctype = "dual",u2=3,kc=c(0.9,0,0),h=1,Zr=0.1)
FAO56Example32$output$fc
FAO56Example32$output$Kc_adj
#FAO56 Example 33: Calculation of the crop coefficient (Kcb + Ke) under furrow irrigation
FAO56Example33=kc(ETo=7,P=0,RHmin=20,soil="sandy loam",crop="Broccoli",I=10,
         kctype = "dual",u2=3,kc=c(0.9,0,0),h=1,Zr=0.1,fw=0.3)
#FAO56 Example 34: Calculation of the crop coefficient (Kcb + Ke) under drip irrigation
FAO56Example34=kc(ETo=7,P=0,RHmin=20,soil="sandy loam",crop="Broccoli",I=10,
           kctype = "dual",u2=3,kc=c(0.9,0,0),h=1,Zr=0.1,fw="drip")
         
file=system.file("extdata","sys","irrigation.txt",package="sebkc")
data=read.table(file,header=TRUE)  
P=data$P
rc=0
I=data$I
ETo=data$ETo
Zr=data$Zr
p=0.6
FC=0.23
WP=0.10
u2=1.6  
RHmin=35
soil="sandy loam"
crop="Broccoli"
############Single crop coefficient model###########
modsingle=kc(ETo,P=P,RHmin,soil,crop,I)
#dataframe of the return values
modsingle$output 
modsingle$output$TAW #Total Avalaible water [mm]
#Single with runoff coefficient
############Dual crop coefficient model###########
moddual=kc(ETo,P=P,RHmin,soil,crop,I,kctype = "dual",FC=FC,p=p,WP=WP,u2=u2,kc=c(0.3,0,0),Zr=Zr)
#Reproducing FAO56 Example 36
fc=(0.00667*1:12)+0.07333
h=1:12/1:12*0.3
 moddual=kc(ETo,P=P,RHmin,soil,crop,I,kctype = "dual",FC=FC,p=p,WP=WP,u2=u2,rc=0,
 kc=data$Kcb,Zr=Zr,lengths = c(4,4,2,2),h=h,hmodel="linear",fc=fc,fw=0.8)
 plot(moddual$output$Day,moddual$output$Kc_adj,type="l",ylim=c(0,1.2))
 lines(moddual$output$Day,moddual$output$Kcb,type="l", lty=2)
###########Spatial model I: Evaporation Fraction (EF) Model###################
#landsat folder with original files but a subset
folder=system.file("extdata","stack",package="sebkc") 

sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28) #sebi model
kc2=stack(sebiauto$EF/2,sebiauto$EF,sebiauto$EF/3) #assign EF to kc
modEF=kc(ETo,P=P,RHmin,soil,crop,I,kc=kc2,Rn=8,lengths = c(4,4,2,2))
spplot(modEF$EF)

#############Spatial model II: interpolating precipitation ######################
h=1:12
Zr=1:12/12
xydata=data.frame(cbind(longitude=data$longitude,latitude=data$latitude))
print(xydata) #take a look at xydata
P2=data.frame(matrix(rep(P,12),ncol=12))
print(P2) #take a look at the format of precipitation data
modEFh=kc(ETo,P=P2,RHmin,soil,crop,I,kc=kc2,Rn=8,lengths = c(4,4,2,2),h=h,Zr=Zr,xydata=xydata)
#Amount of irrigation water needed 
spplot(modEFh$Drend)
#Amount of water Irrigation that is critically needed to reach maximum allowed depletion level
modEFh$Ineedxy
ETcxy=modEFh$ETcxy ##get ETc for each location at each time step
TAW=modEFh$TAW ## 


############FAO56 Example 41: Estimation of mid-season crop coefficient for Trees or weeds ########
FAOExample41 =kc(fc=0.3,DOY=200,latitude=40,u2=1.5,h=2,density="row",lengths=c(0,0,1,0),
ETo=0,P=0,RHmin=55,soil="sand",crop="Broccoli",kctype="dual")
FAOExample41$output$Kcb

#FAO56 Example 43: Estimation of Kcb mid from ground cover with reduction for stomatal control
FAOExample43 =kc(fc=0.196,DOY=180,latitude=30,u2=2,h=5,density="round",y=0.0676,slope=0.189,
r1=420,RHmin=25,lengths=c(0,0,1,0),ETo=0,P=0,soil="sand",crop="Broccoli",kctype="dual")
FAOExample43$output$Kcb
}
}
\references{
\itemize{
\item Allen, R. G., PEREIRA, L. S., RAES, D., & SMITH, M. (1998). 
Crop Evapotranspiration (guidelines for computing crop water requirements) 
FAO Irrigation and Drainage Paper No. 56: FAO.
\item Allen, R.G., Wright, J.L., Pruitt, W.O., Pereira, L.S., Jensen, M.E., 2007. Water requirements.
In: Hoffman, G.J., Evans, R.G., Jensen, M.E., Martin, D.L., Elliot, R.L. (Eds.),
Design and Operation of Farm Irrigation Systems. , 2nd edition. ASABE, St. Joseph,
MI, pp. 208-288.
\item Liu, Y., Pereira, L.S., Fernando, R.M., 2006. Fluxes through the bottom boundary of 
the root zone in silty soils: parametric approaches to estimate groundwater 
contribution and percolation. Agric. Water Manage. 84, 27-40.
 
}
}
\seealso{
\code{\link{cal.kc}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SEBI.R
\name{sebi}
\alias{sebi}
\alias{sebi.default}
\title{SEBI Evapotranspiration Fraction Estimation in R}
\usage{
sebi(
  albedo,
  Ts,
  Tmax,
  Tmin,
  RHmax = NULL,
  RHmin = NULL,
  latitude = NULL,
  NDVI,
  SAVI,
  iter.max = 7,
  xyhot = "auto",
  xycold = "auto",
  DOY,
  sunelev,
  welev,
  zx = 10,
  u = 2,
  zomw = 2,
  zom = NULL,
  LAI = NULL,
  DEM = NULL,
  lapse = 0.0065,
  Rn24 = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  model = "sebi",
  clip = NULL,
  folder = NULL
)

\method{sebi}{default}(
  albedo = NULL,
  Ts,
  Tmax,
  Tmin,
  RHmax = NULL,
  RHmin = NULL,
  latitude = NULL,
  NDVI,
  SAVI,
  iter.max = 7,
  xyhot = "auto",
  xycold = "auto",
  DOY,
  sunelev,
  welev,
  zx = 10,
  u = 2,
  zomw = 2,
  zom = NULL,
  LAI = NULL,
  DEM = NULL,
  lapse = 0.0065,
  Rn24 = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  model = model,
  clip = NULL,
  folder = NULL
)
}
\arguments{
\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{Tmax}{Numeric. Maximum air Temperature in degree Celsius}

\item{Tmin}{Numeric. Minimum air Temperature in degree Celsius}

\item{RHmax}{Maximum Relative Humidity in percent}

\item{RHmin}{Minimum Relative Humidity in percent}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{NDVI}{A RasterLayer data that indicates NDVI 
(Normalized Difference Vegetation Index) values. You can also point to a 
file on your computer.}

\item{SAVI}{A RasterLayer data that indicates 
SAVI(Soil-adjusted Vegetation Index) values. 
 You can also point to file on your computer.}

\item{iter.max}{maximum iterations of sensible heat calculation.}

\item{xyhot}{numeric or "auto". A list of x and y coordinates of a 
hot pixel in the form c(x,y). If it is set to "auto", \code{\link{hotTs}} 
will be used to compute it.}

\item{xycold}{numeric or "auto". A list of x and y coordinates of a 
cold pixel in the form of c(x,y). If it set to "auto", \code{\link{coldTs}} 
will be used to compute it.}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{sunelev}{Angle of Sun elevation in degrees, 
as found in the meta data of the satellite image}

\item{welev}{Weather station elevation in meters}

\item{zx}{The height above the weather station where the 
wind speed is measured [m]}

\item{u}{The satellites  overpass time wind speed at the weather station [m/s]}

\item{zomw}{the roughness length for the weather station surface [m]}

\item{zom}{the momentum roughness length for each pixel [m]. 
You can also point to a raster file on your computer}

\item{LAI}{Leaf Area Index, dimensionless}

\item{DEM}{A digital elevation model[m]}

\item{lapse}{A local lapse rate that is applied to correct DEM [K/m]. 
Default value is 0.0065}

\item{Rn24}{A 24 hour net radiation [W/m2]. It is needed to estimate daily ET. 
It can be computed with \code{\link{ETo}} with even a minimal data of 
Tmax and Tmin. 
 You can also point to raster file on your computer}

\item{wmo}{numeric. World Meteorological Organization weather station code.
You can use the  Worldwide Station List at 
\url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
\url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]}

\item{airport}{numeric. IATA or ICAO code. They are alphabetically  listed at 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]}

\item{Krs}{Relationship between the fraction of extraterrestrial radiation that reaches the 
earth's surface, Rs/Ra, and the air temperature difference Tmax - Tmin for
interior (Krs = 0.16) and coastal (Krs = 0.19) regions}

\item{surface}{character. It is either "grass" or "alfalfa"}

\item{model}{character. The type of model. It takes either "SEBAL" or "METRIC"}

\item{clip}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform the data to be cropped. for example clip can take the form of c(xmin, xmax, ymin, ymax)}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}
}
\value{
EF Evapotranspiration Fraction
}
\description{
\code{sebi} computes Evapotranspiration Fraction Estimation
based on Menenti& Choudhury (1993) parameterization of 
land surface evaporation by means of location dependent potential 
evaporation and surface temperature range.
}
\details{
Function \code{\link{ETo}} will be automatically used to 
estimate  parameters y (psychrometric constant), 
slope (slope of saturation vapour pressure curve) and 
vpd (Vapour pressure deficit).
}
\examples{
\dontrun{
#using folder parameter 
folder=system.file("extdata","stack",package="sebkc")
sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28)

#manual input of parameters
#data
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
#model
modsebi=sebi(albedo=albedo,Ts=Ts,Tmax=31,Tmin=28,RHmax=84,
RHmin=63,NDVI=NDVI,SAVI=NULL,iter.max=7,xyhot="full",
xycold="full",DOY=37,sunelev=50.71154048,welev=317.1,zx=10,
u=2,zomw=2,zom=NULL,LAI=LAI,Rn24=NULL,model="sebi")


#Another interactive example
#get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
welev=278
data=landsat578(rawdata,welev=welev)
 #perform semi-auto simulation 
#Determine xyhot. Digitize polygon on the Ts map
modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#determine the cold. Digitize polygon on the Ts map
modcold=coldTs(data,welev=275,extent="auto",cluster=2)
xyhot=modhot$xyhot
xycold=modcold$xycold
#use object of \code{\link{coldTs}} or \code{\link{hotTs}} in different ways
modsebi=sebi(data,xyhot=modhot,xycold=xycold,welev=welev,
Tmax=Tmax,Tmin=Tmin,RHmax=RHmax,RHmin=RHmin)
}
}
\references{
Menenti, M., & Choudhury, B. 1993. Parameterization of land surface 
evaporation by means of location dependent potential evaporation and 
surface temperature range. Paper presented at the Proceedings of 
IAHS conference on Land Surface Processes.
}
\seealso{
\code{\link{sebal}} , \code{\link{ETo}}, \code{\link{sebs}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomass.R
\name{invdist}
\alias{invdist}
\title{Inverse distance interpolation}
\usage{
invdist(longitude, latitude, var, ext, idp = 0.5)
}
\arguments{
\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{var}{numeric variable. The length should be same as the length of 
latitude and longitude}

\item{ext}{RasterLayer object.}

\item{idp}{Neighbourhood parameter}
}
\value{
var: interpolated values
}
\description{
This function performs inverse distance interpolation based 
on projection and geometry of input spatial data 'ext'.
}
\examples{
 \dontrun{
#Get ext data
folder=system.file("extdata","stack",package="sebkc")
data=landsat578(data=folder, welev=362)
temp=data$Ts
latitude=seq(7.544,7.590,0.001)
longitude=seq(-1.211,-1.187,0.001)
var=seq(20,30,0.2)
latitude=latitude[1:length(longitude)]
var=var[1:length(longitude)]
map=invdist(longitude,latitude,var,ext=temp)
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cal.kc.R
\name{cal.kc}
\alias{cal.kc}
\title{Calibration of crop coefficients}
\usage{
cal.kc(
  ETo,
  P,
  RHmin,
  soil,
  crop,
  I = 0,
  CNII = c(30, 100, 1),
  u2 = 2,
  FC = NULL,
  WP = NULL,
  h = NULL,
  Zr = NULL,
  Ze = c(0.1, 0.15, 0.01),
  kc = c(c(0.1, 0.3, 0.05), c(0.1, 1, 0.1), c(0.1, 1, 0.1)),
  p = c(0.1, 1, 0.1),
  lengths = NULL,
  fw = 0.8,
  fc = NULL,
  kctype = "dual",
  region = NULL,
  regexpr = "sweet",
  initgw = 50,
  LAI = 3,
  rc = NULL,
  Kb = 0,
  Rn = NULL,
  hmodel = "gaussian",
  Zrmodel = "linear",
  xydata = NULL,
  REW = c(2, 12, 0.5),
  a1 = 360,
  b1 = c(-1, 1, 0.1),
  a2 = 240,
  b2 = c(-1, 1, 0.1),
  a3 = c(-2, 1, 0.1),
  b3 = 6.6,
  a4 = 4.6,
  b4 = c(-1.5, 1, 0.1),
  ETa = NULL,
  report = "stages",
  time = "06:00",
  HWR = 1,
  latitude = NULL,
  density = "full",
  DOY = NULL,
  slope = NULL,
  y = NULL,
  r1 = 100,
  nongrowing = "weeds",
  theta = NULL,
  profile = "variable",
  Kcmin = 0.15,
  Ky = NULL
)
}
\arguments{
\item{ETo}{Numeric. Reference evapotranspiration [mm]. This can be estimated with \code{\link{ETo}}}

\item{P}{numeric. Precipitation [mm]}

\item{RHmin}{numeric. Minimum Relative Humidity [per cent]}

\item{soil}{character. Name of soil texture as written in FAO56 TABLE 19}

\item{crop}{character. Name of the crop as written on 
FAO56 Table 11, TABLE 12 and TABLE 17}

\item{I}{numeric. Irrigation [mm]}

\item{CNII}{numeric. The initial Curve Number of the study site. 
This can be calibrated with \code{cal.kc}.}

\item{u2}{numeric. Wind speed of the nearby weather station [m/s] measured at 2m}

\item{FC}{numeric. soil water content at field capacity [-][m3 m-3]. 
This can be calibrated with \code{cal.kc}.}

\item{WP}{numeric. soil water content at wilting point [-][m3 m-3]. 
This can be calibrated with \code{cal.kc}.}

\item{h}{numeric. Maximum crop height [m]. See details below.
This can be calibrated with \code{cal.kc}}

\item{Zr}{numeric. Rooting depth.  See details below.
This can be calibrated with \code{cal.kc}}

\item{Ze}{numeric. The depth of the surface soil layer that is subject to drying 
by way of evaporation [0.10-0.15 m]. This can be calibrated with \code{\link{cal.kc}}}

\item{kc}{list. The tabulated Kc values for single crop c(kcini,kcmid,kcend)
or  dual crop model c(kcbini,kcbmid,kcbend). To use with \code{\link{sebal}} or 
any of the Surface Energy Balance models, it should be linked as list(model1,model2,model3) or
 c(model1$EF,model2$EF,model3$EF) but not a mixture of both. 
This can be calibrated with \code{\link{cal.kc}}}

\item{p}{The depletion coefficient. This can be calibrated with \code{\link{cal.kc}}}

\item{lengths}{list. The lengths of the crop growth stages. It can take dates in the form of 
c(start of planting ('YYYY-mm-dd), start of rapid growth, 
start of mid season, start of maturity, harvesting) 
or in the form number of days: c(initial days, dev days, mid days, late days). 
See FAO56 TABLE 11. If performing time series simulation, different dates or 
one season dates can be set; for instance two seasons simulation can be of the form 
c(initial days, dev days, mid days, late days, initial days, dev days, mid days, late days) 
or just c(initial days, dev days, mid days, late days)}

\item{fw}{list or numeric or character. This can take different fw values
throughout the growing season. It can also take one numeric value and this will
be transform depending on the type of wetting. If the wetting event is only Precipitation,
it is set to 1; if it is only Irrigation it is set to the provided fw value. If it is
both Precipitation and Irrigation weights are assigned for calculation of fw. 
fw can also take a character such as "Drip", "Sprinkler", "Basin", "Border", 
"alternated furrows", "Trickle", "narrow bed" and "wide bed". 
In this case fw values from FAO56 are assigned.}

\item{fc}{numeric. fraction of ground covered by tree canopy}

\item{kctype}{character. The type of model either "single" or "double". In spatial modelling involving
Evaporation Fraction, kctype can be set to "METRIC" or "SSEB" so that 
Actual Evapotranspiration will be estimated as ETo*EF else it will be
estimated with Rn as ETa=(EF*Rn)/2.45. Note that Rn is in MJm-2 day-1.}

\item{region}{character. The region of the crop as on FAO56 TABLE 11}

\item{regexpr}{character. Extra term that can be used to search crop names on FAO56 Tables}

\item{initgw}{numeric. Initial ground water level [m]}

\item{LAI}{numeric. Leaf Area Index}

\item{rc}{numeric Runoff coefficient [0-1]. 
This must be used if there is no information on Curve Number.
This can be calibrated with \code{\link{cal.kc}}}

\item{Kb}{ground water (base flow) recession parameter. 
The default is zero where no baseflow occurs}

\item{Rn}{Net daily radiation [w/m2]. Estimated this with  \code{\link{ETo}}}

\item{hmodel}{character. The type of crop growth model during the simulation. 
It takes "gaussian","linear", "exponential"}

\item{Zrmodel}{character. The type of root growth model during the simulation. 
It takes "gaussian","linear", "exponential"}

\item{xydata}{list c(longitude, latitude). The observation points of the model. 
This should be set only if the model is spatial.}

\item{REW}{numeric Readily Evaporable Water. It can be calibrated with \code{\link{cal.kc}}}

\item{a1}{soil water storage to maximum root depth (Zr) at field capacity}

\item{b1}{Liu et al. (2006) parameter for computing capillary rise [-0.17]. It can calibrated with
\code{\link{cal.kc}}}

\item{a2}{Liu et al. (2006) parameter for computing capillary rise [240]. It can be calibrated with
\code{\link{cal.kc}}}

\item{b2}{Liu et al. (2006) parameter for computing capillary rise [-0.27]. It can be calibrated with
\code{\link{cal.kc}}}

\item{a3}{Liu et al. (2006) parameter for computing capillary rise [-1.3]. It can be calibrated with
\code{\link{cal.kc}}}

\item{b3}{Liu et al. (2006) parameter for computing capillary rise [6.6]. It can be calibrated with
\code{\link{cal.kc}}}

\item{a4}{Liu et al. (2006) parameter for computing capillary rise [4.6]. It can be calibrated with
\code{\link{cal.kc}}}

\item{b4}{Liu et al. (2006) parameter for computing capillary rise [-0.65]. It can be calibrated with
\code{\link{cal.kc}}}

\item{ETa}{Measured Actual Evapotranspiration [mm]}

\item{report}{list. In case of spatial modelling, which days' map should be reported. 
The default is "stages" where the periods of stages maps are reported as values. 
In an ascending order it can take values such as 1:12 days or c(1,3,6:12) days.}

\item{time}{character. Time of the occurrence Irrigation or Precipitation.}

\item{HWR}{numeric. Default is 1. Height to width ratio of individual plants or groups of 
plants when viewed from the east or from the west [-].}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{density}{character. density factor for the trees, 
vines or shrubs on how they are arranged.
It can take "full", "row" or "random" or "round".}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{slope}{numeric. slope of saturation vapour pressure curve at air temperature T [kPa oC-1].}

\item{y}{numeric. psychometric constant  [kPa oC-1]}

\item{r1}{leaf resistance FOR STOMATAL CONTROL [s/m]}

\item{nongrowing}{character. The type of surface during non-growing season. 
It can take "weeds", "bare", or "mulch"}

\item{theta}{numeric. The measured soil water content [m3/m3] in the 
profile. Leave those days without measured data blank.}

\item{profile}{numeric or character. The profile depth to perform water balance.
It takes "variable" when updating the depth with root growth (default). Or taking "fixed" when
the maximum root zone is used. It can also take numeric value and this can be longer than
maximum root length.}

\item{Kcmin}{Minimum  Kc value}

\item{Ky}{list or array. yield response factor [-] in the form of 
c(initial Ky,crop dev't Ky, mid season Ky, late season Ky, total season Ky). 
For instance Maize Ky is c(0.4,0.4,1.3,0.5,1.25). Ky values for other crops
are tabulated in FAO Irrigation and Drainage Paper 33 (Doorenbos and Kassam,
1979) and Paper 56 (Allen et al., 1998).
If only one Ky value is supplied it will be repeated for all the seasons.}
}
\value{
The function returns optimised parameters as "parameters", goodness of fit 
as "fit", and calibrated model as "mod". It also prints candidates of parameters and 
goodness of fit of the local optimum values. For the explanation of calibrated model 
output and optimised parameters see the return values of \code{\link{kc}}. Here, only
goodness of fit tests explanations are given.
\describe{
\item{mod:}{ The optimised model. See return of \code{\link{kc}} for more details}
\item{parameters:}{ parameters of the optimised model}
\item{fit:}{  Goodness of fit tests. See \code{\link{fitting}} for explanation of variables}
}
}
\description{
This function can estimate up to 23 parameters of FAO single and dual crop
parameters. The parameters include soil parameters: WP,FC, REW; 
crop parameters: kcini,kcmid,kcend,kcbini,kcbmid,kcbend,h,Zr; 
the evaporation parameters: p and Ze, 
runoff coefficients: CNII or rc, as well as groundwater and capillary rise 
parameters: initgw,a1,b1,a2,b2,a3,b3,a4,b4. For each parameter initial, maximum and range 
values must be provided if one wants to optimise it.
There are also 12 goodness of fit tests for the evaluation of the model with
\code{\link{fitting}} :r-square, index of agreement, F-value, p-value, sse,sst, AIC,
BIC, RMSE, NRMSE and nashSutcliffe coefficients. A calibrated model that uses more 
parameters gets punished by AIC and BIC as well as adjusted R2, RMSE, NRMSE.

Both point and spatial model can be calibrated. The actual model is available at \code{\link{kc}}
}
\details{
\describe{
There are several ways  to use cal.kc:
 \item{\strong{Measured Data}}{The kc model can be calibrated against either Actual Evapotranspiration (ETa) or
 Available Soil Water; the user must specify theta. If the calibration is done on a point data then time series for
 that location must be provided. If multiple locations are provided with xydata and corresponding 
 multiple locations as input data, then ETa or theta must be provided for all locations. See also \code{kc} 
 for more details on locations.     }
 \item{\strong{Number of optimised parameters and Goodness of fit}}{ 
 There is a need to limit the number of calibrated parameters.
  One can compare models by observing the adjusted goodness of fit tests. 
  The more parameters that one uses the less are the values of R-square but
  RMSE, BIC, AIC increases. Unadjusted and adjusted goodness of fit tests
   are provided with this function.
    }
  }
}
\examples{
\dontrun{
#fewer iteration
file=system.file("extdata","sys","irrigation.txt",package="sebkc")
data=read.table(file,header=TRUE)  
P=data$P
rc=0
I=data$I
ETo=data$ETo
Zr=data$Zr
p=0.6
FC=0.23
WP=0.10
u2=1.6  
RHmin=35
soil="sandy loam"
crop="Broccoli"
#single kc calibration with fewer iteration
cal=cal.kc(ETo,P,RHmin,soil,crop,I=0,kc=NULL,CNII=67,
u2=2,FC=NULL,WP=NULL,h=NULL,Zr=NULL,lengths=NULL,fc=NULL,
kctype="single",region=NULL,regexpr=NULL,initgw=50,LAI=3,
rc=NULL,Rn=NULL,hmodel="gaussian",Zrmodel="linear",
xydata=NULL,ETa=ETo,a1=360,b1=-0.17,a2=240,b2=-0.27,
a3=-1.3,b3=6.6,a4=4.6,b4=-0.65)
parameters=cal$parameters
output=cal$mod$output
gof=cal$fit
ETc=cal$mod$output$ETc
TAW=cal$mod$output$TAW

#Single kc with different kcs. kc uses default values
#RMSE_adj and NRMSE_adj equal to NAN because the number of parameters more than time steps(days)
cal=cal.kc(ETo,P,RHmin,soil,crop,I=0,CNII=67,u2=2,FC=NULL,WP=NULL,
h=NULL,Zr=NULL,lengths=NULL,fc=NULL,kctype="single",region=NULL,
regexpr=NULL,initgw=50,a1=c(100,400,10),b1=c(-1,-0.1,0.1),
a2=c(100,400,10),b2=c(-1,1,0.1),a3=c(-2,1,0.1),b3=c(1,10,1),
a4=c(1,10,1),b4=c(-1.5,1,0.1),LAI=3,rc=NULL,
Rn=NULL,hmodel="gaussian",Zrmodel="linear",xydata=NULL,ETa=ETo)

#Example with fixed kcs
cal=cal.kc(ETo,P,RHmin,soil,crop,I=0,CNII=67,kc=c(0.1,1.15,1),
u2=2,FC=NULL,WP=NULL,h=NULL,Zr=NULL,lengths=NULL,
fc=NULL,kctype="single",region=NULL,regexpr=NULL,initgw=50,
b1=c(-1,-0.1,0.1),b2=c(-1,1,0.1),LAI=3,rc=NULL,Rn=NULL,
hmodel="gaussian",Zrmodel="linear",xydata=NULL,ETa=ETo)

#Example with spatial kcs
#landsat folder with original files but a subset
folder=system.file("extdata","stack",package="sebkc") 
sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28) #sebi model
kc2=stack(sebiauto$EF/2,sebiauto$EF,sebiauto$EF/3) #assign EF to kc
xydata=data.frame(cbind(longitude=data$longitude,latitude=data$latitude))
modEF=kc(ETo,P=P,RHmin,soil,crop,I,kc=kc2,Rn=8,lengths = c(4,4,2,2),xydata=xydata)
ETa=modEF$ETcxy[3:14]

calsp=cal.kc(ETo,P,RHmin,soil,crop,I=0,CNII=67,kc=kc2,u2=2,FC=NULL,
 WP=WP,h=NULL,Zr=NULL,lengths = c(4,4,2,2),fc=NULL,kctype="single",
 region=NULL,regexpr=NULL,initgw=50,b1=c(-1,-0.1,0.1),b2=c(-1,1,0.1),
 LAI=3,rc=NULL,Rn=7,hmodel="gaussian",Zrmodel="linear",xydata=xydata,ETa=ETa)
 gof=cal$fit
r2=cal$fit$r2
AIC=cal$fit$AIC
ETcxy=cal$mod$ETcxy
TAW=cal$mod$TAW
}
}
\seealso{
\code{\link{kc}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sebal.R
\name{sebal}
\alias{sebal}
\alias{sebal.default}
\title{R computation of Surface Energy Balance Algorithm for Land 
and METRIC}
\usage{
sebal(
  albedo,
  Ts,
  NDVI,
  SAVI,
  welev,
  xyhot = "auto",
  xycold = "auto",
  DOY = NULL,
  sunelev,
  zx = 10,
  u = 2,
  zomw = 2,
  zom = NULL,
  LAI = NULL,
  DEM = NULL,
  lapse = 0.0065,
  Rn24 = NULL,
  ETr = NULL,
  ETr24 = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  latitude = NULL,
  t1 = 1,
  time = NULL,
  Lz = NULL,
  Lm = NULL,
  model = "SEBAL",
  iter.max = 7,
  clip = NULL,
  folder = NULL
)

\method{sebal}{default}(
  albedo = NULL,
  Ts = NULL,
  NDVI = NULL,
  SAVI = NULL,
  welev,
  xyhot = "auto",
  xycold = "auto",
  DOY = NULL,
  sunelev = NULL,
  zx = 10,
  u = 2,
  zomw = 2,
  zom = NULL,
  LAI = NULL,
  DEM = NULL,
  lapse = 0.0065,
  Rn24 = NULL,
  ETr = NULL,
  ETr24 = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  latitude = NULL,
  t1 = 1,
  time = NULL,
  Lz = NULL,
  Lm = NULL,
  model = "SEBAL",
  iter.max = 7,
  clip = NULL,
  folder = NULL
)
}
\arguments{
\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{NDVI}{A RasterLayer data that indicates NDVI 
(Normalized Difference Vegetation Index) values. You can also point to a 
file on your computer.}

\item{SAVI}{A RasterLayer data that indicates 
SAVI(Soil-adjusted Vegetation Index) values. 
 You can also point to file on your computer.}

\item{welev}{Weather station elevation in meters}

\item{xyhot}{numeric or "auto". A list of x and y coordinates of a 
hot pixel in the form c(x,y). If it is set to "auto", \code{\link{hotTs}} 
will be used to compute it.}

\item{xycold}{numeric or "auto". A list of x and y coordinates of a 
cold pixel in the form of c(x,y). If it set to "auto", \code{\link{coldTs}} 
will be used to compute it.}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{sunelev}{Angle of Sun elevation in degrees, 
as found in the meta data of the satellite image}

\item{zx}{The height above the weather station where the 
wind speed is measured [m]}

\item{u}{The satellites  overpass time wind speed at the weather station [m/s]}

\item{zomw}{the roughness length for the weather station surface [m]}

\item{zom}{the momentum roughness length for each pixel [m]. 
You can also point to a raster file on your computer}

\item{LAI}{Leaf Area Index, dimensionless}

\item{DEM}{A digital elevation model[m]}

\item{lapse}{A local lapse rate that is applied to correct DEM [K/m]. 
Default value is 0.0065}

\item{Rn24}{A 24 hour net radiation [W/m2]. It is needed to estimate daily ET. 
It can be computed with \code{\link{ETo}} with even a minimal data of 
Tmax and Tmin. 
 You can also point to raster file on your computer}

\item{ETr}{an alfalfa or grass reference ET at the time of satellites  overpass. 
It only needed if you set model to "metric" [mm/hr]}

\item{ETr24}{daily grass or alfalfa reference Evapotranspiration (mm/day)}

\item{wmo}{numeric. World Meteorological Organization weather station code.
You can use the  Worldwide Station List at 
\url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
\url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]}

\item{airport}{numeric. IATA or ICAO code. They are alphabetically  listed at 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]}

\item{Krs}{Relationship between the fraction of extraterrestrial radiation that reaches the 
earth's surface, Rs/Ra, and the air temperature difference Tmax - Tmin for
interior (Krs = 0.16) and coastal (Krs = 0.19) regions}

\item{surface}{character. It is either "grass" or "alfalfa"}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{t1}{numeric. The length of the calculation period in hour; 1 for hour, 0.5 for 30 minutes 0.25 for 15 minutes}

\item{time}{The midpoint of the time of measurement[hour]; for example
time is 12.5 for a period between 12:00 and 13:00}

\item{Lz}{longitude of the centre of the local time zone [degrees west of Greenwich].
MUST BE POSITIVE
Lz = 0 degrees for Greenwich.}

\item{Lm}{longitude of the measurement site [degrees west of Greenwich]. MUST BE POSITIVE}

\item{model}{character. The type of model. It takes either "SEBAL" or "METRIC"}

\item{iter.max}{maximum iterations of sensible heat calculation.}

\item{clip}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform the data to be cropped. for example clip can take the form of c(xmin, xmax, ymin, ymax)}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}
}
\value{
\describe{
\item{LE:}{  latent heat flux [W/m2]}
\item{Rn:}{ Net Radiation [W/m2]}
\item{H: }{ Sensible Heat Flux [W/m2]}
\item{G:}{ Soil heat flux [W/m2]}
\item{ETins:}{ instantaneous ET if model = "METRIC" [mm/hr]}
\item{ET24:}{ 24 hour Evapotranspiration only if Rn24 is not NULL [mm/day]}
\item{ETrF:}{ METRIC ET fraction [dimensionless]}
\item{EF:}{  ET fraction [dimensionless]}
\item{Ta:}{ Air temperature = Ts-dT [K]}
\item{iter.coef:}{ Parameters from sensible heat iteration}
}
}
\description{
This R function computes Bastiaanssen (1998) surface energy 
balance components of evapotranspiration (ET), Sensible heat(H), 
soil heat flux(G) and Net Radiation(Rn).
}
\details{
\describe{
 \item{\strong{Computing SEBAL or METRIC}}{\code{sebal} can compute either SEBAL(Bastiaanssen et al,1998)
 or METRIC (Allen et al 2007). In the case of METRIC, input data such as DEM,
 local lapse rate, and ETr must be provided. See Allen et al (2007) for more
 details.}
 \item{\strong{Using Landsat Folder as input data}}{Though this sebkc 
 function is not sensor dependent, one can point the
 folder parameter to a Landsat TM(5), ETM(7) or 8 folder that contains 
 landsat bands and metadata. One can also point the folder parameter 
 to object of  \code{\link{landsat578}} }
 \item{\strong{Automatic computation}}{When xycold and xyhot are set to "auto", about 80 per cent of the input
 image pixels are used to compute cold and hot pixels. This avoids using
 NAs in classification of pixels. }
 \item{\strong{Taking long time to Simulate?}}{If you point the folder to landsat files and 
 set xycold and xyhot to "auto", it will take about 2 hours for 
 the function to run. Without these "autos", it will take about 45
 minutes. It is advisable to perform 3 step analyses of using three 
 functions as follows: \code{\link{sebkcstack}}, \code{\link{landsat578}},
 \code{\link{hotTs}}, \code{\link{coldTs}}. You need not to go through this if you have your 
 albedo, Ts, NDVI, SAVI from MODIS, ASTER etc}
 }
}
\examples{
\dontrun{
#use original landsat 8 data by specifying folder path
folder=system.file("extdata","stack",package="sebkc")
modauto=sebal(folder = folder,welev = 380,xycold="full",xyhot="full")
#plot ET fraction
plot(modauto$EF)

#Semi-auto
#load individual input files, for example from MODIS, ASTER etc
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
mod=sebal(albedo=albedo,Ts=Ts,NDVI=NDVI,SAVI=NULL,
iter.max=7,xyhot="full",xycold="full",
DOY=37,sunelev=50.71154048,welev=317.1,zx=10,
u=2,zomw=3,zom=NULL,LAI=LAI,DEM=NULL,
lapse=0.0065,Rn24=NULL,ETr=NULL,model="SEBAL")
#get latent heat flux
mod$LE
#evaporation fraction
mod$EF

#3 step simulation using landsat 7 with SLC OFF
##step one: prepare the input data 
### point to landsat 7 data with strips 
folder=system.file("extdata","slc",package="sebkc")
#####get the data stacked and remove strips
rawdata=sebkcstack(folder,remove_cloud = 2,gap_fill = TRUE)
###### check if strips are removed
plot(rawdata$data)
#get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
welev=278
 data=landsat578(rawdata,welev=welev)
 #perform semi-auto simulation mod=sebal(data,welev=welev)
# Determine xyhot. Digitize polygon on the Ts map
modhot=hotTs(data,welev=welev,extent="auto",cluster=2)
#determine the cold. Digitize polygon on the Ts map
modcold=coldTs(data,welev=welev,extent="auto",cluster=2)
xyhot=modhot$xyhot
xycold=modcold$xycold
modsebal=sebal(folder=data,xyhot=xyhot,xycold=xycold,welev=welev)
}
}
\references{
\itemize{
\item Bastiaanssen, W. G. M., Menenti, M., Feddes, R. A., & Holtslag,
 A. A. M. 1998. A remote sensing surface energy balance algorithm for  
 land (SEBAL), part 1: formulation. Journal of 
 Hydrology: 212-213: 198-212.
\item Bastiaanssen, W. G. M., Pelgrum, H., Wang, J., Ma, Y., Moreno,
 J. F., Roerink, G. J., & van der Wal, T. 1998. A remote sensing 
 surface energy balance algorithm for land (SEBAL).: Part 2: 
 Validation. Journal of Hydrology, 212-213(0): 213-229.
 \item Allen, R., Tasumi, M., & Trezza, R. 2007. Satellite-Based 
Energy Balance for Mapping Evapotranspiration with Internalized 
Calibration (METRIC)-Model. Journal of Irrigation and Drainage 
Engineering, 133(4): 380-394.
\item Allen, R., Tasumi, M., Morse, A., Trezza, R., Wright, J., 
Bastiaanssen, W., Kramber, W., Lorite, I., & Robison, C. 2007. 
Satellite-Based Energy Balance for Mapping Evapotranspiration 
with Internalized Calibration (METRIC)-Applications. 
Journal of Irrigation and Drainage Engineering, 133(4): 395-406.
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional.R
\name{rastercon}
\alias{rastercon}
\title{Spatial Conditional function on raster object}
\usage{
rastercon(condition, trueValue, falseValue)
}
\arguments{
\item{condition}{logical}

\item{trueValue}{logical}

\item{falseValue}{logical}
}
\value{
Return True or False
}
\description{
It computes equivalent of logical if function in R
}
\examples{
\dontrun{
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
  eNB=rastercon( NDVI<0 & albedo<0.47,0.99,rastercon(LAI>=3,0.98,0.97+(LAI*0.0033)))
  }
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETo.R
\name{ETo}
\alias{ETo}
\alias{ETo.default}
\title{24 Hour FAO 56 and ASCE-EWRI Reference Evapotranspiration ETo}
\usage{
ETo(
  Tmax,
  Tmin,
  DOY,
  latitude,
  longitude = NULL,
  z = NULL,
  uz = 2,
  altitude = NULL,
  RHmax = NULL,
  RHmin = NULL,
  n = NULL,
  Krs = 0.16,
  albedo = 0.23,
  as = 0.25,
  bs = 0.5,
  Rs = NULL,
  Rn = NULL,
  G = 0,
  EF = NULL,
  wmo = NULL,
  airport = NULL,
  scale = "sine function",
  Kc = NULL,
  surface = "grass"
)

\method{ETo}{default}(
  Tmax,
  Tmin,
  DOY,
  latitude,
  longitude = NULL,
  z = NULL,
  uz = 2,
  altitude = NULL,
  RHmax = NULL,
  RHmin = NULL,
  n = NULL,
  Krs = 0.16,
  albedo = 0.23,
  as = 0.25,
  bs = 0.5,
  Rs = NULL,
  Rn = NULL,
  G = 0,
  EF = NULL,
  wmo = NULL,
  airport = NULL,
  scale = "evaporative fraction",
  Kc = NULL,
  surface = "grass"
)
}
\arguments{
\item{Tmax}{Numeric. Maximum air Temperature in degree Celsius}

\item{Tmin}{Numeric. Minimum air Temperature in degree Celsius}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{z}{The height above surface where wind speed was measured, in metres.}

\item{uz}{The wind speed in m/s}

\item{altitude}{The elevation of the weather station, in metres}

\item{RHmax}{Maximum Relative Humidity in percent}

\item{RHmin}{Minimum Relative Humidity in percent}

\item{n}{Sunshine hours}

\item{Krs}{Relationship between the fraction of extraterrestrial radiation that reaches the 
earth's surface, Rs/Ra, and the air temperature difference Tmax - Tmin for
interior (Krs = 0.16) and coastal (Krs = 0.19) regions}

\item{albedo}{albedo values. For grass it is 0.23[dimensionless]}

\item{as}{Regression constant, intercept, expressing the fraction of extra-terrestrial 
radiation reaching the earth on overcast days (n = 0); 
either a calibrated value or as = 0.25 can be used.}

\item{bs}{regression constant, slope, expressing 
the fraction of extraterrestrial radiation
reaching the earth on overcast days (n = 0); either a calibrated value or bs = 0.5 can be used.}

\item{Rs}{shortwave incoming radiation [MJ/m2/day]}

\item{Rn}{Net daily radiation in MJ/m2/day. If it is NULL, 
it will be computed}

\item{G}{Daily Soil heat flux in W/m2}

\item{EF}{Spatial. ET fraction [-] from \link{sebal} or \link{tseb} \link{sebs}.
It is advisable to link it to any of the two functions.}

\item{wmo}{numeric. World Meteorological Organization weather station code.
You can use the  Worldwide Station List at 
\url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
\url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]}

\item{airport}{numeric. IATA or ICAO code. They are alphabetically  listed at 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]}

\item{scale}{character The type of scale models to upscale instantaneous ET to 
daily ET. it takes "Evaporative Fraction" (Bastiaanssen et al 1998); 
"Sine Function"  (Jackson et al 1983); "Solar Radiation" (French et al 2005). 
Remember the blank spaces between the parameters. 
You can also respectively use "evaporative" or "solar" or "sine".}

\item{Kc}{Crop coefficient}

\item{surface}{character. It is either "grass" or "alfalfa"}
}
\value{
\describe{
\item{ETo:}{ 24 hour reference ETo [mm/day]}
\item{ETa:}{ 24 hour actual ET. It is not NULL if EF has a numeric value}
\item{ETc:}{ 24 hour crop ET. It is not NULL if Kc has a numeric value}
\item{Rn:}{ Net daily Radiation  [W/m2] }
\item{Rn_mj_m2_day:}{ Net daily Radiation [MJ/m2/day]}
\item{y:}{ psychrometric constant [kPa/degree Celsius]}
\item{slope:}{ slope of saturation vapour pressure 
curve [kPa/degree Celsius]}
\item{vpd:}{ Vapour pressure deficit[kPa]}
}
}
\description{
24 Hour FAO 56 and ASCE-EWRI Reference Evapotranspiration ETo
}
\examples{
\donttest{
#with minimal data
PET=ETo(Tmax=31,Tmin=28,latitude=5.6,DOY="6-2-2001")
PET$ETo
#with enough data as in FAO56 EXAMPLE 18 on 6 July in Uccle (Brussels, Belgium) 
#located at 50 degrees 48 minutes N and at 100 m above sea level
PET24=ETo(Tmax=21.5,Tmin=12.3,z=10,uz=2.78,altitude=100,
RHmax=84,RHmin=63,n=9.25,DOY=187,latitude=50.8)
PET24$ETo
}
}
\references{
ALLEN, R. G., PEREIRA, L. S., RAES, D., & SMITH, M. 1996. 
Crop Evapotranspiration (guidelines for computing crop water requirements) 
FAO Irrigation and Drainage Paper No. 56: FAO.
}
\seealso{
\code{\link{ETohr}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xyextract.R
\name{xyextract}
\alias{xyextract}
\title{Reprojecting and extracting Raster values}
\usage{
xyextract(map, longitude, latitude)
}
\arguments{
\item{map}{a raster object.}

\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}
}
\value{
data
}
\description{
The function reprojects longitudes and latitudes into map
coordinates and extract raster values
}
\examples{
\dontrun{
lonlat=read.table(system.file("extdata","sys","xyvalues.txt",package="sebkc"),header=TRUE)
longitude=lonlat$longitude
latitude=lonlat$latitude
folder=system.file("extdata","stack",package="sebkc")
data=landsat578(data=folder, welev=362)
Ts=data$Ts
Tsdata=xyextract(Ts,longitude,latitude)

}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DL.R
\name{dl}
\alias{dl}
\title{Day length calculation}
\usage{
dl(latitude, DOY, model = "CBM", Tmax = NULL, Tmin = NULL, p = 0.5)
}
\arguments{
\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{model}{character Type of model:"CBM" (Schoolfield, 1982),"BGC" (Running & Coughlan, 1988),
"CERES" (Ritchie, 1991) or "FAO56"}

\item{Tmax}{Numeric. Maximum air Temperature in degree Celsius}

\item{Tmin}{Numeric. Minimum air Temperature in degree Celsius}

\item{p}{numeric. CMB parameter}
}
\value{
DL
}
\description{
Day length calculation
}
\examples{
DOY="2001-8-1"
latitude=0
model="CBM"
Tmax=31
Tmin=26
mod=dl(latitude,DOY,model,Tmax,Tmin=Tmin)
}
\references{
\itemize{
\item Schoolfield R. (1982). Expressing daylength as a function of latitude and Julian date.
\item Running S. W. and Coughlan J. C. (1988). A general model of forest ecosystem processes 
for regional applications I. Hydrologic balance, canopy gas exchange and primary production 
processes. Ecological Modelling, 42(2), 125-154 http://dx.doi.org/10.1016/0304-3800(88)90112-3.
\item Ritchie J. T. (1991). Wheat Phasic Development. Modeling Plant and Soil Systems. 
 Agronomy Monograph, 31.
\item Allen R. G., Pereira L. S., Raes D. and Smith M. (1998). Crop evapotranspiration: 
Guidelines for computing crop water requirements. FAO Irrigation and Drainage Paper, 56, 300.
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weather.R
\name{weather}
\alias{weather}
\title{Retrieval of Weather Data  from WMO and NASA Surface meteorology and Solar Energy}
\usage{
weather(
  data = NULL,
  wmo = NULL,
  airport = NULL,
  date = "YYYY-m-d",
  time = NULL,
  latitude = NULL,
  longitude = NULL,
  NASA.SSE = list(from = "YYYY-m-d", to = "YYYY-m-d"),
  folder = NULL
)
}
\arguments{
\item{data}{dataframe that can contain all or part of the input data}

\item{wmo}{numeric. World Meteorological Organization weather station code.
You can use the  Worldwide Station List at 
\url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
\url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]}

\item{airport}{numeric. IATA or ICAO code. They are alphabetically  listed at 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]}

\item{date}{date in the form of "YYYY-mm-dd" for example "2016-01-01"}

\item{time}{time in the form of "H:M" or decimal hours only. For example "14:21" or 14.35}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{NASA.SSE}{list of  NASA SSE data dates in the form of "YYYY-mm-dd".
For example NASA.SSE=list(from=""2001-01-01"", to="2001-04-01"). Note that SSE date
is from 1983 to 2005: 
\url{https://power.larc.nasa.gov/}
[Accessed on 2016-5-6] 
if the  parameter is not provided, date parameter will be used 
provided longitude and latitude are provided.}

\item{folder}{The path of the folder where the data can be written.}
}
\value{
WMO and NASE.SSE. The WMO contains hour and day data. The NASE.SSE contains 
meta and data. See example below.
}
\description{
The function retrieves weather data from the World Meteorological Organization
 and NASA SSE. The retrieval is based on airports  IATA or ICAO codes or WMO weather station 
 codes. The actual WMO data is from Weather Underground . Internet connection  is therefore 
 needed before data can be accessed
}
\examples{
\dontrun{
#all hours data at Lincoln Airport, Nabraska
lincoln=weather(airport = "KLNK", date="2011-08-04")
lincoln$WMO$hour
lincoln$WMO$day
#current hour data at London
london.now=weather(airport = "LON", date=format(Sys.time(), "\%Y-\%m-\%d"),
time=format(Sys.time(), "\%H:\%M"))
london.now$WMO$hour
#NASA SSE data for Kumasi, Ghana
kumasi=weather(longitude = -1.5,latitude = 6.7,NASA.SSE = list(from="2001-04-01",
to="2002-04-01"))

#Hour, daily and SSE data at JFK 
JFK=weather(airport = "JFK", date="2001-02-02",time="17:20", latitude=40.6413, 
longitude=73.7781)
JFK$WMO$hour
JFK$WMO$day
JFK$NASA.SEE$data
JFK$NASA.SEE$meta

#' #Hour, daily and SSE slice data at Kotoka International Airport, ACCRA 
accra=weather(airport = "ACC", date="2001-02-02",time="17:20", latitude=40.6413, longitude=73.7781, 
NASA.SSE=list(from="2001-01-01",to="2004-01-01"))
accra$WMO$hour
accra$WMO$hour
accra$NASA.SEE$data
accra$NASA.SEE$meta

#Using wmo code
kumasi.now=weather(wmo="65442", date=format(Sys.time(), "\%Y-\%m-\%d"),time=format(Sys.time(), 
"\%H:\%M"))
kumasi.now$WMO$hour

#retrieve daily data for a specific period i.e 1 month
lincoln=weather(airport = "KLNK", date=c("2011-08-04","2011-09-04"))
#kumasi and accra data for a time slice. May not work for different 
#geographical regions; write the results to a file
accrakumasi=weather(airport ="ACC", date=c("2011-08-04","2011-10-04"),
folder="C:/Users/george/Documents/")
}
}
\references{
\itemize{
\item Worldwide Station List.\url{https://www.wetterzentrale.de/} 
[Accessed on 2016-5-6] 
\item List of airports by IATA code. 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code} [Accessed on 2016-5-6] 
\item Weather Underground \url{https://www.wunderground.com/} [Accessed on 2016-5-6]
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S-SEBI.R
\name{ssebi}
\alias{ssebi}
\alias{ssebi.default}
\title{S-SEBI Evapotranspiration Fraction}
\usage{
ssebi(
  Ts,
  albedo = NULL,
  threshold = 0.2,
  plot = TRUE,
  Rn24 = NULL,
  folder = NULL,
  welev = NULL
)

\method{ssebi}{default}(
  Ts = NULL,
  albedo = NULL,
  threshold = 0.2,
  plot = TRUE,
  Rn24 = NULL,
  folder = NULL,
  welev = NULL
)
}
\arguments{
\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{threshold}{numeric. Albedo inflection point threshold}

\item{plot}{logical:TRUE or FALSE. To indicate if histograms or triangle 
should be plotted}

\item{Rn24}{A 24 hour net radiation [W/m2]. It is needed to estimate daily ET. 
It can be computed with \code{\link{ETo}} with even a minimal data of 
Tmax and Tmin. 
 You can also point to raster file on your computer}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}

\item{welev}{Weather station elevation in meters}
}
\value{
EF, amax,bmax,amin,bmin
}
\description{
\code{ssebi} computes Roerink et. al (2000) S-SEBI: A simple
 remote sensing algorithm to estimate the surface energy balance.
}
\examples{
\dontrun{
#Manual data specification
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
mod=ssebi(Ts=Ts,albedo=albedo,threshold=0,plot=TRUE)

#Using landsat folder
folder=system.file("extdata","stack",package="sebkc")
ssebiauto=ssebi(folder=folder,welev=278)
}
}
\references{
Roerink, G. J., Su, Z., & Menenti, M. 2000. S-SEBI: 
A simple remote sensing algorithm to estimate the surface energy balance.
Physics and Chemistry of the Earth, Part B: Hydrology, Oceans and Atmosphere
, 25(2): 147-157.
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coldhotxy.R
\name{coldhot}
\alias{coldhot}
\alias{coldhot.default}
\title{Automatic computation of hot and cold pixels from Radiometric Surface Temperature (Ts)}
\usage{
coldhot(Tmin, Tmax, Ts = NULL, folder = NULL, welev = NULL)

\method{coldhot}{default}(Tmin, Tmax, Ts = NULL, folder = NULL, welev = NULL)
}
\arguments{
\item{Tmin}{Numeric. Minimum surface Temperature in  map units (eg. Kilvin) that you want computation from. 
You can keep changing untill you get your desired values}

\item{Tmax}{Numeric. Maximum surface Temperature in map units (eg. Kilvin) that you want computation from. 
You can keep changing untill you get your desired values}

\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}

\item{welev}{Weather station elevation in meters}
}
\value{
\describe{
\item{Ts:}{ The selected or input Ts}
\item{Tshot:}{ The Temperature of hot pixel}
\item{Tscold:}{ The Temperature of cold pixel}
\item{xyhot: }{combined x and y coordinates}
\item{candidates:}{ The similar candidates' pixels that can be used}
}
}
\description{
This function computes temperature coordinates of a hot and cold pixels based on only
Surface Temperature. This function uses ranges of temperature.
}
\details{
The function first divides the selected area into a 
number of clusters. The mean and standard deviations are
computed. The cluster means with the lowest standard deviations
and qualifying for upper (Ts) and lower (NDVI) limits 
(see above) are selected. The provided upper and lower probabilities
 are also used to select the final Ts.
}
\examples{
\dontrun{
 folder=system.file("extdata","stack",package="sebkc")
  model=coldhot(Tmin=300,Tmax=310,folder=folder,welev=170)
  spplot(model$Tshot)
  spplot(model$Tscold)
  modauto=sebal(folder = folder,welev = 380,xycold=model$xycold,xyhot=model$xyhot)}
}
\references{
Allen, R. G., Burnett, B., Kramber, W., Huntington, J., 
Kjaersgaard, J., Kilic, A., Kelly, C., & Trezza, R. 2013. Automated 
Calibration of the METRIC-Landsat Evapotranspiration Process. 
JAWRA Journal of the American Water Resources Association, 49(3): 563-576.
}
\seealso{
\code{\link{coldTs}},  \code{\link{hotTs}}, \code{\link{coldTs2}}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsebWrite.R
\name{writesebkc}
\alias{writesebkc}
\title{Writing sebkc object to a folder}
\usage{
writesebkc(object, folder = NULL, xy = NULL, overwrite = TRUE)
}
\arguments{
\item{object}{sebkc object}

\item{folder}{Folder the files should be written to.If it is set to NULL,
it is written to the input folder of sebkc object.}

\item{xy}{A dataframe of of xy coordinates in in decimal degrees or 
meters in the order of c(x,y). If it is not set NULL,
 the corresponding values are extracted and written to the folder}

\item{overwrite}{logical whether the file should be over written}
}
\value{
Writes output to a folder
}
\description{
This function writes spatial output of the following functions
\code{\link{sebal}}, \code{\link{sebi}},\code{\link{ssebi}},
\code{\link{wdi}},\code{\link{sebs}},\code{\link{sseb}},
\code{\link{tseb}} and \code{\link{landsat578}}
}
\examples{
\dontrun{
folder=system.file("extdata","stack",package="sebkc")
stack=landsat578(data=folder, welev=362)
writesebkc(stack)
sebaloutput=sebal(folder = folder,welev = 317)
writesebkc(sebaloutput)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TSEB.R
\name{tseb}
\alias{tseb}
\alias{tseb.default}
\title{Computing Two Source Surface Energy Balance (TSEB) in R}
\usage{
tseb(
  albedo,
  Ts,
  LAI,
  DOY,
  xyhot = "auto",
  xycold = "auto",
  TA = "sebal",
  Tmax,
  Tmin,
  RHmax = NULL,
  RHmin = NULL,
  n = NULL,
  latitude = NULL,
  zom = NULL,
  NDVI,
  SAVI,
  hc = 2,
  DEM = NULL,
  viewangle = 0,
  sunelev,
  welev,
  u = 2,
  s = 2,
  C = 90,
  lapse = 0.0065,
  Rn24 = NULL,
  ETr = NULL,
  ETr24 = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  time = NULL,
  Lz = NULL,
  Lm = NULL,
  t1 = 1,
  zx,
  zomw,
  xPT = 1.3,
  rc = c(70, 1000, 100),
  series = "xPT",
  clump = 1,
  fg = 1,
  fc = "auto",
  network = "parallel",
  clip = NULL,
  folder = NULL,
  iter.max = 7
)

\method{tseb}{default}(
  albedo = NULL,
  Ts = NULL,
  LAI = NULL,
  DOY = NULL,
  xyhot = "auto",
  xycold = "auto",
  TA = "sebal",
  Tmax,
  Tmin,
  RHmax = NULL,
  RHmin = NULL,
  n = NULL,
  latitude = NULL,
  zom = NULL,
  NDVI,
  SAVI,
  hc = 2,
  DEM = NULL,
  viewangle = 0,
  sunelev,
  welev,
  u = 2,
  s = 2,
  C = 90,
  lapse = 0.0065,
  Rn24 = NULL,
  ETr = NULL,
  ETr24 = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  time = NULL,
  Lz = NULL,
  Lm = NULL,
  t1 = 1,
  zx = 200,
  zomw = 2,
  xPT = c(1.26, 0, 0.1),
  rc = c(50, 1000, 100),
  series = "PM",
  clump = 1,
  fg = 1,
  fc = "auto",
  network = "series",
  clip = NULL,
  folder = NULL,
  iter.max = 7
)
}
\arguments{
\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{LAI}{Leaf Area Index, dimensionless}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{xyhot}{numeric or "auto". A list of x and y coordinates of a 
hot pixel in the form c(x,y). If it is set to "auto", \code{\link{hotTs}} 
will be used to compute it.}

\item{xycold}{numeric or "auto". A list of x and y coordinates of a 
cold pixel in the form of c(x,y). If it set to "auto", \code{\link{coldTs}} 
will be used to compute it.}

\item{TA}{Air temperature[K]. It can take "sebal" or 'Tmax' or 'Tmin' or numeric value.
When it is set to sebal, TA = Ts-dT. The 'Tmax' and 'Tmin' correspond to maximum and 
minimum temperature.}

\item{Tmax}{Numeric. Maximum air Temperature in degree Celsius}

\item{Tmin}{Numeric. Minimum air Temperature in degree Celsius}

\item{RHmax}{Maximum Relative Humidity in percent}

\item{RHmin}{Minimum Relative Humidity in percent}

\item{n}{Sunshine hours}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{zom}{the momentum roughness length for each pixel [m]. 
You can also point to a raster file on your computer}

\item{NDVI}{A RasterLayer data that indicates NDVI 
(Normalized Difference Vegetation Index) values. You can also point to a 
file on your computer.}

\item{SAVI}{A RasterLayer data that indicates 
SAVI(Soil-adjusted Vegetation Index) values. 
 You can also point to file on your computer.}

\item{hc}{numeric. it is canopy height[m]}

\item{DEM}{A digital elevation model[m]}

\item{viewangle}{angle of view of the sensor in degrees}

\item{sunelev}{Angle of Sun elevation in degrees, 
as found in the meta data of the satellite image}

\item{welev}{Weather station elevation in meters}

\item{u}{The satellites  overpass time wind speed at the weather station [m/s]}

\item{s}{numeric. Mean leaf size; it is four times the leaf 
area divided by the perimeter. The default is 2 (Grace et al., 1981).}

\item{C}{coefficient. Weighting a coefficient in the equation for leaf 
boundary layer. A value of 90 s^0.5/m (Norman et al, 1995).}

\item{lapse}{A local lapse rate that is applied to correct DEM [K/m]. 
Default value is 0.0065}

\item{Rn24}{A 24 hour net radiation [W/m2]. It is needed to estimate daily ET. 
It can be computed with \code{\link{ETo}} with even a minimal data of 
Tmax and Tmin. 
 You can also point to raster file on your computer}

\item{ETr}{an alfalfa or grass reference ET at the time of satellites  overpass. 
It only needed if you set model to "metric" [mm/hr]}

\item{ETr24}{daily grass or alfalfa reference Evapotranspiration (mm/day)}

\item{wmo}{numeric. World Meteorological Organization weather station code.
You can use the  Worldwide Station List at 
\url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
\url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]}

\item{airport}{numeric. IATA or ICAO code. They are alphabetically  listed at 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]}

\item{Krs}{Relationship between the fraction of extraterrestrial radiation that reaches the 
earth's surface, Rs/Ra, and the air temperature difference Tmax - Tmin for
interior (Krs = 0.16) and coastal (Krs = 0.19) regions}

\item{surface}{character. It is either "grass" or "alfalfa"}

\item{time}{The midpoint of the time of measurement[hour]; for example
time is 12.5 for a period between 12:00 and 13:00}

\item{Lz}{longitude of the centre of the local time zone [degrees west of Greenwich].
MUST BE POSITIVE
Lz = 0 degrees for Greenwich.}

\item{Lm}{longitude of the measurement site [degrees west of Greenwich]. MUST BE POSITIVE}

\item{t1}{numeric. The length of the calculation period in hour; 1 for hour, 0.5 for 30 minutes 0.25 for 15 minutes}

\item{zx}{The height above the weather station where the 
wind speed is measured [m]}

\item{zomw}{the roughness length for the weather station surface [m]}

\item{xPT}{coefficient(s). The Priestley-Taylor equation coefficient (Norman et al, 2005).
It can take a single coefficient or in the form c(initial, minimum, increment). 
Example is xPT=c(1.26,1,0.1). It must be noted this is a decreasing increment 
from the initial value .i.e 1.26. The subsequent increment is only meant for 
a cell where LEs is negative.}

\item{rc}{coefficient. Penman-Monteith equation 
coefficient (Colaizzi et al,2012). It can take a single coefficient 
or in the form c(initial, maximum, increment). 
Example is rc=c(0,1000,100). Unlike xPT, this is positively incremented.}

\item{series}{character.The type of series. It takes "xPT" for 
Priestley-Taylor equation(Norman, 1995) or 
"PM" for Penman-Monteith equation (Colaizzi et al, 2012)}

\item{clump}{coefficient. A canopy clumping coefficient used for 
calculation of directional  fc (fractional cover of vegetation)}

\item{fg}{numeric. The fraction of LA1 that is green. If no information is
available on fg, then it is assumed to be unity (1).}

\item{fc}{fraction (0-1).Directional vegetation cover. 
The default uses Norman et al(1995) fc equation}

\item{network}{character. It takes either "parallel" (Norman et al, 1995)  or 
"series" (Colaizzi et al, 2012; Norman et al, 1995).}

\item{clip}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform the data to be cropped. for example clip can take the form of c(xmin, xmax, ymin, ymax)}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}

\item{iter.max}{maximum iterations of sensible heat calculation.}
}
\value{
\describe{
\item{LEc:}{ canopy latent heat flux [W/m2]}
\item{LEs:}{ Soil latent heat flux[W/m2]}
\item{T24:}{ 24 hour Transpiration [mm/day]}
\item{E24:}{ 24 hour Evaporation [mm/day]}
\item{ET24:}{ 24 hour Evapotranspiration [mm/day]}
\item{EF:}{ ET24 (daily Evapotranspiration) Evaporative Fraction. 
EF corresponds to the type of scaling method used: see \code{ETohr} for details}
\item{EFs:}{ E(Evaporation) Evaporative Fraction}
\item{EFc: }{ T (Transpiration) Evaporative Fraction}
}
}
\description{
This function estimates evaporation (E), transpiration (T), 
and evapotranspiration(ET) of land surface. The computation is based on
Norman et al (1995) and Colaizzi et al(2012) models.
}
\details{
\describe{
 \item{\strong{Theory}}{This function separately computes LEs, LEs using
 Net radiation (Rn), sensible heat flux (H) and soil heat flux (G).
 The Rn was first computed with \code{sebal}. The soil net radiation computed
 as  \code{Rns=(sebal$Rn*exp(0.9*log(1-fc)))} . The canopy radiation was computed 
 as  \code{Rnc=sebal$Rn-Rns} . The G was computed from \code{sebal}. 
 The canopy sensible heat (Hc) and soil sensible heat (Hs) were computed
 based on corresponding canopy Temperature (Tc) and soil temperature (Tsoil).
 Two different approaches were employed to sensible heat fluxes. The first
  is Norman et al(1995) parallel network. The second is Norman (1995) 
  secant series but uses Priestley-Taylor equation (xPT).   }
 \item{\strong{Default Parameter}}{These parameters have been 
 set based on literature }
 }
}
\examples{
\dontrun{
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
Parallel=tseb(Ts=Ts,LAI=LAI,DOY=37,xyhot="full",
xycold="full",albedo=albedo,Tmax=31,
Tmin=28,RHmax=NULL,RHmin=NULL,zom=NULL,NDVI=NDVI,SAVI=NULL,hc=20,
DEM=NULL,viewangle=0,sunelev=50,welev=345,u=2,
s=2,C=90,lapse=0.0065,Rn24=NULL,zx=200,zomw=2,
xPT=1.3,clump=1,fg=1,fc="auto",network="parallel",
latitude=5.6,n=6.5)

series=tseb(Ts=Ts,LAI=LAI,DOY=37,xyhot="full",
xycold="full",albedo=albedo,Tmax=31,
Tmin=28,RHmax=NULL,RHmin=NULL,zom=NULL,NDVI=NDVI,SAVI=NULL,hc=20,
DEM=NULL,viewangle=0,sunelev=50,welev=345,u=2,
s=2,C=90,lapse=0.0065,Rn24=NULL,zx=200,zomw=2,
xPT=1.3,clump=1,fg=1,fc="auto",network="series",
latitude=5.6,n=6.5)

#using landsat folder parameters
folder=system.file("extdata","stack",package="sebkc")
tsebautoseries=tseb(folder=folder,welev=317,Tmax=31,Tmin=27,
latitude=5.6,n=6.5)
tsebautoparallel=tseb(folder=folder,welev=317,Tmax=31,Tmin=27,
latitude=5.6,n=6.5,network="parallel")

#3 step simulation using landsat 7 with SLC OFF
##step one: prepare the input data 
### point to landsat 7 data with strips 

#####get the data stacked and remove strips
rawdata=sebkcstack(folder)
###### check if strips are removed
plot(rawdata$data)
#get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
welev=278
 data=landsat578(rawdata,welev=welev)
 #perform semi-auto simulation 
# Determine xyhot. Digitize polygon on the Ts map
modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#determine the cold. Digitize polygon on the Ts map
modcold=coldTs(data,welev=275,extent="auto",cluster=2)
xyhot=modhot$xyhot
xycold=modcold$xycold
modTSEB2=tseb(data,Tmax=31,Tmin=28,
latitude=5.6,n=6.5)
#the same
modTSEB2=tseb(data,Tmax=33,Tmin=28,welev=345,xyhot=xyhot,xycold=xycold,
latitude=5.6,n=6.5)
#the same
modTSEB2=tseb(data,Tmax=33,Tmin=28,welev=345,xyhot=modhot,xycold=modcold,
latitude=5.6,n=6.5)
}

}
\references{
\itemize{
\item Norman JM, Kustas WP, Humes KS (1995). A two-source approach for estimating 
soil and vegetation energy fluxes from observations of directional 
radiometric surface temperature. Agric For Meteorol 1995;77:263-93.
\item Colaizzi, Paul; Kustas, William P.; Anderson, Martha C.; 
Agam, Nurit; Tolk, Judy A.; Evett, Steven R.; Howell, Terry A.; 
Gowda, Prasanna H.; and O'Shaughnessy, Susan A., 
"Two-source energy balance model estimates of evapotranspiration using 
component and composite surface temperatures" (2012). Publications 
from USDA-ARS / UNL Faculty. Paper 1147.
http://digitalcommons.unl.edu/usdaarsfacpub/1147
\item Grace, J., 1981. Some effects of wind on plants. 
In: J. Grace, E.D. Ford and P.G. Jarvis (Editors), Plants and
their Atmospheric Environment. Blackwell Scientific, London, pp. 31-56.
\item Ronglin Tang, Zhao-Liang Li, Yuanyuan Jia, Chuanrong Li, 
Kun-Shan Chen, Xiaomin Sun & Jinyong Lou (2012): 
Evaluating one- and two-source energy balance models in
estimating surface evapotranspiration from Landsat-derived 
surface temperature and field
measurements, International Journal of Remote Sensing.
http://dx.doi.org/10.1080/01431161.2012.716529
\item KUSTAS, W. & ANDERSON, M. 2009. Advances in thermal infrared remote 
sensing for land surface modeling. Agricultural and Forest Meteorology, 149,
 2071-2081.
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/landsat578.R
\name{landsat578}
\alias{landsat578}
\alias{landsat578.default}
\title{Radiometric correction and processing of Landsat Images}
\usage{
landsat578(
  data,
  welev,
  sensor = NULL,
  gain = "low",
  sunelev = NULL,
  date = NULL,
  weights = "auto",
  grescale = "auto",
  brescale = "auto",
  ESUNx = "auto",
  Mp = NULL,
  Ap = NULL,
  K1 = NULL,
  K2 = NULL,
  Rp = 0.91,
  Rsky = 1.32,
  tNB = 0.886,
  DOY = NULL,
  clip = NULL,
  folder = NULL
)

\method{landsat578}{default}(
  data = NULL,
  welev,
  sensor = NULL,
  gain = "low",
  sunelev = NULL,
  date = NULL,
  weights = "auto",
  grescale = "auto",
  brescale = "auto",
  ESUNx = "auto",
  Mp = NULL,
  Ap = NULL,
  K1 = NULL,
  K2 = NULL,
  Rp = 0.91,
  Rsky = 1.32,
  tNB = 0.886,
  DOY = NULL,
  clip = NULL,
  folder = NULL
)
}
\arguments{
\item{data}{An original folder containing Landsat data or
Raster \code{\link[raster]{brick}}  or raster 
\code{\link[raster]{stack}}. Arrange the bands in ascending order:
 c(band1,band2,band3,band4,band5,band6,band7) for Landsat 5 and 7.
 And a similar  order for Landsat 8.  On folder see \code{\link{sebkcstack}}
 See also details.}

\item{welev}{Weather station elevation in meters}

\item{sensor}{Numeric. Type of landsat satellite; it takes 5,7, or 8}

\item{gain}{Band-specific sensor gain. It takes either "high" or "low"}

\item{sunelev}{Angle of Sun elevation in degrees, 
as found in the meta data of the satellite image}

\item{date}{DATE_ACQUIRED of the image, see metadata}

\item{weights}{albedo bands weights for each band.
list as in order c(band1,band2,band3,band4,band5,band6,band7) 
for landsat 5 and 7.
 The default is "auto" where 
sebal weights are used.}

\item{grescale}{list as in order weights. Band-specific sensor values. 
The default for landsat 5 and 7 is from Chander et.al (2009).
For Landsat 8 use ML Radiance scale in the metadata;
i.e. Band specific multiplicative rescaling factor from the metadata 
(MTL file) (RADIANCE_MULT_BAND_x, where x is the band number).}

\item{brescale}{list. Band-specific sensor values.
The default for landsat 5 and 7 is from Chander et.al (2009).
for Landsat 8 use AL Radiance scale in the metadata;
i.e. Band specific multiplicative rescaling factor from the metadata 
(MTL file) (RADIANCE_MULT_BAND_x, where x is the band number).}

\item{ESUNx}{list. The mean solar exo-atmospheric irradiance for each band.
It is used for reflectance correction. The default is taken from SEBAL manual}

\item{Mp}{Reflectance scale for Landsat 8. Band specific multiplicative 
rescaling factor from the metadata (MTL file) (REFLECTANCE_MULT_BAND_x, 
where x is the band number).}

\item{Ap}{Reflectance scale for Landsat 8. Band specific additive rescaling
factor from the metadata (MTL file) (REFLECTANCE_ADD_BAND_x,
where x is the band number)}

\item{K1}{Temperature constants for each Landsat image. 
Default is taken from landsat handbook}

\item{K2}{Temperature constants for each Landsat image. 
Default is taken from landsat handbook}

\item{Rp}{path radiance in the 10.4-12.5 um band.
MODTRAN runs can accurately estimate this. 
The default is taken from Allen et al(2007) 
to be 0.91,}

\item{Rsky}{narrow band downward thermal radiation. 
MODTRAN runs can accurately estimate this.
The default is taken from Allen et al(2007) to be 1.32}

\item{tNB}{narrow band transmissivity of air (10.4-12.5) um range.
MODTRAN runs can accurately estimate this.
The default is taken from Allen et al(2007) to be 0.866}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{clip}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform the data to be cropped. for example clip can take the form of c(xmin, xmax, ymin, ymax)}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}
}
\value{
\describe{
\item{albedo:}{ albedo values}
\item{Ts:}{ Surface Temperature}
\item{NDVI:}{ NDVI values}
\item{SAVI:}{ SAVI values}
\item{fc:}{ Fractional vegetation cover}
\item{radiance:}{ radiance values}
\item{reflectance:}{ TOA reflectance}
}
}
\description{
\code{landsat578} performs radiometric collection of 
landsat 5,7,and 8 images. It returns albedo, NDVI, 
fractional vegetation cover, surface temperature
SAVI,radiance and reflectance of the image
}
\details{
If you set the data to landsat folder that contains 
landsat band and meta data \code{\link{sebkcstack}} is used to
compute the parameters. The user must specify welev and may
use default parameters. 
 The data must always be a raster brick 
or stack. To use the default values please order bind the bands
 c(band1,band2,band3,band4,band5,band6,band7). 
 The defaults values are for landsat 5 and 7. 
For landsat 8, use the meta data.
}
\examples{
\dontrun{
#Automatic detection of parameters
folder=system.file("extdata","stack",package="sebkc")
modauto=landsat578(data=folder, welev=362)

#Manual estimation with input parameters
landsat7_6feb2004=brick(system.file("extdata","landsat7_6feb2004.grd",
package="sebkc"))
mod=landsat578(data=landsat7_6feb2004,welev=317.1,sensor=7,gain="high",
sunelev=50.71154048,weights="auto",grescale="auto",brescale="auto",
ESUN="auto",K1=666.09,K2=1282.71,date="2002-2-6")
#acces albedo value
albedo=mod$albedo
}
}
\references{
\itemize{
\item Chander, G., Markham, B. L., & Helder, D. L. 2009. 
Summary of current radiometric calibration coefficients for 
Landsat MSS, TM, ETM+, and EO-1 ALI sensors. 
Remote Sensing of Environment, 113(5): 893-903.
\item Waters, R. 2002. SEBAL: Surface Energy Balance Algorithms for Land. 
Idaho Implementation, Advanced Training and Users Manual. Kimberly, Idaho: 
University of Idaho.
\item Survey, U. S. G. 2015. Landsat 8 (L8) data users handbook: 97p.
 from https://landsat.usgs.gov/documents/Landsat8DataUsersHandbook.pdf
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kumasi.R
\docType{data}
\name{kumasircp452016_2050}
\alias{kumasircp452016_2050}
\title{A 4.5 RCP downscaled Weather data of Kumasi from 2016 to 2050}
\format{
A data frame with 12784 rows and 8 variables:
\describe{
  \item{DOY}{Day of the Year in dates}
  \item{Tmax}{Maximum Temperature [oC]}
  \item{Tmin}{Minimum Temperature [oC]}
  \item{RHmin}{Minimum Relative Humidity [per cent]}
  \item{RHmax}{Maximum Relative Humidity [per cent]}
  \item{n}{sunshine hours}
  \item{uz}{wind speed [m/s]}
  \item{P}{Daily precipitation [mm]}
}
}
\description{
A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
and minimum and maximum Relative Humidity. The RCP 4.5 assumes that green house 
gas will peak in 2040 and declines throughout 21st Century.
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitting.R
\name{fitting}
\alias{fitting}
\title{Goodness of fit Test}
\usage{
fitting(obs, est, P = 1)
}
\arguments{
\item{obs}{Observed data}

\item{est}{Estimated data}

\item{P}{Number of parameters}
}
\value{
\describe{
\item{r2:}{  R-square [-]}
 \item{r2_adj:}{ An adjusted R-square [-]}
\item{d:}{ Index of agreement[-]}
\item{p_value:}{ p value}
\item{sse:}{ Sum of square error}
\item{sst:}{ Sum of square total}
\item{AIC:}{ AIC}
\item{BIC:}{ BIC}
\item{RMSE:}{ Root Mean Square Error}
\item{RMSE_adj:}{ adjusted Root Mean Square Error}
\item{NRMSE}{ Normalised Root Mean Square Error}
\item{NRMSE_adj:}{ adjusted Normalised Root Mean Square Error}
\item{E:}{ Nash Sutcliffe Coefficient}
\item{b:}{ coefficients of regression}
\item{AAE:}{ average absolute error}
\item{ARE:}{ average relative error}
\item{MBE:}{ Mean Bias  error}
\item{MPE:}{ Mean Percentage error}
}
}
\description{
Goodness of fit Test
}
\examples{
file=system.file("extdata","sys","irrigation.txt",package="sebkc")
data=read.table(file,header=TRUE)  
obs=data$ETo
est=data$ETc
P=4
mod=fitting(obs,est,P)
r2=mod$r2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SSEB.R
\name{sseb}
\alias{sseb}
\alias{sseb.default}
\title{Simplified Surface Energy Balance}
\usage{
sseb(
  Ts,
  welev = NULL,
  TH = "auto",
  TC = "auto",
  sunelev = NULL,
  NDVI = NULL,
  DEM = NULL,
  albedo = NULL,
  NDVImax = 0.7,
  cc = 0.65,
  KL = 0.0065,
  ETo = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  x = 1,
  Tmax = NULL,
  Tmin = NULL,
  zx = NULL,
  u = 2,
  DOY = NULL,
  latitude = NULL,
  n = NULL,
  RHmax = NULL,
  RHmin = NULL,
  clip = NULL,
  folder = NULL
)

\method{sseb}{default}(
  Ts = NULL,
  welev = NULL,
  TH = "auto",
  TC = "auto",
  sunelev = NULL,
  NDVI = NULL,
  DEM = NULL,
  albedo = NULL,
  NDVImax = 0.7,
  cc = 0.65,
  KL = 0.0065,
  ETo = NULL,
  wmo = NULL,
  airport = NULL,
  Krs = 0.16,
  surface = "grass",
  x = 1.2,
  Tmax = NULL,
  Tmin = NULL,
  zx = NULL,
  u = 2,
  DOY = NULL,
  latitude = NULL,
  n = NULL,
  RHmax = NULL,
  RHmin = NULL,
  clip = NULL,
  folder = NULL
)
}
\arguments{
\item{Ts}{A RasterLayer data that indicates radiometric 
surface temperature values from remote sensing image, 
preferably in Kelvin (K). You can also point to raster file on your 
computer.}

\item{welev}{Weather station elevation in meters}

\item{TH}{the average of the representative 
3 hot pixels selected for hot "bare" areas. 
If it is set to "auto" TH will be estimated from \code{\link{hotTs}}}

\item{TC}{the average of representative 3 cold 
pixels selected from the irrigated fields. 
When it is set to "auto" TC will be estimated from \code{\link{coldTs}}}

\item{sunelev}{Angle of Sun elevation in degrees, 
as found in the meta data of the satellite image}

\item{NDVI}{A RasterLayer data that indicates NDVI 
(Normalized Difference Vegetation Index) values. You can also point to a 
file on your computer.}

\item{DEM}{A digital elevation model[m]}

\item{albedo}{A RasterLayer data that has albedo values. 
You can also provide file path or location on your computer}

\item{NDVImax}{numeric, NDVI value for heavily green-vegetated area in the image}

\item{cc}{numeric, the correction coefficient if the NDVI approaches 0.0}

\item{KL}{numeric, the assumed lapse rate of air moving along the terrain}

\item{ETo}{numeric, a standardized clipped grass reference ET. when it is set to NULL,
it is automatically estimated from \code{\link{ETo}}}

\item{wmo}{numeric. World Meteorological Organization weather station code.
You can use the  Worldwide Station List at 
\url{https://www.wunderground.com/}  [Accessed on 2016-5-6] or 
\url{https://www.wetterzentrale.de/} [Accessed on 2016-5-6]}

\item{airport}{numeric. IATA or ICAO code. They are alphabetically  listed at 
\url{https://en.wikipedia.org/wiki/List_of_airports_by_IATA_code:_A} [Accessed on 2016-5-6]}

\item{Krs}{Relationship between the fraction of extraterrestrial radiation that reaches the 
earth's surface, Rs/Ra, and the air temperature difference Tmax - Tmin for
interior (Krs = 0.16) and coastal (Krs = 0.19) regions}

\item{surface}{character. It is either "grass" or "alfalfa"}

\item{x}{multiplier needed  to estimate ET for tall, full cover crops 
such as alfalfa, corn and wheat. Default is 1.2.}

\item{Tmax}{Numeric. Maximum air Temperature in degree Celsius}

\item{Tmin}{Numeric. Minimum air Temperature in degree Celsius}

\item{zx}{The height above the weather station where the 
wind speed is measured [m]}

\item{u}{The satellites  overpass time wind speed at the weather station [m/s]}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{n}{Sunshine hours}

\item{RHmax}{Maximum Relative Humidity in percent}

\item{RHmin}{Minimum Relative Humidity in percent}

\item{clip}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform the data to be cropped. for example clip can take the form of c(xmin, xmax, ymin, ymax)}

\item{folder}{An original  directory of the images that contains all 
landsat 5,7 or 8 bands and metadata.
At the moment only Landsat folder is supported}
}
\value{
\describe{
\item{ET24:}{ Actual 24 hour ET}
\item{EF:}{ ET fraction}
\item{ETo:}{ 24 hour reference ETo}
}
}
\description{
Senay etal (2011) Simplified Surface Energy Balance (SSEB) 
approach for estimating landscape ET
}
\examples{
\dontrun{
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
Ts=raster(system.file("extdata","Ts.grd",package="sebkc"))
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
#minimal data
modsseb=sseb(Ts=Ts,TH="full",TC="full",sunelev=50,NDVI=NDVI,DEM=NULL,
albedo=albedo,NDVImax=0.7,cc=0.65,KL=0.0065,
ETo="auto",x=1.2,Tmax=31,Tmin=28,zx=10,
u=2,DOY=37,latitude=5.6,n=NULL,RHmax=NULL,RHmin=NULL)

#uaing the folder parameters
folder=system.file("extdata","stack",package="sebkc")
ssebauto=sseb(folder=folder,welev=317,Tmax=31,Tmin=28, latitude=5.6)

#Another interactive example
#get Ts, Albedo, NDVI, SAVI, sunelev, DOY from landsat data 
welev=278
data=landsat578(rawdata,welev=welev)
#perform semi-auto simulation 
#Determine xyhot. Digitize polygon on the Ts map
modhot=hotTs(data,welev=300,extent="auto",cluster=2)
#determine the cold. Digitize polygon on the Ts map
modcold=coldTs(data,welev=275,extent="auto",cluster=2)
xyhot=modhot$Tshot
xycold=modcold$Tscold
#use object of \code{\link{coldTs}} or \code{\link{hotTs}} in different ways
modsseb=sseb(Ts=Ts,TH=xyhot,TC=modcold,sunelev=50,NDVI=NDVI,DEM=NULL,
albedo=albedo,NDVImax=0.7,cc=0.65,KL=0.0065,
ETo="auto",x=1.2,Tmax=31,Tmin=28,zx=10,
u=2,DOY=37,latitude=5.6,n=NULL,
RHmax=NULL,RHmin=NULL)
}
}
\references{
Senay, G. B., Budde, M. E., & Verdin, J. P. 2011. 
Enhancing the Simplified Surface Energy Balance (SSEB) approach 
for estimating landscape ET: Validation with the METRIC model. 
Agricultural Water Management, 98(4): 606-618.
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomass.R
\name{biomass}
\alias{biomass}
\title{Biomass and yield estimation of crops and plants}
\usage{
biomass(
  Tday = NULL,
  T24,
  Rs,
  latitude,
  longitude = NULL,
  HI = NULL,
  plantdate,
  harvestdate,
  c30 = 0.0108,
  adaptability = 3,
  LAI = 5,
  crop = NULL
)
}
\arguments{
\item{Tday}{numeric. Average Day time temperature in degrees Celsius 
for the growing period. A Remote sensing surface Temperature can also be used}

\item{T24}{numeric. Average 24-hour (including night hours) temperature 
in degrees Celsius for the growing period.}

\item{Rs}{A shortwave radiation data [ca^-2 day^-1]. 
RasterLayer data can be supplied.}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{HI}{numeric. Harvest Index. It can be list or spatial}

\item{plantdate}{character. A date of planting in the form "YYYY-mm-dd"}

\item{harvestdate}{character. A date of harvest in the form "YYYY-mm-dd"}

\item{c30}{numeric. A maintenance respiration coefficient (c30) at 30 oC. 
it takes either a value of 0.0108 for non-legume crop or 0.0283 for legumes}

\item{adaptability}{numeric. The  climate adaptability group  number. 
It takes values of 1,2,3,or 4. it depends on the temperature profile of 
study area. See references (eg. FAO (1981)) for more explanation.}

\item{LAI}{Leaf Area Index, dimensionless}

\item{crop}{character. A crop type. For example "maize"}
}
\value{
yield and biomass in t/ha
}
\description{
This function uses	Wageningen procedure 
(De Wit, 1968; Slabbers et al., 1979) to compute the 
point or spatial possible maximum yield of a standard crop based on a 
Rs data. This function can also perform internal interpolation of 
data if longitude and latitude are provided. See datails and examples below.
}
\details{
\describe{
 The function can be used in several ways:
 \item{\strong{Point Estimation}}{\code{biomass} can compute biomass 
 and yield for  one location. Each input data must be given only one value}
 \item{\strong{Point Estimation for several locations }}{The function can 
 compute biomass and yields for more than one location. Input data must 
 be a list or array.}
 \item{\strong{Spatial Estimation}}{Input data such as Rs, Tday, T24,
 HI or LAI can be spatial.}
 \item{\strong{Spatial Estimation with internal interpolation}}{It takes Tday,
  for example from satellite image, and do interpolation for 
 the rest of the data. Both longitude, latitude, and the variables must be of
 the same length. If longitude is not provided no interpolation will be done.}
 }
 
 @references 
\itemize{
\item De Wit, C. T. (1968). Plant production Miscellaneous Papers: 
Landbouw Hogeschool, Wageningen.
\item Slabbers, P. J., Herrendorf, V. S., & Stapper, M. (1979). 
Evaluation of simplified water-crop yield models. 
Agricultural Water Management, 2(2), 95-129. 
doi: http://dx.doi.org/10.1016/0378-3774(79)90026-X
\item FAO (1981). Report on the Agro-ecological Zones Project: 
Food and Agriculture Organization of the United Nations.
}
}
\examples{
\dontrun{
################# Point Estimation ###################
# Identify the climatic input data on 
latitude=11.18 #in decimal degrees
Rs= 452 #Average shortwave radiation over the growing period (cal-2day-1)
Tday = 26.9 #	Average day time difference (oC) over the growing period
T24 =25.3 #Average 24 hour mean temperature over the growing period
# Gather crop Information
crop="maize"
plantdate="2015-05-1" #Start date
harvestdate="2015-08-29" #end date 120 days
HI=0.4 #Harvest index for maize
adaptability=3 #crop climate adaptability group III
LAI = 4 #Leaf Area Index
pointmod=biomass(Tday,T24,Rs,latitude,longitude=NULL,HI,
plantdate,harvestdate,c30=0.0108,adaptability=3,LAI=5,crop=NULL)

################# Point Estimation for several locations#################
latitude=c(10,11.18,12,9)
longitude=c(-1.4,-2,-1,-2.6)
Rs= c(450,452,400,500)
Tday = c(25,26.9,24,20,30)
T24=c(24,25.3,23,28)
HI=c(0.4,0.4,0.5,0.25)
plantdate=c("1-03-2015","1-04-2015","1-05-2015")
harvestdate=c("29-08-2015","29-08-2015","29-08-2015")

# computing several points
pointSmod=biomass(Tday,T24,Rs,latitude,longitude=longitude,HI,
plantdate,harvestdate,c30=0.0108,adaptability=3,LAI=5,crop=NULL)
yield=pointSmod$yield #tabular data
biomass=pointSmod$output #tabular data
#access on biomass
pointSmod$output$yield
}
\dontrun{
############## Spatial Estimation with internal interpolation ###########################
#generate spatial data
folder=system.file("extdata","stack",package="sebkc")
modauto=landsat578(data=folder, welev=362)
Tday=modauto$Ts-273.15 #temperature in degree celcius
LAI=modauto$LAI
latitude=seq(7.544,7.590,0.001) #for interpolation
longitude=seq(-1.211,-1.187,0.001) #for interpolation
latitude=latitude[1:length(longitude)] #to make sure they are of the same length
spatialmodint=biomass(Tday,T24,Rs,latitude,longitude=longitude,HI,
plantdate,harvestdate,c30=0.0108,adaptability=3,LAI=LAI,crop=NULL)
# Rs, T24, and HI are interpolated
plot(spatialmodint$yield)
############## Spatial Estimation  ###########################
Rs= 452
T24 =25.3
HI=0.4 
spatialmod=biomass(Tday,T24,Rs,latitude,longitude=longitude,HI,
plantdate,harvestdate,c30=0.0108,adaptability=3,
LAI=LAI,crop=NULL)
plot(spatialmod$yield)
}
}
\author{
George Owusu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sebkc-package.R
\docType{package}
\name{sebkc-package}
\alias{sebkc}
\alias{sebkc-package}
\title{sebkc: Surface Energy Balance and Crop Coefficient Evapotranspiration
Estimation}
\description{
Computes and integrates surface energy balance components of
evapotranspiration into the FAO56 water balance model, providing the
SEBAL, METRIC, SEBI, SSEB, SEBS and TSEB models.
}
\details{
Generics that the 'raster' package also provides as methods (such as plot,
hist, quantile, text, lines and points) are intentionally not imported
here, because they are made available through import(raster); importing
them again would trigger "replacing previous import" warnings.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/gowusu/sebkc}
  \item Report bugs at \url{https://github.com/gowusu/sebkc/issues}
}

}
\author{
\strong{Maintainer}: George Owusu \email{owusugeorge@ug.edu.gh}

Authors:
\itemize{
  \item George Owusu \email{owusugeorge@ug.edu.gh}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomass.R
\name{lonlatreproject}
\alias{lonlatreproject}
\title{Reproject WGS 84 longitude and latitude  into a new raster coordinates and vice versa}
\usage{
lonlatreproject(x, y = NULL, var = NA, map, epsg = 4326)
}
\arguments{
\item{x}{a longitude or raster or vector object}

\item{y}{a latitude or raster or vector object}

\item{var}{numeric variable. The length should be same as the length of 
latitude and longitude}

\item{map}{Raster* object that will be used to define the new coordinates.}

\item{epsg}{coordinate reference number. The default is WGS 84: 4326}
}
\value{
New x, y and map
}
\description{
Reproject WGS 84 longitude and latitude  into a new raster coordinates and vice versa
}
\details{
If x is a list of longitudes then new  horizontal  coordinates based on map
coordinates. If x is a raster object with  horizontal  coordinates then it will be projected to
WGS 84.
}
\examples{
folder=system.file("extdata","stack",package="sebkc")
data=landsat578(data=folder, welev=362)
temp=data$Ts
latitude=seq(7.544,7.590,0.001)
longitude=seq(-1.211,-1.187,0.001)
var=seq(20,30,0.2)
latitude=latitude[1:length(longitude)]
var=var[1:length(longitude)]
newlonlat=lonlatreproject(longitude,latitude,var=var,map=temp)
longitude=newlonlat$longitude
map=newlonlat$map


}
\author{
George Owusu
}
